/* 
 * $Id: sportsdb-28-29-transition-postgresql-8.sql 3015 2013-01-14 22:34:57Z ian $
 *  
 * Script to update the sportsdb schema between version 28 and 29.
 */

/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_awards_part_type_part_id ON awards (participant_type, participant_id);

CREATE INDEX IDX_awards_dt_cov_type_dt_cov_id ON awards (date_coverage_type, date_coverage_id);

/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_locations_country_code ON locations (country_code);

CREATE INDEX IDX_locations_city ON locations (city);

CREATE INDEX IDX_locations_state ON locations (state);

CREATE INDEX IDX_locations_country ON locations (country);

/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_sports_property_sp_prop_type_sp_prop_id ON sports_property (sports_property_type, sports_property_id);

CREATE INDEX IDX_sports_property_formal_name ON sports_property (formal_name);

CREATE INDEX IDX_sports_property_value ON sports_property (value);

CREATE TABLE american_football_field_goal_stats (
    id SERIAL UNIQUE NOT NULL PRIMARY KEY,
    attempts INTEGER,
    made INTEGER
);

ALTER TABLE basketball_event_states ADD COLUMN score_team INTEGER;
ALTER TABLE basketball_event_states ADD COLUMN score_team_opposing INTEGER;
ALTER TABLE basketball_event_states ADD COLUMN score_team_home INTEGER;
ALTER TABLE basketball_event_states ADD COLUMN score_team_away INTEGER;
ALTER TABLE basketball_event_states ADD COLUMN record_type VARCHAR(40);

CREATE TABLE basketball_action_plays (
    id SERIAL UNIQUE NOT NULL PRIMARY KEY,
    basketball_event_state_id INTEGER,
    team_id INTEGER NOT NULL,
    play_key VARCHAR(100),
    play_type VARCHAR(100),
    score_attempt_type VARCHAR(100),
    points INTEGER,
    penalty_level VARCHAR(100),
    recipient_type VARCHAR(100),
    recipient_id INTEGER,
    comment VARCHAR(512)
);

ALTER TABLE basketball_action_plays
    ADD CONSTRAINT FK_bask_ev_sta_bask_act_plays
    FOREIGN KEY (basketball_event_state_id) REFERENCES basketball_event_states (id);

ALTER TABLE basketball_action_plays
    ADD CONSTRAINT FK_teams_basketball_action_plays
    FOREIGN KEY (team_id) REFERENCES teams (id);

CREATE TABLE basketball_action_participants (
    id SERIAL UNIQUE NOT NULL PRIMARY KEY,
    basketball_action_play_id INTEGER,
    person_id INTEGER,
    team_id INTEGER NOT NULL,
    participant_role VARCHAR(100)
);

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_persons_bask_act_play_part
    FOREIGN KEY (person_id) REFERENCES persons (id);

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_teams_bask_act_part
    FOREIGN KEY (team_id) REFERENCES teams (id);

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_bask_act_play_bask_act_part
    FOREIGN KEY (basketball_action_play_id) REFERENCES basketball_action_plays (id);

ALTER TABLE soccer_event_states ADD COLUMN score_team INTEGER;
ALTER TABLE soccer_event_states ADD COLUMN score_team_opposing INTEGER;
ALTER TABLE soccer_event_states ADD COLUMN score_team_home INTEGER;
ALTER TABLE soccer_event_states ADD COLUMN score_team_away INTEGER;
ALTER TABLE soccer_event_states ADD COLUMN record_type VARCHAR(40);

ALTER TABLE soccer_action_plays ADD COLUMN team_id INTEGER;
ALTER TABLE soccer_action_plays ADD COLUMN play_key VARCHAR(100);
ALTER TABLE soccer_action_plays ADD COLUMN points INTEGER;
ALTER TABLE soccer_action_plays ADD COLUMN penalty_level VARCHAR(100);
ALTER TABLE soccer_action_plays ADD COLUMN recipient_type VARCHAR(100);

/*  
    Run this command separately. 
    It should create a foreign key that allows null values in the soccer_action_plays table (MATCH SIMPLE)
    but if you get a complaint about null values leave this foreign key out. 
*/
ALTER TABLE soccer_action_plays
    ADD CONSTRAINT FK_teams_soccer_action_plays
    FOREIGN KEY (team_id) REFERENCES teams (id) MATCH SIMPLE;

ALTER TABLE soccer_action_participants ADD COLUMN team_id INTEGER;

/*  
    Run this command separately. 
    It should create a foreign key that allows null values in the soccer_action_participants table (MATCH SIMPLE)
    but if you get a complaint about null values leave this foreign key out. 
*/
ALTER TABLE soccer_action_participants
    ADD CONSTRAINT FK_teams_soccer_action_participants
    FOREIGN KEY (team_id) REFERENCES teams (id) MATCH SIMPLE;

ALTER TABLE soccer_offensive_stats ADD COLUMN goals_own INTEGER;
ALTER TABLE soccer_offensive_stats ADD COLUMN free_kicks INTEGER;

ALTER TABLE soccer_defensive_stats ADD COLUMN goals_own INTEGER;

ALTER TABLE documents ALTER title TYPE CHARACTER VARYING(512);
ALTER TABLE documents ALTER doc_id TYPE CHARACTER VARYING(255);

ALTER TABLE events ADD COLUMN end_date_time TIMESTAMP;

ALTER TABLE american_football_special_teams_stats ADD COLUMN fumbles_forced_special_teams INTEGER;

ALTER TABLE american_football_fumbles_stats ADD COLUMN fumbles_forced_other INTEGER;
ALTER TABLE american_football_fumbles_stats ADD COLUMN fumbles_forced_special_teams INTEGER;

ALTER TABLE american_football_offensive_stats ADD COLUMN tackles_team INTEGER;
ALTER TABLE american_football_defensive_stats ADD COLUMN sacks_team INTEGER;
ALTER TABLE american_football_defensive_stats ADD COLUMN sacks_yards_team INTEGER;

DELETE FROM db_info;
INSERT INTO db_info (version) VALUES ('29');
