/*
 * $Id: sportsdb-28-29-transition-sql-server-2k.sql 3015 2013-01-14 22:34:57Z ian $
 *  
 * Script to update the sportsdb schema between version 28 and 29.
 */

/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_awards_part_type_part_id ON awards (participant_type, participant_id);
GO
CREATE INDEX IDX_awards_dt_cov_type_dt_cov_id ON awards (date_coverage_type, date_coverage_id);
GO
/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_locations_country_code ON locations (country_code);
GO
CREATE INDEX IDX_locations_city ON locations (city);
GO
CREATE INDEX IDX_locations_state ON locations (state);
GO
CREATE INDEX IDX_locations_country ON locations (country);
GO

/* IMPORTANT!!! Add indexes for this table only if no conflict with existing data */

CREATE INDEX IDX_sports_property_sp_prop_type_sp_prop_id ON sports_property (sports_property_type, sports_property_id);
GO
CREATE INDEX IDX_sports_property_formal_name ON sports_property (formal_name);
GO
CREATE INDEX IDX_sports_property_value ON sports_property (value);
GO

CREATE TABLE american_football_field_goal_stats (
    id INT IDENTITY (1,1) NOT NULL PRIMARY KEY,
    attempts INTEGER,
    made INTEGER
);
GO

ALTER TABLE basketball_event_states ADD score_team INTEGER;
ALTER TABLE basketball_event_states ADD score_team_opposing INTEGER;
ALTER TABLE basketball_event_states ADD score_team_home INTEGER;
ALTER TABLE basketball_event_states ADD score_team_away INTEGER;
ALTER TABLE basketball_event_states ADD record_type VARCHAR(40);
GO

CREATE TABLE basketball_action_plays (
    id INT IDENTITY (1,1) NOT NULL PRIMARY KEY,
    basketball_event_state_id INTEGER,
    team_id INTEGER NOT NULL,
    play_key VARCHAR(100),
    play_type VARCHAR(100),
    score_attempt_type VARCHAR(100),
    points INTEGER,
    penalty_level VARCHAR(100),
    recipient_type VARCHAR(100),
    recipient_id INTEGER,
    comment VARCHAR(512)    
);
GO

ALTER TABLE basketball_action_plays
    ADD CONSTRAINT FK_bask_ev_sta_bask_act_plays
    FOREIGN KEY (basketball_event_state_id) REFERENCES basketball_event_states (id);
GO

ALTER TABLE basketball_action_plays
    ADD CONSTRAINT FK_teams_basketball_action_plays
    FOREIGN KEY (team_id) REFERENCES teams (id);
GO

CREATE TABLE basketball_action_participants (
    id INT IDENTITY (1,1) NOT NULL PRIMARY KEY,
    basketball_action_play_id INTEGER,
    person_id INTEGER,
    team_id INTEGER NOT NULL,
    participant_role VARCHAR(100)
);
GO

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_persons_bask_act_play_part
    FOREIGN KEY (person_id) REFERENCES persons (id);
GO

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_teams_bask_act_play_part
    FOREIGN KEY (team_id) REFERENCES teams (id);
GO

ALTER TABLE basketball_action_participants
    ADD CONSTRAINT FK_bask_act_play_bask_act_play_part
    FOREIGN KEY (basketball_action_play_id) REFERENCES basketball_action_plays (id);
GO

ALTER TABLE soccer_event_states ADD score_team INTEGER;
ALTER TABLE soccer_event_states ADD score_team_opposing INTEGER;
ALTER TABLE soccer_event_states ADD score_team_home INTEGER;
ALTER TABLE soccer_event_states ADD score_team_away INTEGER;
ALTER TABLE soccer_event_states ADD record_type VARCHAR(40);
GO

ALTER TABLE soccer_action_plays ADD team_id INTEGER;
ALTER TABLE soccer_action_plays ADD play_key VARCHAR(100);
ALTER TABLE soccer_action_plays ADD points INTEGER;
ALTER TABLE soccer_action_plays ADD penalty_level VARCHAR(100);
ALTER TABLE soccer_action_plays ADD recipient_type VARCHAR(100);
GO

/*  
    Run this command separately. 
    It should create a foreign key that allows null values in the soccer_action_plays table
    but if you get a complaint about null values leave this foreign key out. 
*/
ALTER TABLE soccer_action_plays
    ADD CONSTRAINT FK_teams_soccer_action_plays
    FOREIGN KEY (team_id) REFERENCES teams (id);
GO

ALTER TABLE soccer_action_participants ADD team_id INTEGER;
GO

/*  
    Run this command separately. 
    It should create a foreign key that allows null values in the soccer_action_participants table
    but if you get a complaint about null values leave this foreign key out. 
*/
ALTER TABLE soccer_action_participants
    ADD CONSTRAINT FK_teams_soccer_action_participants
    FOREIGN KEY (team_id) REFERENCES teams (id);
GO

ALTER TABLE soccer_offensive_stats ADD goals_own INTEGER;
ALTER TABLE soccer_offensive_stats ADD free_kicks INTEGER;
GO

ALTER TABLE soccer_defensive_stats ADD goals_own INTEGER;
GO

ALTER TABLE documents ALTER COLUMN title VARCHAR(512);
ALTER TABLE documents ALTER COLUMN doc_id VARCHAR(255);
GO

ALTER TABLE events ADD end_date_time DATETIME;
GO

ALTER TABLE american_football_special_teams_stats ADD fumbles_forced_special_teams INTEGER;
GO

ALTER TABLE american_football_fumbles_stats ADD fumbles_forced_other INTEGER;
ALTER TABLE american_football_fumbles_stats ADD fumbles_forced_special_teams INTEGER;
GO

ALTER TABLE american_football_offensive_stats ADD tackles_team INTEGER;
ALTER TABLE american_football_defensive_stats ADD sacks_team INTEGER;
ALTER TABLE american_football_defensive_stats ADD sacks_yards_team INTEGER;
GO

DELETE FROM db_info;
INSERT INTO db_info (version) VALUES ('29');
