/*
 * $Id: sportsdb-29-30-transition-mysql-3.sql 3108 2016-04-18 16:48:53Z ian $
 *  
 * Script to update the sportsdb schema between version 29 and 30.
 */

/*
   NOTE
   -----------------
   All of the statements in this transition file have been tested for syntax. It is still possible that
   there are errors. If you find any, please report them to support@xmlteam.com.
   
   It is not possible to test against all production scenarios. Problems may arise, especially with 
   statements that modify a field's type.
   
   The following schema adjustment SQL is not intended to be run all at once. Please examine each statement or
   statement collection and run it on its own, examining the outcome each time to ensure that the desired
   change has been made. Please also note that some of the changes modify existing fields and/or indexes. These
   changes are more sensitive and need to be tested out on a non-production database before being run on
   the target database. 
*/

/*
   Add new wins_regulation_overtime field to outcome_totals
*/

ALTER TABLE outcome_totals ADD COLUMN wins_regulation_overtime INTEGER;


/*
--  NOTE
--  These changes need to be tested on a test database to ensure they works correctly.

--  Change the sequence_number field in the baseball_event_states table to INTEGER from NUMERIC

-- american_football_event_states
--  Change the sequence_number to VARCHAR(100) from INTEGER
DROP INDEX IDX_american_football_event_states_seq_num ON american_football_event_states;
UPDATE american_football_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE american_football_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE american_football_event_states ADD INDEX IDX_american_football_event_states_seq_num (sequence_number);


-- baseball_event_states
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
DROP INDEX IDX_baseball_event_states_seq_num ON baseball_event_states;
UPDATE baseball_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE baseball_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE baseball_event_states ADD INDEX IDX_baseball_event_states_seq_num (sequence_number);


-- baseball_action_pitches
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
UPDATE baseball_action_pitches SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE baseball_action_pitches MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;


-- baseball_action_substitutions
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
UPDATE baseball_action_substitutions SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE baseball_action_substitutions MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;


-- basketball_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_basketball_event_states_seq_num ON basketball_event_states;
UPDATE basketball_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE basketball_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE basketball_event_states ADD INDEX IDX_basketball_event_states_seq_num (sequence_number);


-- event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_event_states_seq_num ON event_states;
UPDATE event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE event_states ADD INDEX IDX_event_states_seq_num (sequence_number);


-- motor_racing_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_motor_racing_event_states_seq_num ON motor_racing_event_states;
UPDATE motor_racing_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE motor_racing_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE motor_racing_event_states ADD INDEX IDX_motor_racing_event_states_seq_num (sequence_number);


-- soccer_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_soccer_event_states_seq_num ON soccer_event_states;
UPDATE soccer_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE soccer_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE soccer_event_states ADD INDEX IDX_soccer_event_states_seq_num (sequence_number);


-- tennis_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_tennis_event_states_seq_num ON tennis_event_states;
UPDATE tennis_event_states SET sequence_number = '0' WHERE trim(coalesce(sequence_number, '')) = '';
ALTER TABLE tennis_event_states MODIFY sequence_number VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE tennis_event_states ADD INDEX IDX_tennis_event_states_seq_num (sequence_number);

*/

/*
   Add start_date field to person_phases table to allow values from publisher which are simple years to be
   represented.
*/

ALTER TABLE person_phases ADD COLUMN start_date VARCHAR(64);

DELETE FROM db_info;
INSERT INTO db_info (version) VALUES ('30');
