/*
 * $Id: sportsdb-29-30-transition-postgresql-8.sql 3108 2016-04-18 16:48:53Z ian $
 *  
 * Script to update the sportsdb schema between version 29 and 30.
 */

/*
   NOTE
   -----------------
   All of the statements in this transition file have been tested for syntax. It is still possible that
   there are errors. If you find any, please report them to support@xmlteam.com.
   
   It is not possible to test against all production scenarios. Problems may arise, especially with 
   statements that modify a field's type.

   The following schema adjustment SQL is not intended to be run all at once. Please examine each statement or
   statement collection and run it on its own, examining the outcome each time to ensure that the desired
   change has been made. Please also note that some of the changes modify existing fields and/or indexes. These
   changes are more sensitive and need to be tested out on a non-production database before being run on
   the target database. 
*/

/*
   Add new wins_regulation_overtime field to outcome_totals
*/

ALTER TABLE outcome_totals ADD COLUMN wins_regulation_overtime INTEGER;

/*
-- The following are changes to standardize sequence_number fields to be VARCHAR(100). The transitions are from
-- INTEGER or DECIMAL/NUMERIC type. It is difficult to know what form a publisher's sequence-number values will
-- be in SportsML. Sometimes they take the form of '00000126' or '3.5'. These patterns can change at the publishers'
-- discretion and differ from publisher to publisher. For this reason we are moving to using a VARCHAR field for
-- all sequence number fields. This is the standard for the SportsDB schema v. 30.
-- 
-- NOTE
-- These changes MUST to be tested one table at a time on a NOT PRODUCTION database to ensure they works correctly.
-- Once a test database has been adjusted you should run it with its own FeedFetcher until you are satisfied that
-- it is working properly with the adjusted sequence_number fields.
-- 

-- american_football_event_states
--  Change the sequence_number to VARCHAR(100) from INTEGER
DROP INDEX IDX_american_football_event_states_seq_num;
ALTER TABLE american_football_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_american_football_event_states_seq_num ON american_football_event_states (sequence_number);


-- baseball_event_states
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
DROP INDEX IDX_baseball_event_states_seq_num;
ALTER TABLE baseball_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_baseball_event_states_seq_num ON baseball_event_states (sequence_number);


-- baseball_action_pitches
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
ALTER TABLE baseball_action_pitches ALTER COLUMN sequence_number TYPE VARCHAR(100);


-- baseball_action_substitutions
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
ALTER TABLE baseball_action_substitutions ALTER COLUMN sequence_number TYPE VARCHAR(100);


-- basketball_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_basketball_event_states_seq_num;
ALTER TABLE basketball_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_basketball_event_states_seq_num ON basketball_event_states (sequence_number);


-- event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_event_states_seq_num;
ALTER TABLE event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_event_states_seq_num ON event_states (sequence_number);


-- motor_racing_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_motor_racing_event_states_seq_num;
ALTER TABLE motor_racing_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_motor_racing_event_states_seq_num ON motor_racing_event_states (sequence_number);


-- soccer_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_soccer_event_states_seq_num;
ALTER TABLE soccer_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_soccer_event_states_seq_num ON soccer_event_states (sequence_number);


-- tennis_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX IDX_tennis_event_states_seq_num;
ALTER TABLE tennis_event_states ALTER COLUMN sequence_number TYPE VARCHAR(100);
CREATE INDEX IDX_tennis_event_states_seq_num ON tennis_event_states (sequence_number);

*/

/*
   Add start_date field to person_phases table to allow values from publisher which are simple years to be
   represented.
*/

ALTER TABLE person_phases ADD COLUMN start_date VARCHAR(64);

-- Update the version number to reflect its change.
DELETE FROM db_info;
INSERT INTO db_info (version) VALUES ('30');
