/*
 * $Id: sportsdb-29-30-transition-sql-server-2k.sql 3108 2016-04-18 16:48:53Z ian $
 *  
 * Script to update the sportsdb schema between version 29 and 30.
 */

/*
   NOTE
   -----------------
   All of the statements in this transition file have been tested for syntax. It is still possible that
   there are errors. If you find any, please report them to support@xmlteam.com.
   
   It is not possible to test against all production scenarios. Problems may arise, especially with 
   statements that modify a field's type.
   
   The following schema adjustment SQL is not intended to be run all at once. Please examine each statement or
   statement collection and run it on its own, examining the outcome each time to ensure that the desired
   change has been made. Please also note that some of the changes modify existing fields and/or indexes. These
   changes are more sensitive and need to be tested out on a non-production database before being run on
   the target database. 
*/


/*
   Add new wins_regulation_overtime field to outcome_totals
*/

ALTER TABLE outcome_totals ADD wins_regulation_overtime INTEGER;


/*
-- The following are changes to standardize sequence_number fields to be VARCHAR(100). The transitions are from
-- INTEGER or DECIMAL/NUMERIC type. It is difficult to know what form a publisher's sequence-number values will
-- be in SportsML. Sometimes they take the form of '00000126' or '3.5'. These patterns can change at the publishers'
-- discretion and differ from publisher to publisher. For this reason we are moving to using a VARCHAR field for
-- all sequence number fields. This is the standard for the SportsDB schema v. 30.
-- 
-- NOTE
-- These changes MUST to be tested one table at a time on a NOT PRODUCTION database to ensure they works correctly.
-- Once a test database has been adjusted you should run it with its own FeedFetcher until you are satisfied that
-- it is working properly with the adjusted sequence_number fields.

-- american_football_event_states
--  Change the sequence_number to VARCHAR(100) from INTEGER
DROP INDEX american_football_event_states.IDX_american_football_event_states_seq_num;
GO
ALTER TABLE american_football_event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE american_football_event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE american_football_event_states DROP COLUMN sequence_number;
GO
ALTER TABLE american_football_event_states ADD sequence_number VARCHAR(100);
GO
UPDATE american_football_event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE american_football_event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_american_football_event_states_seq_num ON american_football_event_states (sequence_number);
GO


-- baseball_event_states
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
DROP INDEX baseball_event_states.IDX_baseball_event_states_seq_num;
GO
ALTER TABLE baseball_event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE baseball_event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE baseball_event_states DROP COLUMN sequence_number;
GO
ALTER TABLE baseball_event_states ADD sequence_number VARCHAR(100);
GO
UPDATE baseball_event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE baseball_event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_baseball_event_states_seq_num ON baseball_event_states (sequence_number);
GO


-- baseball_action_pitches
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
ALTER TABLE baseball_action_pitches ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE baseball_action_pitches SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE baseball_action_pitches DROP COLUMN sequence_number;
GO
ALTER TABLE baseball_action_pitches ADD sequence_number VARCHAR(100);
GO
UPDATE baseball_action_pitches SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE baseball_action_pitches DROP COLUMN sequence_number_tmp;
GO


-- baseball_action_substitutions
--  Change the sequence_number field to VARCHAR(100) from NUMERIC/DECIMAL
ALTER TABLE baseball_action_substitutions ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE baseball_action_substitutions SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE baseball_action_substitutions DROP COLUMN sequence_number;
GO
ALTER TABLE baseball_action_substitutions ADD sequence_number VARCHAR(100);
GO
UPDATE baseball_action_substitutions SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE baseball_action_substitutions DROP COLUMN sequence_number_tmp;
GO


-- event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX event_states.IDX_event_states_seq_num;
GO
ALTER TABLE event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE event_states DROP COLUMN sequence_number;
GO
ALTER TABLE event_states ADD sequence_number VARCHAR(100);
GO
UPDATE event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_event_states_seq_num ON event_states (sequence_number);
GO


-- motor_racing_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX motor_racing_event_states.IDX_motor_racing_event_states_seq_num;
GO
ALTER TABLE motor_racing_event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE motor_racing_event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE motor_racing_event_states DROP COLUMN sequence_number;
GO
ALTER TABLE motor_racing_event_states ADD sequence_number VARCHAR(100);
GO
UPDATE motor_racing_event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE motor_racing_event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_motor_racing_event_states_seq_num ON motor_racing_event_states (sequence_number);
GO


-- soccer_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX soccer_event_states.IDX_soccer_event_states_seq_num;
GO
ALTER TABLE soccer_event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE soccer_event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE soccer_event_states DROP COLUMN sequence_number;
GO
ALTER TABLE soccer_event_states ADD sequence_number VARCHAR(100);
GO
UPDATE soccer_event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE soccer_event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_soccer_event_states_seq_num ON soccer_event_states (sequence_number);
GO


-- tennis_event_states
--  Change the sequence_number field to VARCHAR(100) from INTEGER
DROP INDEX tennis_event_states.IDX_tennis_event_states_seq_num;
GO
ALTER TABLE tennis_event_states ADD sequence_number_tmp VARCHAR(100);
GO
UPDATE tennis_event_states SET sequence_number_tmp = cast(sequence_number as VARCHAR);
GO
ALTER TABLE tennis_event_states DROP COLUMN sequence_number;
GO
ALTER TABLE tennis_event_states ADD sequence_number VARCHAR(100);
GO
UPDATE tennis_event_states SET sequence_number = sequence_number_tmp;
GO
ALTER TABLE tennis_event_states DROP COLUMN sequence_number_tmp;
GO
CREATE INDEX IDX_tennis_event_states_seq_num ON tennis_event_states (sequence_number);
GO

*/

/*
   Add start_date field to person_phases table to allow values from publisher which are simple years to be
   represented.
*/

ALTER TABLE person_phases ADD start_date VARCHAR(64);
GO

-- Update the version number to reflect its change.
DELETE FROM db_info;
GO
INSERT INTO db_info (version) VALUES ('30');
GO
