/*
  $Id: sp_mysql_sportsdb_delete_document_data_by_date_range.sql 2873 2011-04-09 01:54:06Z ian $
  
  WARNING:
  This code removes data from a SportsDB database. It has been reviewed and tested in an attempt
  to ensure that it does not harm the integrity of a SportsDB database or remove data that it
  should not, but it may not function as anticipated. This code comes with absolutely no
  guarantees. It is your responsibility to check this code to ensure that you are satisfied that
  it will work as expected. XML Team Inc. takes no responsibility for any unexpected data loss
  or data corruption that may occur as a result of the execution of this code on your database.  

  This is a stored procedure to remove documents by date range. Removes all document-related data and all 
  event-related date for events with a start_date_time within the range specified.
*/

delimiter $$

DROP PROCEDURE IF EXISTS sportsdb_delete_document_data_by_date_range$$

/*
  NOTES:
  documents - date_time is stored as UTC
  events - start_date_time is stored as UTC
  
  PARAMETERS:
  in_start_date - start date for date range to find documents for deletion - Will assume UTC
  in_end_date - end date for date range to find documents for deletion - Will assume UTC
*/

CREATE PROCEDURE sportsdb_delete_document_data_by_date_range
(IN in_start_date DATE, IN in_end_date DATE)
/*
Further work could add in removing documents by league or fixture by selecting the document list into
a temporary table then running the cursor from that recordset.
(IN in_start_date DATE, IN in_end_date DATE, IN in_league_key VARCHAR(100))
*/
BEGIN
  main: BEGIN

  DECLARE v_counter INTEGER;
  DECLARE v_no_more_rows BOOLEAN;
  DECLARE v_document_id INTEGER;
  DECLARE v_event_id INTEGER;
  DECLARE v_test_integer INTEGER;

  DECLARE cur_documents CURSOR FOR
    SELECT id from documents
    WHERE DATE(date_time) >= in_start_date
    AND DATE(date_time) <= in_end_date;

  /* A cursor to gather up all events tied to a document */
  DECLARE cur_events CURSOR FOR
    SELECT events.id FROM events
    JOIN events_documents ON (events_documents.document_id  = v_document_id)
    WHERE DATE(start_date_time) >= in_start_date
    AND DATE(start_date_time) <= in_end_date
    GROUP BY events.id
    ORDER BY events.id;
  
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_no_more_rows := TRUE;
  SET v_counter := 0;

  /* Ensure the start and end date are properly related with start date > end date */
  SELECT (in_start_date < in_end_date) INTO v_test_integer;
  IF v_test_integer = 0 THEN
    LEAVE main;
  END IF;

  /* Remove all documents in the date range. */
  OPEN cur_documents;
  documents_loop: LOOP

    FETCH cur_documents INTO v_document_id;

    IF v_no_more_rows THEN
      SET v_no_more_rows := FALSE;
      CLOSE cur_documents;
      LEAVE documents_loop;
    END IF;
    SET v_counter := v_counter + 1;

    /* Find all events related to the document by document.date_time and events.start_date_time */
    OPEN cur_events;
    events_loop: LOOP
      FETCH cur_events INTO v_event_id;
      
      IF v_no_more_rows THEN
        SET v_no_more_rows := FALSE;
        CLOSE cur_events;
        LEAVE events_loop;
      END IF;
    
      /* Delete the current event. */
      CALL sportsdb_delete_event_data_by_id(v_event_id);
    END LOOP events_loop;
    
    /* Delete the current document. */
    CALL sp_mysql_sportsdb_delete_document_data_by_id(v_document_id);
  END LOOP documents_loop;

  /* Report count of duplicate sets processed for this condition */
  SELECT CONCAT('Documents proccessed: ', v_counter) AS Counter;
  
  END main;
END;$$

delimiter ;