/*
  $Id: sp_mysql_sportsdb_delete_duplicate_participants_events.sql 2868 2011-04-07 21:36:19Z ian $
  
  A stored procedure to remove duplicate participants_events records in the case where
  there are duplicate team type records for a home or away event for an event and particpant.
*/

delimiter $$

DROP PROCEDURE IF EXISTS sportsdb_delete_duplicate_participants_events$$

CREATE PROCEDURE sportsdb_delete_duplicate_participants_events
()
BEGIN
  main: BEGIN

  DECLARE v_counter INTEGER;
  DECLARE v_no_more_rows BOOLEAN;
  DECLARE v_pe_count INTEGER;
  DECLARE v_participant_event_id INTEGER;
  DECLARE v_participant_id INTEGER;
  DECLARE v_event_id INTEGER;

  /* A cursor to gather up all of the home participants for an event */
  DECLARE cur_participants_events_home CURSOR FOR
  SELECT pe1.participant_id, pe1.event_id 
  FROM participants_events pe1, participants_events pe2 
  WHERE pe1.event_id = pe2.event_id 
  AND pe1.participant_id = pe2.participant_id
  AND pe1.participant_type = 'teams'
  AND pe2.participant_type = pe1.participant_type
  AND pe1.alignment='home' 
  AND pe2.alignment = 'home'
  AND pe1.id != pe2.id 
  GROUP BY pe1.participant_id, pe1.event_id 
  ORDER BY pe1.participant_id, pe1.event_id;

  /* A cursor to gather up all of the away participants for an event */
  DECLARE cur_participants_events_away CURSOR FOR
  SELECT pe1.participant_id, pe1.event_id 
  FROM participants_events pe1, participants_events pe2 
  WHERE pe1.event_id = pe2.event_id 
  AND pe1.participant_id = pe2.participant_id
  AND pe1.participant_type = 'teams'
  AND pe2.participant_type = pe1.participant_type
  AND pe1.alignment='away'
  AND pe2.alignment = 'away' 
  AND pe1.id != pe2.id
  GROUP BY pe1.participant_id, pe1.event_id 
  ORDER BY pe1.participant_id, pe1.event_id;
  
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET v_no_more_rows := TRUE;
  SET v_counter := 0;

  /* Remove duplicate home participants */
  OPEN cur_participants_events_home;
  participants_events_home_loop: LOOP

    FETCH cur_participants_events_home INTO v_participant_id, v_event_id;

    IF v_no_more_rows THEN
      SET v_no_more_rows := FALSE;
      CLOSE cur_participants_events_home;
      LEAVE participants_events_home_loop;
    END IF;
    SET v_counter := v_counter + 1;
    
    /* Make extra sure that there are two or more participant/event records for this condition */
    SELECT COUNT(*) INTO v_pe_count 
    FROM participants_events 
    WHERE participant_id = v_participant_id
    AND participant_type = 'teams'
    AND event_id = v_event_id
    AND alignment = 'home';

    IF v_pe_count >= 2 THEN
      /* Find the id of the FIRST record in the duplicate set */
      SELECT id INTO v_participant_event_id
      FROM participants_events
      WHERE participant_id = v_participant_id
      AND event_id = v_event_id
      AND participant_type = 'teams'
      AND alignment = 'home'
      ORDER BY id ASC
      LIMIT 1;
      
      /* If we found something, remove all others */
      If v_participant_event_id IS NOT NULL THEN
/*
        SELECT v_participant_event_id AS 'HOME_ID'; 
*/        
        DELETE FROM participants_events
        WHERE participant_id = v_participant_id
        AND event_id = v_event_id
        AND participant_type = 'teams'
        AND alignment = 'home'
        AND id != v_participant_event_id;
      END IF;
    ELSE
      SELECT CONCAT('Not 2: ', v_pe_count) AS NOT_TWO;      
    END IF;

  END LOOP participants_events_home_loop;

  /* Report count of duplicate sets processed for this condition */
  SELECT CONCAT('Home Count: ', v_counter) AS Counter;
    
  SET v_counter := 0;

  /* Remove duplicate away participants */
  OPEN cur_participants_events_away;
  participants_events_away_loop: LOOP

    FETCH cur_participants_events_away INTO v_participant_id, v_event_id;
    IF v_no_more_rows THEN
      SET v_no_more_rows := FALSE;
      CLOSE cur_participants_events_away;
      LEAVE participants_events_away_loop;
    END IF;
    SET v_counter := v_counter + 1;

    /* Make extra sure that there are two or more participant/event records for this condition */
    SELECT COUNT(*) INTO v_pe_count 
    FROM participants_events 
    WHERE participant_id = v_participant_id
    AND event_id = v_event_id
    AND participant_type = 'teams'
    AND alignment = 'away';

    IF v_pe_count >= 2 THEN
      /* Find the id of the FIRST record in the duplicate set */
      SELECT id INTO v_participant_event_id
      FROM participants_events
      WHERE participant_id = v_participant_id
      AND event_id = v_event_id
      AND participant_type = 'teams'
      AND alignment = 'away'
      ORDER BY id ASC
      LIMIT 1;

      /* If we found something, remove all others */
      If v_participant_event_id IS NOT NULL THEN
/*
        SELECT v_participant_event_id AS 'AWAY_ID'; 
*/
        DELETE FROM participants_events
        WHERE participant_id = v_participant_id
        AND event_id = v_event_id
        AND participant_type = 'teams'
        AND alignment = 'away'
        AND id != v_participant_event_id;
      END IF;
    ELSE
      SELECT CONCAT('Not 2: ', v_pe_count) AS NOT_TWO;
    END IF;
  END LOOP participants_events_away_loop;
  
  /* Report count of duplicate sets processed for this condition */
  SELECT CONCAT('Away Count: ', v_counter) AS Counter;
  
  END main;
END;$$

delimiter ;