/*
  $Id: sp_postgresql_sportsdb_delete_document_data_by_id.sql 2876 2011-04-12 21:36:57Z ian $
  
  WARNING:
  This code removes data from a SportsDB database. It has been reviewed and tested in an attempt
  to ensure that it does not harm the integrity of a SportsDB database or remove data that it
  should not, but it may not function as anticipated. This code comes with absolutely no
  guarantees. It is your responsibility to check this code to ensure that you are satisfied that
  it will work as expected. XML Team Inc. takes no responsibility for any unexpected data loss
  or data corruption that may occur as a result of the execution of this code on your database.  
  
  This is a stored procedure to remove document data tied to a document record by its id.
*/

delimiter $$

DROP PROCEDURE IF EXISTS sportsdb_delete_document_data_by_id$$

CREATE PROCEDURE sportsdb_delete_document_data_by_id
(IN in_document_id INTEGER)
BEGIN
  main: BEGIN

  DECLARE v_document_id INTEGER;
  
  SELECT id INTO v_document_id FROM documents
  WHERE id = in_document_id;
  
  /* Leave if no document id found to match the one passed in */
  IF v_document_id IS NULL THEN
    LEAVE main;
  END IF;


  DELETE FROM document_contents
  WHERE document_id = in_document_id;

  DELETE FROM documents_media
  WHERE document_id = in_document_id;

  DELETE FROM document_package_entry
  WHERE document_id = in_document_id;

  DELETE FROM document_fixtures_events
  WHERE latest_document_id = in_document_id;

  DELETE FROM affiliations_documents
  WHERE document_id = in_document_id;

  DELETE FROM teams_documents
  WHERE document_id = in_document_id;

  DELETE FROM persons_documents
  WHERE document_id = in_document_id;

  DELETE FROM events_documents
  WHERE document_id = in_document_id;

  DELETE FROM documents
  WHERE id = in_document_id;
  
  END main;
END;$$

delimiter ;