# $Id: CircularQueue.pm 2125 2008-07-23 19:27:14Z ian $
# Circular Queue

package Core::CircularQueue;

use strict;

sub new {
	my ($class, $length) = @_;

	my $self = {};
	bless $self, $class;
	$self->initialize($length);
	return $self;
}

sub initialize {
	my ($self, $length) = @_;
	$self->{'queue'} = [];
	$self->{'max_len'} = $length;
	$self->{'head'} = 0;
	$self->{'tail'} = 0;
	$self->{'len'} = 0;
}

sub add {
	my ($self, $rec) = @_;

	my $deleted;

	# Make room
	$deleted = $self->del() if $self->{'len'} == $self->{'max_len'};

	$self->{'queue'}->[$self->{'tail'}] = $rec;
	$self->{'tail'} += 1;
	$self->{'tail'} = 0 if $self->{'tail'} == $self->{'max_len'};
	$self->{'len'} += 1 if $self->{'len'} < $self->{'max_len'};

	return $deleted;
}

sub del {
	my ($self) = @_;
	my $deleted = $self->{'queue'}->[$self->{'head'}];
	$self->{'head'} += 1;
	$self->{'head'} = 0 if $self->{'head'} == $self->{'max_len'};

	return $deleted;
}

sub len {
	my ($self) = @_;
	return $self->{'len'};
}

sub get {
	my ($self, $index) = @_;
	return if $index >= $self->len() || $index < 0;
	$index += $self->{'head'};
	$index = $index % $self->{'max_len'};
	return $self->{'queue'}->[$index];
}

1;

