# $Id$

package Core::Configurator_overrides;

use strict;

# Set this module as one that will merge with Core::Configurator
use base 'Exporter';
our @EXPORT = qw(do_override_config);


sub do_override_config {
	my ($self) = @_;

    # This is meant to be set in a build script, replacing the OVERRIDES line with the override values
    # DO NOT change that line, it will be replaced with the values to override in the build script
    # sample override
    #xt_user|bob^xt_bob|carl
    # Note that separate key/value pairs separated by ^ char rather than newlines (plays nice with sed)
    # An empty value is valid (e.g. "vehicle|car^food|" - in this case, the food key will be removed from the config hash
    my $overrides = "#OVERRIDES#";

    my @override_set = split(/\^/, $overrides);
    
    for my $item (@override_set)
    {
        # Eliminate leading and trailing spaces
        $item =~ s/^\s*(.*?)\s*$/$1/;
        # Skip if for some reason an empty item is encountered
        next if ($item eq '' || $item =~ /^\s*#/);
        my ($key, $val) = split(/\|/, $item);
        $key =~ s/\s+//g;
        $val =~ s/^\s*(.*?)\s*$/$1/;
        # Set the actual config hash value (delete key if value empty)
        if ($val eq '') {
            delete $self->{'keys'}->{$key};
        }
        else {
            $self->{'keys'}->{$key} = $val;
        }
    }
}