# $Id: LineParser.pm 2125 2008-07-23 19:27:14Z ian $
package Core::LineParser;

use FileHandle;

use Core::Util;

sub load {
	my ($filepath) = @_;

	my $fh = new FileHandle $filepath or die "Can't open $filepath: $!";
	my @lines = ();
	while (<$fh>) {
		chomp;
		next if /^#/ || /^\s*$/;

		push @lines, Core::Util::trim($_);
	}

	return @lines;
}

sub load_table {
	my ($cfg, $param_key) = @_;

	my $fp = $cfg->required_param($param_key);
	my @lines = load($fp);
	my $table = {};
	foreach my $line (@lines) {
		chomp $line;
		my @recs = split(/:/, $line);
		die "Bad format for $fp" if @recs != 2;
		$table->{$recs[0]} = $recs[1];
	}

	return $table;
}

sub load_table_list {
	my ($cfg, $param_key) = @_;

	my $fp = $cfg->required_param($param_key);
	my @lines = load($fp);
	my $table = {};
	foreach my $line (@lines) {
		chomp $line;
		my @recs = split(/:/, $line);
		die "Bad format for $fp" if @recs != 2;
		$table->{$recs[0]} = [] if !defined $table->{$recs[0]};
		push @{$table->{$recs[0]}}, $recs[1];
	}

	return $table;
}

sub load_list {
	my ($cfg, $param_key) = @_;

	my $fp = $cfg->required_param($param_key);
	my @lines = load($fp);
	my $table = {};
	foreach my $line (@lines) {
		chomp $line;
		$table->{$line} = 1
	}

	return $table;
}

1;