# $Id: Util.pm 2817 2010-12-16 22:58:27Z ian $
package Core::Util;

use strict;

use File::Basename;
use File::Spec;
use FindBin;
use Date::Pcalc qw( Delta_DHMS );
use Time::Local;
use FindBin;
use Carp;
use Util::DateUtils;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
use Exporter;
$VERSION = 1.1;
@ISA = qw(Exporter);

@EXPORT = qw(trim trims remove_duplicates underscore2dash dash2underscore);
@EXPORT_OK = ();

sub trim {
	my ($str) = @_;

	return undef if !defined $str;
	$str =~ s/^\s+//;
	$str =~ s/\s+$//; 

	return $str;
}

sub trims {
	my @list = @_;
	my @results = ();

	foreach my $str (@list) {
		push @results, trim($str);
	}

	return @results;
}

# Returns the current date/time in ISO 8601 format,
# with the correct timezone offset
sub current_ISO8601_time {
	my ($time) = @_;

	$time = time if !defined $time;
	return epoch2iso8601($time);
}

sub epoch2iso8601 {
	my ($time) = @_;

	croak "No epoch given" if !defined $time;
	my ($utc_sec, $utc_min, $utc_hour, $utc_day, $utc_mon, 
		$utc_year, $utc_wday, $utc_yday) = gmtime($time);
	my ($sec, $min, $hour, $day, $mon, $year, $wday, $yday) = localtime($time);
	$utc_year += 1900;
	$utc_mon++;
	$year += 1900;
	$mon++;

	my ($Dd,$Dh,$Dm,$Ds) = Delta_DHMS(
		$utc_year, $utc_mon, $utc_day, $utc_hour, $utc_min, $utc_sec,
		$year, $mon, $day, $hour, $min, $sec);

	my $sign = '+';
	if ($Dh < 0) { $Dh = -$Dh; $Dm = -$Dm; $sign = '-'; }
	return sprintf("%04d%02d%02dT%02d%02d%02d%s%02d%02d", 
		$year, $mon, $day, $hour, $min, $sec, $sign, $Dh, $Dm);
}


# Converts ISO 8601 date to Perl epoch date
sub date2epoch {
	my ($date_time) = @_;

    croak "No date-time given" if !defined $date_time;

#    print "date2epoch\n";
    
    my $date_utils = Util::DateUtils->new();
    my ($year, $mon, $day, $hour, $min, $sec) = $date_utils->get_UTC_time_parts($date_time);
    
#    my $year;
#    my $mon;
#    my $day;
#    my $hour;
#    my $min;
#    my $sec;
#    my $z;
#    my $zhr;
#    my $zmin;
#    
#	# AT zero TZ offset from CST apparently the [Z+-] plus offset amount indication can be completely missing,
#	# so allow them all to be missing.
#	my $match = ($date_time =~ 
#		/(\d\d\d\d)-?(\d\d)-?(\d\d)T(\d\d):?(\d\d):?(\d\d)([Z+-])?((\d\d):?(\d\d))?/);
#    ($year, $mon, $day, $hour, $min, $sec, $z, $zhr, $zmin) = ($1, $2, $3, $4, $5, $6, $7, $9, $10);
#	croak "Bad time-date format: $date_time" if !$match;
	
	# Calculate timestamp without any TZ offset
	my $epoch = timegm($sec, $min, $hour, $day, $mon - 1, $year);
#	# If there is a TZ offset
#	if ($z) {
#        if ($z eq '-') {
#            croak "Bad time-date format: $date_time" if !defined $zhr || !defined $zmin;
#            $epoch += ($zhr * 60 + $zmin) * 60;
#        } elsif ($z eq '+') {
#           croak "Bad time-date format: $date_time" if !defined $zhr || !defined $zmin;
#           $epoch -= ($zhr * 60 + $zmin) * 60;
#        }
#    }

	return $epoch;
}

# Converts Perl epoch date to ISO 8601 date
sub epoch2date {
	my ($epoch) = @_;

	return if !$epoch;
	return if $epoch !~ /^\d+$/;
	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday) =
		gmtime($epoch);
	return sprintf("%04d%02d%02dT%02d%02d%02dZ", 
		($year + 1900), $mon + 1, $mday, $hour, $min, $sec);
}

sub get_home_path {
	# Assumes script is in ff_home/perl/<dir>

    if (defined $ENV{'HOMEDIR'}) {
    	return $ENV{'HOMEDIR'};
    }
    
	my ($vol, $dir) = File::Spec->splitpath($FindBin::RealBin, 1);
	my @dirs = File::Spec->splitdir($dir);
	return File::Spec->catpath($vol, 
		File::Spec->catdir(@dirs[0 .. $#dirs - 2]), '');
}

sub remove_duplicates {
	my ($list_ref) = @_;

	my %hash = map { $_ => 1 } @$list_ref;
	my @new_list = keys %hash;
	@$list_ref = @new_list;
}

sub underscore2dash {
	my ($txt) = @_;

	$txt =~ s/_/-/g;
	return $txt;
}

sub dash2underscore {
	my ($txt) = @_;

	$txt =~ s/-/_/g;
	return $txt;
}

1;
