# $RCSfile$; $Date: 2006-02-08 07:50:37 -0500 (Wed, 08 Feb 2006) $; $Revision: 1006 $; $Name$
package Core::XmlUtil;

use strict;
use XML::LibXML;
use Carp;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
use Exporter;
$VERSION = 1.1;
@ISA = qw(Exporter);

@EXPORT = qw(nodes solonode solonode_optional firstnode 
	lastnode attribute set_attribute required_attribute attribute_xpath 
	attributes_xpath required_attribute_xpath );
@EXPORT_OK = ();

sub nodes {
	my ($node, $path) = @_;

	my @nodes;
	eval { @nodes = $node->findnodes($path); };
	croak $@ if $@;
	return @nodes;
}

sub solonode {
	my ($node, $path, $no_croak) = @_;

	croak "Undefined node" if !$node;
	my @nodes = undef;
	eval { @nodes = $node->findnodes($path); };
	croak $@ if $@;

	return if $no_croak && (!@nodes || @nodes > 1);
	croak "$path not in document" if !@nodes;
	croak "More than one node for $path in document" if @nodes > 1;
	return shift @nodes;
}

sub solonode_optional {
	my ($node, $path) = @_;

	croak "Undefined node" if !$node;
	my @nodes = $node->findnodes($path);

	croak "More than one node for $path in document" if @nodes > 1;
	return shift @nodes;
}

sub firstnode {
	my ($node, $path) = @_;

	croak "Undefined node" if !$node;
	my @nodes = $node->findnodes($path);

	return shift @nodes;
}

sub lastnode {
	my ($node, $path) = @_;

	croak "Undefined node" if !$node;
	my @nodes = $node->findnodes($path);

	return pop @nodes;
}

sub attribute {
	my ($node, $name) = @_;

	croak "Undefined node" if !$node;
	return $node->getAttribute($name);
}

sub set_attribute {
	my ($node, $name, $val) = @_;

	croak "Undefined node" if !$node;
	return $node->setAttribute($name, $val);
}

sub required_attribute {
	my ($node, $name) = @_;

	croak "Undefined node" if !$node;
	my $value = $node->getAttribute($name);
	my $node_name = $node->nodeName;
	croak "No attribute $name in $node_name" if 
		!defined $value || length $value == 0;

	return $value;
}

sub attribute_xpath {
	my ($node, $xpath) = @_;
	
	my $attr_node = solonode_optional($node, $xpath) or return undef;
	return $attr_node->getValue();
}

sub attributes_xpath {
	my ($node, $xpath) = @_;
	
	my @attr_nodes = nodes($node, $xpath);
	my @atts = map { $_->getValue() } @attr_nodes;
	return @atts;
}

sub required_attribute_xpath {
	my ($node, $xpath) = @_;
	
	my $attr_node = solonode($node, $xpath);
	return $attr_node->getValue();
}

1;
