# $RCSfile$; $Date: 2005-11-01 13:17:12 -0500 (Tue, 01 Nov 2005) $; $Revision: 967 $; $Name$
package Feedfetcher::Plugins::Mlbam;

use strict;
use Text::ParseWords;
use Feedfetcher::Plugins::Retriever;
our @ISA        = ("Feedfetcher::Plugins::Retriever");
our $DEBUG_MODE = "FALSE";

use Carp;

use File::Find;
use LWP::Simple;
use URI;
use URI::file;
use XML::LibXSLT;
use XML::LibXML;
use XML::Writer;

sub name { return "mlbam"; }
sub code { return "mlbam2retrosheet"; }
sub code_player_files { return "mlbam_players2sml"; }

sub save_raw {
	my ($self) = @_;
	return $self->{'plugin_module'} ? $self->{'plugin_module'}->save_raw : 0;
}

sub initialize {
	my ( $self, $plugin_module ) = @_;

	$self->{plugin_module} = $plugin_module;
}

sub get_list {
	my ($self) = @_;

	my $cfg  = $self->cfg;
	my $path = $cfg->param('mlbam_source_dir');

	die "No such directory" if !-d $path;
	my @mlbam_files = ();
	my @xml_files   = ();

	find(
		sub {
			push @mlbam_files, URI::file->new_abs($File::Find::name)
			  if /\.txt$/i;
		},
		$path
	);

	@mlbam_files = sort { get_mod($a) <=> get_mod($b) } @mlbam_files;

	foreach my $mlbamfileuri (@mlbam_files) {

		# local vars
		my $newGame          = 1;
		my $currentGamepk    = '';
		my $currentGameid    = '';
		my @currentGameStack = ();

		my $mlbamfile = $mlbamfileuri->file;

		my $startpart = rindex( $mlbamfile, '/' ) + 1;
		my $basename = substr( $mlbamfile, $startpart );
		my $lengthpart = rindex( $basename, '.' );
		my $basename = substr( $basename, 0, $lengthpart );
		print "file: $basename\n";

		my $file_type = ( $basename =~ /player_(xref|changes)/ ) ? 'player' : 'game';

# $self->log("[DBG] *** Found a file type: $file_type with basename: $basename for input file: $mlbamfile");

		if ( $file_type eq 'player' ) {
#			print "player file, name: $basename\n";
			open( MLBAMFILE, $mlbamfile );
			# go through file line by line
			my @lines = ();
			while (<MLBAMFILE>) {
				my $line = $_;

				chomp($line);
#				print "line: $line\n" if $basename =~ /changes/;
				if ( $line ne '' ) {
					push( @lines, $line);
				}
			}
			my $sml = transform_player_file( $self, \@lines, $basename );
			if ($sml) {
				push( @xml_files, $sml );
			}
		} else {
			print("opening $mlbamfile\n");
			open( MLBAMFILE, $mlbamfile );
			                                  # go through file line by line
			while (<MLBAMFILE>) {
				my $line = $_;

				$line =~ s/\n//g;

				if ( $line != '' ) {
					my @lineParts = split( /\|/, $line, 65 );
					pop(@lineParts);

					my $gamepk = shift(@lineParts);
					my $gameid = shift(@lineParts);

					if ($newGame) {
						$newGame       = 0;
						$currentGamepk = $gamepk;
						$currentGameid = $gameid;
					}

					if ( $currentGamepk != $gamepk ) {
						$newGame = 1;

						my $sml =
						  transform( $self, $currentGamepk, $currentGameid,
							\@currentGameStack, $basename );

						if ($sml) {
							push( @xml_files, $sml );
						}

						# reset the game stack
						@currentGameStack = ();
					}

					push( @currentGameStack, \@lineParts );
				}
			}
			close(MLBAMFILE);

			my $sml = transform( $self, $currentGamepk, $currentGameid,
				\@currentGameStack, $basename );
#			print "sml: $sml\n";
			if ($sml) {
				push( @xml_files, $sml );
			}
		}

		# finally delete each .txt file
		unlink($mlbamfile) if ( $DEBUG_MODE eq "FALSE" );
	}

	return @xml_files;
}

sub transform {
	my ( $self, $currentGamepk, $currentGameid, $currentGameStack, $basename ) =
	  @_;

	my $cml =
	  buildTree( $self, $currentGamepk, $currentGameid, $currentGameStack );

	my $cfg         = $self->cfg;
	my $path        = $cfg->param('mlbam_source_dir');
	my $localsource = $cfg->param('source_dir');
	my $filename    = "$path/$basename.$currentGamepk.evx";

	my $stylesheet = $self->get_stylesheet( $self->code() );
	my $parser     = XML::LibXML->new();
	my $doc        = $parser->parse_string($cml);
	my $tcml       = $stylesheet->transform($doc);
	my $evx        = $stylesheet->output_string($tcml);

	# imarsman - 2007.06.20
	# Extract the year from the first
	# There may indeed be a more efficient way to do this
	my @evx_lines = split( /\n|\r\n/, $evx );
	my $evx_fline = $evx_lines[0];
	my $year      = $evx_fline;
	$year =~ s/id,\D+(\d\d\d\d).*/$1/;

	open( EVXFILE, ">$filename" );
	print EVXFILE $evx;
	close(EVXFILE);

	my $sml;
	if ( $DEBUG_MODE eq "TRUE" ) {
		$sml = `cd $path; cwbox -S -y $year $filename`;
	} else {
		$sml = `cd $path; cwbox -S -y $year $filename 2>/dev/null`;
	}

	if ( $? == 0 ) {
		open( SMLFILE, ">$path/$basename.$currentGamepk.xml" );
		print SMLFILE $sml;
		close(SMLFILE);

		unlink $filename if ( $DEBUG_MODE eq "FALSE" );

		if ( $DEBUG_MODE eq "TRUE" ) {
			my $cmlfile = "$path/$basename.$currentGamepk" . "_cml.xml";
			open( CMLFILE, ">$cmlfile" );
			print CMLFILE $cml;
			close(CMLFILE);
		}

		return URI::file->new_abs("$path/$basename.$currentGamepk.xml");
	} else {
		$self->log("[ERR] Could not run cwbox on  $filename");
		my $cmlfile = "$path/$basename.$currentGamepk" . "_cml.xml";
		open( CMLFILE, ">$cmlfile" );
		print CMLFILE $cml;
		close(CMLFILE);
	}
	return;
}

sub transform_player_file {
	my ( $self, $player_data, $fname ) = @_;

	my $cml = build_player_tree( $self, $player_data, $fname );

	my $cfg         = $self->cfg;
	my $path        = $cfg->param('mlbam_source_dir');
	my $localsource = $cfg->param('source_dir');
	my $filename    = "$path/$fname.xml";

	# BEGIN test
#	open( FILE, ">$path/$fname.xml" );
#	print FILE $cml;
#	close(FILE);
#	return undef;
	# END test

	my $stylesheet = $self->get_stylesheet( $self->code_player_files() );
	my $parser     = XML::LibXML->new();
	my $doc        = $parser->parse_string($cml);
	my $tcml       = $stylesheet->transform($doc);
	my $sml        = $stylesheet->output_string($tcml);

	open( FILE, ">$path/$fname.xml" );
	print FILE $sml;
	close(FILE);
	return URI::file->new_abs("$path/$fname.xml");
}

sub build_player_tree {
	my ( $self, $player_data, $fname ) = @_;

	my @player_data_array = @$player_data;
	my $caseyml;
	my $xwriter = new XML::Writer(
		OUTPUT      => \$caseyml,
		DATA_MODE   => 1,
		DATA_INDENT => 2
	);
	$xwriter->xmlDecl();

	$xwriter->startTag( 'file', 'name' => $fname );
#	print "size for $fname: ".scalar(@player_data_array)."\n";
	while ( scalar(@player_data_array) > 0 ) {
		my $line = shift(@player_data_array);
#		$line =~ s/\"//g;
##		print "line $line\n" if $fname =~ /changes/;
#		my @parts = split(/,/, $line);

		# Using Text::ParseWords
		# See http://search.cpan.org/dist/Text-ParseWords/ParseWords.pm
		my @parts = quotewords ',', 0, $line;

		$xwriter->startTag( 'player' );
		foreach my $field (@parts) {
			if ( $field ne '' ) {
				$field =~ s/^\s*//;
				$field =~ s/\s*$//;
				$xwriter->startTag('field');
				$xwriter->characters($field);
				$xwriter->endTag('field');
			} else {
				$xwriter->emptyTag('field');
			}
		}
		$xwriter->endTag('player');
	}

	$xwriter->endTag('file');
	$xwriter->end();

	return $caseyml;
}

sub buildTree {
	my ( $self, $currentGamepk, $currentGameid, $currentGameStack ) = @_;

	my @gameArray = @$currentGameStack;
	my $caseyml;
	my $xwriter = new XML::Writer(
		OUTPUT      => \$caseyml,
		DATA_MODE   => 1,
		DATA_INDENT => 2
	);
	$xwriter->xmlDecl();

	$xwriter->startTag(
		'game',
		'game_pk' => $currentGamepk,
		'game_id' => $currentGameid
	);

	while ( scalar(@gameArray) > 0 ) {
		my $curRecordRef = shift(@gameArray);
		my @curRecord    = @$curRecordRef;
		my $rec_seq      = shift(@curRecord);
		my $rec_type     = shift(@curRecord);

		$xwriter->startTag(
			'record',
			'rec_seq'  => $rec_seq,
			'rec_type' => $rec_type
		);

		foreach my $field (@curRecord) {
			if ( $field ne '' ) {
				$xwriter->startTag('field');
				$xwriter->characters($field);
				$xwriter->endTag('field');
			} else {
				$xwriter->emptyTag('field');
			}
		}

		$xwriter->endTag('record');
	}

	$xwriter->endTag('game');
	$xwriter->end();

	return $caseyml;
}

# Return and cache a stylesheet object for a given transformation
sub get_stylesheet {
	my ( $self, $name ) = @_;

	return $self->{'xslts'}->{$name} if $self->{'xslts'}->{$name};
	my $base_path = $self->cfg->param('stylesheet_dir')
	  or die "Stylesheet_dir not defined in config file";

	my $xslt      = XML::LibXSLT->new();
	my $path      = "$base_path/$name.xsl";
	my $style_ref = $xslt->parse_stylesheet_file($path)
	  or die "Can't load stylesheet for $name: $!";
	$self->{'xslts'}->{$name} = $style_ref;

	return $style_ref;
}

# Return last-modification time for file
sub get_mod {
	my ($file) = @_;

	my $mod = ( stat($file) )[9];
	return $mod;
}

sub retrieve_doc {
	my ( $self, $uri_str ) = @_;

	my $uri = URI->new($uri_str);
	die "URL is not file: $uri" if $uri->scheme ne 'file';

	$self->{'last_url'} = $uri;
	my $xml = get($uri) or die "Couldn't retrieve $uri: $!";

	return $xml;
}

sub filter {
	my ( $self, $xml ) = @_;

	my $uri = $self->{'last_url'} or die "No last_url";
	my $filterable = $uri =~ /\.raw$/
	  || $self->{cfg}->param('local_always_raw');

	my $plugin = $self->{'plugin_module'};

	die "No plugin defined" if $filterable && !$plugin;
	my @filtered =
	    $filterable
	  ? $self->{'plugin_module'}->filter($xml)
	  : ($xml);

	return @filtered;
}

sub post_retrieve {
	my ($self) = @_;

	my $uri = $self->{'last_url'} or return;

	# delete file just processed
	unlink $uri->file if ( $DEBUG_MODE eq "FALSE" );
}

1;
