# $RCSfile$; $Date: 2008-01-10 22:23:56 -0800 (Thu, 10 Jan 2008) $; $Revision: 1675 $; $Name$
package Feedfetcher::Plugins::Olympics;

use strict;
use Feedfetcher::Plugins::Retriever;
our @ISA = ("Feedfetcher::Plugins::Retriever");

use Core::LineParser;

use News::NNTPClient;
use MIME::Entity;
use MIME::Parser;
use XML::LibXML;
use URI;
use FileHandle;

sub name { return "Olympics wnpaML feed"; }
sub code { return "olympics"; }

sub initialize {
	my ($self) = @_;

	my $xsltp = new XML::LibXSLT;
	my $short_code = $self->shortcode;
	my $xsl_path = $self->cfg->required_param("${short_code}_xsl");
	$self->{'xsl'} = $xsltp->parse_stylesheet_file($xsl_path);

	$self->load_last_articles;
}

sub nntp_connect {
	my ($self) = @_;

	my $cfg = $self->cfg;
	my $short_code = $self->shortcode;
	my $server = $cfg->required_param("${short_code}_server");
	my $user = $cfg->required_param("${short_code}_user");
	my $pass = $cfg->required_param("${short_code}_pass");
	my $c = new News::NNTPClient($server, "", 0);
	my $msg = $c->message;
	chomp $msg;
	my $code = $c->code;
	my $status = $msg ? "$msg ($code)" : $code;
	my $name = $self->name;
	$c->ok or die "Can't connect to $name server $server: $status";
	$c->authinfo($user, $pass) or 
		die "Can't authenticate: $!";

	return $c;
}

sub get_list {
	my ($self) = @_;

	my @files = ();
	my @groups = $self->get_groups;
	$self->{'server'} = $self->retry(\&nntp_connect);
	foreach my $group (@groups) {
		my $first_last = $self->retry(
			sub { 
				my ($self, $group) = @_; 
				my $first_last = $self->{'server'}->group($group) or die $!; 
				return $first_last;
			}, $group);
		my ($first, $last) = split(/-/, $first_last);
		$first = $self->get_first($first, $last, $group);
		for (my $i = $first; $i <= $last; $i++) {
			my $art = $self->retry(
				sub { 
					my ($self, $art_number) = @_; 
					return $self->{'server'}->stat($art_number) or die $!; 
				}, $i) or next;
			push @files, $art;
			$self->map_article($art, $group, $i);
		}
	}

	return @files;
}

sub retrieve_doc {
	my ($self, $art_id) = @_;

	$self->{'last_article'} = $self->map_article($art_id);
	my $lines_ref = $self->retry(
		sub {
			my ($self, $art_id) = @_;
			my $lines_ref = $self->{'server'}->article($art_id); 
			return $lines_ref;
		}, $art_id) or next;
	my $article = join("", @$lines_ref);

	my $parser = new MIME::Parser;
	$parser->output_to_core(1);	# Don't print out to disk
	my $entity = $parser->parse_data($article);
	my $xml_entity  = $entity->parts(1);
	my $body = $xml_entity->bodyhandle;
	my $xml = $body->as_string;

	return $xml
}

sub post_retrieve {
	my ($self) = @_;
	my ($group, $last) = @{$self->{'last_article'}};
	$self->set_last($group, $last);
	$self->store_last_articles;
}

sub cleanup {
	my ($self) = @_;

	$self->store_last_articles;
	undef $self->{'server'};
}

sub map_article {
	my ($self, $art_id, $group, $art_num) = @_;

	$self->{'map_article'} = {} if !defined $self->{'map_article'};
	if (defined $group) {
		$self->{'map_article'}->{$art_id} = [ $group, $art_num ];
	} else {
		return $self->{'map_article'}->{$art_id};
	}
}

sub get_groups {
	my ($self) = @_;

	my $short_code = $self->shortcode;
	my $groups_file = $self->cfg->required_param("${short_code}_groups_file");
	my @groups = Core::LineParser::load($groups_file);
	return @groups;
}

sub get_first {
	my ($self, $first, $last, $group) = @_;

	my $new_first = undef;

	my $force_last = $self->arg('last');
	$new_first = $last - ($force_last - 1) if $force_last;

	if (!defined $new_first) {
		my $last_loaded = $self->{'last_article_table'}->{$group};
		$new_first = $last_loaded + 1 if defined $last_loaded;
	} 

	if (!defined $new_first) {
		my $short_code = $self->shortcode;
		my $cfg_last = $self->cfg->required_param("${short_code}_last");
		return $last - ($cfg_last - 1);
	}

	return $new_first > $first ? $new_first : $first;
}

sub load_last_articles {
	my ($self) = @_;

	$self->{'last_article_table'} = {};
	my $home_path = Core::Util::get_home_path();
	my $short_code = $self->shortcode;
	my $last_article_file = "$home_path/${short_code}_last_article";
	return if !-f $last_article_file;

	my @lines = Core::LineParser::load($last_article_file);
	foreach my $line (@lines) {
		my ($group, $last_art) = split(/:/, $line);
		my $short_code = $self->shortcode;
		die "Corrupt ${short_code}_last_article file: $line" if 
			!defined $group || !defined $last_art;
		$self->{'last_article_table'}->{$group} = $last_art;
	}
}

sub store_last_articles {
	my ($self) = @_;

	return if !$self->{'last_article_table'};
	my $home_path = Core::Util::get_home_path();
	my $short_code = $self->shortcode;
	my $last_article_file = "$home_path/${short_code}_last_article";
	my $fh = new FileHandle ">$last_article_file" or 
		die "Can't write $last_article_file: $!";
	my %last_article_table = %{$self->{'last_article_table'}};
	foreach my $group (keys %last_article_table) {
		my $last_article = $last_article_table{$group};
		print $fh "$group:$last_article\n";
	}
}

sub set_last {
	my ($self, $group, $last) = @_;
	
	$self->{'last_article_table'}->{$group} = $last;
}

sub filter {
	my ($self, $xml) = @_;

	my $doc = $self->parser->parse_string($xml);	  
	my $xsl = $self->{'xsl'};
	my $results = $xsl->transform($doc);
    return ($xsl->output_string($results));
}

1;