# $RCSfile$; $Date: 2005-11-01 13:17:12 -0500 (Tue, 01 Nov 2005) $; $Revision: 967 $; $Name$
package Feedfetcher::Plugins::Statsinc;

use strict;
use Feedfetcher::Plugins::Retriever;
our @ISA = ("Feedfetcher::Plugins::Retriever");
our $DEBUG_MODE = "FALSE";

use Carp;

use File::Find;
use File::Basename;
use LWP::Simple;
use URI;
use URI::file;
use XML::LibXSLT;
use XML::LibXML;
use XML::Writer;

sub name { return "statsinc"; }
sub code { return "stats2retrosheet"; }

sub save_raw {
	my ($self) = @_;
	return $self->{'plugin_module'} ? $self->{'plugin_module'}->save_raw : 0;
}

sub initialize {
	my ($self, $plugin_module) = @_;

	$self->{plugin_module} = $plugin_module;
}

sub get_list {
	my ($self) = @_;

	my $cfg = $self->cfg;
	my $path = $cfg->param('statsinc_source_dir');

	die "No such directory" if !-d $path;

  # check for a players.csv file, and create the player stats xml files if
  # it exists
#  $self->parsePlayersStats();
    $self->process_players_dat();
    
	my @statsinc_files = ();
	my @xml_files = ();

	find(sub { push @statsinc_files, URI::file->new_abs($File::Find::name)
		if /games.*\.dat$/i }, $path);

	@statsinc_files = sort { get_mod($a) <=> get_mod($b) } @statsinc_files;

  foreach my $statsincfileuri (@statsinc_files) {
    # local vars
    my $statsincfile = $statsincfileuri->file;
    my ($basename, $path) = fileparse($statsincfile);
    my $startpart =  index($basename, '_') + 1;
    my $lengthpart = index($basename, '.') - $startpart;
    my $datepart = substr($basename, $startpart, $lengthpart);

    # figure out other dependant file names corresponding to current games file
    # and store it for later use
    $self->{'gamesfile'} = $statsincfile;
    $self->{'eventsfile'} = URI::file->new_abs("$path" . "events_$datepart.dat");
    $self->{'defensefile'} = URI::file->new_abs("$path" . "defense_$datepart.dat");
    $self->{'pitchesfile'} = URI::file->new_abs("$path" . "pitches_$datepart.dat");

    # open statsinc game files and for each game line, build the sportsml and
    # then add it to the xml_files array to be returned for processing
    open(STATSFILE, $statsincfile);
      while (<STATSFILE>) {
        my $line = $_;
        $line =~ s/\n//g;

        if ($line != '') {
          my $tyrgam = substr($line, 0, 9);

	# $self->log("[DBG] *** Found a game for $tyrgam -- basename is $basename, datepart is $datepart");

          my $sml = $self->transform($tyrgam, $line, $datepart);

          if ($sml) {
            push(@xml_files, $sml);
          }
        }
      }
    close(STATSFILE);

    # finally delete the .dat files

    # unlink $statsincfile; # the gamesfile
    unlink $statsincfile if ($DEBUG_MODE eq "FALSE");

    if (-e $self->{'eventsfile'}->file) {
      unlink $self->{'eventsfile'}->file if ($DEBUG_MODE eq "FALSE");
    }

    if (-e $self->{'defensefile'}->file) {
      unlink $self->{'defensefile'}->file if ($DEBUG_MODE eq "FALSE");
    }

    if (-e $self->{'pitchesfile'}->file) {
      unlink $self->{'pitchesfile'}->file if ($DEBUG_MODE eq "FALSE");
    }
  }

	return @xml_files;
}


sub process_players_dat {
    my ($self) = @_;

    my $cfg = $self->cfg;
    my $xsl_dir = $cfg->param('stylesheet_dir');
    my $path = $cfg->param('statsinc_source_dir');

#    my $xsl_dir = './xsl';
#    my $path = '.';

    my $dat_file = "$path/players.dat";
        
    if (! -e $dat_file) {
       $self->log("[LOW] Can't find player DAT file $dat_file.");  	
        return;
    }
    $self->log("[LOW] DAT file found, parsing out players stats files");
   
    my $count = unlink <$xsl_dir/players/players_stats_*.xml>;
    $self->log("[LOW] Removed $count previous players_stats*xml files");

    open(FILE, $dat_file) || die "Cannot open dat file";
    my @lines = <FILE>;
    close(FILE);

    my $template =  "A20 A20 A25 A15 A15 A2 A5 A1 ";
    $template .= "A1 A3 A4 A4 A8 A25 ";
    $template .= "A4 A20 A3 A30 A8 A1 ";
    $template .= "A1 A3 A3 A3 A1 A20 A2 A2 ";
    $template .= "A1";

    my $out_val;
    my $xwriter = XML::Writer->new(
                     #              ENCODING => "utf-8",
                                   OUTPUT => \$out_val,
                                   DATA_MODE => 1,
                                   DATA_INDENT => 2
                                   );

    foreach my $line (@lines) {
        $out_val = '';
        chomp;
        my ($surname, $familiar_name, $first_name, $other_names_1, $other_names_2, $initials, $player_id, $bat_side,
            $throwing_hand, $team_now_id, $year_1, $year_last, $birth_date, $birth_city,
            $birth_country, $state, $birth_state, $college, $death_date, $roster_status,
            $pitcher, $ht, $wt, $uniform_number, $rookie, $ln_caps, $position_1, $position_2, $roster_position,
            $ps_status) = unpack($template, $line);


        # header declaractions
#        $xwriter->xmlDecl();

        my $contents = '';

        $xwriter->startTag('PLAYER');
        $self->simple_tag(\$xwriter, 'SURNAME', $surname);
        $self->simple_tag(\$xwriter, 'FAMILIAR_NAME', $familiar_name);
        $self->simple_tag(\$xwriter, 'FIRST_NAME', $first_name);
        $self->simple_tag(\$xwriter, 'OTHER_NAMES', $other_names_1, 1);
        $self->simple_tag(\$xwriter, 'OTHER_NAMES', $other_names_2, 1);
        $self->simple_tag(\$xwriter, 'NITIALS', $initials);
        $self->simple_tag(\$xwriter, 'PLAYER_ID', $player_id);
        $self->simple_tag(\$xwriter, 'BAT_SIDE', $bat_side);
        $self->simple_tag(\$xwriter, 'THROWING_HAND', $throwing_hand);
        $self->simple_tag(\$xwriter, 'TEAM_NOW_ID', $team_now_id);
        $self->simple_tag(\$xwriter, 'YEAR_1', $year_1);
        $self->simple_tag(\$xwriter, 'YEAR_LAST', $year_last);
        $self->simple_tag(\$xwriter, 'BIRTH_DATE', $birth_date);
        $self->simple_tag(\$xwriter, 'BIRTH_CITY', $birth_city);
        $self->simple_tag(\$xwriter, 'BIRTH_COUNTRY', $birth_country);
        $self->simple_tag(\$xwriter, 'STATE', $state);
        $self->simple_tag(\$xwriter, 'BIRTH_STATE', $birth_state);
        $self->simple_tag(\$xwriter, 'COLLEGE', $college);
        $self->simple_tag(\$xwriter, 'ROSTER_STATUS', $roster_status);
        $self->simple_tag(\$xwriter, 'PITCHER', $pitcher);
        $self->simple_tag(\$xwriter, 'HT', $ht);
        $self->simple_tag(\$xwriter, 'WT', $wt);
        $self->simple_tag(\$xwriter, 'UNIFORM_NUMBER', $uniform_number);
        $self->simple_tag(\$xwriter, 'ROOKIE', $rookie);
        $self->simple_tag(\$xwriter, 'LN_CAPS', $ln_caps);
        $self->simple_tag(\$xwriter, 'POSITION', $position_1, 1);
        $self->simple_tag(\$xwriter, 'POSITION', $position_2, 1);
        $self->simple_tag(\$xwriter, 'ROSTER_POSITION', $roster_position);
        $self->simple_tag(\$xwriter, 'PS_STATUS', $ps_status);
        $xwriter->endTag('PLAYER');
#        $xwriter->close();
        $xwriter->end();

        my $file_prefix = substr(sprintf('%05d', $player_id), 0, 3);
        my $file_path = "$xsl_dir/players/players_stats_$file_prefix.xml";
        my @content_array;

        if (-e $file_path) {
            open(SPECIFICPLAYERFILE, "<$file_path");
            @content_array = <SPECIFICPLAYERFILE>;
            $contents = join("", @content_array);
            close(SPECIFICPLAYERFILE);
            open(SPECIFICPLAYERFILE, ">$file_path");
            $contents =~ s/\<\/PLAYERS\>//g;

            $contents .= "\n$out_val\n</PLAYERS>";
            print SPECIFICPLAYERFILE $contents;
            close(SPECIFICPLAYERFILE);        
        }
        else {
            open(SPECIFICPLAYERFILE, ">$file_path");
            $contents = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<PLAYERS>\n".$out_val."\n</PLAYERS>";
            print SPECIFICPLAYERFILE $contents;
            close(SPECIFICPLAYERFILE);        
        }
    }
    
    unlink <$path/players.dat>;    
    
}

sub simple_tag {
    my ($self, $xwriter, $tag, $value, $skip_empty) = @_;

    $skip_empty ||= '0';
    $value =~ s/^\s+//;
    $value =~ s/\s+$//;
    return if ($skip_empty eq '1' && $value eq '');

    $$xwriter->startTag($tag);
    $$xwriter->characters($value);
    $$xwriter->endTag($tag);
}

sub parsePlayersStats_old_csv {
  my ($self) = @_;

	my $cfg = $self->cfg;
	my $xsldir = $cfg->param('stylesheet_dir');
	my $path = $cfg->param('statsinc_source_dir');
  my $csvfile = "$path/players.csv";

  if (-e  $csvfile) {
    $self->log("[LOW] CSV file found, parsing out players stats files");

    # remove all current player_stats files
    my $count = unlink <$xsldir/players/players_stats_*.xml>;
    $self->log("[DBG] deleted $count old players_stats files");

    open(PLAYERFILE, $csvfile);
      my @headers = ();

      while (<PLAYERFILE>) {
        my $line = $_;
        chomp($line);

        # ignore the first line of header information
        if (($line =~ /^SURNAME/)) {
          my @fields = split(',', $line);
          for (my $i=0; $i<scalar(@fields); $i++) {
            $headers[$i] = $fields[$i];
          }
        } else {
          my @fields = split(',', $line);

          # concatenate fields that have quotes in them
          while (scalar(@fields) != 41) {
            for (my $i=0; $i<scalar(@fields); $i++) {
              if ($fields[$i] =~ /"/) {
                my $newfield = $fields[$i] . $fields[$i+1];
                splice (@fields, $i, 2, $newfield);
              }
            }
          }

          # get player id
          my $playerid = $fields[6];

          # add line to appropriate file
          $self->appendPlayersStatsFile($playerid, \@fields, \@headers);
        }
      }
    close(PLAYERFILE);

    # now we have to close off every created players_stats file with a closing
    # </PLAYERS> element
    my @players_stats_files = ();

	  find(sub { push @players_stats_files, URI::file->new_abs($File::Find::name) if /players_stats_.*\.xml$/i }, $xsldir);

    foreach my $players_stats_file_uri (@players_stats_files) {
      my $players_stats_file = $players_stats_file_uri->file;
      open(SPECIFICPLAYERFILE, ">>$players_stats_file");
        print SPECIFICPLAYERFILE "</PLAYERS>\n";
      close(SPECIFICPLAYERFILE);
    }

    # remove the players.csv when we're done
    unlink ($csvfile);
  }
}

sub appendPlayersStatsFile {
  my ($self, $id, $fieldsref, $headersref) = @_;

  my @fields = @$fieldsref;
  my @headers = @$headersref;

	my $cfg = $self->cfg;
	my $xsldir = $cfg->param('stylesheet_dir');

  my $fileprefix = substr(sprintf('%05d', $id), 0, 3);
  my $filepath = "$xsldir/players/players_stats_$fileprefix.xml";

  if (!-e $filepath) {
    open(SPECIFICPLAYERFILE, ">>$filepath");
      print SPECIFICPLAYERFILE "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      print SPECIFICPLAYERFILE "<PLAYERS>\n";
    close(SPECIFICPLAYERFILE);
  }

  open(SPECIFICPLAYERFILE, ">>$filepath");
    print SPECIFICPLAYERFILE "  <PLAYER>\n";
    for (my $i=0; $i<scalar(@fields); $i++) {
      if ($fields[$i] ne '') {
        print SPECIFICPLAYERFILE "    <$headers[$i]>$fields[$i]</$headers[$i]>\n";
      }
    }
    print SPECIFICPLAYERFILE "  </PLAYER>\n";
  close(SPECIFICPLAYERFILE);
}

sub buildTree {
  my ($self, $tyrgam, $gameLine) = @_;

  # extract data items from fixed width fields of the game line
  my $teama = substr($gameLine, 9, 2);
  my $teamh = substr($gameLine, 11, 2);
  my $gamedate = substr($gameLine, 13, 8);
  my $year = substr($gameLine, 21, 4);
  my $hora = substr($gameLine, 25, 1);
  my $game_id1 = substr($gameLine, 26, 3);
  my $game_id2 = substr($gameLine, 29, 3);
  my $sch = substr($gameLine, 32, 1);
  my $schtyp = substr($gameLine, 33, 1);
  my $dorn = substr($gameLine, 34, 1);
  my $postponed = substr($gameLine, 35, 1);
  my $outcomea = substr($gameLine, 36, 1);
  my $outcomeh = substr($gameLine, 37, 1);
  my $scorea = substr($gameLine, 38, 2);
  my $scoreh = substr($gameLine, 40, 2);
  my $inning = substr($gameLine, 42, 2);
  my $startera = substr($gameLine, 44, 5);
  my $starterh = substr($gameLine, 49, 5);
  my $finishera = substr($gameLine, 54, 5);
  my $finisherh = substr($gameLine, 59, 5);
  my $winner = substr($gameLine, 64, 5);
  my $loser = substr($gameLine, 69, 5);
  my $savpit = substr($gameLine, 74, 5);
  my $temp = substr($gameLine, 79, 3);
  my $humidity = substr($gameLine, 82, 3);
  my $clouds = substr($gameLine, 85, 2);
  my $windir = substr($gameLine, 87, 2);
  my $winvel = substr($gameLine, 89, 3);
  my $attendance = substr($gameLine, 92, 6);
  my $time_g = substr($gameLine, 98, 5);
  my $umpire0 = substr($gameLine, 103, 15);
  my $umpire1 = substr($gameLine, 118, 15);
  my $umpire2 = substr($gameLine, 133, 15);
  my $umpire3 = substr($gameLine, 148, 15);
  my $umpid0 = substr($gameLine, 163, 5);
  my $umpid1 = substr($gameLine, 168, 5);
  my $umpid2 = substr($gameLine, 173, 5);
  my $umpid3 = substr($gameLine, 178, 5);

  # store the YEAR to use in $self for later use
  $self->{'yeartouse'} = $year;

  # start new caseyml document
  my $caseyml;
  my $xwriter = new XML::Writer(
                                OUTPUT => \$caseyml,
                                DATA_MODE => 1,
                                DATA_INDENT => 2
                               );

  # header declaractions
  $xwriter->xmlDecl();

  # start game tag
  $xwriter->startTag('game',
                     'game_id' => $tyrgam);

  # start record tag for GAMESRD
  $xwriter->startTag('record',
                     'rec_type' => 'GAMESRD');

  # -BEGIN- field tags for GAMESRD
  $xwriter->startTag('field',
                     'name' => 'TYRGAM');
  $xwriter->characters($tyrgam);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'TEAM');
  $xwriter->characters($teama);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'TEAM');
  $xwriter->characters($teamh);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'GAMEDATE');
  $xwriter->characters($gamedate);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'YEAR');
  $xwriter->characters($year);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'HORA');
  $xwriter->characters($hora);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'GAME_ID');
  $xwriter->characters($game_id1);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'GAME_ID');
  $xwriter->characters($game_id2);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'SCH');
  $xwriter->characters($sch);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'SCHTYP');
  $xwriter->characters($schtyp);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'DORN');
  $xwriter->characters($dorn);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'POSTPONED');
  $xwriter->characters($postponed);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'OUTCOME');
  $xwriter->characters($outcomea);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'OUTCOME');
  $xwriter->characters($outcomeh);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'SCORE');
  $xwriter->characters($scorea);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'SCORE');
  $xwriter->characters($scoreh);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'INNING');
  $xwriter->characters($inning);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'STARTER');
  $xwriter->characters($startera);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'STARTER');
  $xwriter->characters($starterh);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'FINISHER');
  $xwriter->characters($finishera);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'FINISHER');
  $xwriter->characters($finisherh);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'WINNER');
  $xwriter->characters($winner);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'LOSER');
  $xwriter->characters($loser);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'SAVPIT');
  $xwriter->characters($savpit);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'TEMP');
  $xwriter->characters($temp);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'HUMIDITY');
  $xwriter->characters($humidity);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'CLOUDS');
  $xwriter->characters($clouds);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'WINDIR');
  $xwriter->characters($windir);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'WINVEL');
  $xwriter->characters($winvel);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'ATTENDANCE');
  $xwriter->characters($attendance);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'TIME_G');
  $xwriter->characters($time_g);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPIRES');
  $xwriter->characters($umpire0);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPIRES');
  $xwriter->characters($umpire1);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPIRES');
  $xwriter->characters($umpire2);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPIRES');
  $xwriter->characters($umpire3);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPID');
  $xwriter->characters($umpid0);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPID');
  $xwriter->characters($umpid1);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPID');
  $xwriter->characters($umpid2);
  $xwriter->endTag('field');

  $xwriter->startTag('field',
                     'name' => 'UMPID');
  $xwriter->characters($umpid3);
  $xwriter->endTag('field');
  # -END- field tags for GAMESRD

  # end record tag for GAMESRD
  $xwriter->endTag('record');

  # get lines from events file
  my @eventLines = $self->getDataLines($tyrgam, 'events');

  # output each event line as an event record
  foreach my $eventline (@eventLines) {
    my $teama = substr($eventline, 9, 3);
    my $teamh = substr($eventline, 12, 3);
    my $batter = substr($eventline, 15, 5);
    my $e = substr($eventline, 20, 2);
    my $evp0 = substr($eventline, 22, 1);
    my $evp1 = substr($eventline, 23, 1);
    my $evp2 = substr($eventline, 24, 1);
    my $evp3 = substr($eventline, 25, 1);
    my $evp4 = substr($eventline, 26, 1);
    my $evp5 = substr($eventline, 27, 1);
    my $evp6 = substr($eventline, 28, 1);
    my $evp7 = substr($eventline, 29, 1);
    my $evp8 = substr($eventline, 30, 1);
    my $evp9 = substr($eventline, 31, 1);
    my $nevent = substr($eventline, 32, 3);
    my $pitcher = substr($eventline, 35, 5);
    my $defense = substr($eventline, 40, 3);
    my $inning = substr($eventline, 43, 2);
    my $outs0 = substr($eventline, 45, 1);
    my $outs1 = substr($eventline, 46, 1);
    my $bc0 = substr($eventline, 47, 1);
    my $bc1 = substr($eventline, 48, 1);
    my $ron0 = substr($eventline, 49, 5);
    my $ron1 = substr($eventline, 54, 5);
    my $ron2 = substr($eventline, 59, 5);
    my $score0 = substr($eventline, 64, 2);
    my $score1 = substr($eventline, 66, 2);
    my $score2 = substr($eventline, 68, 2);
    my $rbi = substr($eventline, 70, 1);
    my $fielding = substr($eventline, 71, 5);
    my $battype = substr($eventline, 76, 1);
    my $error = substr($eventline, 77, 1);
    my $direction = substr($eventline, 78, 1);
    my $distance = substr($eventline, 79, 3);
    my $hardhit = substr($eventline, 82, 2);
    my $balls = substr($eventline, 84, 1);
    my $strikes = substr($eventline, 85, 1);
    my $npitches0 = substr($eventline, 86, 2);
    my $npitches1 = substr($eventline, 88, 2);
    my $pitch_result = substr($eventline, 90, 1);
    my $pitch_type = substr($eventline, 91, 1);
    my $zone1 = substr($eventline, 92, 1);
    my $zone2 = substr($eventline, 93, 2);
    my $fieldpos = substr($eventline, 95, 2);
    my $bpos = substr($eventline, 97, 1);
    my $tob0 = substr($eventline, 98, 1);
    my $tob1 = substr($eventline, 99, 1);
    my $tob2 = substr($eventline, 100, 1);
    my $skill = substr($eventline, 101, 5);
    my $seq = substr($eventline, 106, 3);
    my $dist_1b = substr($eventline, 109, 4);
    my $dist_3b = substr($eventline, 113, 4);

    # start record tag for EVENTSRD
    $xwriter->startTag('record',
                       'rec_type' => 'EVENTSRD');

    # -BEGIN- field tags for EVENTSRD
    $xwriter->startTag('field',
                       'name' => 'TYRGAM');
    $xwriter->characters($tyrgam);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TEAM');
    $xwriter->characters($teama);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TEAM');
    $xwriter->characters($teamh);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BATTER');
    $xwriter->characters($batter);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'E');
    $xwriter->characters($e);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp3);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp4);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp5);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp6);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp7);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp8);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'EVP');
    $xwriter->characters($evp9);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NEVENT');
    $xwriter->characters($nevent);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PITCHER');
    $xwriter->characters($pitcher);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DEFENSE');
    $xwriter->characters($defense);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'INNING');
    $xwriter->characters($inning);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'OUTS');
    $xwriter->characters($outs0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'OUTS');
    $xwriter->characters($outs1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BC');
    $xwriter->characters($bc0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BC');
    $xwriter->characters($bc1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'RON');
    $xwriter->characters($ron0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'RON');
    $xwriter->characters($ron1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'RON');
    $xwriter->characters($ron2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SCORE');
    $xwriter->characters($score0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SCORE');
    $xwriter->characters($score1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SCORE');
    $xwriter->characters($score2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'RBI');
    $xwriter->characters($rbi);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'FIELDING');
    $xwriter->characters($fielding);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BATTYPE');
    $xwriter->characters($battype);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'ERROR');
    $xwriter->characters($error);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DIRECTION');
    $xwriter->characters($direction);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DISTANCE');
    $xwriter->characters($distance);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'HARDHIT');
    $xwriter->characters($hardhit);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BALLS');
    $xwriter->characters($balls);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'STRIKES');
    $xwriter->characters($strikes);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NPITCHES');
    $xwriter->characters($npitches0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NPITCHES');
    $xwriter->characters($npitches1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PITCH_RESULT');
    $xwriter->characters($pitch_result);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PITCH_TYPE');
    $xwriter->characters($pitch_type);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'ZONE1');
    $xwriter->characters($zone1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'ZONE2');
    $xwriter->characters($zone2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'FIELDPOS');
    $xwriter->characters($fieldpos);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BPOS');
    $xwriter->characters($bpos);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TOB');
    $xwriter->characters($tob0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TOB');
    $xwriter->characters($tob1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TOB');
    $xwriter->characters($tob2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SKILL');
    $xwriter->characters($skill);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SEQ');
    $xwriter->characters($seq);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DIST_1B');
    $xwriter->characters($dist_1b);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DIST_3B');
    $xwriter->characters($dist_3b);
    $xwriter->endTag('field');
    # -END- field tags for EVENTSRD

    # end record tag for EVENTSRD
    $xwriter->endTag('record');
  }

  # get lines from defense file
  my @defenseLines = $self->getDataLines($tyrgam, 'defense');

  # output each defense line as an event record
  foreach my $defenseline (@defenseLines) {
    my $defense = substr($defenseline, 9, 3);
    my $team = substr($defenseline, 12, 3);
    my $player0 = substr($defenseline, 15, 5);
    my $player1 = substr($defenseline, 20, 5);
    my $player2 = substr($defenseline, 25, 5);
    my $player3 = substr($defenseline, 30, 5);
    my $player4 = substr($defenseline, 35, 5);
    my $player5 = substr($defenseline, 40, 5);
    my $player6 = substr($defenseline, 45, 5);
    my $player7 = substr($defenseline, 50, 5);

    # start record tag for DEFENSERD
    $xwriter->startTag('record',
                       'rec_type' => 'DEFENSERD');

    # -BEGIN- field tags for DEFENSERD
    $xwriter->startTag('field',
                       'name' => 'TYRGAM');
    $xwriter->characters($tyrgam);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'DEFENSE');
    $xwriter->characters($defense);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'TEAM');
    $xwriter->characters($team);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player3);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player4);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player5);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player6);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PLAYERS');
    $xwriter->characters($player7);
    $xwriter->endTag('field');
    # -END- field tags for DEFENSERD

    # end record tag for DEFENSERD
    $xwriter->endTag('record');
  }

  # get lines from pitches file
  my @pitchesLines = $self->getDataLines($tyrgam, 'pitches');

  # output each pitch line as an event record
  foreach my $pitchline (@pitchesLines) {
    my $nevent = substr($pitchline, 9, 3);
    my $nballs = substr($pitchline, 12, 1);
    my $nstrikes = substr($pitchline, 13, 1);
    my $cs0 = substr($pitchline, 14, 1);
    my $cs1 = substr($pitchline, 15, 1);
    my $cs2 = substr($pitchline, 16, 1);
    my $cs3 = substr($pitchline, 17, 1);
    my $cs4 = substr($pitchline, 18, 1);
    my $cs5 = substr($pitchline, 19, 1);
    my $cs6 = substr($pitchline, 20, 1);
    my $cs7 = substr($pitchline, 21, 1);
    my $cs8 = substr($pitchline, 22, 1);
    my $cs9 = substr($pitchline, 23, 1);
    my $cs10 = substr($pitchline, 24, 1);
    my $cs11 = substr($pitchline, 25, 1);
    my $result = substr($pitchline, 26, 1);
    my $pitchtype = substr($pitchline, 27, 1);
    my $zone1 = substr($pitchline, 28, 1);
    my $zone2 = substr($pitchline, 29, 2);
    my $velocity = substr($pitchline, 31, 3);
    my $seq = substr($pitchline, 34, 4);
    my $batter_id = substr($pitchline, 38, 5);
    my $pitcher_id = substr($pitchline, 43, 5);
    my $ispa = substr($pitchline, 48, 1);
    my $islhb = substr($pitchline, 49, 1);

    # start record tag for PITCHESRD
    $xwriter->startTag('record',
                       'rec_type' => 'PITCHESRD');

    # -BEGIN- field tags for PITCHESRD
    $xwriter->startTag('field',
                       'name' => 'TYRGAM');
    $xwriter->characters($tyrgam);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NEVENT');
    $xwriter->characters($nevent);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NBALLS');
    $xwriter->characters($nballs);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'NSTRIKES');
    $xwriter->characters($nstrikes);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs0);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs3);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs4);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs5);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs6);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs7);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs8);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs9);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs10);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'CS');
    $xwriter->characters($cs11);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'RESULT');
    $xwriter->characters($result);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PITCHTYPE');
    $xwriter->characters($pitchtype);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'ZONE1');
    $xwriter->characters($zone1);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'ZONE2');
    $xwriter->characters($zone2);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'VELOCITY');
    $xwriter->characters($velocity);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'SEQ');
    $xwriter->characters($seq);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'BATTER_ID');
    $xwriter->characters($batter_id);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'PITCHER_ID');
    $xwriter->characters($pitcher_id);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'isPA');
    $xwriter->characters($ispa);
    $xwriter->endTag('field');

    $xwriter->startTag('field',
                       'name' => 'isLHB');
    $xwriter->characters($islhb);
    $xwriter->endTag('field');
    # -END- field tags for PITCHESRD

    # end record tag for PITCHESRD
    $xwriter->endTag('record');
  }

  # end game tag
  $xwriter->endTag('game');

  return $caseyml;
}

sub getDataLines {
  my ($self, $tyrgam, $type) = @_;
  my @lineStack = ();

  my $typeparam = $type . 'file';
  my $datafileuri = $self->{$typeparam};
  my $datafile = $datafileuri->file;

  my $basename = basename($datafile);

  if (-e $datafile) {
    open(DATAFILE, $datafile);
      while(<DATAFILE>) {
        if ($_ =~ /^$tyrgam/) {
          push(@lineStack, $_);
        }
      }
    close(DATAFILE);
  }

  return @lineStack;
}

sub transform {
  my ($self, $tyrgam, $gameline, $datepart) = @_;

  my $cml = $self->buildTree($tyrgam, $gameline);

  my $cfg = $self->cfg;
  my $path = $cfg->param('statsinc_source_dir');
  my $localsource = $cfg->param('source_dir');
  my $filename = "$path/$datepart.$tyrgam.evx";

  my $stylesheet = $self->get_stylesheet($self->code());
  my $parser = XML::LibXML->new();
  my $doc = $parser->parse_string($cml);

  my $tcml = $stylesheet->transform($doc);
  my $evx = $stylesheet->output_string($tcml);

  open(EVXFILE, ">$filename");
  print EVXFILE $evx;
  close(EVXFILE);

    my $sml;
    if ($DEBUG_MODE eq "TRUE") {
	$sml = `cd $path; cwbox -S -y $self->{'yeartouse'} $filename`;
	}
    else {
	  $sml = `cd $path; cwbox -S -y $self->{'yeartouse'} $filename 2>/dev/null`;
	}

  if ($? == 0) {
    open(SMLFILE, ">$path/$datepart.$tyrgam.xml");
    print SMLFILE $sml;
    close(SMLFILE);

    unlink $filename if ($DEBUG_MODE eq "FALSE");

    if ($DEBUG_MODE eq "TRUE") {
	    my $cmlfile = "$path/$datepart.$tyrgam" . "_cml.xml";
	    open(CMLFILE, ">$cmlfile");
	    print CMLFILE $cml;
	    close(CMLFILE);
	}


    return URI::file->new_abs("$path/$datepart.$tyrgam.xml");
  } else {
    $self->log("[ERR] Could not run cwbox on  $filename");

    # DEBUG
    # write out the caseyml for debuging
    # uncomment 4 lines below to have the caseyml be written out upon error
    # DEBUG
    my $cmlfile = "$path/$datepart.$tyrgam" . "_cml.xml";
    open(CMLFILE, ">$cmlfile");
      print CMLFILE $cml;
    close(CMLFILE);
  }

  return;
}

# Return and cache a stylesheet object for a given transformation
sub get_stylesheet {
  my ($self, $name) = @_;

  return $self->{'xslts'}->{$name} if $self->{'xslts'}->{$name};
  my $base_path = $self->cfg->param('stylesheet_dir') or
    die "Stylesheet_dir not defined in config file";

  my $xslt = XML::LibXSLT->new();
  my $path = "$base_path/$name.xsl";
  my $style_ref = $xslt->parse_stylesheet_file($path) or
    die "Can't load stylesheet for $name: $!";
  $self->{'xslts'}->{$name} = $style_ref;

  return $style_ref;
}

# Return last-modification time for file
sub get_mod {
	my ($file) = @_;

	my $mod = (stat($file))[9];
	return $mod;
}

sub retrieve_doc {
	my ($self, $uri_str) = @_;

	my $uri = URI->new($uri_str);
	die "URL is not file: $uri" if $uri->scheme ne 'file';

	$self->{'last_url'} = $uri;
	my $xml = get($uri) or die "Couldn't retrieve $uri: $!";

	return $xml;
}

sub filter {
	my ($self, $xml) = @_;

	my $uri = $self->{'last_url'} or die "No last_url";
	my $filterable = $uri =~ /\.raw$/ || $self->{cfg}->param('local_always_raw');

	my $plugin = $self->{'plugin_module'};

	die "No plugin defined" if $filterable && !$plugin;
	my @filtered = $filterable ?
		$self->{'plugin_module'}->filter($xml) :
		($xml);

	return @filtered;
}

sub post_retrieve {
	my ($self) = @_;

	my $uri = $self->{'last_url'} or return;

  # delete file just processed
	unlink $uri->file if ($DEBUG_MODE eq "FALSE");
}

1;
