# $Id: LocalGeneric.pm 2130 2008-07-24 21:03:59Z ian $
package Feedfetcher::Plugins::LocalGeneric;

use strict;
use Feedfetcher::Plugins::Retriever;
our @ISA = ("Feedfetcher::Plugins::Retriever");

use Carp;

use File::Find;
use LWP::Simple;
use URI;
use URI::file;

sub name { return "local generic"; }
sub code { return "localgeneric"; }
sub forced { return 1; }
sub generic { return 1; }

sub save_raw { 
	my ($self) = @_; 
	return $self->{'plugin_module'} ? $self->{'plugin_module'}->save_raw : 0; 
}

sub initialize {
	my ($self, $plugin_module) = @_;

	$self->{plugin_module} = $plugin_module;
}

sub get_list {
	my ($self) = @_;

	my $cfg = $self->cfg;
	my $path = $cfg->param('generic_source_dir');

	die "No such directory" if !-d $path;
	my @files = ();

	find(sub { push @files, URI::file->new_abs($File::Find::name) 
		if /\.raw$/i }, $path); 
	find(sub { push @files, URI::file->new_abs($File::Find::name) 
		if /\.xml$/i }, $path); 
	@files = sort { get_mod($a) <=> get_mod($b) } @files;

	return @files;
}

# Return last-modification time for file
sub get_mod {
	my ($file) = @_;

	my $mod = (stat($file))[9];
	return $mod;
}

sub retrieve_doc {
	my ($self, $uri_str) = @_;

	my $uri = URI->new($uri_str);
	die "URL is not file: $uri" if $uri->scheme ne 'file';

	$self->{'last_url'} = $uri;
	my $xml = get($uri) or die "Couldn't retrieve $uri: $!";

	return $xml;
}

sub filter {
	my ($self, $xml) = @_;

	my $uri = $self->{'last_url'} or die "No last_url";
	my $filterable = $uri =~ /\.raw$/ || $self->{cfg}->param('local_always_raw');

	my $plugin = $self->{'plugin_module'};

	die "No plugin defined" if $filterable && !$plugin;
	my @filtered = $filterable ? 
		$self->{'plugin_module'}->filter($xml) : 
		($xml);

	return @filtered;
}

sub post_retrieve {
	my ($self) = @_;

	my $uri = $self->{'last_url'} or return;
	unlink $uri->file if !$self->arg('leave_local');
}

1;
