# $Id: Retriever.pm 2130 2008-07-24 21:03:59Z ian $
package Feedfetcher::Plugins::Retriever;

use Carp;
use strict;

use FileHandle;
use File::Path;

sub new {
    my ($class, $check_abort_callback, $args, $cfg, $log, $parser, @others) = @_;

	my $self = {};
	bless $self, $class;
	$self->{'check_abort_callback'} = $check_abort_callback or 
		croak "No abort callback given";
	$self->{'args'} = $args or croak "No command line args given";
	$self->{'cfg'} = $cfg or croak "No configuration given";
	$self->{'log'} = $log or croak "No logger given";
	$self->{'parser'} = $parser or croak "No parser given";
	$self->initialize(@others);
	return $self;
}

sub arg { my ($self, $key) = @_; return $self->{'args'}->{$key}; }
sub cfg { my ($self) = @_; return $self->{'cfg'}; }
sub log { my ($self, $msg) = @_; $self->{'log'}->log($msg); }
sub get_log { my ($self) = @_; return $self->{'log'}; }
sub parser { my ($self) = @_; $self->{parser}; }

sub name { croak "name not defined"; }
sub code { croak "code not defined"; }
sub shortcode { my ($self) = @_; return $self->code; }
sub forced { return 0; }
sub generic { return 0; }
sub get_list { croak "get_list not defined"; }
sub filter { my ($self, $doc) = @_; return ($doc); }	# Do nothing
sub retrieve_doc { croak "retrieve_doc not defined"; }
sub save_raw { 
	my ($self) = @_; 
	return $self->cfg->param($self->shortcode . "_save_raw");
}

sub retrieve { 
	my ($self, @p) = @_;
	my $doc = $self->retrieve_doc(@p);
}

sub filter_doc {
	my ($self, $doc) = @_;

	my $result = { 'raw' => $self->save_raw ? $doc : undef };
	my @docs = undef;
	eval { @docs = $self->filter($doc); };
	if ($@) {
		my $code = $self->code;
		my $dir = $self->cfg->required_param('filter_error_dir');
		$dir = "$dir/$code";
		mkpath($dir);
		my $file = $self->gen_filename();
		my $path = "$dir/$file";
		my $fh = new FileHandle ">$path" or die "Can't write $path: $!";
		print $fh $doc;
		die "$@ [$path]";
	}
	$result->{'filtered'} = \@docs;

	return $result;
}

sub gen_filename {
	my ($self) = @_;

	my $cnt = $self->{'gen_filename_counter'};
	$cnt = $cnt ? $cnt + 1 : 1;
	my $time = time();
	return "raw.${time}_$cnt.txt";
}

sub path2doc_id { croak "path2doc_id not defined"; }
sub log_docs	{ return 0; }

sub initialize { }		# Optional
sub post_retrieve { }	# Optional 
sub cleanup { }			# Optional

sub retry {
	my ($self, $callback_ref, @args) = @_;

	my $retries = $self->cfg->param('pull_retries') || 3;
	my $result = undef;
	for (my $i = 0; $i < $retries; $i++) {
		eval {
			$result = &$callback_ref($self, @args); 
		};
		last if !$@;
		my $count = $i + 1;
		$self->log('[ADV] Attempt #' . $count . ': ' . $@) if $@;
		sleep 1;
	}
	die "Failed after $retries retries" if $@;
	return $result;
}

sub check_abort {
	my ($self) = @_;

	&{$self->{'check_abort_callback'}}();
}

1;