# $Id: XT.pm 2129 2008-07-24 21:03:46Z ian $
package Feedfetcher::Plugins::TXT;

use strict;
use Feedfetcher::Plugins::Retriever;
our @ISA = ("Feedfetcher::Plugins::XT");

use Feedfetcher::DocLogger;
use Core::Log;
use Core::XmlUtil;

use LWP::UserAgent;
use HTTP::Request;
use FileHandle;
use Carp;
use XML::LibXML;

sub name { return "XML Team TXT feed"; }
sub code { return "txt"; }

# TODO - REMOVE
#sub initialize {
#	my ($self) = @_;
#
#	my $cfg = $self->cfg;
#	my $log = $self->get_log;
#	my @servers = $cfg->required_params('xt_server'); 
#	$self->{'servers'} = \@servers;	
#	my @servers_copy = @servers;
#	$self->{'original_server_order'} = \@servers_copy;
#	$self->{'revert_primary'} = $cfg->required_params('xt_alt_server_runs'); 
#}

sub get_list {
	my ($self) = @_;

	my $cfg = $self->cfg;
	my $feed_path = $cfg->required_param("xt_feed_path");
	$self->check_revert_servers();
	my @servers = @{$self->servers};
	foreach my $server (@servers) {
		my @files = undef;
		eval {
			my $current_time = $self->get_server_time($server);
			my $since = $self->since($current_time);
			$self->{'last_access'} = $current_time;	# New last access is now
			my @queries = ();
			my $delay = $cfg->param("pull_delay");
			push @queries, "delay=$delay" if $delay;
			my $now = $cfg->param("pull_now");
			push @queries, "now=$now" if $now;
#			my $xml = $cfg->param("pull_xml");
#			push @queries,"xml=$xml" if $xml;
            #push @queries, "last-accessed=$since";
            my $txt_feed_path = $self->cfg->required_param('txt_feed_path');
            push @queries, $txt_feed_path;
            push @queries, "last-accessed=$since";
			my $query = join("&", @queries); 
			my $url = "http://$server/$feed_path?$query";
#			print "url: $url\n";
			@files = $self->get_recursive_files($url, 1);
			my $file_count = @files;
#			print "file count: $file_count\n";
			return undef if !@files;
		};
		if ($@) {
			$self->log("[ADV] Couldn't connect to $server: $@");
			$self->{'last_access'} = undef;
			$self->demote_server();
		} else {
			return @files;
		}
	}
	die "Could not process files: could not connect to any server";
}

sub log_docs { return 1; }

# TODO - REMOVE
#sub path2doc_id {
#	my ($self, $path) = @_;
#
#	$path =~ /([^\/]+).txt$/;
#	return $1;
#}

# TODO - REMOVE
#sub retrieve_doc {
#	my ($self, $uri_str) = @_;
#
#	return $self->retrieve_url($uri_str);
#}

# KEEP
sub filter {
	my ($self, $txt) = @_;

    return $txt;
}

# TODO - REMOVE
#sub cleanup {
#	my ($self) = @_;
#	
#	$self->set_last_access();
#}

# KEEP
# Recursively fetches remote files, by parsing directory-type html pages
sub get_recursive_files {
	my ($self, $url, $is_directory) = @_;
	my @files = ();

#	print "url: $url\n";
#	print "is_directory: $is_directory\n";
	if ($is_directory || $url !~ /\.txt$/) {		# If it's a directory
		my @listing = $self->get_dir_listing($url);
		foreach my $file (@listing) {
			return undef if $self->check_abort;
			push @files, $self->get_recursive_files($file, 0);
		}
		return @files;
	} else {							# If it's a file
		return ($url);
	}
}

# TODO - REMOVE
# Return a listing of URLS from a directory-type HTML file.
# This is done by returning the locations of all of the links in the HTML file.
sub get_dir_listing {
	my ($self, $url) = @_;

	my $content = $self->retrieve_url($url);
	return () if !$content;
	my @listing = ();

	my $listing_text = extract_listing($content);
	while ($listing_text =~ /href=["']([^"']+)["']/igs) {
		my $filename = $1;
		next if $filename eq '..';
		my $uri;

#		print "filename: $filename\n";
		if (substr($filename, 0, 1) eq '/') {	# Root path
			$uri = URI->new($filename);
			$uri->scheme('http');
			my $url_uri = URI->new($url);
			$uri->host($url_uri->host);
		} else {								# Relative path
			$uri = URI->new_abs($filename, $url);
		}
		push @listing, $uri->as_string;
	}

	return @listing;
}

# KEEP
sub extract_listing {
	my ($content) = @_;

	my $ret_val = "";
	while ($content =~ m:<ul.*?>(.+?)</ul>:isg) {
		$ret_val .= $1
	}
#	if ($content =~ m:<ul.*?>(.+?)</ul>:is) {
#		return $1;
#	} else {
#		return "";
#	}
	$ret_val =~ s/\n\n/\n/g;
#	print "ret_val: $ret_val\n";
#	exit;
	return $ret_val;
}

# TODO - REMOVE
#sub retrieve_url {
#	my ($self, $url) = @_;
#
#	my $cfg = $self->cfg;
#	my $user = $cfg->required_param('xt_user');
#	my $pass = $cfg->required_param('xt_pass');
#	my $ua = LWP::UserAgent->new(env_proxy => 1, timeout => 60);
#	my $req = HTTP::Request->new(GET => $url);
#	$req->authorization_basic($user, $pass);
#	
#	my $res = $self->retry(sub { 
#		my ($self, $req) = @_;
#		return $ua->request($req);
#	}, $req);
#	die "Couldn't access $url: " . $res->status_line if $res->is_error;
#
#    # BEGIN charset preserving attempt
#    my $content = $res->content;
##    my $encoding = 'utf8'; # assume this is the default
##    if($content =~ /encoding=\"([^\"]+)\"/) {
##    	# Apache can give something like 'UTF-8'
##        $encoding = $1;
##    }
##    $content = $res->decoded_content((charset => $encoding));
#    # END charset preserving attempt
#
#    return $content;
##	
##	return $res->content;
#}

# TODO - REMOVE
# Returns when to get files from, in ISO 8601 format
#sub since {
#	my ($self, $current_time) = @_;
#
#	my $last_access = get_last_access();
#	my $since;
#	my $current_time_epoch = Core::Util::date2epoch($current_time);
#	my $cfg = $self->cfg;
#
#	my $since_value = $self->arg('since');
#	if (!($self->arg('use_since') || $since_value) && $last_access) {
#		die "Last_access must be in ISO 8601 format" if 
#			$last_access !~ 
#			/\d\d\d\d-?\d\d-?\d\dT\d\d:?\d\d:?\d\d[Z+-](\d\d:?\d\d)?/;
#		$since = $last_access;
#	} else {
#		my $adjust = $since_value || $cfg->param('since');	# In minutes
#		$adjust *= 60;										# Convert to seconds
#		my $epoch = $current_time_epoch - $adjust;		
#		$since = Core::Util::epoch2iso8601($epoch); 	
#	}
#
#	my $since_epoch = Core::Util::date2epoch($since);
#	my $cap = $cfg->param('last_access_cap');	# In minutes
#	my $wink = $self->arg('wink');
#	if ($cap && !$wink) {
#		$cap *= 60;								# Convert to seconds
#		my $cap_epoch = $current_time_epoch - $cap;
#		if ($since_epoch < $cap_epoch) {
#			$since = Core::Util::epoch2iso8601($cap_epoch);
#		}
#	}
#
#	return $since;
#}

# TODO - REMOVE
#sub set_last_access {
#	my ($self) = @_;
#
#	my $last_access = $self->{'last_access'} or return;
#
#	# Add 5 second safety overlap
#	my $epoch = Core::Util::date2epoch($last_access);
#	$epoch -= 5;
#	$last_access = Core::Util::epoch2iso8601($epoch);
#
#	my $home_path = Core::Util::get_home_path();
#	my $path = "$home_path/xt_last_access";
#	my $fh = new FileHandle ">$path";
#	print $fh $last_access;
#	undef $fh;
#}

# TODO - REMOVE
#sub get_last_access {
#	my $home_path = Core::Util::get_home_path();
#	my $path = "$home_path/xt_last_access";
#	my $fh = new FileHandle $path or return;
#	my $last_access = <$fh>;
#	undef $fh;
#
#	return $last_access;
#}

# TODO - REMOVE
#sub get_server_time {
#	my ($self, $server) = @_;
#
#	my $date = $self->retrieve_url("http://$server/feedfetcher/time.pl");
#	return if !defined $date;
#	chomp $date;
#	return $date;
#}

# TODO - REMOVE
#sub servers { my ($self) = @_; return $self->{'servers'}; }
# TODO - REMOVE
#sub demote_server {
#	my ($self) = @_;
#
#	push @{$self->{'servers'}}, shift @{$self->{'servers'}};
#}

# TODO - REMOVE
#sub check_revert_servers {
#	my ($self) = @_;
#
#	my $cnt = $self->{'revert_count'} || 0;
#	if ($cnt > $self->{'revert_primary'}) {
#		my @servers_copy = @{$self->{'original_server_order'}};
#		$self->{'servers'} = \@servers_copy;
#		$cnt = 0;
#	}
#	$self->{'revert_count'} = $cnt + 1;
#}

sub save_raw { 
	my ($self) = @_; 
	return 1;
}
1;