# $Id: XT.pm 3097 2016-04-15 16:48:12Z ian $
package Feedfetcher::Plugins::XT;

use strict;
use Feedfetcher::Plugins::Retriever;
our @ISA = ("Feedfetcher::Plugins::Retriever");
use Feedfetcher::DocLogger;
use Core::Log;
use Core::XmlUtil;
use LWP::UserAgent;
use HTTP::Request;
use FileHandle;
use Carp;
use XML::LibXML;
use Core::Util;
#use diagnostics;

sub name { return "XML Team feed"; }
sub code { return "xt"; }

sub initialize {
	my ($self) = @_;

	my $cfg = $self->cfg;
	my $log = $self->get_log;
	my @servers = $cfg->required_params('xt_server'); 
	$self->{'servers'} = \@servers;	
	my @servers_copy = @servers;
	$self->{'original_server_order'} = \@servers_copy;
	$self->{'revert_primary'} = $cfg->required_params('xt_alt_server_runs'); 
#	if (! defined $self->{'ua'}) {
#		# Define a user agent with 1 keep_alive session cached
#		print "Creating ua\n";
#		$self->{'ua'} = LWP::UserAgent->new(env_proxy => 1, timeout => 180, keep_alive => 1);
#	}
}

sub get_list {
	my ($self) = @_;

#    $self->{'ua'} = LWP::UserAgent->new(env_proxy => 1, timeout => 180, keep_alive => 1);

	my $cfg = $self->cfg;
	my $feed_path = $cfg->required_param("xt_feed_path");
	if ($self->{'args'}->{'xt_feed_path'}) {
        $feed_path = $self->{'args'}->{'xt_feed_path'};
    }
    my $extra_args;
    if ($self->{cfg}->param("extra_args")) {
        $extra_args = $cfg->param("extra_args");
    }
    elsif ($self->{'args'}->{'extra_args'}) {
        $extra_args = $self->{'args'}->{'extra_args'};
    }
	$self->check_revert_servers();
	my @servers = @{$self->servers};
	foreach my $server (@servers) {
		my @files = undef;
            my $current_time = $self->get_server_time($server);
            my $since = $self->since($current_time);
            $self->{'last_access'} = $current_time; # New last access is now
            my @queries = ();
            my $delay = $cfg->param("pull_delay");
            push @queries, "delay=$delay" if $delay;
            my $now = $cfg->param("pull_now");
            push @queries, "now=$now" if $now;
            my $xml = $cfg->param("pull_xml");
            push @queries,"xml=$xml" if $xml;
            my $pulltransform = $cfg->param("pull_transform");
            push @queries,"transform=$pulltransform" if $pulltransform;
            my $pullformat = $cfg->param("pull_format");
            push @queries,"format=$pullformat" if $pullformat;
            my $ext = $cfg->param("pull_ext");
            push @queries,"ext=$ext" if $ext;
            unless ($extra_args && $extra_args =~ /start/) {
                 push @queries, "last-accessed=$since";
            }
            my $query = join("&", @queries);
            my $url = "http://$server/$feed_path?$query";
#            print "url: $url\n";
            if ($extra_args) {
            	# Trim extra_args value to get rid of leading & and ? and trailing &
            	$extra_args =~ s/^\[&\?]+//;
                $extra_args =~ s/\&+$//;
            	$url = $url."&".$extra_args;
            	$url =~ s/\&{2,}/\&/;
                $url =~ s/\?\&/\?/;
#                print "extra_args: ".$extra_args."\n";
#                print "url: $url\n";
            }
		eval {
			@files = $self->get_recursive_files($url, 1);
			return undef if !@files;
		};
		# Report any error, including url used.
		if ($@) {
			$self->log("[ADV] Couldn't connect to $server: $@. URL: $url");
			$self->{'last_access'} = undef;
			$self->demote_server();
		} else {
			return @files;
		}
	}
	die "Could not process files: could not connect to any server";
}

sub log_docs { return 1; }

sub path2doc_id {
	my ($self, $path) = @_;

	$path =~ /([^\/]+).xml$/;
	return $1;
}

sub retrieve_doc {
	my ($self, $uri_str) = @_;

	return $self->retrieve_url($uri_str);
}

sub filter {
	my ($self, $xml) = @_;

	$xml =~ s/xmlns=\".*?\"//;
	my $doc = $self->parser->parse_string($xml);
#	$doc->documentElement->setNamespace("http://iptc.org/std/SportsML/2009-06-18/","");

#	print "found $count nodes to process\n";

	# Pull out sports-contents out of sports-content-set if applicable
	my @nodes = nodes($doc, '//sports-content');
	$self->log("[MED] Processing a content-set of " . (scalar @nodes) . " nodes") if
		@nodes > 1;
	my @filtered = map { $_->toString; } reverse @nodes;
	my $count = scalar @nodes;

	return @filtered;
}

sub cleanup {
	my ($self) = @_;
	
	$self->set_last_access();
}

# Recursively fetches remote files, by parsing directory-type html pages
sub get_recursive_files {
    my ($self, $url, $is_directory) = @_;
    my @files = ();

    my $cfg = $self->cfg;
    my $ext = $cfg->param("pull_ext") || 'xml';
    my $regex = qr/^.*?\.$ext\?.*$/;
    if ($url =~ /$regex/) {
        $regex = '.*?\.'.$ext.'\?.*$';
    }
    else {
#        print "regex: $regex\n";
        $regex = '\.'.$ext.'$';
    }
#    my $regex = 'anpa$';
##    if ($url !~ /$regex/) {
##    	print "no match: $url\n";
##    }
#    print "url from get_recursive_files: $url is_directory: $is_directory\n";
	if ($is_directory || $url !~ /$regex/) {		# If it's a directory
		my @listing = $self->get_dir_listing($url);
		foreach my $file (@listing) {
#			print "file: $file\n";
			return undef if $self->check_abort;
			push @files, $self->get_recursive_files($file, 0);
		}
		return @files;
	} else {							# If it's a file
		return ($url);
	}
}

# Return a listing of URLS from a directory-type HTML file.
# This is done by returning the locations of all of the links in the HTML file.
sub get_dir_listing {
	my ($self, $url) = @_;

	my $content = $self->retrieve_url($url);
#	print "url: $url content: $content\n";
	return () if !$content;
	my @listing = ();

	my $listing_text = extract_listing($content);
	while ($listing_text =~ /href=["']([^"']+)["']/igs) {
		my $filename = $1;
		next if $filename eq '..';
		my $uri;

		if (substr($filename, 0, 1) eq '/') {	# Root path
			$uri = URI->new($filename);
			$uri->scheme('http');
			my $url_uri = URI->new($url);
			$uri->host($url_uri->host);
		} else {								# Relative path
			$uri = URI->new_abs($filename, $url);
		}
		push @listing, $uri->as_string;
	}

	return @listing;
}

sub extract_listing {
	my ($content) = @_;

	if ($content =~ m:<ul.*?>(.+?)</ul>:is) {
		return $1;
	} else {
		return "";
	}
}

sub retrieve_url {
	my ($self, $url) = @_;

#    enable diagnostics;
	my $cfg = $self->cfg;
	my $user = $cfg->required_param('xt_user');
	my $pass = $cfg->required_param('xt_pass');
#	print "user: $user pass: $pass\n";
	
    my $ua = LWP::UserAgent->new(env_proxy => 1, timeout => 60);
#    my $ua =  $self->{'ua'};
	# Tell server that client can accept gzip encoded responses
	# See http://search.cpan.org/~gaas/libwww-perl-6.03/lib/LWP/UserAgent.pm
	$ua->default_header('Accept-Encoding' => 'gzip');
	my $req = HTTP::Request->new(GET => $url);
	$req->authorization_basic($user, $pass);
	
#	print "user: $user pass: $pass url: $url\n";
	my $res = $self->retry(sub { 
		my ($self, $req) = @_;
		return $ua->request($req);
	}, $req);
	die "Couldn't access $url: " . $res->status_line if $res->is_error;
#	print $res->as_string."\n";
#	print "status line: ".$res->status_line."\n";
    my $content_encoding = $res->header('Content-encoding');

    # BEGIN charset preserving attempt
    my $content = $res->content;
    if (defined $content_encoding && $content_encoding =~ /gzip/) {
        $content = Compress::Zlib::memGunzip($content);
#        $self->log("[DBG] Gunzipped");
    }
#    else {
#        $self->log("[DBG] Not Gunzipped");     
#    }

#    my $encoding = 'utf8'; # assume this is the default
#    if($content =~ /encoding=\"([^\"]+)\"/) {
#    	# Apache can give something like 'UTF-8'
#        $encoding = $1;
#    }
#    $content = $res->decoded_content((charset => $encoding));
    # END charset preserving attempt

#    print "content encoding: $content_encoding url: $url fetched content: $content length: ".length($content)." success: ".$res->as_string."\n";
#    print "url: $url fetched content: $content length: ".length($content)." success: ".$res->as_string."\n";
#    disable diagnostics;
    
    return $content;
#	
#	return $res->content;
}

# Returns when to get files from, in ISO 8601 format
sub since {
	my ($self, $current_time) = @_;

	my $last_access = $self->get_last_access();
	my $since;
	my $current_time_epoch = Core::Util::date2epoch($current_time);
	my $cfg = $self->cfg;

	my $since_value = $self->arg('since');
	if (!($self->arg('use_since') || $since_value) && $last_access) {
		die "Last_access must be in ISO 8601 format" if 
			$last_access !~ 
			/\d\d\d\d-?\d\d-?\d\dT\d\d:?\d\d:?\d\d[Z+-](\d\d:?\d\d)?/;
		$since = $last_access;
	} else {
		my $adjust = $since_value || $cfg->param('since');	# In minutes
		$adjust *= 60;										# Convert to seconds
		my $epoch = $current_time_epoch - $adjust;		
		$since = Core::Util::epoch2iso8601($epoch); 	
	}

	my $since_epoch = Core::Util::date2epoch($since);
	my $cap = $cfg->param('last_access_cap');	# In minutes
	my $wink = $self->arg('wink');
	if ($cap && !$wink) {
		$cap *= 60;								# Convert to seconds
		my $cap_epoch = $current_time_epoch - $cap;
		if ($since_epoch < $cap_epoch) {
			$since = Core::Util::epoch2iso8601($cap_epoch);
		}
	}

	return $since;
}

sub set_last_access {
	my ($self) = @_;

	my $last_access = $self->{'last_access'} or return;

	# Add 5 second safety overlap
	my $epoch = Core::Util::date2epoch($last_access);
	$epoch -= 5;
	$last_access = Core::Util::epoch2iso8601($epoch);

	my $home_path = Core::Util::get_home_path();
	my $path = "$home_path/xt_last_access_".$self->cfg->{name};
	my $fh = new FileHandle ">$path";
	print $fh $last_access;
	undef $fh;
}

sub get_last_access {
	my ($self) = @_;
	
	my $home_path = Core::Util::get_home_path();
	my $path = "$home_path/xt_last_access_".$self->cfg->{name};
	my $fh = new FileHandle $path or return;
	my $last_access = <$fh>;
	undef $fh;

	return $last_access;
}

sub get_server_time {
	my ($self, $server) = @_;

	my $date = $self->retrieve_url("http://$server/feedfetcher/time.pl");
	return if !defined $date;
	chomp $date;
	return $date;
}

#sub get_server_time_new {
#	my ($self, $server) = @_;
#
#    my $ua =  $self->{'ua'};
#    my $req = new HTTP::Request('HEAD', "http://$server/feedfetcher/time.pl");
#    
#    my $res = $self->retry(sub { 
#        my ($self, $req) = @_;
#        return $ua->request($req);
#    }, $req);
#    die "Couldn't access $server: " . $res->status_line if $res->is_error;
#
##    print $res->date . " $server\n";
##    my $ts = $res->date;
##
##	my $date = $self->retrieve_url("http://$server/feedfetcher/time.pl");
##	return if !defined $date;
##	chomp $date;
##	print "date: $date and from header:".Core::Util::epoch2iso8601($ts)."\n";
#
#    # The date value is a unix timestamp representing the date header value
#    my $ts = $res->date;
#	my $server_time = Core::Util::epoch2iso8601($ts);
#	
##   return $date;
#    return $server_time;
#}

sub servers { my ($self) = @_; return $self->{'servers'}; }
sub demote_server {
	my ($self) = @_;

	push @{$self->{'servers'}}, shift @{$self->{'servers'}};
}

sub check_revert_servers {
	my ($self) = @_;

	my $cnt = $self->{'revert_count'} || 0;
	if ($cnt > $self->{'revert_primary'}) {
		my @servers_copy = @{$self->{'original_server_order'}};
		$self->{'servers'} = \@servers_copy;
		$cnt = 0;
	}
	$self->{'revert_count'} = $cnt + 1;
}

1;