# $RCSfile$; $Date: 2016-04-07 11:58:44 -0400 (Thu, 07 Apr 2016) $; $Revision: 3072 $; $Name$

package Feedfetcher::Push;

use strict;

use HTTP::Daemon;
use HTTP::Status;
use XML::LibXML;
use IO::Select;
use File::Path;
use File::Temp;
use Core::XmlUtil;
use Core::Util;
use Compress::Zlib;
use LWP::Simple;

# Launch HTTP PUT push receiver in thread 
sub launch_incoming_push {
	my ($work_data) = @_;

	my $data = $work_data->{data};

	my $cfg = $data->{cfg};
	my $log = $data->{log};

	#   my $last_dbstore_time = $data->{last_dbstore_time};

	my $use_database = ( ( $cfg->param('use_database') ) =~ /true/i );

#   my $db_storage_warning_threshold;
#   my $last_dbstore_time;
#   my $last_fetched_time;
#   my $db_store_finished;
#   if ($use_database) {
#       $db_storage_warning_threshold = $cfg->param('db_storage_warning_threshold') || 240;
#       $last_dbstore_time = scalar ${$work_data->{data}->{last_dbstore_time}};
#       $last_fetched_time = scalar ${$work_data->{data}->{last_fetched_time}};
#       $db_store_finished = scalar ${$work_data->{data}->{db_store_finished}};
#   }

	my $host    = $cfg->param('push_host')    || 'localhost';
	my $port    = $cfg->param('push_port')    || 8082;
	my $timeout = $cfg->param('push_timeout') || 30;

	my $push_active = $cfg->param('push_mode') eq 'active';

	my $d = HTTP::Daemon->new(
							   LocalAddr => $host,
							   LocalPort => $port,
							   Timeout   => $timeout
	) || die "$@";
	$log->log(   "[LOW] Starting FF webservice at: <URL: "
			   . $d->url
			   . ">. (Timeout set to $timeout seconds.)" );

	my $select = IO::Select->new($d);

	my $dirty = 1;
	until ( $work_data->check_abort ) {

        # For debugging
		#   $self->log("[DBG] BEGINNING A PROCESSING RUN FOR PULL");

		$log->log("[DBG] Awaiting push document...") if $dirty && $push_active;
		$dirty = 0;
		until ( $select->can_read(1) || $work_data->check_abort ) { }
		last if $work_data->check_abort;

        # For debugging
        # $log->log("[DBG] last_dbstore_time from push: ".${$last_dbstore_time});

		# Connection established
		my $c = $d->accept;
		# Restart loop if no connection
		next if !$c;
		# Connection request
		my $r = $c->get_request;
		# Error logged if no request, close connection and re-loop
		if ( !$r ) {
			$log->log(
				"[ERR] FF Push: Couldn't process HTTP request: " . $c->reason);
			$c->close;
			undef($c);
			next;
		}

        # If push is active and the incoming request is using the PUT protocol, proceed
		if ( $push_active && $r->method eq 'PUT' ) {
			# Assume for now that a full file is being sent
			my $push_method = "FILE";

			#           if ($use_database) {
##              $last_dbstore_time = scalar ${$work_data->{data}->{last_dbstore_time}};
##              $last_fetched_time = scalar ${$work_data->{data}->{last_fetched_time}};
##              $db_store_finished = scalar ${$work_data->{data}->{db_store_finished}};
#
#               # PROBLEM FOUND HERE:
#               # Can't use an undefined value as a SCALAR reference at /opt/FeedFetcherDeluxe/perl/Feedfetcher/Push.pm line 82.
##              my $time_since_last_dbstore = ($last_fetched_time - $last_dbstore_time);
#       #       $self->log("[DBG] last fetched time: ".${$self->{last_fetched_time}});
#       #       $self->log("[DBG] last_dbstore_time from pull: ".${$self->{last_dbstore_time}});
#       #       $self->log("[DBG] db_store_finished: ".${$self->{db_store_finished}});
#
#       #       $self->log("[DBG] DIFF: ".$time_since_last_dbstore);
#               if (($time_since_last_dbstore - 0) > $db_storage_warning_threshold) {
#                   $log->log("[ERR] Push DB_STORAGE_WARNING. No completed document storage in ".$time_since_last_dbstore." seconds.");
#               }
#               my $last_fetched_time = time;
#               print "last fetched time: $last_fetched_time time_since_last_dbstore: $time_since_last_dbstore\n";
#               $work_data->{data}->{last_fetched_time} = \$last_fetched_time;
#               my $test = scalar ${$work_data->{data}->{last_fetched_time}};
#               print "stored last fetched time: $test\n";
#           }
            
            # Old method, got raw bytes but did not handle character set and encoding
			#           my $xml = $r->content;

			# Get content - decoded_content handles character set and encoding, plus decompresses if necessary
			my $xml = $r->decoded_content();

            # Old code to do manual gunzipping. Now handled by using HTTP::Response->decoded_content()
            # See http://search.cpan.org/~gaas/libwww-perl-5.815/lib/HTTP/Headers.pm
            # In a scalar context, $r->header will return a list, separated by commas if there is more than one value for a key
            # Thus, use a regex to check for gzip, not 'eq'
            #            my $content_encoding = $r->header('Content-encoding');
            #            $xml = Compress::Zlib::memGunzip($xml) if (defined $content_encoding && $content_encoding =~ /gzip/);
            # $log->log("[DBG] Encoding: $content_encoding");
			
			# For debugging
			# my $test_str = substr( $xml, 0, 200 );
			# $log->log("[DBG] sample: $test_str");

			# 202: The request has been accepted for processing, but the processing has not been completed
			$c->send_status_line(202);

            # No longer used. Remove at some point.
			#           $c->force_last_request;

			my $content_test = $xml;

			# If content begins with 'http' do a GET request for the actual content
			if ( $content_test =~ /^http.*?/ ) {
				# Push method is no loger 'FILE'
				$push_method = "URL";
				my $url = $content_test;

				# Check against XT_Server_1 and XT_Server_2 to see if file to be downloaded comes from one of those two
				my @servers      = $cfg->required_params('xt_server');
				my $valid_source = 0;
				foreach my $source (@servers) {
					my $domain = $source;
					# Just get the main domain, such as xmlteam.com to test against url domain.
					$domain =~ s/^.*?\.*([\d\w-_]+\.[\d\w-_]+)$/$1/;
					
					# Decide if URL is at least from same domain as main feed servers
					if ( $url =~ /http(?:s){0,1}\:\/\/([\d\w-_]+\.)*$domain.*/ )
					{
						$valid_source = 1;
						last;
					}
				}
				# Error if URL domain is not from feed server domain and loop to wait for next push 
				if ( $valid_source eq '0' ) {
					$log->log(
						"[ERR] URL To be fetched does not originate from XT_Server_1 or XT_Server_2. Rejecting."
					);
					$log->log("[ERR] Rejected URL is: $url");
					next;
				}
				chomp $url;
				my $nr_of_lines = $url =~ tr/\n//;
				# If more than one line in the case of URL push, Error and loop to wait for next push
				if ( $nr_of_lines > 1 ) {
					$log->log(
						"[ERR] Found more than 1 line ($nr_of_lines) in pushed URL. Skipping."
					);
					next;
				}
				# Prepare to download the file from the URL
				$log->log("[DBG] Downloading pushed URL $url");
				my $user = $cfg->required_param('xt_user');
				my $pass = $cfg->required_param('xt_pass');
				# Set up user agent to do the download
				my $ua   = LWP::UserAgent->new( env_proxy => 1, timeout => 60 );
				# Allow for compressed content
				$ua->default_header( 'Accept-Encoding' => 'gzip' );

                # Use GET request and basic auth
                # For future could also use SSL
				my $req = HTTP::Request->new( GET => $url );
				$req->authorization_basic( $user, $pass );

                # Make the request
				my $r = $ua->request($req);

                # If there was a problem with the request, report it and loop to wait for next push
				if ( $r->is_error ) {
					$log->log( "[ERR] Couldn't access pushed url: $url: "
							   . $r->status_line );
					next;
				}

 				# Get the decoded content we just downloaded via the GET reqeust
				$xml = $r->decoded_content();
			}

			$dirty = 1;

			eval {
				# Get doc-id to be used to ensure that this document has not recently been processed
				$xml =~ /sports-metadata[^<>]+doc-id=\"([^\"]+)\"/;
				my $doc_id = $1;
				# If we have not yet recently (based on a list of recently processed docs) processed, proceed
				if ( ! $data->{doc_logger}->was_processed($doc_id) ) {
					my $queue = $data->{push_queue};
					$xml =~ s/\<\?[^\?]*\?\>//;
					$xml =~ s/(\<\!DOCTYPE[^\>]*\>)//;
					# Wrap the document xml before enqueuing
					$xml =
                        qq{<?xml version="1.0" encoding="UTF-8"?>$1<sports-ff-wrapper>$xml</sports-ff-wrapper>};
                    # Enqueue the document
					$queue->enqueue($xml);
					$data->{queue_semaphore}->up;

					$doc_id = 'unknown' if !$doc_id;
					# Log the doc-id
					$log->log("[LOW] Via Push: $doc_id via $push_method");
				}
				# If we have processed this document recently, don't enqueue it and logg this action 
				else {
					$log->log(
                        "[LOW] Via Push: $doc_id (already received) via $push_method"
                        );
				}
			};
			# If there was an error, log it and save the failed document content
			if ($@) {
				my $error_msg = $@;
				my $base_path = $cfg->required_param('feed_error_dir');
				mkpath($base_path);

				my $tmp = new File::Temp(
										  DIR    => $base_path,
										  SUFFIX => '.xml',
										  UNLINK => 0
				);
				print $tmp "$xml";
				$log->log("[ERR] FF Push: Can't parse xml ($tmp): $error_msg");
				undef $tmp;
			}
		}
		# Handle request for log sample 
		elsif ( $r->method eq 'GET' and $r->url->path eq "/xtslogs" ) {
			$c->send_response( logs_response( $log, $r->uri->query_form ) );
		} 
		# Handle request for list of logs
		elsif ( $r->method eq 'GET' and $r->url->path eq "/xtsloglist" ) {
			$c->send_response( list_logs($log) );
		} else {
			$c->send_error;
		}

		$c->close;
		undef($c);
	}

    # Report shutdown of PUSH process
	$log->log("[LOW] Push was shut down") if $push_active;
}

# For push polling (long polling) handling. Not available in production currently.
sub launch_polling_push {
	my ($work_data) = @_;

	my $data = $work_data->{data};

	my $cfg        = $data->{cfg};
	my $log        = $data->{log};
	my $date_utils = Util::DateUtils->new();

	my $user               = $cfg->required_param('xt_user');
	my $pass               = $cfg->required_param('xt_pass');
	my $xt_server          = $cfg->required_param('xt_server_1');
	my $push_polling_token = $cfg->param('push_polling_token') || 'feedfetcher';

	# Check time offset every 1 minute
	my $sec_between_time_checks = 60;

	my $ua = LWP::UserAgent->new( env_proxy => 1, timeout => 65 );
	$ua->default_header( 'Accept-Encoding' => 'gzip' );

	my $last_lag_time_check = ( time() - $sec_between_time_checks - 1 );
	my $time_offset_ms      = 0;

	my $announce_push_wait = 1;
	until ( $work_data->check_abort ) {
		$log->log("[DBG] Awaiting push document...") if $announce_push_wait;

		my $push_polling_target =
		  "http://" . $xt_server . "/push?token=$push_polling_token";

	# Check on time difference between this server and the one sending the docs.
	# Any offset found will be used to help ensure a lag notice is not alarming
	# for a spurious reason.
		if ( $announce_push_wait
			 && ( ( time() - $last_lag_time_check ) > $sec_between_time_checks )
		  )
		{
			eval {
				my $time_req =
				  HTTP::Request->new(
							   GET => "http://$xt_server/feedfetcher/time.pl" );
				$time_req->protocol('HTTP/1.1');

				my $time_res = $ua->request($time_req);

				if ( !$time_res->is_error ) {

					my $server_time =
					  $date_utils->ISO8601_date2time(
												 $time_res->decoded_content() );
					$last_lag_time_check = time();
					$time_offset_ms      = $last_lag_time_check - $server_time;
				}
			};
			if ($@) {
				$last_lag_time_check = time();
				$time_offset_ms      = 0;
			}
		}

		my $req = HTTP::Request->new( GET => $push_polling_target );
		$req->protocol('HTTP/1.1');
		$req->header( Connection => 'Keep-alive' );
		$req->authorization_basic( $user, $pass );

		my $res = $ua->request($req);

		if ( !$res->is_error ) {
			my $content = $res->decoded_content();
			my $xml;

			my @lines = split /\n/, $content;
			foreach my $message (@lines) {
				my @parts = split /;/, $message;
				my $file_mtime = @parts[0];
				$file_mtime =~ s/\.\d\d\d//;
				my $file_path = @parts[1];

				my $url =
				  "http://" . $xt_server . "/sportsml/files/" . $file_path;

				$log->log("[DBG] Downloading pushed URL $url");
				my $req_file = HTTP::Request->new( GET => $url );
				$req_file->authorization_basic( $user, $pass );

				my $res_file = $ua->request($req_file);
				if ( $res_file->is_error ) {
					$log->log( "[ERR] Couldn't access pushed url: $url: "
							   . $res_file->status_line );
					next;
				}

				eval {
					$xml = $res_file->decoded_content();

					#                    my $test_str = substr( $xml, 0, 200 );
					#                    $log->log("[DBG] test xml: $test_str");

		  #					my $doc_id = $xml =~ /sports-metadata[^<>]+doc-id=\"([^\"]+)\"/;
					$xml =~ /sports-metadata[^<>]+doc-id=\"([^\"]+)\"/;
					my $doc_id = $1;

					#					$log->log("[DBG] doc_id: $doc_id");
					if ( !$data->{doc_logger}->was_processed($doc_id) ) {
						my $queue = $data->{push_queue};
						$xml =~ s/\<\?[^\?]*\?\>//;
						$xml =~ s/(\<\!DOCTYPE[^\>]*\>)//;
						$xml =
qq{<?xml version="1.0" encoding="UTF-8"?>$1<sports-ff-wrapper>$xml</sports-ff-wrapper>};
						$queue->enqueue($xml);
						$data->{queue_semaphore}->up;

						$doc_id = 'unknown' if !$doc_id;

						eval {

				 # Note that any lag measurement is tied to the accuracy of both
				 # the feed server's clock and the clock of the computer running
				 # FeedFetcher.
							my $current_time_stamp = time();
							my $current_date_time =
							  $date_utils->get_ISO8601_date(
														   $current_time_stamp);
							my $date_time_stamp =
							  $date_utils->ISO8601_date2time($file_mtime);

							my $publishing_lag =
							  $current_time_stamp -
							  $date_time_stamp -
							  $time_offset_ms;
							$log->log(
"[LOW] Via Push: $doc_id via polling push. Publishing lag: $publishing_lag" );
						};
						if ($@) {
							my $error_msg = $@;
							$log->log(
								  "[DBG] Problem getting publishing lag value: "
									. $error_msg );
							$log->log(
								   "[LOW] Via Push: $doc_id via polling push.");
						}

					} else {
						$log->log(
"[LOW] Via Push: $doc_id (already received) via polling push" );
					}
				};
				if ($@) {
					my $error_msg = $@;
					my $base_path = $cfg->required_param('feed_error_dir');
					mkpath($base_path);

					my $tmp =
					  new File::Temp(
									  DIR    => $base_path,
									  SUFFIX => '.xml',
									  UNLINK => 0
					  );
					print $tmp "$xml";
					$log->log(
						   "[ERR] FF Push: Can't parse xml ($tmp): $error_msg");
					undef $tmp;
				}
			}
			$announce_push_wait = 1;
		} else {

			# Wait a while before re-trying
			#			$log->log( "[WARN] Couldn't access: " . $res->status_line );
			$announce_push_wait = 0;
			next;
		}
	}

	$log->log("[LOW] Pollling push was shut down");
}

#sub get_server_time {
#    my ($self, $server) = @_;
#
#    my $date = $self->retrieve_url("http://$server/feedfetcher/time.pl");
#    return if !defined $date;
#    chomp $date;
#    return $date;
#}

sub launch {
	my ($work_data) = @_;

	my $data = $work_data->{data};
	my $cfg  = $data->{cfg};
	my $log  = $data->{log};

	$log->log("[LOW] Initiating possible push start.");

	my $push_active = $cfg->param('push_mode') eq 'active';

	my $push_method = $cfg->param('push_method') || 'incoming';
	$push_method = lc($push_method);

	if ($push_active) {
		if ( $push_method eq 'incoming' ) {
			launch_incoming_push($work_data);
		} elsif ( $push_method eq 'polling' ) {
			launch_polling_push($work_data);
		} else {
			$log->log(
				   "[ERR] No valid push method specified (found $push_method)");
		}
	}
}

# Get lines at tail of log file for troubleshooting purposes
sub logs_response {
	my ( $log, %query ) = @_;

	my $num_lines = $query{lines};
	my $file      = $query{file};
	my $path      = $file ? $log->{path} . $file : $log->_logpath;
	$path =~ s/\\/\//g;
	my @log_lines = tail( $path, $num_lines );
	$num_lines = @log_lines;
	my $log_tail = join "", @log_lines;
	$log_tail =~ s/</&lt;/g;
	$log_tail =~ s/>/&gt;/g;
	$log_tail =~ s/\n/<br\/>/g;
	$log_tail = "Log file is empty." if !$log_tail;
	my $content =
qq{<html><body><h2>$path (<a href="/xtsloglist">List all logs</a>)</h2><h3>Displaying $num_lines lines</h3><pre>$log_tail</pre></body></html>};
	my $r = HTTP::Response->new( 200, undef, undef, $content );

	return $r;
}

# Get list of log files for troubleshooting purposes
sub list_logs {
	my ($log) = @_;

	my $path = $log->{path};
	$path =~ s/\\/\//g;

	opendir DIR, $path
	  or return HTTP::Response->new( 500, "Couldn't open directory", undef,
									 undef );
	my @files = sort ( grep { -f "$path/$_" } readdir DIR );
	close DIR;
	my $list = join "<br/>",
	  map { qq{<a href="/xtslogs?file=$_">$_</a>} } @files;

	my $content =
	  "<html><body><h2>File listings ($path)</h2>$list</body></html>";
	my $r = HTTP::Response->new( 200, undef, undef, $content );

	return $r;
}

sub tail {
	my ( $fp, $num_lines ) = @_;

	$num_lines = 50 if !defined $num_lines;

	$fp =~ s/\\/\//g;
	return "No such file: $fp" if !-f $fp;

	my $avg_line_length = 400;                             # in bytes
	my $size            = -s $fp;
	my $block           = $num_lines * $avg_line_length;
	open FH, $fp or return "Can't open file: $fp -- $!";
	seek FH, -$block, 2 if $block < $size && $block != 0;
	my @lines = <FH>;
	shift @lines
	  if $block < $size && $block != 0;    # First line is probably incomplete
	$num_lines = @lines if $num_lines > @lines || $num_lines == 0;
	return @lines[ -$num_lines .. -1 ];
}

1;
