# $RCSfile$; $Date: 2010-05-27 10:57:52 -0700 (Thu, 27 May 2010) $; $Revision: 2793 $; $Name$
# Package: Feedfetcher::TransformPlugin
#
# Perl language plugins for transforming SportsML

package Feedfetcher::TransformPlugin;

use strict;

my %plugins = ();

# Subroutine: is_plugin
#
# Indicates whether the supplied string is the name of a supported plugin
#
# Parameters: 
#
# $key - The name of the plugin to check for
#
# Returns:
#
# True if $key is a supported plugin, false otherwise
sub is_plugin {
	my ($key) = @_;
	return exists $plugins{$key};
}

# Subroutine: transform
#
# Transforms the supplied string with a specified plugin
#
# Parameters: 
#
# $key - The name of the plugin to transform with
# $str - The string to perform the transformation on
# $params - An option hashref of parameters
#
# Returns:
# 
# The results of the transformation, as a string
sub transform {
	my ($key, $str, $params) = @_;

	die "No such plugin: $key" if !is_plugin($key);
	return &{$plugins{$key}}($str, $params);
}

# Group: Plugins
#
# Def: identity
#
# Returns the original SportsML *as is* 
$plugins{'identity'} = sub {
	my ($str) = @_;

	return $str;
};

# Def: html2text
#
# Transform a simple HTML document to wire-style ASCII text 
use Core::SimpleHtmlFormatter;
$plugins{'html2text'} = sub {
	my ($str, $params) = @_;

	my $shf = new Core::SimpleHtmlFormatter($params);
	return $shf->transform($str);
};

use Core::SimpleAnpaFormatter;
$plugins{'html2anpa'} = sub {
	my ($str, $params) = @_;
	
	my $shf = new Core::SimpleAnpaFormatter($params);
	return $shf->transform($str);
};

$plugins{'ttml2tt'} = sub {
	my ($str, $params) = @_;

	# Replace [ with <
	$str =~ s/\[/</gs;

	# Replace ] with >
	$str =~ s/\]/>/gs;

	#Remove all end tags
	$str =~ s/<\/[^>]*>//gs;

	# Replace value=" with = and Remove closing " mark
	$str =~ s/value=\"([^\"]*)\"/$1/gs;

	return $str;
};
