# $RCSfile$; $Date: 2005-02-10 16:59:44 -0800 (Thu, 10 Feb 2005) $; $Revision: 817 $; $Name$

package Feedfetcher::Validation::ApQa;

use strict;

use File::Slurp;
use File::Spec;

use Feedfetcher::Validation::SchemaValidator;
use Feedfetcher::Validation::SchematronValidator;
use Core::ActiveFixtures;

sub new {
    my ($class, $cfg, $log) = @_;

	my $self = { cfg => $cfg, log => $log };
	bless $self, $class;
	$self->init();
	return $self;
}

sub init {
	my ($self) = @_;
	$self->{lookup} = Core::ActiveFixtures::load($self->{cfg}, 'apqa_lookup');
}

sub validate {
	my ($self, $doc, $xml) = @_;

	# Schema validation
	my $schema_val = $self->{schema_val} ||= 
		Feedfetcher::Validation::SchemaValidator->new($self->{cfg}, 
		$self->{log});
	$schema_val->validate($doc, $xml);					  

	# Schematron validation
	my $schematron_val = $self->{schematron_val} ||= 
		Feedfetcher::Validation::SchematronValidator->new($self->{cfg}, 
		$self->{log});
	$schematron_val->validate($doc, $xml);					  

	# Use lookup file to run other tests
	my $tests_str = 
		Core::ActiveFixtures::doc_lookup($self->{lookup}, $doc, $self->{cfg});
	
	my @tests = split(/\s*,\s*/, $tests_str);
	my $vdir = 'Feedfetcher::Validation';
	my $vdir_path = 'Feedfetcher/Validation';
	foreach my $test (@tests) {
		my $vpath = "Feedfetcher/Validation/${test}.pm";
		my $vpack = "Feedfetcher::Validation::${test}";
		my $vmodule = $self->{modules}->{$test};
		if (!$vmodule) {
			require $vpath;
			$vmodule = $self->{modules}->{$test} = 
				${vpack}->new($self->{cfg}, $self->{log});
		}

		$vmodule->validate($doc, $xml);
	}
}

1;
	