# $RCSfile$; $Date: 2005-09-06 16:06:08 -0700 (Tue, 06 Sep 2005) $; $Revision: 935 $; $Name$

package Feedfetcher::Validation::Validator;

use strict;

use Core::XmlUtil;

use Net::SMTP;

sub new {
    my ($class, $cfg, $log) = @_;

	my $self = { cfg => $cfg, log => $log };
	bless $self, $class;
	$self->init;
	$self->init_plugin;
	return $self;
}

sub init {}
sub init_plugin {}

sub cfg { return $_[0]->{cfg}; }
sub xml { return $_[0]->{xml}; }
sub doc { return $_[0]->{doc}; }

sub pre_validate {
	my ($self, $doc, $xml) = @_;

	$self->{doc} = $doc;
	$self->{xml} = $xml;
	
	# Get doc-id, timestamp, sports-title, league, fixture-key
	my @meta = firstnode($doc, '/sports-content/sports-metadata');
	if (!@meta) {
		error('No sports-metadata node in document');
		return;
	} elsif (@meta > 1) {
		error('More than one sports-metadata node in document');
	}

	my $meta = shift @meta;
	$self->{doc_id} = attribute($meta, 'doc-id');
	$self->{timestamp} = attribute($meta, 'date-time');
	$self->{fixture_key} = attribute($meta, 'fixture-key');
	$self->{league_key} = attribute_xpath($meta,
		'/sports-content/sports-metadata/' .
		'sports-content-codes/sports-content-code[@code-type="league"]/@code-key');
	$self->{sports_title} = solonode_optional($meta, 'sports-title/text()');
}

sub validate {
	my ($self, $doc, $xml) = @_;

	$self->pre_validate($doc, $xml);
	$self->test($doc, $xml);
}

sub _alert { 
	my ($self, $type, $level, $msg) = @_;

	my $doc_id = $self->{doc_id} || 'unknown';
	my $league_key = $self->{league_key} || 'unknown';
	my $fixture_key = $self->{fixture_key} || 'unknown';
	my $timestamp = $self->{timestamp} || 'unknown';

	$msg = "ALERT: doc-id: $doc_id (league: $league_key, " . 
		"fixture-key: $fixture_key, timestamp: $timestamp): $msg";
	$self->{log}->log("[ADV] $msg);

	$self->_mail("$level $type alert: $league_key, $fixture_key", $msg);
}

sub alert { 
	my ($self, $level, $msg) = @_; 
	$self->_alert('dataset', $level, $msg); 
}

sub schema_alert { 
	my ($self, $msg) = @_; 
	$self->_alert('schema', 'hi', $msg); 
}

sub error {
	my ($self, $msg) = @_; 

	my $doc_id = $self->{doc_id} || 'unknown';
	my $league_key = $self->{league_key} || 'unknown';
	my $fixture_key = $self->{fixture_key} || 'unknown';
	my $timestamp = $self->{timestamp} || 'unknown';

	$msg = "doc-id: $doc_id (league: $league_key, " . 
		"fixture-key: $fixture_key, timestamp: $timestamp): $msg";
	$self->{log}->log("[ADV] $msg");
}

sub _mail {
	my ($self, $subject, $msg) = @_;

	my $cfg = $self->{cfg};
	return if !$cfg->param('smtp_host');
	my $from = $cfg->required_param('apqa_from');
	my $to = $cfg->required_param('apqa_to');
	my $cc = $cfg->param('apqa_cc');

    my $smtp = Net::SMTP->new($cfg->required_param('smtp_host')) or die $!;
	$smtp->auth($cfg->required_param('smtp_user'),
		$cfg->required_param('smtp_pass')) if 
		$cfg->required_param('smtp_user');
	$smtp->mail( $from );
	$smtp->to( $to );
	$smtp->cc( $cc ) if $cc;
	$smtp->data();
	$smtp->datasend("To: $to\n");
	$smtp->datasend("Cc: $cc\n") if $cc;
	$smtp->datasend("From: $from\n");
	$smtp->datasend("Subject: $subject\n");
	$smtp->datasend("\n"); # done with header
	$smtp->datasend($msg);
	$smtp->dataend();
	$smtp->quit(); # all done. message sent.
}

1;
