# $Id: Affiliation_phase.pm 2440 2009-04-21 20:59:44Z ian $

package FeedfetcherDeluxe::Entities::Affiliation_phase;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Season;
use FeedfetcherDeluxe::Entities::Display_name;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::Team_phase;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Affiliation

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub store_from_doc {
	my ($self) = @_;
	
	
}

sub process_league_directory_group {
	my ($self, $group_node, $publisher_id) = @_;

    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    my $display_name_entity = FeedfetcherDeluxe::Entities::Display_name->new($self->{dbh}, $self->{log});
    my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new($self->{dbh}, $self->{log});
    my $team_phase_entity = FeedfetcherDeluxe::Entities::Team_phase->new($self->{dbh}, $self->{log});

	my $group_key = attribute_xpath($group_node, 'group-metadata/@group-key') or
		die "No group-key supplied";

	my $group_type = attribute_xpath($group_node, 'group-metadata/@group-type') or
		die "No group-type supplied";

    my $league_key;
    if ($group_type eq 'league') {
    	$league_key = $group_key;
    }
    else {
        my $league_node = solonode_optional($group_node, 'group-metadata/affiliation[@membership-type="league"]');
        $league_key = attribute($league_node, 'membership-key'); # or die "No team-key supplied";
    }

    my $league_id = $affiliation_entity->retrieve_id_by_key({'affiliation_key'=>$league_key, 'affiliation_type'=>'league', 'publisher_id'=>$publisher_id});
    if (! $league_id) {
        $league_id = $affiliation_entity->create(({'affiliation_key'=>$league_key, 'affiliation_type'=>'league', 'publisher_id'=>$publisher_id}));
    }

	my $display_name_full = attribute_xpath($group_node, 'group-metadata/name/@full');
	my $display_name_first = attribute_xpath($group_node, 'group-metadata/name/@first');
	my $display_name_last = attribute_xpath($group_node, 'group-metadata/name/@last');
	my $display_name_nickname = attribute_xpath($group_node, 'group-metadata/name/@nickname');
	my $display_name_abbreviation = attribute_xpath($group_node, 'group-metadata/name/@abbreviation');

    my @affiliation_args = ($group_key, $group_type);
    my $affiliation_id = $affiliation_entity->retrieve_id_by_key({'affiliation_key'=>$group_key, 'affiliation_type'=>$group_type, 'publisher_id'=>$publisher_id});
    if (! $affiliation_id) {
    	# Create an affiliation record
    	$affiliation_id = $affiliation_entity->create({
    		'affiliation_key'=>$group_key, 
    		'affiliation_type'=>$group_type,
    		'publisher_id'=>$publisher_id
    	});
    }

    my $display_name_id = $display_name_entity->retrieve_id_by_key({'entity_type'=>'affiliations', 'entity_id'=>$affiliation_id});
    if (! $display_name_id) {
        # Create display name record to store more detailed naming information about the league, division, etc.
        $display_name_entity->create({
            'entity_type'=>'affiliations',
            'entity_id'=>$affiliation_id,
            'language'=>'en-US',
            'first_name'=>$display_name_first,
            'last_name'=>$display_name_last,
            'full_name'=>$display_name_full,
            'alias'=>$display_name_nickname,
            'abbreviation'=>$display_name_abbreviation
        });
    }
    else {
        $display_name_entity->update({
            'entity_type'=>'affiliations',
            'entity_id'=>$affiliation_id,
            'language'=>'en-US',
            'first_name'=>$display_name_first,
            'last_name'=>$display_name_last,
            'full_name'=>$display_name_full,
            'alias'=>$display_name_nickname,
            'abbreviation'=>$display_name_abbreviation
        },{
            'id'=>$display_name_id
        });
    }


    
	# HERE: if affiliations table already has a row with these matches:
	#	affiliations.affilation_key == $group_key
	#	affiliations.affiliation_type == $group_type
	#	affiliations.publisher_id == $publisher_id
	# then update:
	#	display_names.entity_type, entity_id, full_name, first_name, last_name, abbreviation
	# else:
	#	create new row in affiliations
	#	create new row with display_names.entity_type, entity_id, full_name, first_name, last_name, abbreviation
	#
	# ALSO: wipe out any old affiliation_phases for this group
	
	$self->delete({'ancestor_affiliation_id'=>$affiliation_id, 'root_id'=>$league_id});

	my $organizational_phase_start_season = attribute_xpath($group_node, 'group-metadata/organizational-phase/@start-season');
	my $organizational_phase_end_season = attribute_xpath($group_node, 'group-metadata/organizational-phase/@end-season');
	my $organizational_phase_start_date_time = attribute_xpath($group_node, 'group-metadata/organizational-phase/@start-date-time');
	my $organizational_phase_end_date_time = attribute_xpath($group_node, 'group-metadata/organizational-phase/@end-date-time');
	
	$organizational_phase_start_date_time =~ s/\s//g;
    $organizational_phase_end_date_time =~ s/\s//g;
	
	# Fragile, need to check for well-formedness
	$organizational_phase_start_date_time = undef if $organizational_phase_start_date_time eq '';
    $organizational_phase_end_date_time = undef if $organizational_phase_start_date_time eq '';
    	
    # To deal with imperfect testing data. Remove.
    $organizational_phase_start_season =~ s/\s//g;
    $organizational_phase_start_season = '' if $organizational_phase_start_season !~ /\d+/;
    $organizational_phase_end_season =~ s/\s//g;
    $organizational_phase_end_season = '' if $organizational_phase_end_season !~ /\d+/;
    
    my $start_season_id;
    my $end_season_id;
    # Need to look this up
    if ($organizational_phase_start_season ne '') {
        $start_season_id = $season_entity->retrieve_id_by_key({'season_key' => $organizational_phase_start_season, 'league_id' => $league_id, 'publisher_id' => $publisher_id});
        if (! $start_season_id) {
            $start_season_id = $season_entity->create({'season_key'=>$organizational_phase_start_season, 'league_id'=>$league_id, 'publisher_id'=>$publisher_id});
        }
    }
    # Need to look this up
    if ($organizational_phase_end_season ne '') {
        $end_season_id = $season_entity->retrieve_id_by_key({'season_key' => $organizational_phase_end_season, 'league_id' => $league_id, 'publisher_id' => $publisher_id});
        if (! $end_season_id) {
            $end_season_id = $season_entity->create({'season_key'=>$organizational_phase_end_season, 'league_id'=>$league_id, 'publisher_id'=>$publisher_id});
        }
    }
	
    $self->delete({'affiliation_id'=>$affiliation_id, 'root_id'=>$league_id});

	my @sub_affliation_nodes = nodes($group_node, 'group-metadata/affiliation');
	foreach my $sub_affliation_node (@sub_affliation_nodes) {
        # HERE: for each child affiliation, 
        # insert: affiliation_phases.ancestor_affiliation_id, start_season, end_season, start_date_time, end_date_time

        my $affiliation_key = attribute_xpath($sub_affliation_node, '@membership-key');
        my $affiliation_type = attribute_xpath($sub_affliation_node, '@membership-type');
		
		my $ancestor_affiliation_id = $affiliation_entity->retrieve_id_by_key({'affiliation_key'=>$affiliation_key, 'affiliation_type'=>$affiliation_type, 'publisher_id'=>$publisher_id});
        if (! $ancestor_affiliation_id) {
            # Create an affiliation record
            $ancestor_affiliation_id = $affiliation_entity->create({
                'affiliation_key'=>$affiliation_key, 
                'affiliation_type'=>$affiliation_type,
                'publisher_id'=>$publisher_id
            });
        }
    
        $self->create({
            'affiliation_id'=>$affiliation_id,
            'ancestor_affiliation_id'=>$ancestor_affiliation_id,
            'root_id'=>$league_id,
            'start_season_id'=>$start_season_id,
            'end_season_id'=>$end_season_id,
            'start_date_time'=>$organizational_phase_start_date_time,
            'end_date_time'=>$organizational_phase_end_date_time
        });
	}

	my @sub_group_nodes = nodes($group_node, 'group');
	foreach my $sub_group_node (@sub_group_nodes) {
		$self->process_league_directory_group($sub_group_node,$publisher_id)
	}

	my @sub_team_nodes = nodes($group_node, 'team');
	foreach my $sub_team_node (@sub_team_nodes) {
		$team_phase_entity->process_league_directory_team($sub_team_node,$publisher_id)
	}
}

1;