# $Id: American_football_action_participant.pm 2628 2009-11-17 21:41:04Z ian $

package FeedfetcherDeluxe::Entities::American_football_action_participant;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::American_football_action_play;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::American_football_action_participant

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;
}

sub store_from_doc {
    my ($self, $node, $publisher_id, $play_id) = @_;

    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});

#    $self->{log}->log("[DBG] event_state_id: $event_state_id");
    
    # Get attributes to store
    my $player_idref = attribute($node, 'player-idref');
    my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
    if (! $person_id) {
        $self->{log}->log("[ADV] american_football_action_participant add failure: no player information found in document for player-idref $player_idref");    	    	
        return;
    }
    my $role = attribute($node, 'role');
    my $yardage = attribute($node, 'yardage');
   	my $yards_gained = attribute($node, 'yards-gained');

    if (! defined $role || $role =~ /^\s*$/ ) {
        $self->{log}->log("[ADV] american_football_action_participant add failure: no participant role found in document for player-idref $player_idref");    	    	
        return;
    }

    my $score_credit = attribute($node, 'score-credit') || '0';
#    $score_credit = '0' if ! $score_credit;
    ($score_credit eq '1') ? '1' : '0';

    my $american_football_action_participant_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new(
        $self->{dbh}, $self->{log}, 'american_football_action_participants'
    );

    $american_football_action_participant_record_mapper->retrieve(
        {
            'american_football_action_play_id'  => $play_id,
            'person_id' => $person_id,
            'participant_role'=>$role
        }
    );
    # Get a count of the rows that are there, in case cleanup is needed
    my $action_participant_row_count =  $self->retrieve_row_count(
        {
            'american_football_action_play_id'  => $play_id,
            'person_id' => $person_id,
            'participant_role'=>$role
        }
    );

    # This should not happen, but if old duplicate rows are in the db, this will clear them out
    if ($action_participant_row_count > 1) {
    	$self->delete(
    	{
            'american_football_action_play_id'  => $play_id,
            'person_id' => $person_id,
            'participant_role'=>$role
    	}
    	);
    	$action_participant_row_count = 0;
    }
    $american_football_action_participant_record_mapper->set(
        {
            'yardage'=>$yardage,
            'yards_gained'=>$yardage,
            'score_credit'=>$score_credit,
            'participant_role'=>$role           
        }
    );
    $american_football_action_participant_record_mapper->save();
#    if ($action_participant_row_count == 0) {
#    	$self->create(
#            {
#                'american_football_action_play_id'  => $play_id,
#                'person_id' => $person_id,
#                'yardage'=>$yardage,
#                'yards_gained'=>$yards_gained,
#                'score_credit'=>$score_credit,
#                'participant_role'=>$role           
#            }
#        );
#    }
#    else {
#    	$self->update(
#    	{
#            'yardage'=>$yardage,
#            'yards_gained'=>$yardage,
#            'score_credit'=>$score_credit,
#            'participant_role'=>$role    		
#    	}
#    	, {
#            'american_football_action_play_id'  => $play_id,
#            'person_id' => $person_id,
#            'participant_role'=>$role
#    	}
#    	);
#   }
}

# NOTE: Assumes proper doc context for node argument
sub store_all_from_doc {
    my ($self, $node, $publisher_id, $event_state_id) = @_;

    my $american_football_action_play_entity = FeedfetcherDeluxe::Entities::American_football_action_play->new($self->{dbh}, $self->{log});
    my $play_id;
    if ($event_state_id) {
        $play_id = $american_football_action_play_entity->retrieve_id_by_key({'american_football_event_state_id'=>$event_state_id});
    }
    else {
        $play_id = $american_football_action_play_entity->retrieve_max_id();        
    } 
 
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
    my $action_listing = $sportsml_document->get_content_code_key('action-listing');
    
    if ($action_listing) {
         # Clear out participants in case any have disappeared and should not be there
#        $self->{log}->log("[DBG] action_listing: $action_listing play_id: $play_id");
        $self->delete({'american_football_action_play_id'=>$play_id}) if $action_listing eq 'complete';
    }
    
    my @participant_nodes = $node->findnodes("child::action-american-football-play-participant");
    foreach my $participant_node (@participant_nodes) {
        $self->store_from_doc($participant_node, $publisher_id, $play_id);
    }
}

1;