# $Id: Award.pm 3066 2014-11-14 18:38:14Z ian $

package FeedfetcherDeluxe::Entities::Award;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
#use Time::HiRes qw( gettimeofday);

=head1 NAME

FeedFetcherDeluxe::Entities::Award

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub  store_from_doc {
    my ( $self, $participant_type, $node, $publisher_id, @award_nodes ) = @_;

    my $team_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});
    my $participant_id;

    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    
    my $date_coverage_node = firstnode( @award_nodes[0], 'ancestor::*[@date-coverage-type and (@date-coverage-value or @event-key)]' );
    if (! $date_coverage_node) {
    	$date_coverage_node = $node;
    }
    
    my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($date_coverage_node, $publisher_id);

#    $self->{log}->log("[DBG] date_coverage_type, date_coverage_value, date_coverage_value_id: $date_coverage_type, $date_coverage_value, $date_coverage_value_id");
     
     
    my $participant_root = ($participant_type eq 'teams') ? 'team': 'player';

    my $metadata_node = solonode($node, $participant_root."-metadata");
    my $participant_key = attribute($metadata_node, $participant_root."-key");
    
    # Just in case, if there's no key, get out.
    return $participant_id if (! $participant_key || $participant_key eq '');

    if ($participant_type eq 'teams') {
        $participant_id = $team_entity->retrieve_id_by_key({'team_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }

    if ($participant_type eq 'players') {
        $participant_id = $person_entity->retrieve_id_by_key({'person_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }
    
#    my @award_nodes = $node->findnodes($participant_root."-stats/award");
#    $self->{log}->log("[DBG] award participant_id: $participant_id");
    
#    return if (! @award_nodes);
        
    $self->delete(
    {
        'participant_type' => ($participant_type eq 'players') ? 'persons' : $participant_type,
        'participant_id' => $participant_id,
        'date_coverage_type' => $date_coverage_type,
        'date_coverage_id' => $date_coverage_value_id
    }
    );

    foreach my $node (@award_nodes) {
        my $award_type = attribute($node, 'award-type');
        my $award_name = attribute($node, 'name');        
        my $award_rank = attribute($node, 'place');
        my $award_total = attribute($node, 'total');
        my $award_value = attribute($node, 'value');
        my $award_currency = attribute($node, 'currency');
        
        $self->create(
        {
            'participant_type' => ($participant_type eq 'players') ? 'persons' : $participant_type,
            'participant_id' => $participant_id,
            'date_coverage_type' => $date_coverage_type,
            'date_coverage_id' => $date_coverage_value_id,
            
        	'award_type' => $award_type,
        	'name' => $award_name,
        	'total' => $award_total,
        	'rank' => $award_rank,
        	'award_value' => $award_value,
        	'currency' => $award_currency
        }
        );
        
    }        
}

sub store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
#    $self->{log}->log("[DBG] awards");
    my @team_nodes = $node->findnodes("//team");
    my @player_nodes = $node->findnodes("//player");
    
    foreach my $node (@team_nodes) {
        # Save .03 ms by passing in the tested-for award nodes!
    	my @award_nodes = $node->findnodes("team-stats/award");
    	# Only process if there is at least one award node, otherwise could wipe out previous entries
        $self->store_from_doc('teams', $node, $publisher_id, @award_nodes) if (@award_nodes);
    }    
    foreach my $node (@player_nodes) {    	
        # Save .03 ms by passing in the tested-for award nodes!
        my @award_nodes = $node->findnodes("player-stats/award");
        # Only process if there is at least one award node, otherwise could wipe out previous entries
        $self->store_from_doc('players', $node, $publisher_id, @award_nodes) if (@award_nodes);
    }    
}

1;