# $Id: Baseball_action_pitches.pm 2630 2009-11-17 21:42:13Z ian $

package FeedfetcherDeluxe::Entities::Baseball_action_pitches;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
#use FeedfetcherDeluxe::Entities::Baseball_action_participant;
use FeedfetcherDeluxe::Entities::Baseball_action_play;
use FeedfetcherDeluxe::Entities::SportsmlDocument;

#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Baseball_action_pitches

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub initialize_local {
    my ($self) = @_;
#    $self->{table_name} = "baseball_action_pitches";
}

sub setup_sql {
    my ($self) = @_;
}

sub store_from_doc {
    my ($self, $node, $publisher_id, $play_id, $count) = @_;
#!play:action-baseball-pitch://action-baseball-pitch:15007000
#umpire-call:varchar(100)
#pitch-type:varchar(100)
#pitch-velocity:integer
# NOTE: for xtoss, this is taken and concatenated with the y value. Don't need to add the y one here.
#pitch-coordinate-x:integer
    my $pitch_type = attribute($node, 'pitch-type');
    my $pitch_velocity = attribute($node, 'pitch-velocity');
    my $pitch_coordinate_x = attribute($node, 'pitch-coordinate-x');
    my $pitch_coordinate_y = attribute($node, 'pitch-coordinate-y');
    my $trajectory_coordinates = $pitch_coordinate_x.",".$pitch_coordinate_y;
    my $ball_type = attribute($node, 'ball-type');
    my $strike_type = attribute($node, 'strike-type');
    my $umpire_call = attribute($node, 'umpire-call');
    

    # Since old pitch records have been removed in store_all_from_doc,
    # we just need here to create a new record for the node
    $self->create(
        {
            'baseball_action_play_id'  => $play_id,
            'pitch_type' => $pitch_type,
            'pitch_velocity'=>$pitch_velocity,
            'trajectory_coordinates'=>$trajectory_coordinates,
            'ball_type'=>$ball_type,
            'strike_type'=>$strike_type,
            'umpire_call'=>$umpire_call,
            'sequence_number'=>$count
        }
    );
}

sub store_all_from_doc {
    my ($self, $node, $publisher_id, $event_state_id) = @_;
    
    my $baseball_action_play_entity = FeedfetcherDeluxe::Entities::Baseball_action_play->new($self->{dbh}, $self->{log});

    my $play_id;
    if ($event_state_id) {
        $play_id = $baseball_action_play_entity->retrieve_id_by_key({'baseball_event_state_id'=>$event_state_id});
    }
    else {
        $play_id = $baseball_action_play_entity->retrieve_max_id();        
    } 

#    $self->{log}->log("[DBG] play id: $play_id");

   my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
   my $action_listing = $sportsml_document->get_content_code_key('action-listing');
    
    # Clear out existing pitch data
   $self->delete({'baseball_action_play_id'=>$play_id}); #if $action_listing eq 'complete';
    
    my @pitch_nodes = $node->findnodes("child::action-baseball-pitch");
    my $count = 1;
    foreach my $pitch_node (@pitch_nodes) {
#        $self->{log}->log("[DBG] processing pitches");
        $self->store_from_doc($pitch_node, $publisher_id, $play_id, $count);
        $count += 1;
    }
    
 
}

1;