# $Id: Basketball_action_participant.pm 2994 2012-08-14 22:36:35Z ian $

package FeedfetcherDeluxe::Entities::Basketball_action_participant;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Basketball_action_play;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Basketball_action_participant

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;
}

sub store_from_doc {
    my ($self, $node, $publisher_id, $play_id) = @_;

    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});

#    $self->{log}->log("[DBG] event_state_id: $event_state_id");
    
    # Get attributes to store
    my $player_idref = attribute($node, 'player-idref');
    my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
    if (! $person_id) {
        $self->{log}->log("[ADV] basketball_action_participant add failure: no player information found in document for player-idref $player_idref");    	    	
        return;
    }

    my $team_idref = attribute($node, 'team-idref');
    my $team_id;
    # Make an extra effort to find the team for a player if that attribute has not been attached to the node
    if (! defined $team_idref) {
        $team_idref = $person_entity->find_team_idref_from_player_idref($node, $player_idref);
#        print "team_idref: $team_idref\n";
    }
    if (defined $team_idref) {
        my $team_entity = FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
        $team_id = $team_entity->find_id_from_idref($node, $team_idref, $publisher_id);
    }
    else {
        $self->{log}->log("[ADV] basketball_action_participant add failure: no team information found in document for player-idref '$player_idref'");             
        return;
    }
    
    my $role = attribute($node, 'role');

# TODO: Ensure that this works for ALL feeds - the main issue is the possibility of more than 1 instance of a player per play

# Ticket 187
# http://redmine.xmlteam.com/redmine/issues/187
# 2. It appears that basketball-action-participants processing requires the existence of role. The source data doesn't have role for every play; it uses 
# it primarily when there are two players involved in the play and each player's role must be defined. Can we remove this requirement for processing?
#    if (! defined $role || $role =~ /^\s*$/ ) {
#        $self->{log}->log("[ADV] basketball_action_participant add failure: no participant role found in document for player-idref $player_idref");    	    	
#        return;
#    }

    my $basketball_action_participant_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new(
        $self->{dbh}, $self->{log}, 'basketball_action_participants'
    );

    # NOTE
    # The assumtion is that there will be only one instance of a person per play, so role is not not needed 
    
    # Get a count of the rows that are there, in case cleanup is needed
    my $action_participant_row_count =  $self->retrieve_row_count(
        {
            'basketball_action_play_id'  => $play_id,
            'person_id' => $person_id #,
#            'participant_role'=>$role # Remove role from search since it can be null
        }
    );

    # This should not happen, but if old duplicate rows are in the db, this will clear them out
    if ($action_participant_row_count > 1) {
        $self->delete(
        {
            'basketball_action_play_id'  => $play_id,
            'person_id' => $person_id #,
#            'participant_role'=>$role # Remove role from search since it can be null
        }
        );
        $action_participant_row_count = 0;
    }
    $basketball_action_participant_record_mapper->retrieve(
        {
            'basketball_action_play_id'  => $play_id,
            'person_id' => $person_id #,
#            'participant_role'=>$role # Remove role from search since it can be null
        }
    );
    $basketball_action_participant_record_mapper->set(
        {
#            'participant_role'=>$role,
            'team_id' => $team_id,
            'participant_role'=>$role           
        }
    );
    my $basketball_action_participant_id = $basketball_action_participant_record_mapper->save();
    
    return $person_id;
}

# NOTE: Assumes proper doc context for node argument
sub store_all_from_doc {
    my ($self, $node, $publisher_id, $event_state_id) = @_;

    my $skip_delete = 0;
    
    my $basketball_action_play_entity = FeedfetcherDeluxe::Entities::Basketball_action_play->new($self->{dbh}, $self->{log});

    my $play_id;
    if ($event_state_id) {
        $play_id = $basketball_action_play_entity->retrieve_id_by_key({'basketball_event_state_id'=>$event_state_id});
    }
    else {
    	# This should not happen
#    	print "max\n";
        $play_id = $basketball_action_play_entity->retrieve_max_id();        
    } 
 
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
    my $action_listing = $sportsml_document->get_content_code_key('action-listing');
    
    if ($action_listing) {
         # Clear out participants in case any have disappeared and should not be there
#        $self->{log}->log("[DBG] action_listing: $action_listing play_id: $play_id");
        $self->delete({'basketball_action_play_id'=>$play_id}) if $action_listing eq 'complete';
    }
    
    my @participant_id_list = ();

    my @participant_nodes = $node->findnodes("child::action-basketball-play-participant");
    foreach my $participant_node (@participant_nodes) {
        my $person_id = $self->store_from_doc($participant_node, $publisher_id, $play_id);
        push(@participant_id_list, "$person_id") unless (! defined($person_id));
    }
    my $list_count = @participant_id_list;
#    print "list_count: $list_count\n";
    if ($list_count > 0) {
        my $id_str = join(', ', @participant_id_list);
#        print "deleting by play id: $play_id and id list: $id_str\n";
        $self->delete_by_id_list($play_id, $id_str) unless ($skip_delete eq '1');
    }
}

sub delete_by_id_list {
    my ( $self, $play_id, $id_str ) = @_;

    my $delete_sql = qq{
        DELETE FROM basketball_action_participants
        WHERE basketball_action_play_id=?
        AND person_id NOT IN ($id_str)     
    };

#    print $delete_sql."\n";
    my $sth = $self->{dbh}->prepare($delete_sql);
    $sth->execute($play_id);    
    $sth->finish;
}

1;