# $Id: Event.pm 2948 2012-05-16 19:04:21Z ian $

package FeedfetcherDeluxe::Entities::Event;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Document;
use FeedfetcherDeluxe::Entities::Season;
use Util::DateUtils;
#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

Core::Entities::Event

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.
=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

    $self->{'later_event_select'} = $self->{dbh}->prepare(qq{
        SELECT id
        FROM events
        WHERE id=?
        AND last_update > ?
    });

}

sub store_from_doc {
    my ($self, $event_node, $publisher_id) = @_;
	
    my $document_entity = FeedfetcherDeluxe::Entities::Document->new($self->{dbh}, $self->{log});
	my $sportsml_document_entity = FeedfetcherDeluxe::Entities::SportsmlDocument->new($event_node, $self->{log});
	my $doc_data = $sportsml_document_entity->data();
	my $doc_key = $doc_data->{'doc_id'};
	
	my $doc_id = $document_entity->retrieve_id_by_key(
        {
            'doc_id'=>$doc_key, 
            'publisher_id'=>$publisher_id
        }
	);
}

sub store_all_from_doc {
    my ($self, $node, $publisher_id) = @_;
	
	my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
    my $action_listing = $sportsml_document->get_content_code_key('schedule-listing');
    
	
    my @event_nodes = $node->findnodes("//event");
    foreach my $event_node (@event_nodes) {
        $self->store_from_doc($event_node, $publisher_id);
    }
}

sub find_event_id {
    my ($self, $node, $publisher_id) = @_;
	
	my $event_metadata_node = firstnode($node, "ancestor::sports-event/event-metadata");
	my $event_key = $event_metadata_node->getAttribute('event-key');
	return undef unless (defined $event_key);
	
	my $id = $self->retrieve_id_by_key({'event_key'=>$event_key, 'publisher_id'=>$publisher_id});
	
	return $id;
}

sub store_events_sub_seasons {
    my ($self, $node, $publisher_id) = @_;

    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    my $context;
    my $date_coverage_type;
    my $date_coverage_value;
    my $date_coverage_value_id;
    
    my @event_nodes = $node->findnodes("//sports-event");
    # This will return seasons, since the coverage value for an event is a season (contextually)
    foreach my $event_node (@event_nodes) {
        ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($event_node, $publisher_id);
    }
}

# Store the last_update times the document's events, based on this document's date-time value
sub store_event_last_updates {
	my ( $self, $node, $publisher_id ) = @_;
	
    my $sports_content_node = solonode( $node, "//sports-content/sports-metadata");
    my $date_time = $sports_content_node->getAttribute('date-time');
    my $date_utils = Util::DateUtils->new();
    
	my @sports_event_nodes = $node->findnodes("//sports-content//sports-event");
	foreach my $event_node (@sports_event_nodes) {
        my $event_metanode = solonode($event_node, "event-metadata");
        next unless defined $event_metanode;
        my $event_key = attribute($event_metanode, 'event-key');
        next unless defined $event_key;
        next if $event_key =~ /^\s*$/;
        
		my $event_id = $self->retrieve_id_by_key( {'event_key'=>$event_key, 'publisher_id'=>$publisher_id} );
		$self->update(
		{
			'last_update' => $date_utils->sportsml_date2sql_date($date_time)
		}, 
		{
			'id' => $event_id
		}
		);

	}
}

sub check_for_old {
	my ( $self, $node, $publisher_id ) = @_;
	
	my $ret_val;
	
	
	
    my $sports_content_node = solonode( $node, "//sports-content/sports-metadata");
    my $date_time = $sports_content_node->getAttribute('date-time');
    
	my @sports_event_nodes = $node->findnodes("//sports-content//sports-event");
	foreach my $event_node (@sports_event_nodes) {
        my $event_metanode = solonode($event_node, "event-metadata");
        next unless defined $event_metanode;
        my $event_key = attribute($event_metanode, 'event-key');
        next unless defined $event_key;
        next if $event_key =~ /^\s*$/;
		my $event_id = $self->retrieve_id_by_key( {'event_key'=>$event_key, 'publisher_id'=>$publisher_id} );
		if ( $event_id ) {
			$ret_val = $self->more_recent_event_updates( $event_id, $date_time );
			return 1 if defined $ret_val;
		}
	}
	
	return $ret_val;
}

sub more_recent_event_updates {
    my ($self, $event_id, $date_time) = @_;

    my $date_utils = Util::DateUtils->new();

    my $sql_date_time = $date_utils->sportsml_date2sql_date($date_time);
    my $ref = $self->{dbh}->selectrow_arrayref($self->{'later_event_select'},
        undef, $event_id, $sql_date_time
    );
    return $ref ? 1 : undef;
}
sub scan_for_delete {
    my ($self, $node, $publisher_id) = @_;
    
    my $specific_affiliation;
    my $specific_affiliation_id;
    my $league_id;
    my $sub_season_id;
    
    my $sportsml_document_entity = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{log});
    my $event_listing = $sportsml_document_entity->get_content_code_key('schedule-listing');
    my $schedule_affiliation = $sportsml_document_entity->get_content_code_key('schedule-affiliation');
#    $self->{log}->log("[DBG] event_listing: $event_listing");
    return unless (($event_listing eq 'complete'));
    
    # Find affiliation and exit if found and not valid
    # First look to s-c-c 'sport-category' to find out which s-c-c to look up thru join in affiliations_documents
    if (defined $schedule_affiliation) {
#        $self->{log}->log("[DBG] schedule_affiliation: $schedule_affiliation found");
        
        $specific_affiliation = $sportsml_document_entity->get_content_code_key($schedule_affiliation);
        if (defined $specific_affiliation) {
            my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, $self->{log} );
            $specific_affiliation_id = $affiliation_entity->retrieve_id_by_key(
            {
                'affiliation_key'  => $specific_affiliation,
                'affiliation_type' => $schedule_affiliation,
                'publisher_id'     => $publisher_id
            }
        );
        $self->{log}->log("[DBG] specific_affiliation: $specific_affiliation type: $schedule_affiliation specific_affiliation_id: $specific_affiliation_id");
        }
    }
    
    # If no sport-category, look in sub_season for league id and exit if not found
    
    # Find sub_season thru schedule-metadata - exit if not found
    # <schedule>
    #    <schedule-metadata date-coverage-type="season" date-coverage-value="2010.winter" competition-scope="sport-category"/>
    
    # Look for league id. It is ASSUMED here that there is one league per file
    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    $league_id = $season_entity->get_league_id($node, $publisher_id);
    if (! defined $league_id) {
        $self->{log}->log("[DBG] No league found for document. Exiting Event->scan_for_delete.");
        return unless (defined $league_id);
    }

    my @schedule_nodes = $node->findnodes('//schedule');
    
    foreach my $schedule_node (@schedule_nodes) {

#        $self->{log}->log("[DBG] Looking for season");
        my $schedule_metadata_node = firstnode($schedule_node, "schedule-metadata");
        if (defined $schedule_metadata_node) {
            my $date_coverage_type = $schedule_metadata_node->getAttribute('date-coverage-type');
            my $date_coverage_value = $schedule_metadata_node->getAttribute('date-coverage-value');
#        my $seasons_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'sub_seasons');
        
#            $self->{log}->log("[DBG] date_coverage_value: $date_coverage_value");
            my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
            my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($schedule_metadata_node, $publisher_id);
            if ($date_coverage_type =~ /sub_season/) {
                $sub_season_id = $date_coverage_value_id;
            }
            else {
                next;
            }
#        $self->{log}->log("[DBG] context: $context, date_coverage_type: $date_coverage_type, date_coverage_value: $date_coverage_value, date_coverage_value_id: $date_coverage_value_id");
#            $self->{log}->log("[DBG] sub_season_id: $sub_season_id");
        }
    
        my @sports_event_nodes = $schedule_node->findnodes('descendant::sports-event');
    
        my $event_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'events');
        my @event_id_list = ();
        
        foreach my $sports_event_node (@sports_event_nodes) {
            my $event_metadata_node = firstnode($sports_event_node, "event-metadata");
            my $event_key = $event_metadata_node->getAttribute("event-key");
            $event_record_mapper->reset();
            $event_record_mapper->retrieve(
            {
                'event_key'=>$event_key,
                'publisher_id'=>$publisher_id
            }
            );
            my $event_id = $event_record_mapper->get_id();
            push(@event_id_list, $event_id);
        }
    
        my $event_count = @event_id_list;
        next unless ($event_count > 0);
        
        my $extra_join = '';
        my $extra_where = '';
        my $select_sql = <<EOS;
SELECT events.id
FROM events
INNER JOIN events_sub_seasons ON (events_sub_seasons.sub_season_id = $sub_season_id AND events_sub_seasons.event_id = events.id)
EOS
        # If no sport-category s-c-c use season league - exit if no league
        my $event_id_list_str = join(', ', @event_id_list);
#        $self->{log}->log("[DBG] id_list string: $event_id_list_str");
        
        # Join by specific_affiliation if possible
        # e.g. 
        # <sports-content-code code-type="schedule-affiliation" code-key="sport-category" />
        # <sports-content-code code-type="sport-category" code-key="FS" code-name="Figure Skating" />
        # In the above example, the affiliation join would be to FS, pointed to via 'schedule-affiliaton'
        if (defined $specific_affiliation_id) {
            $extra_join .= "JOIN affiliations_events ae_scc ON (ae_scc.affiliation_id=$specific_affiliation_id AND ae_scc.event_id=events.id)\n";
        }
        if (defined $league_id) {
            $extra_join .= "JOIN affiliations_events ae_league ON (ae_league.affiliation_id=$league_id AND ae_league.event_id=events.id)\n";
        }
        $select_sql .="$extra_join\n";
        $select_sql .= "WHERE events.id NOT IN($event_id_list_str)\n";

        $self->{log}->log("[DBG] $select_sql");
        
        my $sth = $self->{dbh}->prepare($select_sql);
        $sth->execute();
        my @old_event_id_list = ();
        while(my @row = $sth->fetchrow_array) {
            push(@old_event_id_list, $row[0]);
        }
        
        my $old_event_count = @old_event_id_list;
#        $self->{log}->log("[DBG] old_event_count: $old_event_count");
        next unless ($old_event_count > 0);

        my $old_event_id_list_str = join(', ', @old_event_id_list);
#        $self->{log}->log("[DBG] old_event_id_list_str: $old_event_id_list_str");
#        return;
    
#        # Find existing event list
#        my @event_ids = [];
#    
#        # Update events in list to have event_status='deleted'
#        return if (scalar @event_ids == 0);
#        my $id_str = join(',', @event_ids);

        my $delete_sql;
        
## affiliations_events
#        $delete_sql = <<EOS;
#            DELETE FROM affiliations_events
#            WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
#        $delete_sql = <<EOS;
#            DELETE FROM events_sub_seasons
#            WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
#        $delete_sql = <<EOS;
#            DELETE FROM document_fixtures_events
#            WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
#        $delete_sql = <<EOS;
#            DELETE FROM events_documents
#            WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#        
## periods
#        $delete_sql = <<EOS;
#            DELETE FROM periods
#            WHERE periods.participant_event_id IN
#            (SELECT participants_events.id FROM participants_events WHERE participants_events.event_id IN ($old_event_id_list_str))
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
## participants_events
#        $delete_sql = <<EOS;
#DELETE FROM participants_events
#WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
#        $delete_sql = <<EOS;
#DELETE FROM person_event_metadata
#WHERE event_id IN ($old_event_id_list_str)
#EOS
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();
#
##        $delete_sql = <<EOS;
##EOS

#        $delete_sql = <<EOS;
#DELETE FROM events
#WHERE id IN ($old_event_id_list_str)
#EOS
##        print "$delete_sql\n";
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute();

        my $update_sql = <<EOS;
UPDATE events
SET event_status='deleted' WHERE id IN ($old_event_id_list_str)
EOS
#        print "$delete_sql\n";
        $sth = $self->{dbh}->prepare($update_sql);
        $sth->execute();

        # Do a select to find events matching all criteria but NOT in current document
    
        # JOIN sub_seasons ON (sub_seasons.id=)
        # JOIN events_sub_seasons ON (events_sub_seasons.event_id=events.id AND events_sub_seasons.sub_season_id=sub_seasons.id)
        
        # Do update on events table, setting event_status to 'deleted'
    }
}

sub delete_marked_events {
    my ($self) = @_;

    # Here, we benefit from previous work that marked events table event_status field value to 'deleted'
    # The work here involves going through all dependencies first and removing them followed up the chain to the events table
}

1;