# $Id: Event_action_play.pm 2193 2008-09-16 18:18:27Z ian $

package FeedfetcherDeluxe::Entities::Event_action_play;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Publisher;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Event_action_participant;

use FeedfetcherDeluxe::Entities::Entity;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Event_action_plays

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

    $self->{'delete_event_action_participants'} = $self->{dbh}->prepare(qq{
        DELETE FROM event_action_participants
        WHERE event_action_participants.event_action_play_id IN
        (
            SELECT event_action_plays.id
            FROM event_action_plays
            WHERE event_action_plays.event_state_id=?
        )
    });
}

sub store_event_action_participant {
    my ( $self, $event_action_play_id, $person_id, $participant_role, $node, $publisher_id ) = @_;
    
    my $event_action_participant_entity = FeedfetcherDeluxe::Entities::Event_action_participant->new( $self->{dbh}, $self->{log} );
    my $event_action_participant_id = $event_action_participant_entity->create(
    {
        'event_action_play_id' => $event_action_play_id,
        'person_id' => $person_id,
        'participant_role' => $participant_role
    }
    );
    
    return $event_action_participant_id;
}

sub store_from_doc {
    my ( $self, $event_state_id, $node, $publisher_id ) = @_;

    my $node_name = $node->nodeName();
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new( $self->{dbh}, $self->{log} );

    my $event_action_play_id;
    $self->delete_by_event_state_id($event_state_id);


    my $play_type = $node->getAttribute('play-type');
    my $score_attempt_type = $node->getAttribute('score-attempt-type');
    my $play_result = $node->getAttribute('play-result');
    my $comment = $node->getAttribute('comment');
    
    $event_action_play_id = $self->create(
    {
    	'event_state_id' => $event_state_id,
        'play_type' => $play_type,
        'score_attempt_type' => $score_attempt_type,
        'play_result' => $play_result,
        'comment' => $comment
    }
    );
    
    
    my $player_idref = $node->getAttribute('player-idref');
    my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
        
    my $participant_role = $node->getAttribute('participant-role');
    if ( ! defined $participant_role && $node_name =~ /action\-score/ ) {
        $participant_role = 'scorer';
    }
    
    $self->store_event_action_participant($event_action_play_id, $person_id, $participant_role, $node, $publisher_id);

    return $event_action_play_id
}

sub delete_by_event_state_id {
    my ($self, $event_state_id) = @_;
    
    
    my $event_action_play_delete_count = $self->{'delete_event_action_participants'}->execute($event_state_id);
    $self->{'delete_event_action_participants'}->finish;

    # This is not a call to the centtralized Entity->delete, so we need to do an explicit cache invalidate call here
    my $event_action_participant_entity = FeedfetcherDeluxe::Entities::Event_action_participant->new( $self->{dbh}, $self->{log} );
    $event_action_participant_entity->query_cache()->invalidate();
    
#    $self->{log}->log("[DBG] event_action_play_delete_count: $event_action_play_delete_count");
    
    $self->delete(
    {
        "event_state_id" => $event_state_id
    }
    );
}

1;