# $Id: Event_action_substitution.pm 2123 2008-07-23 19:22:08Z ian $

package FeedfetcherDeluxe::Entities::Event_action_substitution;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Position;
#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Event_action_substitution

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub store_from_doc {
    my ( $self, $event_state_id, $node, $publisher_id ) = @_;

    my $node_name = $node->nodeName();

    my $person_entity = FeedfetcherDeluxe::Entities::Person->new( $self->{dbh}, $self->{log} );
    my $position_entity = FeedfetcherDeluxe::Entities::Position->new( $self->{dbh}, $self->{log} );
    my $person_original_idref = $node->getAttribute('person-original-idref');
    my $person_replacing_idref = $node->getAttribute('person-replacing-idref');

#    $self->{log}->log("[DBG] person_original_idref: $person_original_idref person_replacing_idref: $person_replacing_idref");

    return undef unless (defined $person_original_idref && defined $person_replacing_idref);

    my $person_original_id = $person_entity->find_id_from_idref($node, $person_original_idref, $publisher_id);
    return undef unless defined $person_original_id;
    my $person_replacing_id = $person_entity->find_id_from_idref($node, $person_replacing_idref, $publisher_id);
    return undef unless defined $person_replacing_id;

    my $person_original_position = $node->getAttribute('person-original-position');
    return undef unless defined $person_original_position;
    my $person_replacing_position = $node->getAttribute('person-replacing-position');
    $person_replacing_position = $person_original_position if ( ! defined $person_replacing_position );
    
    my $person_original_position_id = $position_entity->get_position_id( $person_original_position, $node, $publisher_id );
    my $person_replacing_position_id;
    if (! $person_replacing_position eq $person_original_position ) {
        my $person_replacing_position_id = $position_entity->get_position_id( $person_replacing_position, $node, $publisher_id );
    }
    else {
        $person_replacing_position_id = $person_original_position_id;
    }

    my $person_type = $node->getAttribute('person-type');
    my $substitution_reason = $node->getAttribute('substitution-reason');
    my $comment = $node->getAttribute('comment');

    my $event_action_substitution_id = $self->retrieve_id_by_key( 
    {
        'event_state_id' => $event_state_id,
        'person_original_id' => $person_original_id,
        'person_replacing_id' => $person_replacing_id,
    }
    );

    if (! defined $event_action_substitution_id) {
        $event_action_substitution_id = $self->create(
        {
            'event_state_id' => $event_state_id,
            'person_type' => $person_type,
            'person_original_id' => $person_original_id,
            'person_original_position_id' => $person_original_position_id,
            'person_replacing_id' => $person_replacing_id,
            'person_replacing_position_id' => $person_replacing_position_id,
            'substitution_reason' => $substitution_reason,
            'comment' => $comment
        }
        );
    }
    else {
        $self->update(
        {
            'person_type' => $person_type,
            'person_original_position_id' => $person_original_position_id,
            'person_replacing_position_id' => $person_replacing_position_id,
            'substitution_reason' => $substitution_reason,
            'comment' => $comment
        },
        {
            'event_state_id' => $event_state_id,
            'person_original_id' => $person_original_id,
            'person_replacing_id' => $person_replacing_id,
        }
    );
    }
    
    return $event_action_substitution_id;
}
1;