# $Id: Event_state.pm 2139 2008-08-08 05:04:26Z ian $

package FeedfetcherDeluxe::Entities::Event_state;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Event_action_play;
use FeedfetcherDeluxe::Entities::Event_action_penalty;
use FeedfetcherDeluxe::Entities::Event_action_substitution;
use FeedfetcherDeluxe::Entities::Event_action_foul;
use FeedfetcherDeluxe::Entities::Publisher;
use FeedfetcherDeluxe::Entities::Event;

use FeedfetcherDeluxe::Entities::Entity;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Event_state

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub get_event_state_id {
    my ( $self, $node, $publisher_id ) = @_;
    
    my $node_name = $node->nodeName();
    my $publisher_entity = FeedfetcherDeluxe::Entities::Publisher->new( $self->{dbh}, $self->{log} );
    my $event_entity = FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );
    
    my $publisher_key = $node->getAttribute('publisher-key');
    if ( ! defined $publisher_key ) {
        $self->{log}->log("[ADV] No publisher_key found. Not storing node $node_name.");
        return undef;
    }

    # Assuming for the moment that the node refers to an event
    my $event_key = $node->getAttribute('date-coverage-value');
    if ( ! defined $event_key ) {
        $self->{log}->log("[ADV] No event_key found. Not storing node $node_name.");
        return undef;
    }
    my $event_id = $event_entity->retrieve_id_by_key(
        {
            'event_key' => $event_key,
            'publisher_id' => $publisher_id
        }
    );
    if ( ! defined $event_id ) {
        $self->{log}->log("[ADV] No event found for $event_key. Not storing node $node_name.");
        return undef;
    }
    
#    if ( ! defined $event_id ) {
#        $self->{log}->log("[ADV] No publisher found for $publisher_key. Not storing node $node_name.");
#        return undef;
#    }

    # For the moment, use minutes_elapsed and sequence_number as tests
    my $current_state = '0';
    my $sequence_number = $node->getAttribute('sequence-number');
    $sequence_number = undef if $sequence_number =~ /^\s*$/;
    my $period_value = $node->getAttribute('period-value');
    my $period_time_elapsed = $node->getAttribute('period-time-elapsed');
    my $period_time_remaining = $node->getAttribute('period-time-remaining');
    my $minutes_elapsed = $node->getAttribute('minutes-elapsed');
    $minutes_elapsed = undef if $minutes_elapsed =~ /^\s*$/;
    my $period_minutes_elapsed = $node->getAttribute('period-minutes-elapsed');
    my $context = $node->getAttribute('date-coverage-type');

    if ( (! defined $minutes_elapsed) && (! defined $sequence_number) ) {
        $self->{log}->log("[ADV] No minutes elapsed or sequence number attribute found. Not storing node $node_name.");
        return undef;
    }


    my $lookup_hash = {};
    $lookup_hash->{'event_id'} = $event_id;
    $lookup_hash->{'minutes_elapsed'} = $minutes_elapsed if (defined $minutes_elapsed);
    $lookup_hash->{'sequence_number'} = $sequence_number if ( defined $sequence_number );
    $lookup_hash->{'context'} = $context; # date-coverage-type attribute


    
    my $event_state_id = $self->retrieve_id_by_key( $lookup_hash );
    
    if ( ! defined $event_state_id ) {
        $self->create(
        {
        	'event_id' => $event_id,
        	'current_state' => '0', # Leave this way for now
        	'sequence_number' => $sequence_number,
        	'period_value' => $period_value,
        	'period_time_elapsed' => $period_time_elapsed,
        	'period_time_remaining' => $period_time_remaining,
        	'minutes_elapsed' => $minutes_elapsed,
        	'period_minutes_elapsed' => $period_minutes_elapsed,
            'context' => $context
        }
        );
        $event_state_id = $self->retrieve_id_by_key( $lookup_hash );
    }
    else {
        $self->update(
        {
            'event_id' => $event_id,
            'current_state' => '0', # Leave this way for now
            'sequence_number' => $sequence_number,
            'period_value' => $period_value,
            'period_time_elapsed' => $period_time_elapsed,
            'period_time_remaining' => $period_time_remaining,
            'minutes_elapsed' => $minutes_elapsed,
            'period_minutes_elapsed' => $period_minutes_elapsed,
            'context' => $context
        }, $lookup_hash
        );
    }
    
    return $event_state_id;
}

sub get_publisher_id {
    my ( $self, $node ) = @_;

    my $publisher_entity = FeedfetcherDeluxe::Entities::Publisher->new( $self->{dbh}, $self->{log} );
    my $publisher_key = $node->getAttribute('publisher-key');
    my $node_name = $node->nodeName();
    
    return undef unless defined $publisher_key;
    
    my $publisher_id = $publisher_entity->retrieve_id_by_key(
        {
            'publisher_key' => $publisher_key
        }
    );
    if ( ! defined $publisher_id ) {
        $self->{log}->log("[ADV] No publisher found for $publisher_key. Not storing node $node_name.");
    }

    return $publisher_id;
}

sub store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;

    my $event_action_play_entity = FeedfetcherDeluxe::Entities::Event_action_play->new( $self->{dbh}, $self->{log} );
    my $event_action_penalty_entity = FeedfetcherDeluxe::Entities::Event_action_penalty->new( $self->{dbh}, $self->{log} );
    my $event_action_substitution_entity = FeedfetcherDeluxe::Entities::Event_action_substitution->new( $self->{dbh}, $self->{log} );
    my $event_action_foul_entity = FeedfetcherDeluxe::Entities::Event_action_foul->new( $self->{dbh}, $self->{log} );

    my @event_action_nodes = $node->findnodes('//event-actions');

    foreach my $event_action_node (@event_action_nodes) {
        my @action_score_nodes = $event_action_node->findnodes('action-score');
        my @action_penalty_nodes = $event_action_node->findnodes('action-penalty');
        my @action_substitution_nodes = $event_action_node->findnodes('action-substitution');

        foreach my $action_score_node (@action_score_nodes) {
            my $publisher_id = $self->get_publisher_id($action_score_node);
            next unless defined $publisher_id;

            my $event_state_id = $self->get_event_state_id($action_score_node, $publisher_id);
            next unless defined $event_state_id;
            
            $event_action_play_entity->store_from_doc($event_state_id, $action_score_node, $publisher_id);
        }
        foreach my $action_penalty_node (@action_penalty_nodes) {
            my $publisher_id = $self->get_publisher_id($action_penalty_node);
            next unless defined $publisher_id;

            my $event_state_id = $self->get_event_state_id($action_penalty_node, $publisher_id);
            next unless defined $event_state_id;
            
            $event_action_penalty_entity->store_from_doc($event_state_id, $action_penalty_node, $publisher_id);
        }
        foreach my $action_substitution_node (@action_substitution_nodes) {
            my $publisher_id = $self->get_publisher_id($action_substitution_node);
            next unless defined $publisher_id;

            my $event_state_id = $self->get_event_state_id($action_substitution_node, $publisher_id);
            next unless defined $event_state_id;
            
            $event_action_substitution_entity->store_from_doc($event_state_id, $action_substitution_node, $publisher_id);
        }
    }

}


1;