# $Id: Ice_hockey_action_participant.pm 3020 2013-03-26 17:33:41Z ian $

package FeedfetcherDeluxe::Entities::Ice_hockey_action_participant;

use vars qw(@ISA);
use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Ice_hockey_action_play;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Team;

#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Ice_hockey_action_participant

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;
}

sub handle_store {
    my ($self, $node, $play_id, $player_idref, $role, $point_credit, $publisher_id) = @_;
	
    return undef unless (defined $player_idref);
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
	my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
    if (! $person_id) {
        $self->{log}->log("[ADV] ice_hockey_action_participant add failure: no player information found in document for player-idref '$player_idref'");             
        return;
    }
	
	my $goals_cumulative = attribute($node, 'goals-cumulative');
	my $assists_cumulative = attribute($node, 'assists-cumulative');

	my $team_idref = attribute($node, 'team-idref');
	my $team_id;
    # Make an extra effort to find the team for a player if that attribute has not been attached to the node
    if (! defined $team_idref) {
        $team_idref = $person_entity->find_team_idref_from_player_idref($node, $player_idref);
#        print "team_idref: $team_idref\n";
    }
	if (defined $team_idref) {
		my $team_entity = FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
		$team_id = $team_entity->find_id_from_idref($node, $team_idref, $publisher_id);
	}
    else {
        $self->{log}->log("[ADV] ice_hockey_action_participant add failure: no team information found in document for player-idref '$player_idref'");             
        return;
    }

    my $ice_hockey_action_participant_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new(
        $self->{dbh}, $self->{log}, 'ice_hockey_action_participants'
    );
    $ice_hockey_action_participant_record_mapper->retrieve(
        {
            'ice_hockey_action_play_id'  => $play_id,
            'person_id' => $person_id,
            'participant_role'=>$role
        }
    );
    $ice_hockey_action_participant_record_mapper->set(
        {
            'team_id' => $team_id,
            'participant_role'=>$role,
            'point_credit'=>$point_credit,
            'goals_cumulative'=>$goals_cumulative,
            'assists_cumulative'=>$assists_cumulative
        }
    );
    $ice_hockey_action_participant_record_mapper->save();

#    my $ice_hockey_action_play_id =  $self->retrieve_id_by_key(
#        {
#            'ice_hockey_action_play_id'  => $play_id,
#            'person_id' => $person_id,
#            'participant_role'=>$role
#        }
#    );
#    if (! $ice_hockey_action_play_id) {
#        $self->create(
#            {
#                'ice_hockey_action_play_id'  => $play_id,
#                'person_id' => $person_id,
#                'team_id' => $team_id,
#                'participant_role'=>$role,
#                'point_credit'=>$point_credit,
#                'goals_cumulative'=>$goals_cumulative,
#                'assists_cumulative'=>$assists_cumulative
#            }
#        );
#    }
#    else {
#        $self->update(
#        {
#                'team_id' => $team_id,
#                'participant_role'=>$role,
#                'point_credit'=>$point_credit,
#                'goals_cumulative'=>$goals_cumulative,
#                'assists_cumulative'=>$assists_cumulative
#        }
#        , {
#            'ice_hockey_action_play_id'  => $play_id,
#            'person_id' => $person_id
#        }
#        );
#   }
}

sub store_from_doc {
    my ($self, $node, $publisher_id, $play_id) = @_;

    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    
    # Get attributes to store
    my $player_idref = attribute($node, 'player-idref');
#    my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
#    if (! $person_id) {
#        $self->{log}->log("[ADV] ice_hockey_action_participant add failure: no player information found in document for player-idref '$player_idref'");    	    	
#        return;
#    }
    my $point_credit = (defined attribute($node, 'point-credit')) ? attribute($node, 'point-credit') : 0;
    my $role = attribute($node, 'role');   
    if ($node->nodeName eq 'action-ice-hockey-assisting-player') {
        $role = (defined attribute($node, 'role')) ? attribute($node, 'role') : 'assist';	
        $point_credit = 1;
    }
    elsif ($node->nodeName eq 'action-ice-hockey-participant') {
        $role =attribute($node, 'role');   
    }
    elsif ($node->nodeName eq 'action-ice-hockey-scored-on-players') {
        $role =attribute($node, 'role') || 'goaltender';   
    }

    $self->handle_store($node, $play_id, $player_idref, $role, $point_credit, $publisher_id);
}

# NOTE: Assumes proper doc context for node argument
sub store_all_from_doc {
    my ($self, $node, $publisher_id, $event_state_id) = @_;
    

    my $ice_hockey_action_play_entity = FeedfetcherDeluxe::Entities::Ice_hockey_action_play->new($self->{dbh}, $self->{log});
    my $play_id;
    if ($event_state_id) {
        $play_id = $ice_hockey_action_play_entity->retrieve_id_by_key({'ice_hockey_event_state_id'=>$event_state_id});
    }
    else {
        $play_id = $ice_hockey_action_play_entity->retrieve_max_id();        
    } 
    $play_id = 0 if ! defined $play_id;

    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});

    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
    my $action_listing = $sportsml_document->get_content_code_key('action-listing');
    
    if ($action_listing && $play_id) {
#    	print "clearing out participant\n";
         # Clear out participants in case any have disappeared and should not be there
        $self->delete({'ice_hockey_action_play_id'=>$play_id}) if $action_listing eq 'complete';
    }
 	
 	
	my @participant_nodes = $node->findnodes("child::action-ice-hockey-participant");
    # Try with older node name
 	if (scalar @participant_nodes eq '0') {
    	@participant_nodes = $node->findnodes("child::action-ice-hockey-play-participant");
 	}
    foreach my $participant_node (@participant_nodes) {
        $self->store_from_doc($participant_node, $publisher_id, $play_id);
    }

 	# Had used 'defined' test, but @participant_nodes is not undef, so failed to work
 	if (scalar @participant_nodes eq '0') {

	    my $player_idref;
	    my $person_id;
#	    $player_idref = attribute($node, 'player-idref');
#	    if (! defined $player_idref) {
#    		# For the score
#    		if ($node->nodeName eq 'action-ice-hockey-score') {
#        		$player_idref = attribute($node, 'player-idref');
#    		}
#    		# For the score attempt 
#   			elsif ($node->nodeName eq 'action-ice-hockey-score-attempt') {
#    			my $child_node = firstnode($node, "child::action-ice-hockey-participant[\@role='shooter']");
#        		$player_idref = attribute($child_node, 'player-idref');
#    		}
#    		elsif ($node->nodeName eq 'action-ice-hockey-penalty') {
#        		$player_idref = attribute($node, 'recipient-idref');
#    		}
#    	}
#    	$person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
#    my $test_nn = $node->nodeName;
#    $self->{log}->log("[DBG] player_idref: $player_idref publisher_id: $publisher_id node_name: $test_nn");               
#    if (! $person_id) {
#    	my $node_name = $node->nodeName;
#        $self->{log}->log("[ADV] node: $node_name ice_hockey_action_plays and ice_hockey_action_participants add failure: no player information found in document for player-idref '$player_idref'");               
#        return;
#    }
#		return unless (defined $person_id);

	    if ($node->nodeName eq 'action-ice-hockey-score') {
	    	
		    $player_idref = attribute($node, 'player-idref');
#    		$person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
	    	if (defined $player_idref) {
	    	    my $role = (defined attribute($node, 'role')) ? attribute($node, 'role') : 'shooter';
    	    	my $point_credit = (defined attribute($node, 'point-credit')) ? attribute($node, 'point-credit') : 1;
 
 		       $self->handle_store($node, $play_id, $player_idref, $role, $point_credit, $publisher_id);

		        my @participant_nodes = $node->findnodes("child::action-ice-hockey-assisting-player");
    		    foreach my $participant_node (@participant_nodes) {
        		    $self->store_from_doc($participant_node, $publisher_id, $play_id);
        		}
        		@participant_nodes = $node->findnodes("child::action-ice-hockey-scored-on-players");
        		foreach my $participant_node (@participant_nodes) {
            		$self->store_from_doc($participant_node, $publisher_id, $play_id);
        		}
	    	}
    	}
    	elsif ($node->nodeName eq 'action-ice-hockey-penalty') {
       		$player_idref = attribute($node, 'recipient-idref');
#			$person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);

	    	if (defined $player_idref) {
	       		$self->handle_store($node, $play_id, $player_idref, 'penalty-committed-by', 0, $publisher_id);
       			if (defined attribute($node, 'served-by-idref')) {
       				my $served_by_idref = $node->get_attribute('served-by-idref');
#			    	my $served_by_id = $person_entity->find_id_from_idref($node, attribute($node, 'served-by-idref'), $publisher_id);
	       			$self->handle_store($node, $play_id, $served_by_idref, 'penalty-served-by', 0, $publisher_id);
       			}
	    	}
    	}
 	}
 	

}

1;