# $Id: Injury_phase.pm 3070 2015-05-11 19:59:44Z ian $

package FeedfetcherDeluxe::Entities::Injury_phase;

use vars qw(@ISA);

use strict;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Season;
use Core::XmlUtil;
use Core::Util;
use Time::Local;
use Util::DateUtils;

=head1 NAME

FeedFetcherDeluxe::Entities::Injury_phase

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN { @ISA = qw ( FeedfetcherDeluxe::Entities::Entity ); }

sub setup_sql {
	my ($self) = @_;

	# Find records for a person with start_date_time NOT equal to arg and NULL end_date_time
	
	# Update records for a person, setting end_date_time for items having NULL end_date_time and
	# start_date_time NOT equal to arg
}

sub store_from_doc {
	my ( $self, $player_node, $person_id, $publisher_id ) = @_;

	my $injury_phase_node = solonode_optional( $player_node,
		'descendant::player-metadata/injury-phase' );

	return unless defined $injury_phase_node;

	my $season_entity =
	  FeedfetcherDeluxe::Entities::Season->new( $self->{dbh}, $self->{log} );

	my $sub_season_entity =
	  FeedfetcherDeluxe::Entities::Sub_season->new( $self->{dbh},
		$self->{log} );

	my ( $context, $date_coverage_type, $date_coverage_value,
		$date_coverage_value_id )
	  = $season_entity->get_season( $player_node, $publisher_id );

	my $injury_start_date = $injury_phase_node->getAttribute('start-date');

	#	$self->{log}->log("[DBG] injury_start_date: $injury_start_date");

	my $date_utils = Util::DateUtils->new();
#	my ( $m, $d, $y ) = split( /\//, $injury_start_date );
    my ( $y, $m, $d ) = $date_utils->get_UTC_date_parts($injury_start_date);
    
#    $self->{log}->log("[DBG] Year: $y Month: $m Day: $d");

	# Month is zero based in perl timelocal call being used by datutils, but
	# this offset is handled there
	my $injury_start_time =
	  $date_utils->get_time_from_date_parts( 0, 0, 0, $d, $m, $y );
	my $sql_injury_start_date = $date_utils->get_SQL_date($injury_start_time);

	my $sql_injury_end_date;

	my $injury_end_date = $injury_phase_node->getAttribute('end-date');
	my $injury_status;
	if ($injury_end_date) {
		$injury_status = "inactive";
#		my ( $m, $d, $y ) = split( /\//, $injury_end_date );
        my ( $y, $m, $d ) = $date_utils->get_UTC_date_parts($injury_end_date);
		# Month is zero based in perl timelocal call being used by datutils, but
		# this offset is handled there
		my $injury_end_time =
		  $date_utils->get_time_from_date_parts( 0, 0, 0, $d, $m, $y );
		my $sql_injury_end_date = $date_utils->get_SQL_date($injury_end_time);
	}
	else {
		$injury_status = "active";
	}
#	$self->{log}->log("[DBG] sql_injury_start_date: $sql_injury_start_date");

	my $injury_type = $injury_phase_node->getAttribute('injury-type');
	my $phase_type  = $injury_phase_node->getAttribute('phase-type');
	my $injury_side = $injury_phase_node->getAttribute('injury-side');
	my $injury_comment = $injury_phase_node->getAttribute('comment');

	my $season_id;
	if ( $date_coverage_type =~ /season/ ) {

	  #            $season_id = $date_coverage_value_id;
	  # The date_coverage_value_id is actually a sub_season id, not a season id.
	  # Therefore, we need to look up the sub_season by id to get its season id
		my $hashref =
		  $sub_season_entity->retrieve_by_id($date_coverage_value_id);
		$season_id = $hashref->{season_id};
	}

	# TODO Set previous injuries' end_date (i.e injuries for persons of same id but diff't start_date_time
	# and NULL end_date_times (queries defined in earlier local setup_sql sub)

	my $injury_phase_id = $self->retrieve_id_by_key(
		{
			'person_id' => $person_id,
			'injury_status' => $injury_status
#			'start_date_time' => $sql_injury_start_date
		}
	);
	if ( !$injury_phase_id ) {
		$self->create(
			{
				'person_id'       => $person_id,
				'season_id'       => $season_id,
				'start_date_time' => $sql_injury_start_date,
				'end_date_time'   => $sql_injury_end_date,
				'injury_status'   => $injury_status,
				'injury_type'     => $injury_type,
				'injury_side'     => $injury_side,
				'phase_type'      => $phase_type,
				'injury_comment'  => $injury_comment
			}
		);
	}
	else {
		# Only update active injuries for now
		if ($injury_status eq 'active') {
	   		# NOTE
   			# Could update by season id, but that does not seem to be reliably available.
   			# Therefore, allowing only one person id. (this may be outdated)
			$self->update(
				{
					'season_id'       => $season_id,
					'start_date_time' => $sql_injury_start_date,
					'end_date_time'   => $sql_injury_end_date,
					'injury_status'   => $injury_status,
					'injury_type'     => $injury_type,
					'injury_side'     => $injury_side,
					'phase_type'      => $phase_type,
					'injury_comment'  => $injury_comment
				},
				{ 'id' => $injury_phase_id }
			);
		}
	}
}

1;
