# $Id: Location.pm 3016 2013-01-31 23:20:40Z ian $

package FeedfetcherDeluxe::Entities::Location;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;
use Core::XmlUtil;

=head1 NAME

Core::Entities::Document

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN { @ISA = qw ( FeedfetcherDeluxe::Entities::Entity ); }

sub setup_sql {
	my ($self) = @_;
}

sub initialize_local {
	my ($self) = @_;

	$self->{location_mapper} =
	  FeedfetcherDeluxe::Entities::MetaRecordMapper->new( $self->{dbh},
		$self->{log}, 'locations' );
}

# Takes a node argument then calles store
sub store_from_doc {
	my ( $self, $node ) = @_;

	my $hash_ref = {
		'city'         => attribute( $node, 'city' ),
		'state'        => attribute( $node, 'state' ),
		'area'         => attribute( $node, 'area' ),
		'country'      => attribute( $node, 'country' ),
		'country_code' => attribute( $node, 'country-code' ),
		'latitude'     => attribute( $node, 'latitude' ),
		'longitude'    => attribute( $node, 'longitude' ),
		'timezone'     => attribute( $node, 'timezone' )
	};

	return $self->store($hash_ref);
}

# Requires a hash argument
sub store {
	my ( $self, $hash ) = @_;

	my $city         = $hash->{city};
	my $state        = $hash->{state};
	my $area         = $hash->{area};
	my $country      = $hash->{country};
	my $country_code = $hash->{country_code};

	my $hr    = {};
	my $found = 0;

    #14:33 ian: I could look for city and if I find that try for area or state to add to the lookup
    #14:34 ian: and if no city look for state or area and try to add country or country_code to that
    #14:34 ian: else just use country

	# Look for city then try to add area or state
	if ( defined $city && $city !~ /^\s*$/ )
	{
		#		$self->{log}->log("[DBG] Could not save location; city undefined");
		$found = 1;
		$hr->{'city'} = $city;
		if ( defined $state && $state !~ /^\s*$/ ) {
			$hr->{state} = $state;
		}
		elsif ( defined $area && $area !~ /^\s*$/ ) {
			$hr->{area} = $area;
		}
	}
	# If no luck with city then try state
    elsif ( defined $state && $state !~ /^\s*$/ ) {
        $found = 1;
        # Set fields that should not be there but would otherwise provide 
        # a result (e.g. state MI matching ANY city or a country matching 
        # ANY record also including a state and/or city) to null
        $hr->{city} = undef;
        
        $hr->{state} = $state;
        if ( defined $area && $area !~ /^\s*$/ ) {
            $hr->{area} = $area;      	
        }
        if ( defined $country && $country !~ /^\s*$/ ) {
            $found = 1;
            $hr->{country} = $country;
        }
        if ( defined $country_code && $country_code !~ /^\s*$/ ) {
            $found = 1;
            $hr->{country_code} = $country_code;
        }
    }
    # If no luck with city then try state
    elsif ( defined $area && $area !~ /^\s*$/ ) {
        $found = 1;
        # Set fields that should not be there but would otherwise provide 
        # a result (e.g. state MI matching ANY city or a country matching 
        # ANY record also including a state and/or city) to null
        $hr->{city} = undef;
        
        $hr->{area} = $area;
        if ( defined $country && $country !~ /^\s*$/ ) {
            $found = 1;
            $hr->{country} = $country;
        }
        if ( defined $country_code && $country_code !~ /^\s*$/ ) {
            $found = 1;
            $hr->{country_code} = $country_code;
        }
    }
    # If no luck with city or state/area try for country
	elsif ( defined $country && $country !~ /^\s*$/ ) {
		$found = 1;
        # Set fields that should not be there but would otherwise provide 
        # a result (e.g. state MI matching ANY city or a country matching 
        # ANY record also including a state and/or city) to null
        $hr->{city} = undef;
        $hr->{area} = undef;
        $hr->{state} = undef;
        
		$hr->{country} = $country;
        if ( defined $country_code && $country_code !~ /^\s*$/ ) {
            $found = 1;
            $hr->{country_code} = $country_code;
        }
	}
    # If no luck with city or state/area or country try for country_code
	elsif ( defined $country_code && $country_code !~ /^\s*$/ ) {
        # Set fields that should not be there but would otherwise provide 
        # a result (e.g. state MI matching ANY city or a country matching 
        # ANY record also including a state and/or city) to null
        $hr->{city} = undef;
        $hr->{area} = undef;
        $hr->{state} = undef;
        $hr->{country} = undef;

		$found = 1;
		$hr->{country_code} = $country_code;
	}
	unless ( $found eq "1" ) {
		return undef;
	}

	my $location_id;
	# Set flag to 1 to allow lookups with NULL for fields set to null to avoid
	# matching a location with just NY to another with, for example, Buffalo NY
	$self->{location_mapper}->retrieve($hr, 1);
	my $can_save = $self->{location_mapper}->can_save();
	
	if ( $self->{location_mapper}->can_save() ) {
		$self->{location_mapper}->set(
			{
				'latitude'    => $hash->{latitude},
				'longitude'    => $hash->{longitude},
				'timezone'     => $hash->{timezone},
				'country'      => $hash->{country},
				'country_code' => $hash->{country_code}
			}
		);
	}
	$location_id = $self->{location_mapper}->save();
#    print "location id: ".$location_id."\n";

	return $location_id;
}

1;
