# $Id: Participants_event.pm 3023 2013-09-11 17:44:52Z ian $

package FeedfetcherDeluxe::Entities::Participants_event;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Period;
use FeedfetcherDeluxe::Entities::Event;


#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Participants_event

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );}

sub setup_sql {
    my ($self) = @_;

    $self->{'retrieve_ids_by_participant_type'} = $self->{dbh}->prepare(qq{
        SELECT id
        FROM participants_events
        WHERE event_id=?
        AND participant_type=?
    });
    $self->{'delete_subscore_score'} = $self->{dbh}->prepare(qq{
        DELETE
        FROM periods
        WHERE participant_event_id=?
        AND period_value=?
        AND (score_attempts IS NULL OR score_attempts = 0)
    });
    $self->{'delete_subscore_score_attempt'} = $self->{dbh}->prepare(qq{
        DELETE
        FROM periods
        WHERE participant_event_id=?
        AND period_value=?
        AND (score_attempts IS NOT NULL AND score_attempts != 0)
    });


}

sub delete_periods {
    my ($self, $event_id, $participant_type) = @_;

     my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});

    $self->{'retrieve_ids_by_participant_type'}->execute($event_id, $participant_type);

     while (my $row = $self->{'retrieve_ids_by_participant_type'}->fetchrow_hashref()) {
     	my $id = $row->{id};
#     	$self->{log}->log("[DBG] deleting periods for participant_event_id: $id");
        $period_entity->delete({'participant_event_id'=>$id});
     }
    $self->{'retrieve_ids_by_participant_type'}->finish;
}

sub find_participants_event_id {
    my ($self, $node, $publisher_id) = @_;

    my $event_entity =
      FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );

	my $event_id = $event_entity->find_event_id($node, $publisher_id);
#	$self->{log}->log("[DBG] event_id: $event_id");
	
	return undef if (! defined $event_id);
	my $participant_node;
	
	if ($node->nodeName eq 'team' || $node->nodeName eq 'player') {
		$participant_node = $node;
	}
	else {
		my $player_node = firstnode($node, "ancestor::player");
		my $player_node_count = $player_node->findvalue("count(preceding::*)");
		my $team_node = firstnode($node, "ancestor::team");
		my $team_node_count = $team_node->findvalue("count(preceding::*)");
#	$self->{log}->log("[DBG] player count: $player_node_count team count: $team_node_count");
	
		$participant_node = ($player_node_count > $team_node_count) ? $player_node: $team_node;
	}
#	$self->{log}->log("[DBG] node: $participant_node");
	return undef unless (defined $participant_node);
	
	my $participant_type = ($participant_node->nodeName eq 'player') ? 'persons': 'teams';

	my $participant_entity;
	my $alignment;
	if ($participant_type eq 'persons') {
		$participant_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
	}
	else {
		$participant_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
	}
	my $participant_key;
	my $key_name = $participant_node->nodeName."-key";
	my $key_column_name = ($participant_type eq 'persons') ? "person_key" : "team_key";
	my $metadata_node = firstnode($participant_node, "child::".$participant_node->nodeName."-metadata");
	$participant_key = $metadata_node->getAttribute($key_name);
	
	if ($participant_type eq 'teams') {
		$alignment = $metadata_node->getAttribute("alignment");
	}
	
	my $participant_id = $participant_entity->retrieve_id_by_key({$key_column_name=>$participant_key, 'publisher_id'=>$publisher_id});
	
#	$self->{log}->log("[DBG] key name: $key_name participant id: $participant_id key: ${participant_key}");
	
	my $id = $self->retrieve_id_by_key({'participant_type'=>$participant_type, 'participant_id'=>$participant_id, 'event_id'=>$event_id});
	
	if (! defined $id) {
		$id = $self->create(
        {
            'participant_type' => $participant_type,
            'participant_id'=>$participant_id,
            'event_id'=>$event_id, 
            'alignment'=>$alignment #,
#            'score'=>$score,
#            'event_outcome'=>$event_outcome,
#            'rank' => $rank,
#            'result_effect'=>$result_effect
        }
		);
	}
	
	return $id;
}

sub  store_from_doc {
    my ( $self, $participant_type, $node, $event_id, $publisher_id ) = @_;


    my $team_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});
    my $participant_id;
    my $participant_event_id;

    my $participant_root = ($participant_type eq 'teams') ? 'team': 'player';

    my $metadata_node = solonode($node, $participant_root."-metadata");
    my $participant_key = attribute($metadata_node, $participant_root."-key");
    
    # Just in case, if there's no key, get out.
    return $participant_id if (! $participant_key || $participant_key eq '');
    
#    print "participant type: $participant_type\n";
    if ($participant_type eq 'teams') {
        $participant_id = $team_entity->retrieve_id_by_key({'team_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }

    if ($participant_type eq 'players') {
        $participant_id = $person_entity->retrieve_id_by_key({'person_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }


    my $alignment = attribute($metadata_node, 'alignment');

    my $stats_node = firstnode($node, $participant_root."-stats");
    my $sort_order;
    my $score_type;

    $participant_event_id = $self->retrieve_id_by_key(
        {
            'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
            'participant_id'=>$participant_id,
            'event_id' => $event_id
        }
    );

    my $score;
    my $score_attempts;
    my $event_outcome;
    my $rank;
    my $result_effect;
    
    my $rank_node = solonode_optional($node, $participant_root."-stats/rank");
    if (defined $rank_node) {
    	$rank = attribute($rank_node, "value");
    	$rank = ($rank =~ /^\s*$/) ? undef: $rank;
        # Avoid things like "DQ" (disqualified) from messing things up
        $rank = ($rank =~ /^\d+$/) ? $rank: undef;
    }    

    if ($stats_node) {
        $score = $stats_node->getAttribute('score');
        $score_attempts = $stats_node->getAttribute('score-attempts');
        $event_outcome = $stats_node->getAttribute('event-outcome');
        $result_effect = $stats_node->getAttribute('result-effect');
        $sort_order = $stats_node->getAttribute('xts:sort-order');
        $score_type = $stats_node->getAttribute('xts:score-type') || $stats_node->getAttribute('score-type');
    }

    if (! defined $participant_event_id) {
    	if (defined $participant_id && defined $event_id) {
#    	$self->{log}->log("[DBG] participant_id, event_id, event_outcome, alignment, score, participant_type, rank: $participant_id, $event_id, $event_outcome, $alignment, $score, $participant_type, $rank");
    	
          $participant_event_id = $self->create(
          {
              'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
              'participant_id'=>$participant_id,
              'event_id'=>$event_id,
              'alignment'=>$alignment,
              'score'=>$score,
              'score_attempts'=>$score_attempts,
              'event_outcome'=>$event_outcome,
              'rank' => $rank,
              'result_effect'=>$result_effect,
              'sort_order'=>$sort_order,
              'score_type'=>$score_type
          });
    	}
    }
    else {
        my $hash_ref = {};
        $hash_ref->{alignment} = $alignment if (defined $alignment);
#    	my $hash_ref = {
#            'alignment'=>$alignment
#    	};
   		$hash_ref->{score} = $score if (defined $score);
   		$hash_ref->{score_attempts} = $score_attempts if (defined $score_attempts);
   		$hash_ref->{event_outcome} = $event_outcome if (defined $event_outcome);
   		# Allow blank ranks to overwrite since disqualified players can lose ranks and this
   		# needs to be reflected in table.
   		$hash_ref->{rank} = $rank if (defined $rank);
   		$hash_ref->{result_effect} = $result_effect if (defined $result_effect);
        $hash_ref->{sort_order} = $sort_order if (defined $sort_order);
        $hash_ref->{score_type} = $score_type if (defined $score_type);

        $self->update_allow_null_values(
	        $hash_ref,
#        {
#            'alignment'=>$alignment,
#            'score'=>$score,
#            'event_outcome'=>$event_outcome,
#            'rank' => $rank,
#            'result_effect'=>$result_effect
#        },
        {
            'id'=>$participant_event_id,
            'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
        }
        ) if (defined $alignment || defined $score || defined $event_outcome || defined $rank);
    }
    
    # Don't proceed unless stats are available to process
#    return $participant_event_id unless defined $stats_node;
    return $participant_id unless defined $stats_node;


    my @subscore_nodes = nodes($stats_node, 'sub-score');
    my @subscore_attempts = nodes($stats_node, 'sub-score-attempts');
    # Merge subscore array with attempts array, since the data all goes in same table
    @subscore_nodes = (@subscore_nodes, @subscore_attempts);
#$self->{log}->log("[DBG] subscore_nodes: @subscore_nodes, participant_type: $participant_type");

#    return $participant_id unless (@subscore_nodes && ($participant_type eq 'teams'));
    return $participant_id unless (@subscore_nodes);

    foreach my $subscore_node (@subscore_nodes) {
        my $period_value = $subscore_node->getAttribute('period-value');

        my $score = $subscore_node->getAttribute('score');
        my $sub_score_key = $subscore_node->getAttribute('sub-score-key');
        my $sub_score_type = $subscore_node->getAttribute('sub-score-type');
        my $sub_score_name = $subscore_node->getAttribute('sub-score-name');
		$score = undef if ($score =~ /^\s*$/);
        my $score_attempts = $subscore_node->getAttribute('score-attempts');
		$score_attempts = undef if ($score_attempts =~ /^\s*$/);
        my $label = $subscore_node->getAttribute('label');
		$label = undef if ($label =~ /^\s*$/);

        # sub-score-attemps nodes have a score-attempts attribute, but subscore nodes don't
        $period_entity->store_period( $participant_event_id, $period_value, $score, $score_attempts, $sub_score_key, $sub_score_type, $sub_score_name );
            
    }
    return $participant_id;
}


sub store_all_from_doc {
    my ( $self, $doc, $publisher_id ) = @_;

    my $sportsml_document =
      FeedfetcherDeluxe::Entities::SportsmlDocument->new( $doc,
        $self->{log} );
    
    my $sports_metadata_node = solonode_optional($doc, "//sports-content/sports-metadata");
    my $fixture_key = $sports_metadata_node->getAttribute('fixture-key');
    my $skip_delete = ("$fixture_key" =~ /(medallists-event|medallists-discipline|medal-standings|record)/) ? "1": "0";
    if ($skip_delete eq '0') {
    	my $code_key = $sportsml_document->get_content_code_key('participant-event-listing');
    	if (defined $code_key) {
            $skip_delete = ($code_key eq 'partial') ? "1": "0";
    	}
    }
#    print "skip_delete: $skip_delete\n";
    

    my $event_entity =
      FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );

    my $event_id;
    my $prev_event_id = 0;
    my $participant_id = 0;
    my @participant_id_list = ();
    my $list_count = 0;
    my $participant_id_list_str;

#    my @player_subscores = $doc->findnodes("//sports-event/player/player-stats/sub-score");
#    my @player_subscore_attempts = $doc->findnodes("//sports-event/player/player-stats/sub-score-attempts");
#    
#    my @player_subscore_set = (@player_subscores, @player_subscore_attempts);
#    
#    my @team_subscores = $doc->findnodes("//sports-event/team/team-stats/sub-score");
#    my @team_subscore_attempts = $doc->findnodes("//sports-event/team/team-stats/sub-score-attempts");
#
#    my @team_subscore_set = (@team_subscores, @team_subscore_attempts);

#    my @player_nodes = $doc->findnodes("//sports-event/player");
#    my @team_nodes = $doc->findnodes("//sports-event/team");

    # Get list of event-metadata nodes, to be able to compile a list of event keys for the document
    my @event_metadata_nodes = $doc->findnodes("//sports-event/event-metadata");

    my %event_key_hash = ();

    # Compile the list of event keys
    foreach my $event_metadata_node (@event_metadata_nodes) {
        my $event_key = $event_metadata_node->getAttribute('event-key');
        next unless defined $event_key;
        next if $event_key =~ /^\s*$/;
        $event_key_hash{$event_key} = $event_key;
    }
    my @event_key_list = keys %event_key_hash;
    
    # Go through the list of event keys, for each, finding players and teams, deleting entries
    # not in the list compiled from the database (participants_events table), and storing the ones found
    foreach my $event_key (@event_key_list) {    	
    	# Find the event id for the key
        $event_id = $event_entity->retrieve_id_by_key(
            {
                'event_key' => $event_key,
                'publisher_id' => $publisher_id
            }            
        );

        # Find all players that are children of the current event (wherever they are in the document), for each, storing in participants_events
        # table, and removing those not in that list (the call to delete_by_id_list)
        my @participant_nodes = $doc->findnodes("//sports-event[child::event-metadata[\@event-key='$event_key']]/descendant::player");
        @participant_id_list = ();
             
        foreach my $node (@participant_nodes) {
            $participant_id = $self->store_from_doc( 'players', $node, $event_id, $publisher_id );
            push(@participant_id_list, "$participant_id") unless (! defined($participant_id));
        }
        
        $list_count = @participant_id_list;
        if ($list_count > 0) {
            $participant_id_list_str = join(', ', @participant_id_list);
            $self->delete_by_id_list('players', $event_id, $participant_id_list_str) unless ($skip_delete eq '1');
        }

        # Find all teams that are children of the current event (wherever they are in the document), for each, storing in participants_events
        # table, and removing those not in that list (the call to delete_by_id_list)
        @participant_nodes = $doc->findnodes("//sports-event[child::event-metadata[\@event-key='$event_key']]/descendant::team");
        @participant_id_list = ();
             
        foreach my $node (@participant_nodes) {
            $participant_id = $self->store_from_doc( 'teams', $node, $event_id, $publisher_id );
            push(@participant_id_list, "$participant_id") unless (! defined($participant_id));
        }
        
        $list_count = @participant_id_list;
#        print "list_count: $list_count\n";
        if ($list_count > 0) {
            $participant_id_list_str = join(', ', @participant_id_list);
            $self->delete_by_id_list('teams', $event_id, $participant_id_list_str) unless ($skip_delete eq '1');
        }
        @participant_id_list = ();

    }
    
    # This is the old method of doing things that assumed that players and teams were grouped together under events. This is not always
    # the case (as first encountered in olympics data), and besides, this method has a more complicated event_id tracking mechanism since
    # it's following players and teams across events. THIS SECTION SHOULD BE DELETED ONCE THE CODE ABOVE IS FOUND TO WORK
#    foreach my $node (@player_nodes) {
#        my $event_metadata_node = solonode_optional($node, "ancestor::sports-event/event-metadata");
#        my $event_key = attribute($event_metadata_node, 'event-key');
#
#        $event_id = $event_entity->retrieve_id_by_key(
#            {
#                'event_key' => $event_key,
#                'publisher_id' => $publisher_id
#            }
#        );
#        
#        $event_id += 0;
#
#        if ($prev_event_id != $event_id) {
#            if ($prev_event_id != 0) {
#                $participant_id_list_str = join(', ', @participant_id_list);
#                $self->delete_by_id_list('players', $prev_event_id, $participant_id_list_str);
#            }
#            $prev_event_id = $event_id;
#            @participant_id_list = ();
#        }
#
#        $participant_id = $self->store_from_doc( 'players', $node, $event_id, $publisher_id );
#        # store_from_doc returns undef if no valid key found, so don't add what will become an empty string to the array
#        push(@participant_id_list, "$participant_id") unless (! defined($participant_id));
#    }
#    $list_count = @participant_id_list;
#    if ($list_count > 0) {
#        $participant_id_list_str = join(', ', @participant_id_list);
#        $self->delete_by_id_list('players', $prev_event_id, $participant_id_list_str);
#    }
#
#    $prev_event_id = 0;
#    $participant_id = 0;
#    @participant_id_list = ();
#    $participant_id_list_str = '';
#    $list_count = 0;
#
#    foreach my $node (@team_nodes) {
#        my $event_metadata_node = solonode_optional($node, "ancestor::sports-event/event-metadata[position()=last()]");
#        my $event_key = attribute($event_metadata_node, 'event-key');
#
#        $event_id = $event_entity->retrieve_id_by_key(
#            {
#                'event_key' => $event_key,
#                'publisher_id' => $publisher_id
#            }
#        );
#
#        $event_id += 0;
#
#        if ($prev_event_id != $event_id) {
#        	if ($prev_event_id != 0) {
#                $participant_id_list_str = join(',', @participant_id_list);
#                $self->delete_by_id_list('teams', $prev_event_id, $participant_id_list_str);
#        	}
#        	$prev_event_id = $event_id;
#        	@participant_id_list = ();
#        }
#
#       $participant_id = $self->store_from_doc( 'teams', $node, $event_id, $publisher_id );
##$self->{log}->log("[DBG] participant_id: $participant_id");
#        # store_from_doc returns undef if no valid key found, so don't add what will become an empty string to the array
#        push(@participant_id_list, "$participant_id") unless (! defined($participant_id));
#    }
#    $list_count = @participant_id_list;
#    if ($list_count > 0) {
#        $participant_id_list_str = join(', ', @participant_id_list);
#        $self->delete_by_id_list('teams', $prev_event_id, $participant_id_list_str);
#    }

}

sub delete_by_id_list {
    my ( $self, $participant_type, $event_id, $id_str ) = @_;

#$self->{log}->log("[DBG] deleting for event_id: $event_id id_str: $id_str");

    $participant_type = ($participant_type eq 'players') ? 'persons' : $participant_type;
     
     my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});

    # return;
    
    my $id;
    my $select_sql = qq{
        SELECT id FROM participants_events
        WHERE participant_type=?
        AND event_id=?
        AND participant_id NOT IN ($id_str)    	
    };

#    my $deleted = undef;
    
    my $select_sth = $self->{dbh}->prepare($select_sql);
    $select_sth->execute($participant_type, $event_id);    
#    $select_sth->bind_columns(\$id);
    
    while (my $data = $select_sth->fetchrow_hashref()) {
    	$id = $data->{id};
      	next unless defined $id;
    	# Needed to do this because of what must be a new foreign key constraint whereby
    	# periods depend on participants_events, so deleting participants_events before
    	# periods is vorboten
    	$period_entity->delete(
    	{
    		'participant_event_id'=>$id
    	}
    	);
    	$self->delete(
    	{
    		'id'=>$id
    	}
    	);

#       if ( ! defined $deleted ) {
            $self->{log}->log("[DBG] An extra $participant_type participant was found for event_id $event_id");
#           $deleted = 1;
#        )
    	
    }
    $select_sth->finish;
}

1;