# $Id: Period.pm 2659 2009-11-30 18:40:42Z ian $

package FeedfetcherDeluxe::Entities::Period;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Participants_event;
#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Period

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

    $self->{'delete_subscore_score'} = $self->{dbh}->prepare(qq{
        DELETE
        FROM periods
        WHERE participant_event_id=?
        AND period_value=?
        AND (score_attempts IS NULL OR score_attempts = 0)
    });
    $self->{'delete_subscore_score_attempt'} = $self->{dbh}->prepare(qq{
        DELETE
        FROM periods
        WHERE participant_event_id=?
        AND period_value=?
        AND (score_attempts IS NOT NULL AND score_attempts != 0)
    });
}

sub find_period_id {
	my ($self, $node, $publisher_id) = @_;
	
	my $node_name = $node->nodeName;	
	my $team_node = firstnode($node, "ancestor::team");
	return undef unless (defined $team_node);
	my $period_value = $node->getAttribute('period-value');
	if (! defined $period_value) {
		$period_value = $node->getAttribute('date-coverage-value');
	}
#	print "period_value not found for node name: node_name\n" unless (defined $period_value);
	return undef unless defined $period_value;
	
	my $participants_event_entity = FeedfetcherDeluxe::Entities::Participants_event->new($self->{dbh}, $self->{log});
	
	my $participant_event_id = $participants_event_entity->find_participants_event_id($team_node, $publisher_id);

#	$self->{log}->log("[DBG] period_value: $period_value node name: $node_name participant_event_id: $participant_event_id");
	return undef unless defined $participant_event_id;

	my $id = $self->store_period($participant_event_id, $period_value, undef, undef, undef, undef, undef);
		
	return $id;
}

sub store_period {
    my ( $self, $participant_event_id, $period_value, $score, $score_attempts, $sub_score_key, $sub_score_type, $sub_score_name ) = @_;
    
#    $score = (defined $score) ? $score : '0';
	$score_attempts = (defined $score_attempts) ? $score_attempts : undef;
#    $label = '' unless defined $label;
#    $self->{'delete_subscore_score'}->execute($participant_event_id, $period_value);
 
    my $lookup_hash = {
        'participant_event_id'=>$participant_event_id
    };
    
    if (defined $sub_score_key) {
        $lookup_hash->{sub_score_key} = $sub_score_key;
    }
    elsif ((defined $period_value) && (defined $sub_score_type)) {
        $lookup_hash->{period_value} = $period_value;
        $lookup_hash->{sub_score_type} = $sub_score_type;
    }
    elsif (defined $period_value) {
        $lookup_hash->{period_value} = $period_value;
    }
    elsif (defined $sub_score_type) {
        $lookup_hash->{sub_score_type} = $sub_score_type;
    }
    else {
        return undef;
    }
#        'participant_event_id'=>$participant_event_id,
#        'period_value'=>$period_value
#    };

 	my $id = $self->retrieve_id_by_key(
        $lookup_hash
    );
#    my $count = $self->retrieve_row_count(
#    {
#         'participant_event_id'=>$participant_event_id,
#         'period_value'=>$period_value
#    }
#    );

#    $self->{log}->log("[DBG] id: $id");

    if ( ! defined $id ) {
    	my $hash_ref = {
            'participant_event_id'=>$participant_event_id,
            'period_value'=>$period_value
    	};
        $hash_ref->{score} = $score if (defined $score);
        $hash_ref->{score_attempts} = $score_attempts if (defined $score_attempts);
        $hash_ref->{sub_score_key} = $sub_score_key if (defined $sub_score_key);
        $hash_ref->{sub_score_type} = $sub_score_type if (defined $sub_score_type);
        $hash_ref->{sub_score_name} = $sub_score_name if (defined $sub_score_name);
        $id = $self->create($hash_ref);
#        {
#            'participant_event_id'=>$participant_event_id,
#            'period_value'=>$period_value,
#            'score'=>$score,
#            'label'=>$label,
#            'score_attempts'=>$score_attempts
#        }
#        );
    }
    else {
    	if ( (defined $score) || (defined $score_attempts) || (defined $label) ) {
	    	my $hash_ref = {};
    		$hash_ref->{score} = $score if (defined $score);
    		$hash_ref->{score_attempts} = $score_attempts if (defined $score_attempts);
#	    	}
#			print "score: $score hashref: ".$hash_ref->{score}." score_attempts: $score_attempts hashref: ".$hash_ref->{score_attempts}."\n";
#    		$hash_ref->{score} = $score if (defined $score);
            $hash_ref->{sub_score_key} = $sub_score_key if (defined $sub_score_key);
            $hash_ref->{sub_score_type} = $sub_score_type if (defined $sub_score_type);
            $hash_ref->{sub_score_name} = $sub_score_name if (defined $sub_score_name);
	        $self->update(
#    	    {
#            	'score'=>$score,
#            	'score_attempts'=>$score_attempts,
#            	'label'=>$label
#        	},
                $hash_ref,
                $lookup_hash
#        	{
#            	'participant_event_id'=>$participant_event_id,
#            	'period_value'=>$period_value,
#        	}
        	);
    	}
    }
 	return $id;
}

# TO DELETE
# Handled now by store_period
sub store_score {
    my ( $self, $participant_event_id, $period_value, $score, $label ) = @_;
    
#    $self->{'delete_subscore_score'}->execute($participant_event_id, $period_value);
 
    my $count = $self->retrieve_row_count(
    {
         'participant_event_id'=>$participant_event_id,
         'period_value'=>$period_value
    }
    );

#    $self->{log}->log("[DBG] count: $count");

    if ( $count == 0 ) {
        $self->create(
        {
            'participant_event_id'=>$participant_event_id,
            'period_value'=>$period_value,
            'score'=>$score,
            'label'=>$label,
            'score_attempts'=>'0'
        }
        );
    }
    else {
        $self->update(
        {
            'score'=>$score,
            'label'=>$label
        },
        {
            'participant_event_id'=>$participant_event_id,
            'period_value'=>$period_value,
        }
        );
    }
 
}

# TO DELETE
# Handled now by store_period
sub store_score_attempts {
    my ( $self, $participant_event_id, $period_value, $score_attempts, $label ) = @_;
    
#    $self->{'delete_subscore_score_attempt'}->execute($participant_event_id, $period_value);

     my $count = $self->retrieve_row_count(
    {
         'participant_event_id'=>$participant_event_id,
         'period_value'=>$period_value
    }
    );

#    $self->{log}->log("[DBG] count: $count");


    if ( $count == 0 ) {
        $self->create(
        {
            'participant_event_id'=>$participant_event_id,
            'period_value'=>$period_value,
            'score'=>0,
            'label'=>$label,
            'score_attempts'=>$score_attempts
        }
        );
    }
    else {
        $self->update(
        {
            'score_attempts'=>$score_attempts,
            'label'=>$label
        },
        {
            'participant_event_id'=>$participant_event_id,
            'period_value'=>$period_value,
        }
        );
    }

#    $self->create(
#    {
#      'participant_event_id'=>$participant_event_id,
#      'period_value'=>$period_value,
#      'score'=>'0',
#      'label'=>$label,
#      'score_attempts'=>$score_attempts
#    }
#    );
}

1;