# $Id: Person.pm 3095 2016-04-15 14:54:00Z ian $

package FeedfetcherDeluxe::Entities::Person;

use vars qw(@ISA);
use strict;

use Core::Util;
use Time::Local;
use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::Position;
use FeedfetcherDeluxe::Entities::Season;
use FeedfetcherDeluxe::Entities::Sub_season;
use FeedfetcherDeluxe::Entities::Event;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Display_name;
use FeedfetcherDeluxe::Entities::Person_phase;
use FeedfetcherDeluxe::Entities::Person_event_metadata;
use FeedfetcherDeluxe::Entities::Injury_phase;
use FeedfetcherDeluxe::Entities::Core_person_stat;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Location;
use Util::SharedData;
use Util::DateUtils;
use Conf::ActiveFixtures;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;
use FeedfetcherDeluxe::Entities::Sports_property;

#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

Core::Entities::Person

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN { @ISA = qw ( FeedfetcherDeluxe::Entities::Entity ); }

sub initialize_local {
	my ($self) = @_;
	
#	my $test = $self->{shared_data}->get('cfg');
#	print "push mode:".$self->{cfg}->param('push_mode')."\n";
	$self->{teams} = ();
	$self->{person_phase_members} = ();
#	my %hash   = map { $_, 1 } @array;
    $self->{person_record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'persons');
    $self->{display_name_record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'display_names');
    $self->{affiliation_record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'affiliations');
    
}

sub setup_sql {
	my ($self) = @_;
	# Find persons not in list who are on a team and are in injuries table
	# and have a NULL injury end dateand set their injury end dates to this
	# docs date
}


sub clear_person_phase_members {
	my ($self, $person_id) = @_;
	
}

sub add_person_phase_member {
	my ($self, $person_id) = @_;
	
}

sub update_person_phase_statuses {
	my ($self, $team_id) = @_;
	
}


sub store_from_doc {
	my ( $self, $player_node, $publisher_id ) = @_;

	# TODO!!!
	# Store persons_documents too

	my $person_id;
	my $date_utils = Util::DateUtils->new();
	

	my $sportsml_document =
	  FeedfetcherDeluxe::Entities::SportsmlDocument->new( $player_node,
		$self->{log} );
	my $affiliation_entity =
	  FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh},
		$self->{log} );
	my $season_entity =
	  FeedfetcherDeluxe::Entities::Season->new( $self->{dbh}, $self->{log} );
	my $sub_season_entity =
	  FeedfetcherDeluxe::Entities::Sub_season->new( $self->{dbh},
		$self->{log} );
	my $team_entity =
	  FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
	my $position_entity =
	  FeedfetcherDeluxe::Entities::Position->new( $self->{dbh}, $self->{log} );
	my $display_name_entity =
	  FeedfetcherDeluxe::Entities::Display_name->new( $self->{dbh},
		$self->{log} );
	my $person_phase_entity =
	  FeedfetcherDeluxe::Entities::Person_phase->new( $self->{dbh},
		$self->{log} );
	my $event_entity =
	  FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );
	my $person_event_metadata_entity =
	  FeedfetcherDeluxe::Entities::Person_event_metadata->new( $self->{dbh},
		$self->{log} );
	my $injury_phase_entity =
	  FeedfetcherDeluxe::Entities::Injury_phase->new( $self->{dbh},
		$self->{log} );
	my $core_person_stat_entity =
	  FeedfetcherDeluxe::Entities::Core_person_stat->new( $self->{dbh},
		$self->{log} );

	# Call to store all teams first so they'll be available
	# TODO This should be done elsewhere once and for all, not here.
	#	$team_entity->store_all_from_doc( $player_node, $publisher_id );
	my $event_key;
	my $event_status;

# TODO Ensure that this is the proper way to find events for all docs (i.e. including for docs with multiple events)
# Look up event id
	my $event_metadata_node = firstnode( $player_node, "ancestor::sports-event[1]/event-metadata" );

	my $sport_code_key = $sportsml_document->get_content_code_key('sport');

	# $self->{log}->log("[DBG] sport_code_key: $sport_code_key");

	if ($event_metadata_node) {
		$event_status = attribute( $event_metadata_node, 'event-status' );
		$event_key    = attribute( $event_metadata_node, 'event-key' );
	}
	my $event_id;
	if ( defined $event_key ) {
		$event_id = $event_entity->retrieve_id_by_key(
			{ 'event_key' => $event_key, 'publisher_id' => $publisher_id } );
	}

	# Not used. Leaving for now, but delete later.
#	my ( $context, $date_coverage_type, $date_coverage_value,
#		$date_coverage_value_id )
#	  = $season_entity->get_season( $player_node, $publisher_id );

	my $player_id;
	my $team_meta;
	my $team_key;
	my $team_id;
	my $affiliation_id;
    my $home_location_id;
	my $team_node = solonode_optional( $player_node, 'parent::team' );
	$team_meta = solonode_optional( $team_node, 'team-metadata' ) if $team_node;
    my $home_location_node = solonode_optional( $player_node, 'player-metadata/home-location' );
    if (defined $home_location_node) {
    	my $location_entity = FeedfetcherDeluxe::Entities::Location->new($self->{dbh}, $self->{log});
        $home_location_id = $location_entity->store_from_doc($home_location_node);
    }

	my $player_metadata = solonode_optional( $player_node, 'player-metadata' );
	
	
#	my $weight = $player_metadata->getAttribute('weight');
	return if (! defined $player_metadata);

	my $player_status = $player_metadata->getAttribute('status');
	my $person_gender = $player_metadata->getAttribute('gender');

	# Used later with person_event_metadata
	$team_key = attribute( $team_meta, 'team-key' ) if $team_meta;

	my $player_key = $player_metadata->getAttribute('player-key');

#    $self->{log}->log("[DBG] event_key: $event_key player_key: $player_key");

# If there is no player key, which happens sometimes, get out of this sub call.
# This avoids a condition whereby a person record with a blank key gets created.
	return undef if !$player_key;

	my $position_key = $player_metadata->getAttribute('position-event');
	$position_key = $player_metadata->getAttribute('position-regular')
	  if !$position_key;
	my $position_depth = $player_metadata->getAttribute('position-depth');
#	$position_key = undef if (! $position_key || $position_key =~ /^\s*$/);
	
    my $weight = $player_metadata->getAttribute('weight');
	my $height = $player_metadata->getAttribute('height');
	my $person_phase_status = $player_metadata->getAttribute('status');
	my $person_date_of_birth = $player_metadata->getAttribute('date-of-birth');
	my $health = $player_metadata->getAttribute('health');
	my $uniform_number = $player_metadata->getAttribute('uniform-number');
	my $lineup_slot = $player_metadata->getAttribute('lineup-slot');
	# Avoid things like "DQ" (disqualified) from messing things up
	$lineup_slot = ($lineup_slot =~ /^\d+$/) ? $lineup_slot : undef;
	my $lineup_slot_sequence = $player_metadata->getAttribute('lineup-slot-sequence');

	# next if !$player_key;
	my $name_node = solonode_optional( $player_metadata, 'name' );
    my $person_full_name;
    my ($first_name, $middle_name, $last_name, $full_name, $nickname);
	if (defined $name_node) {
        ( $first_name, $middle_name, $last_name, $full_name, $nickname ) =
	     $self->_get_name($name_node);


        if (defined $full_name) {
           $person_full_name = "$full_name";
        }
        elsif (defined $last_name) {
            $person_full_name = "$first_name $last_name";
        }
#        else {
#            $person_full_name = "unknown";
#        }
	}	
    $self->{person_record_mapper}->reset();
    
    $self->{person_record_mapper}->retrieve(
        { 
        'person_key' => $player_key, 
        'publisher_id' => $publisher_id 
        }    
    );

    if ($self->{person_record_mapper}->can_save()) {
        # person_key and publisher_id are already known to the mapper
        $self->{person_record_mapper}->set(
            {
            'birth_date'   => $person_date_of_birth,
            'gender' => $person_gender,
            'hometown_location_id' => $home_location_id
            }
        );
        $person_id = $self->{person_record_mapper}->save();
        
        my $sports_property_entity = FeedfetcherDeluxe::Entities::Sports_property->new($self->{dbh}, $self->{log});
        $sports_property_entity->process_all($player_metadata, 'persons', $person_id, $publisher_id);

    }
#   $person_id = $self->retrieve_id_by_key(
#       { 
#       'person_key' => $player_key, 
#       'publisher_id' => $publisher_id 
#       }
#    );
	# Insert new rows into persons and display_names
#	if ( !$person_id ) {
#		$person_id = $self->create(
#			{
#				'person_key'   => $player_key,
#				'publisher_id' => $publisher_id,
#				'birth_date'   => $person_date_of_birth,
#				'gender' => $person_gender
#			}
#		);
#	}
#    else {
#        if ($person_date_of_birth) {
#            $self->update(
#                {
#                    'birth_date'   => $person_date_of_birth,
#                    'gender' => $person_gender
#                }, 
#                {
#                    'id' =>     $person_id,
#                    'publisher_id' => $publisher_id
#                }
#            );
#        }
#    }

	$team_id = $self->find_team_id( $player_node, $publisher_id );
	if ( !$team_id ) {
		# find_team_id tries a few things to get a team_id, so if that doesn't work,
		# stop now and return undef.
		if (! defined $self->{person_team_warning}) {
			$self->{person_team_warning} = 1;
		}
		else {
			$self->{person_team_warning} += 1;
		}
		if ($self->{person_team_warning} < 4) {
    		$self->{log}->log("[ADV] No team key found for player. This may be appropriate for sport or data file.");
		}
		elsif ($self->{person_team_warning} == 4) {
			$self->{log}->log("[ADV] Suppressing further team-player advisements for this data file.");
		}
	}

	# fixit: add full code-name, to provide proper name of person
	#	my ( $first_name, $last_name ) =
	#	  $self->find_first_last_name( $player_node, 'player', $player_key );

    my $display_name_id;
    
#    my $display_name_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'display_names');
    $self->{display_name_record_mapper}->reset();
    $self->{display_name_record_mapper}->retrieve(
        {
            'entity_type' => 'persons',
            'entity_id'   => $person_id
        }
    );

    if ($self->{display_name_record_mapper}->can_save()) {
        # entity_type and entity_id are already known to the mapper
        $self->{display_name_record_mapper}->set(
            {
                'language'    => 'en-US',
                'first_name'  => $first_name,
                'last_name'   => $last_name,
                'full_name'   => $person_full_name,
                'alias'       => $nickname
            }
        );
        $display_name_id = $self->{display_name_record_mapper}->save();
    }

#	my $display_name_id = $display_name_entity->retrieve_id_by_key(
#		{
#			'entity_type' => 'persons',
#			'entity_id'   => $person_id
#		}
#	);
#	if ( !$display_name_id ) {
#
## Create display name record to store more detailed naming information about the league, division, etc.
#		$display_name_entity->create(
#			{
#				'entity_type' => 'persons',
#				'entity_id'   => $person_id,
#				'language'    => 'en-US',
#				'first_name'  => $first_name,
#				'last_name'   => $last_name,
#				'full_name'   => $person_full_name,
#				'alias'       => $nickname
#			}
#		);
#	}
#	else {
#
#		$display_name_entity->update(
#			{
#				'entity_type' => 'persons',
#				'entity_id'   => $person_id,
#				'language'    => 'en-US',
#				'first_name'  => $first_name,
#				'last_name'   => $last_name,
#				'full_name'   => $person_full_name,
#				'alias'       => $nickname
#			},
#			{ 'id' => $display_name_id }
#		);
#	}

#         $self->{log}->log("[DBG] name: $first_name $last_name $person_full_name");
#	}

	# TODO: Add support for storing roles
	# Need:
	# - to look up and substitute role_id for role? - see if there are any

    # TODO Replace this code with new subroutine get_position_id in position entities
	my $position_id;
	if (defined $position_key && ! ($position_key =~ /^\s*$/)) {

#        my $affiliation_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'affiliations');
        $self->{affiliation_record_mapper}->reset();
        $self->{affiliation_record_mapper}->retrieve(
            {
                'affiliation_key'  => $sport_code_key,
                'affiliation_type' => 'sport',
                'publisher_id'     => $publisher_id
            }
        );
        # If a record already exists no update will be done, otherwise an insert will be done using
        # the values used in the retrieve call
        $affiliation_id = $self->{affiliation_record_mapper}->save();
        
        
#		my $affiliation_id = $affiliation_entity->retrieve_id_by_key(
#			{
#				'affiliation_key'  => $sport_code_key,
#				'affiliation_type' => 'sport',
#				'publisher_id'     => $publisher_id
#			}
#		);

# TODO Check to see if this is unecessary - probably done much earlier in document processing when content codes are dealt with
#		if ( !$affiliation_id ) {
#			$affiliation_id = $affiliation_entity->create(
#				{
#					'affiliation_key'  => $sport_code_key,
#					'affiliation_type' => 'sport',
#					'publisher_id'     => $publisher_id
#				}
#			);

#		}
        my $position_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'positions');
        $position_record_mapper->retrieve(
            {
            'affiliation_id' => $affiliation_id,
            'abbreviation'   => $position_key
            }
        );
        # If a record already exists no update will be done, otherwise an insert will be done using
        # the values used in the retrieve call
        $position_id = $position_record_mapper->save();

#		$position_id = $position_entity->retrieve_id_by_key(
#			{
#				'affiliation_id' => $affiliation_id,
#				'abbreviation'   => $position_key
#			}
#		);
#		if ( !$position_id ) {
#			$position_id = $position_entity->create(
#				{
#					'affiliation_id' => $affiliation_id,
#					'abbreviation'   => $position_key
#				}
#			);
#		}
	}

	#	}

	# TODO Get role for saving
	my $role_id;
	my $person_phase_id;

	my $person_phase_node_test = firstnode( $player_metadata, 'career-phase[@phase-status]' );
	if ( defined $person_phase_node_test ) {
        my @career_phase_nodes = $player_metadata->findnodes('career-phase[@phase-status]');

		my $cleared_teams   = undef;
		my $cleared_leagues = undef;
        my @team_membership_list = ();
        my @affiliation_membership_list = ();
        
		foreach my $career_phase_node (@career_phase_nodes) {

			my $phase_membership_name = attribute( $career_phase_node, 'name' );
			my $phase_status = attribute( $career_phase_node, 'phase-status' );
			my $role_status  = attribute( $career_phase_node, 'role-status' );
            my $phase_type = attribute($career_phase_node, 'phase-type');
			my $subphase_type = attribute($career_phase_node, 'subphase-type');
			my $membership_type = 
			  attribute( $career_phase_node, 'phase-caliber' );
			$membership_type = "$membership_type" . 's';
			my $membership_key =
			  attribute( $career_phase_node, 'phase-caliber-key' );
#            print "membership_type: $membership_type membership_name: $phase_membership_name phase status: $phase_status\n";

            # Letting empty strings through can allow things like teams with empty string key values be created
            # That is not a good thing
			$membership_key = ( $membership_key eq '' ) ? undef : $membership_key; 
			
			next unless ( defined $membership_type && defined $membership_key );
            # Only process for teams and leagues
			next unless ( $membership_type eq 'teams' || $membership_type eq 'leagues' );

			my $regular_position_id;


            # Added 2016-04 after being added unofficially 2016-01 by alan
            my $start_date = attribute( $career_phase_node, 'start-date' );
            my $start_date_time = undef;

#            print "start_date: $start_date\n";

            # NOTE: below is code to use when there is more than just the year in the start-date value.
            #       Set start_date_time in this case in addition to start_date.
            #       Assumes ISO-8601 date and datetime format will be present.
            #       It _should_ work. Has been tested with a dummy date value for both date only and date-time.

            # Deal with year, month, and day. Note hard end-of-line marker to avoid catching
            # date-time values here.
            if ($start_date =~ /\d\d\d\d\-?\d\d\-?\d\d$/) {
                $start_date_time = $start_date." 00:00:00";
            }
            # Deal with full datetime.
            elsif ($start_date =~ /\d\d\d\d\-?\d\d\-?\d\dT?\d\d\:?\d\d\:?\d\d.*/) {
                $start_date_time = $date_utils->ISO8601_date2SQL_date($start_date);            
            }

			my $entry_reason = attribute( $career_phase_node, 'entry-reason' );
			my $exit_reason  = attribute( $career_phase_node, 'exit-reason' );
			my $selection_level = attribute( $career_phase_node, 'selection-level' );
			$selection_level =~ s/\D//g if defined $selection_level;
			my $selection_sublevel = attribute( $career_phase_node, 'selection-sublevel' );
			$selection_sublevel =~ s/\D//g if defined $selection_sublevel;
			my $selection_overall = attribute( $career_phase_node, 'selection-overall' );

            # Though redundant with person_metadata variables, these variables need to be here
            # in order that they might be set to null for phase status != 'active'  or 'invitee' 
			my $phase_position_id;
			my $phase_position_depth;
			my $phase_uniform_number;
			my $phase_height;
			my $phase_weight;
			my $phase_duration;

			if ( ( $phase_status eq 'active' || $phase_status eq 'invitee' || $phase_status eq 'injured' || $phase_status eq 'physically-unable' || $phase_status eq 'inactive' ) && $membership_type eq 'teams' )
			{
				$phase_position_id = $position_id;
				$phase_position_depth = $position_depth;
				$phase_uniform_number = $uniform_number;
#				$phase_height = $height;
#				$phase_weight = $weight;
				$phase_duration = attribute( $career_phase_node, 'duration' );
			}
            $phase_height = $height if defined $height;
            $phase_weight = $weight if defined $weight;
			

			my $membership_id;

                # Add current player key to roster list by team to allow later setting of those not
                # on list to 'inactive'.
                if ($self->{roster_listing} eq 'complete') {
                    # Prepare an array to hold player keys for team if it does not yet exist. Index
                    # by team key to make debugging easier.
                    if ($membership_type eq 'teams') {
                        unless (defined $self->{roster_by_team}->{$membership_key}) {
                            $self->{roster_by_team}->{$membership_key} = [];
                        }
                        # Key by team-key and player-key to allow easier debugging by values found in
                        # rosters rather than by db id
                        push(@{$self->{roster_by_team}->{$membership_key}}, $player_key);
                        my @list = @{$self->{roster_by_team}->{$membership_key}};
                    }
                    if ($membership_type eq 'leagues') {
                        unless (defined $self->{roster_by_league}->{$membership_key}) {
                            $self->{roster_by_league}->{$membership_key} = [];
                        }
                        # Key by team-key and player-key to allow easier debugging by values found in
                        # rosters rather than by db id
                        push(@{$self->{roster_by_league}->{$membership_key}}, $player_key);
                        my @list = @{$self->{roster_by_league}->{$membership_key}};
                    }
                }
                
            # Handle id retrieval for teams, including creation of placeholder person and display name records
			if ( $membership_type eq 'teams' ) {
				my $team_entity = FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
				$membership_id = $team_entity->retrieve_id_by_key(
					{
						'team_key'     => $membership_key,
						'publisher_id' => $publisher_id
					}
				);
			
		   # Shouldn't be necessary - remove once confident documents have teams
				if ( !defined $membership_id ) {
					$membership_id = $team_entity->create(
						{
							'team_key'     => $membership_key,
							'publisher_id' => $publisher_id
						}
					);
				}



				if ( $phase_membership_name !~ /^\s*$/ ) {
					my $display_name_initial_id =
					  $display_name_entity->retrieve_id_by_key(
						{
							'entity_type' => 'teams',
							'entity_id'   => $membership_id
						}
					  );

                    # Add a display name if there isn't one yet. It will get updated in a later document with
                    # more complete information.
					if ( !defined $display_name_initial_id ) {
						$display_name_entity->create(
							{
								'entity_type' => 'teams',
								'entity_id'   => $membership_id,
								'language'    => 'en-US',
								'full_name'   => $phase_membership_name
							}
						);
					}
				}

                # Clear teams once for each player's phases to avoid missing logical variations and having
                # useless extra rows show up
				if ( !defined $cleared_teams ) {
#					$person_phase_entity->delete(
#						{
#							'person_id'       => $person_id,
#							'membership_type' => 'teams'
#						}
#					);
					$cleared_teams = 1;
				}
			}

			# Handle id retrieval for leagues
			if ( $membership_type eq 'leagues' ) {
				my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, 	$self->{log} );
				$membership_id = $affiliation_entity->retrieve_id_by_key(
					{
						'affiliation_key'  => $membership_key,
						'affiliation_type' => 'league',
						'publisher_id'     => $publisher_id
					}
				);

                # Clear leagues once for each player's phases to avoid missing logical variations and having
                # useless extra rows show up
				if ( !defined $cleared_leagues ) {
#					$person_phase_entity->delete(
#						{
#							'person_id'       => $person_id,
#							'membership_type' => 'affiliations'
#						}
#					);
					$cleared_leagues = 1;
				}
			}
			$person_phase_entity->update_status_for_other_memberships($person_id, $membership_type, $membership_id, $publisher_id);
            
			next unless defined $membership_id;
            
            # TODO This row_count stuff should perhaps go
			my $row_count = $person_phase_entity->retrieve_row_count(
				{
					'person_id'       => $person_id,
					'membership_type' => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
					'membership_id' => $membership_id #,
#					'phase_status'  => $phase_status
				}
			);

			# Should never happen with simplified pre-clearing logic
			if ( $row_count > 1 ) {
				$person_phase_entity->delete(
					{
						'person_id'       => $person_id,
						'membership_type' => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
						'membership_id' => $membership_id #,
#						'phase_status'  => $phase_status
					}
				);
			}
	            my $person_phase_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'person_phases');
            $person_phase_record_mapper->retrieve(
                {
                    'person_id'       => $person_id,
                    'membership_type' => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
                    'membership_id' => $membership_id
                }
            );

#			my $person_phase_entity_id = $person_phase_entity->retrieve_id_by_key(
#				{
#					'person_id'       => $person_id,
#					'membership_type' => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
#					'membership_id' => $membership_id #,
##					'phase_status'  => $phase_status
#				}
#			);
			# Add to the hash defining items NOT to clear out for teams and affiliations membership types
			# only if an existing id was found (or else we'd add undef to the array)
#            if ( defined $person_phase_entity_id ) {
#            }
            my $person_phase_hash = {
                    'person_id'          => $person_id,
                    'membership_type'    => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
                    'membership_id'      => $membership_id,
                    'phase_status'       => $phase_status,
                    'role_status'        => $role_status,
                    'start_date'         => $start_date,
                    'start_date_time'    => $start_date_time,
                    'entry_reason'       => $entry_reason,
                    'exit_reason'        => $exit_reason,
                    'selection_level'    => $selection_level,
                    'selection_sublevel' => $selection_sublevel,
                    'selection_overall'  => $selection_overall,
#                    'regular_position_depth' => $phase_position_depth,
#                    'regular_position_id' => $phase_position_id

                     # Need only for current
#                        'uniform_number'         => $phase_uniform_number,
#                        'height'                 => $phase_height,
#                        'weight'                 => $phase_weight,
#                        'duration'                 => $phase_duration
            };

#           $person_phase_id = $person_phase_entity->retrieve_id_by_key({'person_id'=>$person_id});

            $person_phase_hash->{'regular_position_depth'} = $phase_position_depth if defined $phase_position_depth;
            $person_phase_hash->{'regular_position_id'} = $phase_position_id if defined $phase_position_id;
            $person_phase_hash->{'uniform_number'} = $phase_uniform_number if defined $phase_uniform_number;
            $person_phase_hash->{'height'} = $phase_height if defined $phase_height;
            $person_phase_hash->{'weight'} = $phase_weight if defined $phase_weight;
            $person_phase_hash->{'duration'} = $phase_duration if defined $phase_duration;
            $person_phase_hash->{'subphase_type'} = $subphase_type if defined $subphase_type;
            $person_phase_hash->{'phase_type'} = $phase_type if defined $phase_type;
            # If there is a full date or a full date-time value for the start-date attribute, populate start_date_time as well
#            $person_phase_hash->{'start_date_time'} = $start_date_time if defined $start_date_time;

            # TODO Which fields should override with nulls?
            $person_phase_record_mapper->set($person_phase_hash);

            # Save, allowing nulls to overwrite in case of update
            my $person_phase_entity_id = $person_phase_record_mapper->save();
            if ($membership_type eq 'teams') {
                # Add team id to list to NOT remove later via call to delete_by_id_list
#                print "clearing teams\n";
                push(@team_membership_list, $membership_id);
#                push(@team_membership_list, $person_phase_entity_id);
            }
            else {
                # Add league id to list to NOT remove later via call to delete_by_id_list
                push(@affiliation_membership_list, $membership_id);
#                push(@affiliation_membership_list, $person_phase_entity_id);
            }
		}
		
		# Now that all nodes have been processed, clear out items NOT found in the file for the person_id in question
		my $team_membership_list_count = @team_membership_list;
        my $affiliation_membership_list_count = @affiliation_membership_list;
#		$self->{log}->log("[DBG] team_membership_list_count: $team_membership_list_count @team_membership_list");
		if ($team_membership_list_count > 0) {
			my $id_list_str = join(', ', @team_membership_list);
#			$self->{log}->log("[DBG] count: $team_membership_list_count list str: $id_list_str list arr: @team_membership_list");
			$self->delete_by_id_list($person_id, 'persons', $id_list_str);
		}
        if ($affiliation_membership_list_count > 0) {
            my $id_list_str = join(', ', @affiliation_membership_list);
            $self->delete_by_id_list($person_id, 'affiliations', $id_list_str);            
        }
	}
	# Do a poor-man's person_phases entry to allow matching of 
	# persons to teams	
#	$self->{log}->log("[DBG] regular_position_id: $position_id uniform_number: $uniform_number height: $height weight: $weight");
    else {
        my $membership_type;
        my $membership_id;
        if ( defined $team_id ) {
            $membership_type = 'teams';
            $membership_id = $team_id;
        }
        else {
            my $league_key = $sportsml_document->get_content_code_key('league');
            
            my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh},   $self->{log} );
            $membership_id = $affiliation_entity->retrieve_id_by_key(
            {
                'affiliation_key'  => $league_key,
                'affiliation_type' => 'league',
                'publisher_id'     => $publisher_id
            }
            );
        }
        if (defined $membership_id) {
            my $row_count = $person_phase_entity->retrieve_row_count(
            {
                'person_id'       => $person_id
            }
            );
            if ( $row_count == 0 ) {
                $person_phase_entity->create(
                {
                    'phase_status'    => 'active',
                    'role_status'     => undef,
				    'person_id'       => $person_id,
				    'membership_type' => ( $membership_type eq 'teams' ) ? 'teams' : 'affiliations',
				    'membership_id'      => $membership_id,

				    # Need only for current
				    'regular_position_id'    => $position_id,
				    'uniform_number'         => $uniform_number,
				    'height'                 => $height,
				    'weight'                 => $weight
                }
                );

            }
        }
	}

#    $self->{log}->log("[DBG] event_id: $event_id team_id: $team_id");

	if ( $event_id ) {

		# Have:
		# person_id (player_id)
		# event_id
		# position_id

		# Not handled at the moment

        my $person_event_metadata_id;
        my $person_event_metadata_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'person_event_metadata');
        $person_event_metadata_record_mapper->retrieve(
            {
                'person_id' => $person_id,
                'event_id'  => $event_id
            }
        );

        if ($person_event_metadata_record_mapper->can_save()) {
            # person_id and event_id are already known to the mapper
            $person_event_metadata_record_mapper->set(
                {
# Putting status back in
                    'status'      => $player_status,
                    'weight' => $weight,
                    'health' => $health,
                    'role_id' => $role_id,
                    'position_id' => $position_id,
                    'team_id' => $team_id,
                    'lineup_slot' => $lineup_slot,
                    'lineup_slot_sequence' => $lineup_slot_sequence
                }            );
            $person_event_metadata_id = $person_event_metadata_record_mapper->save();
        }
	}
	
	# $self->{active_fixtures_injuries} is set from store_all_from_doc, which is the proper starting
	# context. If this document
	if (defined $self->{process_injuries}) {
		my $injury_phase_node = solonode_optional( $player_node,
			'descendant::player-metadata/injury-phase' );
		if (defined $injury_phase_node) {
			$injury_phase_entity->store_from_doc( $player_node, $person_id,
				$publisher_id );
			$self->{injured_persons}->{$person_id} = 1;
			$self->{injured_persons_teams}->{$team_id} = 1;
		}
	}

# Handle player stats. This is also done in DbStore.pm at the moment in the subroutine
# process_players. This is a duplication that needs to be eliminated, with the eventual
# definative code going here.
# So far, we are doing season stats here
	my $player_stats = firstnode( $player_node, 'player-stats' );
	if ($player_stats) {

		#    	$self->{log}->log("[DBG] found player stats");
		# Need to store in core_person_stats
#		my $time_played_event = $player_stats->getAttribute('time-played-event');
#		my $time_played_total = $player_stats->getAttribute('time-played-total');
		
	}

	return $person_id;
}

sub handle_person_event_metadata {
	my ( $self, $doc, $publisher_id ) = @_;

}

sub handle_person_phases {
	my ( $self, $doc, $publisher_id ) = @_;

}

sub store_all_from_doc {
	my ( $self, $doc, $publisher_id ) = @_;

	my $active_fixtures = Conf::ActiveFixtures->new($doc);
	$self->{process_injuries} = undef;
    my $active_fixtures_injuries = $active_fixtures->doc_lookup('active-fixtures-injuries');
#    print "active_fixtures_injuries: ".$self->{active_fixtures_injuries}."\n";

	# Create a sportsml_document object to help us grab content code information related to
	# handling roster and injury entries found in those tables but not in current document
	# (i.e. set person_phase and injury_phase items for team to 'inactive')
	my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($doc, $self->{log});
	# Later will be looking for a value of 'complete'
	my $roster_listing = $sportsml_document->get_content_code_key('roster-listing');
	$self->{roster_listing} = $roster_listing;
	# Instantiate empty hash reference
	$self->{roster_by_team} = {};
    $self->{roster_by_league} = {};

	# Prepare structures for handling injuryies if config file indicates injuries data to be processed
	# for document. 
	if (defined $active_fixtures_injuries) {
		my $injury_listing = $sportsml_document->get_content_code_key('injury-listing');
		$self->{injury_listing} = $injury_listing;
	}
	if (defined $self->{injury_listing}) {
		my $sports_data_node = firstnode($doc, "//sports-content/sports-metadata");
		my $date_utils = Util::DateUtils->new();
		my $doc_date_str = $sports_data_node->getAttribute('date-time');
		$self->{doc_date_sql_fmt} = $date_utils->ISO8601_date2SQL_date($doc_date_str);
		$self->{process_injuries} = 1;
		$self->{injured_persons} = ();
		$self->{injured_persons_teams} = ();
	}
	

	# TODO - See if this is redundant and therefore a waste of cycles (i.e. are teams ALWAYS stored elsewhere for sure)
	my $team_entity = FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
	$team_entity->store_all_from_doc( $doc, $publisher_id );
	
	# Process all players in document.
	my @player_nodes = $doc->findnodes("//player");
	foreach my $player_node (@player_nodes) {
		$self->store_from_doc( $player_node, $publisher_id );
	}

	# If a roster listing status of 'complete' has been found then go ahead and
	# do an update on the roster to set any not mentioned for the team to 'inactive'
	if ($self->{roster_listing} eq 'complete') {
		       
        if (defined $self->{roster_by_team}) {
    		my @team_keys = keys %{$self->{roster_by_team}};
	       	# The listings for players in roster are indexed by team, so run through each
		    # team and produce update SQL.
		    foreach my $team_key (@team_keys) {
			     # Get list of person keys for team
			     my @person_keys = @{$self->{roster_by_team}->{$team_key}};
			
          		# Don't proceed unless there is something to do. This should not be necessary
                # Since the team index in the roster_listing hash is not produced until there
			    # is a person to put into its array.
			    my $person_key_count = @person_keys;
			    next unless ($person_key_count > 0);

			    # Get ready for SQL by quoting all team keys and producing a list separated by commas
			    @person_keys = map("'$_'", @person_keys);
			    my $person_key_str = join(',', @person_keys);
			    my $person_phase_update_sql = <<END;
UPDATE person_phases
SET phase_status = 'inactive'
WHERE membership_type = 'teams'
AND membership_id = (SELECT id FROM teams WHERE team_key = '$team_key' AND publisher_id = $publisher_id)
AND person_id NOT IN (
	SELECT id FROM persons
	WHERE person_key IN ($person_key_str)
	AND publisher_id = $publisher_id
)
END
#			print "$person_phase_update_sql\n";
			    my $sth = $self->{dbh}->prepare($person_phase_update_sql);
			    $sth->execute();
			    my $rows = $sth->rows;
			    $sth->finish;

#			$self->{log}->log("[DBG] rows updated: $rows");
    			if ($rows > 0) {
	       			my $person_phase_entity = FeedfetcherDeluxe::Entities::Person_phase->new( $self->{dbh}, $self->{log} );
			     	$person_phase_entity->query_cache->invalidate();
			    }

#			print "team-key: $team_key person_keys: @person_keys\n";
		    }
        }
            if (defined $self->{roster_by_league}) {
                my @league_keys = keys %{$self->{roster_by_league}};
                # The listings for players in roster are indexed by team, so run through each
                # team and produce update SQL.
                foreach my $league_key (@league_keys) {
                    # Get list of person keys for team
                    my @person_keys = @{$self->{roster_by_league}->{$league_key}};
#                    print "person keys: @person_keys\n";
            
                    # Don't proceed unless there is something to do. This should not be necessary
                    # Since the league index in the roster_listing hash is not produced until there
                    # is a person to put into its array.
                    my $person_key_count = @person_keys;
                    next unless ($person_key_count > 0);

                    # Get ready for SQL by quoting all team keys and producing a list separated by commas
                    @person_keys = map("'$_'", @person_keys);
                    my $person_key_str = join(',', @person_keys);
                    my $person_phase_update_sql = <<END;
UPDATE person_phases
SET phase_status = 'inactive'
WHERE membership_type = 'affiliations'
AND membership_id = (SELECT id FROM affiliations WHERE affiliation_key = '$league_key' AND affiliation_type='league' AND publisher_id = $publisher_id)
AND person_id NOT IN (
    SELECT id FROM persons
    WHERE person_key IN ($person_key_str)
    AND publisher_id = $publisher_id
)
END
#           print "$person_phase_update_sql\n";
                    my $sth = $self->{dbh}->prepare($person_phase_update_sql);
                    $sth->execute();
                    my $rows = $sth->rows;
                    $sth->finish;

#           $self->{log}->log("[DBG] rows updated: $rows");
                    if ($rows > 0) {
                        my $person_phase_entity = FeedfetcherDeluxe::Entities::Person_phase->new( $self->{dbh}, $self->{log} );
                        $person_phase_entity->query_cache->invalidate();
                    }
                }
            }
	}
	if (defined $self->{process_injuries} && $self->{injury_listing} eq 'complete') {
		my @injured_persons = sort {$a <=> $b} keys %{$self->{injured_persons}};
		my $injured_persons_str = join(',', @injured_persons);
#		$self->{log}->log("[DBG] injured persons: $injured_persons_str");
		my @injured_persons_teams = sort {$a <=> $b} keys %{$self->{injured_persons_teams}};
		my $injured_persons_teams_str = join(',', @injured_persons_teams);
#		$self->{log}->log("[DBG] injured persons teams: $injured_persons_teams_str");
		
		my $injured_persons_count = @injured_persons;
		my $injured_persons_teams_count = @injured_persons_teams;
		
		# Update the injury_phases to close out previous injuries if we have any persons or teams
		# Note that this requires injury data and a sports-content-code injury-listing of 'complete'
		if ($injured_persons_count > 0 && $injured_persons_teams_count > 0) {
			my $injury_update_sql = <<END;
UPDATE injury_phases
SET injury_status = 'inactive', end_date_time='$self->{doc_date_sql_fmt}'
WHERE injury_phases.injury_status='active'
AND injury_phases.end_date_time IS NULL
AND injury_phases.person_id IN (
  SELECT persons.id FROM persons
  INNER JOIN person_phases ON (
    person_phases.person_id = persons.id AND
    person_phases.membership_type='teams'
  )
  INNER JOIN teams ON (
    teams.id = person_phases.membership_id AND 
    teams.id IN ($injured_persons_teams_str)
  )
  WHERE persons.id NOT IN ($injured_persons_str)
)
END
#			print $injury_update_sql."\n";
			my $sth = $self->{dbh}->prepare($injury_update_sql);
			$sth->execute();
			my $rows = $sth->rows;
			$sth->finish;
			if ($rows > 0) {
				my $injury_phase_entity =
					FeedfetcherDeluxe::Entities::Injury_phase->new( $self->{dbh},
					$self->{log} );
				$injury_phase_entity->query_cache->invalidate();
			}
		}
	
	}
}

# Store just person information, no affiliated data
sub store_persons {
	my ( $self, $doc, $publisher_id ) = @_;

	my $person_id;

	my @player_nodes = $doc->findnodes("//player");

	foreach my $player_node (@player_nodes) {
		my $player_metadata = solonode_optional( $player_node, 'player-metadata' );
		next if (! defined $player_metadata);
		my $player_key = $player_metadata->getAttribute('player-key');
        my $person_gender = $player_metadata->getAttribute('gender');

		next if !$player_key;

		my $person_date_of_birth = $player_metadata->getAttribute('date-of-birth');

#    $self->{log}->log("[DBG] ***** player_key: $player_key publisher_id: $publisher_id person_date_of_birth: $person_date_of_birth");


#    my $person_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'persons');
    $self->{person_record_mapper}->reset();
    $self->{person_record_mapper}->retrieve(
        { 
        'person_key' => $player_key, 
        'publisher_id' => $publisher_id 
        }    
    );

    if ($self->{person_record_mapper}->can_save()) {
        # person_key and publisher_id are already known to the mapper
        $self->{person_record_mapper}->set(
            {
            'birth_date'   => $person_date_of_birth,
            'gender' => $person_gender
            }
        );
        $person_id = $self->{person_record_mapper}->save();
    }

#        $person_id = $self->retrieve_id_by_key(
#            {
#                'person_key'   => $player_key,
#                'publisher_id' => $publisher_id
#            }
#        );
#		if ( !$person_id ) {
#			# insert new rows into persons and display_names
#			$person_id = $self->create(
#				{
#					'person_key'   => $player_key,
#					'publisher_id' => $publisher_id,
#					'birth_date'   => $person_date_of_birth,
#					'gender' => $person_gender
#				}
#			);
#		}
#        else {
#            if ($person_date_of_birth) {
#                $person_id = $self->update(
#                    {
#                        'birth_date'   => $person_date_of_birth,
#                        'gender' => $person_gender
#                    }, 
#                    {
#                        'id' =>     $person_id,
#                        'publisher_id' => $publisher_id
#                    }
#                );
#            }
#        }		
	}
}

sub find_team_idref_from_player_idref {
    my ( $self, $node, $idref ) = @_;

    # Get the player metadata node from the document
    my $player_metadata_node = $self->find_player_metadata_node($node, $idref);
    
    return undef unless (defined $player_metadata_node);
    
    # Find the team metadata node for the player
    my $team_metadata_node = firstnode( $player_metadata_node, "ancestor::team/team-metadata" );
    
    return undef unless (defined $team_metadata_node);
    
    # Find the team-key for the team-metadata node
    my $team_key = $team_metadata_node->getAttribute("team-key");
    
    # Will be defined or undef (i.e. found or not found)
    return $team_key;
}

sub find_player_metadata_node {
    my ( $self, $node, $idref ) = @_;
    
    my $player_metadata_node =
      firstnode( $node, "//player[\@id='$idref']/player-metadata" );

    if ( !$player_metadata_node ) {
        $player_metadata_node =
          firstnode( $node, "//player-metadata[\@player-key='$idref']" );
    }

    if ( !$player_metadata_node ) {
        $player_metadata_node =
          firstnode( $node, "//player-metadata[\@id='$idref']" );
    }
    return $player_metadata_node;
}

sub find_id_from_idref {
	my ( $self, $node, $idref, $publisher_id ) = @_;

	my $person_id;
	my $person_key;
	my $id;

    return undef if $idref eq '';

#    print "looking for player based on idref: $idref\n";

    my $player_metadata_node = $self->find_player_metadata_node($node, $idref);
#	my $player_metadata =
#	  firstnode( $node, "//player[\@id='$idref']/player-metadata" );
#
#	if ( !$player_metadata ) {
#		$player_metadata =
#		  firstnode( $node, "//player-metadata[\@player-key='$idref']" );
#	}
#
#	if ( !$player_metadata ) {
#		$player_metadata =
#		  firstnode( $node, "//player-metadata[\@id='$idref']" );
#	}


	if ($player_metadata_node) {
		$person_key = required_attribute( $player_metadata_node, 'player-key' );
	}

	# Create a stub person entry if no metadata is found.
	# The assumption is that in a later document proper player metadata will be
	# available which can then be used to populate person related tables.
#	if ( ! $person_key ) {
#		return undef;
#
#        my $person_id_count = $self->retrieve_row_count({'person_key'=>$idref, 'publisher_id'=>$publisher_id});
#
#        if ($person_id_count == 0) {
#            $self->{log}->log("[ADV] No person key and/or player metadata found for player-idref $idref. Creating stub person entry.");
#    		$id = $self->create(
#	   		{
#		  		'person_key'   => $idref,
#				'publisher_id' => $publisher_id
#			}
#		  )
#        }
#        else {
#            $id = $self->retrieve_id_by_key(
#            {
#                 'person_key' => $idref, 
#                 'publisher_id' => $publisher_id 
#            }
#            );
#        }
#		return $id;
#	}
    if (! $person_key) {
        $id = $self->retrieve_id_by_key(
        {
             'person_key' => $idref, 
             'publisher_id' => $publisher_id 
        }
        );
        if ($id) {
            return $id;
        }
        else {
            return undef;
        }
    }
    else {
        # Handle finding person by person-key
        $id = $self->retrieve_id_by_key(
            {
            'person_key' => $person_key, 
            'publisher_id' => $publisher_id 
            }
        );
        # Create a new player entry if there is not one yet. This can happen in cases where the document's fixture-key
        # is not listed in active-fixtures-players.dat
        if ( !$id ) {
            my $player_node = solonode_optional( $player_metadata_node, "ancestor::player" );
            $id = $self->store_from_doc( $player_node, $publisher_id );
        }

        return $id;
    }
    return undef;
}

sub _get_name {
	my ( $self, $name_node ) = @_;

	my ( $first, $middle, $last, $full, $nickname ) = (
		$name_node->getAttribute('first'),
		$name_node->getAttribute('middle'),
		$name_node->getAttribute('last'),
		$name_node->getAttribute('full'),
		$name_node->getAttribute('nickname')
	);

	if ( !$full ) {
		my @names = ();
		push @names, $first  if $first;
		push @names, $middle if $middle;
		push @names, $last   if $last;
		$full = join( " ", @names );
	}

	#$self->{log}->log("[DBG] $first, $middle, $last, $full, $nickname");
	return ( $first, $middle, $last, $full, $nickname );
}

sub find_team_id {
	my ( $self, $player_node, $publisher_id ) = @_;

	my $person_id;
	my $team_id;
	my $team_node = solonode_optional( $player_node, 'parent::team' );
	my $team_meta = solonode( $team_node, 'team-metadata' ) if $team_node;
	my $player_metadata = solonode( $player_node, 'player-metadata' );
	my $player_key = $player_metadata->getAttribute('player-key');
	return undef if !$player_key;

	$person_id = $self->retrieve_id_by_key(
		{ 'person_key' => $player_key, 'publisher_id' => $publisher_id } );
	if ( !$person_id ) {
		$self->{log}->log("[ADV] person_id lookup failed. Has person data for player-key $player_key been stored?");
		return undef;
	}

# Do some extra work here looking in the person_phase table if no nearby team metadata node
# has been found.
# NOTE: more work could be done to search the rest of the document for other players with the same
#            key, then for team metadata nodes nearby
	if ( !defined $team_meta ) {
        # Make an attempt to find team_id based on team-idref in player-metadata, if it exists
        my $team_idref = $player_metadata->getAttribute( 'team-key' );
        # Try for player-metadata[@team-key]
        if ( ! defined $team_idref ) {
            $team_idref = $player_metadata->getAttribute( 'team-idref' );
        }
        if ( defined $team_idref ) {
            my $team_entity =
                FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
            $team_id = $team_entity->find_id_from_idref( $player_metadata, $team_idref, $publisher_id );
#             $self->{log}->log("[DBG] team_idref: $team_idref team_id: $team_id");
        }
        else {
    		my $person_phase_entity =
	       FeedfetcherDeluxe::Entities::Person_phase->new( $self->{dbh},
            $self->{log} );
            my $hashref = $person_phase_entity->retrieve_all_fields_by_key(
                { 'person_id' => $person_id, 'membership_type' => 'teams' } );
            if ( defined $hashref ) {
                $self->{log}->log("[DBG] Team id found through lookup in person_phase table");
                $team_id = $hashref->{membership_id};
            }
			return undef if ! defined $team_id;
		}
	}
	else {
		my $team_entity =
		  FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
		my $team_key = attribute( $team_meta, 'team-key' );
#		print "team key: $team_key\n";
		
		if ( defined $team_key ) {
			$team_id = $team_entity->retrieve_id_by_key(
				{
					'team_key'     => $team_key,
					'publisher_id' => $publisher_id
				}
			);
		}
	}

	return $team_id;
}

sub delete_by_id_list {
    my ( $self, $person_id, $membership_type, $id_str ) = @_;

    my $person_phase_entity =
      FeedfetcherDeluxe::Entities::Person_phase->new( $self->{dbh},
        $self->{log} );
        
    my $delete_sql = qq{
        DELETE FROM person_phases
        WHERE person_id=? 
        AND membership_type=?
        AND membership_id NOT IN ($id_str)     
    };
#    print "delete_sql:\n$delete_sql person_id: $person_id membership_type: $membership_type\n";
    
    my $delete_sth = $self->{dbh}->prepare($delete_sql);
    my $rows = $delete_sth->execute($person_id, $membership_type);
    $delete_sth->finish;
    $person_phase_entity->query_cache()->invalidate();
#    $self->{log}->log("[DBG] Removed $rows records for person_id $person_id from person_phases");
#    $self->{log}->log("[DBG] Delete sql: $delete_sql");
}


1;
