# $Id: Position.pm 3079 2016-04-07 20:01:21Z ian $

package FeedfetcherDeluxe::Entities::Position;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Affiliation;

#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Position

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

    $self->{'retrieve_id_by_key'} = $self->{dbh}->prepare(qq{
        SELECT id
        FROM positions 
        WHERE  affiliation_id=?
        AND abbreviation=?
    });

}

sub get_position_id_from_node {
	my ( $self, $node, $attribute_name, $publisher_id ) = @_;
	
	return undef unless defined $node;
	my $position_key = $node->getAttribute($attribute_name);
	# Deal with undefined position key
	return undef unless defined $position_key;
	# Deal with empty position key as well
	return undef if $position_key =~ /\s+/;
	my $position_id = $self->get_position_id($position_key, $node, $publisher_id);

	return $position_id;
}

sub get_position_id {
    my ( $self, $position_key, $node, $publisher_id ) = @_;

    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new( $node, $self->{log} );
    my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, $self->{log} );
      
    my $sport_code_key = $sportsml_document->get_content_code_key('sport');
    return undef unless defined $sport_code_key;
    
    my $affiliation_id = $affiliation_entity->retrieve_id_by_key(
        {
            'affiliation_key'  => $sport_code_key,
            'affiliation_type' => 'sport',
            'publisher_id'     => $publisher_id
        }
    );

# TODO Check to see if this is unecessary - probably done much earlier in document processing when content codes are dealt with
    if ( !$affiliation_id ) {
        $affiliation_id = $affiliation_entity->create(
            {
                'affiliation_key'  => $sport_code_key,
                'affiliation_type' => 'sport',
                'publisher_id' => $publisher_id
            }
        );
    }

    $position_id = $self->retrieve_id_by_key(
        {
            'affiliation_id' => $affiliation_id,
            'abbreviation' => $position_key
        }
    );
    if ( !$position_id ) {
        $position_id = $self->create(
            {
                'affiliation_id' => $affiliation_id,
                'abbreviation' => $position_key
            }
        );
    }
    return $position_id;
}

1;