# $Id: Record.pm 2123 2008-07-23 19:22:08Z ian $

package FeedfetcherDeluxe::Entities::Ranking;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Document_fixture;


=head1 NAME

FeedFetcherDeluxe::Entities::Record

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}


#        <team-stats>
#          <outcome-totals alignment-scope="league" duration-scope="season-regular" wins="9" losses="0" />
#          <rank competition-scope="league" issuer="BCS" value="1" value-previous="2" />
#          <rank competition-scope="league" issuer="Harris" value="1" />
#          <rank competition-scope="league" issuer="USA" value="1" />
#          <rank competition-scope="league" issuer="Previous" value="2" />
#          <rating competition-scope="league" rating-issuer="Computer" rating-value=".960" />
#          <rating competition-scope="league" rating-issuer="BCS" rating-value=".975" />
#        </team-stats>
sub  store_from_doc {
    my ( $self, $participant_type, $node, $publisher_id ) = @_;

	my $document_fixture_entity = FeedfetcherDeluxe::Entities::Document_fixture->new( $self->{dbh}, $self->{log} );
    my $team_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});

	my $document_fixture_id = $document_fixture_entity->get_document_fixture_id( $node, $publisher_id );
    my $participant_id;

    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($node, $publisher_id);

    my $participant_type_singular = $participant_type;
    $participant_type = $participant_type."s";
    my $participant_root = ($participant_type eq 'teams') ? 'team': 'player';

    my $metadata_node = solonode($node, $participant_root."-metadata");
    my $participant_key = $metadata_node->getAttribute($participant_root."-key");
    
    # Just in case, if there's no key, get out.
    return $participant_id if (! $participant_key || $participant_key eq '');

    if ($participant_type eq 'teams') {
        $participant_id = $team_entity->retrieve_id_by_key({'team_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }

    if ($participant_type eq 'players') {
        $participant_id = $person_entity->retrieve_id_by_key({'person_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }
    
#    # First, delete all rankings for the team for the season for the document fixture id
#	$self->delete({
#		'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#		'participant_id' => $participant_id,
#		'document_fixture_id' => $document_fixture_id,
#		'date_coverage_type' => $date_coverage_type,
#		'date_coverage_id' => $date_coverage_value_id
#	}
#	);

    
    my @rank_nodes = $node->findnodes($participant_type_singular."-stats/rank");
    my @rating_nodes = $node->findnodes($participant_type_singular."-stats/rating");
    
    my @combined_nodes = (@rank_nodes, @rating_nodes);

    foreach my $rank_node (@combined_nodes) {
    	# Get ranking type, sequentially favouring ranking-type and type attributes over the ranking node name
		my $ranking_type = $rank_node->nodeName;
		$ranking_type = $rank_node->getAttribute('ranking-type') if (defined $rank_node->getAttribute('ranking-type'));
		$ranking_type = $rank_node->getAttribute('rating-type') if (defined $rank_node->getAttribute('rating-type'));
		$ranking_type = $rank_node->getAttribute('type') if (defined $rank_node->getAttribute('type'));
		my $issuer = $rank_node->getAttribute('issuer');
		$issuer = $rank_node->getAttribute('rating-issuer') if (! defined $issuer);
		next if (! defined $issuer);
#		my $rating_value_attribute_name = ($ranking_type eq 'rating') ? 'rating-value': 'value';
		my $ranking_value = $rank_node->getAttribute('value');
		$ranking_value = $rank_node->getAttribute('rating-value') if (! defined $ranking_value);
		next if (! defined $ranking_value || $ranking_value =~ /^\s*$/);
		my $ranking_value_previous = $rank_node->getAttribute('value-previous');
#	    $self->{log}->log("[DBG] type: $ranking_type issuer: $issuer value: $ranking_value date_coverage_type: $date_coverage_type date_coverage_value_id $date_coverage_value_id");
	    	    
#	    my $hash_ref = $self->retrieve_all_fields_by_key(
#	    {
#	        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#	    	'participant_id' => $participant_id,
#	    	'document_fixture_id' => $document_fixture_id,
#	    	'issuer' => $issuer,
#	    	'ranking_type' => $ranking_type,
#    		'date_coverage_type' => $date_coverage_type,
#    		'date_coverage_id' => $date_coverage_value_id
#	    }
#	    );
#	    if (defined $hash_ref) {
#	    	my $id = $hash_ref->{id};
#	    	$self->update(
#	    	{
#	    		'ranking_value' => $ranking_value,
#	    		'ranking_value_previous' => $ranking_value_previous
#	    	},
#			{'id' => $id}
##		    {
##		    	'participant_type' => $participant_type,
##	    		'participant_id' => $participant_id,
##	    		'issuer' => $issuer,
##	    		'ranking_type' => $ranking_type,
##    			'date_coverage_type' => $date_coverage_type,
##    			'date_coverage_id' => $date_coverage_value_id
##	    	}
#	    	);
#	    }
#	    else {
	    	$self->create(
		    {
		        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
	    		'participant_id' => $participant_id,
	    		'document_fixture_id' => $document_fixture_id,
	    		'issuer' => $issuer,
	    		'ranking_type' => $ranking_type,
	    		'ranking_value' => $ranking_value,
	    		'ranking_value_previous' => $ranking_value_previous,
	    		'date_coverage_type' => $date_coverage_type,
	    		'date_coverage_id' => $date_coverage_value_id
	    	}
	    	);
#	    }
    }
}

sub store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
    my @team_nodes = $node->findnodes("//team");
    my @player_nodes = $node->findnodes("//player");

	my $document_fixture_entity = FeedfetcherDeluxe::Entities::Document_fixture->new( $self->{dbh}, $self->{log} );
	my $document_fixture_id = $document_fixture_entity->get_document_fixture_id( $node, $publisher_id );
#    my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($node, $publisher_id);
    
	my $sample_node;
    # If findnodes found elements
    if ( @team_nodes ) {
    	$sample_node = $team_nodes[0];
    }
    # If findnodes found elements
    elsif ( @player_nodes ) {
    	$sample_node = $player_nodes[0];
    }
	else {
		$self->{log}->log("[DBG] No teams or players found. Ranking data will not be stored.");
		return;
	}
    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
	my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($sample_node, $publisher_id);
#	$self->{log}->log("[DBG] document_fixture_id: $document_fixture_id date_coverage_type: $date_coverage_type date_coverage_value_id: $date_coverage_value_id");

	if (! defined $date_coverage_value_id) {
		$self->{log}->log("[DBG] No date coverage id found. Ranking data will not be stored.");
		return;
	}
	
    # First, delete all rankings for the team for the season for the document fixture id
	$self->delete({
		'document_fixture_id' => $document_fixture_id,
		'date_coverage_type' => $date_coverage_type,
		'date_coverage_id' => $date_coverage_value_id
	}
	);
            
    foreach my $node (@team_nodes) {
    	my $node_name = $node->nodeName();
        $self->store_from_doc($node_name, $node, $publisher_id);    	
    }    
    foreach my $node (@player_nodes) {    	
        my $node_name = $node->nodeName();
        $self->store_from_doc($node_name, $node, $publisher_id);       
    }    
}

1;