# $Id: Record.pm 2989 2012-08-01 00:27:00Z ian $

package FeedfetcherDeluxe::Entities::Record;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;
use FeedfetcherDeluxe::Entities::SportsmlDocument;


=head1 NAME

FeedFetcherDeluxe::Entities::Record

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub  store_from_doc {
    my ( $self, $participant_type, $node, $publisher_id ) = @_;

    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});

    my $team_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    my $period_entity = FeedfetcherDeluxe::Entities::Period->new($self->{dbh}, $self->{log});
    my $participant_id;

    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
    my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($node, $publisher_id);

    return unless (defined $date_coverage_type && defined $date_coverage_value_id);

    my $participant_root = ($participant_type eq 'teams') ? 'team': 'player';

    my $metadata_node = solonode($node, $participant_root."-metadata");
    my $participant_key = attribute($metadata_node, $participant_root."-key");
    
    # Just in case, if there's no key, get out.
    return $participant_id if (! $participant_key || $participant_key eq '');

    if ($participant_type eq 'teams') {
        $participant_id = $team_entity->retrieve_id_by_key({'team_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }

    if ($participant_type eq 'players') {
        $participant_id = $person_entity->retrieve_id_by_key({'person_key'=>$participant_key, 'publisher_id'=>$publisher_id});
    }
    return unless (defined $participant_id);
    
    my $record_node = firstnode($node, $participant_root."-stats/event-record");
    
    return if (! defined $record_node);
    
    my $stats_node = firstnode($node, $participant_root."-stats");
    
    my $record_type = attribute($record_node, 'type');
    return unless (defined $record_type);
    my $record_label = attribute($record_node, 'xts:label');
    $record_label = ($record_label =~ '^\s*$') ? undef: $record_label;
    my $record_value = attribute($stats_node, 'score');
    my $previous_value = attribute($record_node, 'previous-record');
    my $comment = attribute($record_node, 'xts:comment');
    
    my $record_listing = $sportsml_document->get_content_code_key('record-listing');
    
    # If this is a complete record listing, clear only once (thus the cleared_old flag)
    if ($record_listing && $self->{cleared_old} eq "0") {
        if ($record_listing eq 'complete') {
#            $self->{log}->log("[DBG] record_listing: $record_listing");
            $self->delete(
            {
                'date_coverage_type' => $date_coverage_type,
                'date_coverage_id' => $date_coverage_value_id
            }
            );
            $self->{cleared_old} = 1;
        }
    }

#    my $record_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'records');

#    print "participant_type: $participant_type, participant_id: $participant_id, date_coverage_type: $date_coverage_type, date_coverage_id: $date_coverage_value_id, record_type: $record_type\n";
    $self->{record_record_mapper}->reset();
    my $hash_ref = {};
    $hash_ref->{participant_type} = ($participant_type eq 'players') ? 'persons' : 'teams';
    $hash_ref->{participant_id} = $participant_id;
    $hash_ref->{date_coverage_type} = $date_coverage_type;
    $hash_ref->{date_coverage_id} = $date_coverage_value_id;
    $hash_ref->{record_type} = $record_type;
    $hash_ref->{record_label} = $record_label if (defined $record_label);
    
    $self->{record_record_mapper}->retrieve(
        $hash_ref
#        { 
#        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#        'participant_id' => $participant_id,
#        'date_coverage_type' => $date_coverage_type,
#        'date_coverage_id' => $date_coverage_value_id,
#        'record_type' => $record_type
#        }    
    );

    $self->{record_record_mapper}->set(
        {
#        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#        'participant_id' => $participant_id,
#        'record_type' => $record_type,
        'record_value' => $record_value,
        'record_label' => $record_label,
        'previous_value' => $previous_value,
        'comment' => $comment #,
#        'date_coverage_type' => $date_coverage_type,
#        'date_coverage_id' => $date_coverage_value_id
        }
    );
    $self->{record_record_mapper}->save();
#    $self->delete(
#    {
#        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#        'participant_id' => $participant_id,
#        'date_coverage_type' => $date_coverage_type,
#        'date_coverage_id' => $date_coverage_value_id,
#        'record_type' => $record_type
#    }
#    );

#    $self->create(
#    {
#        'participant_type' => ($participant_type eq 'players') ? 'persons' : 'teams',
#        'participant_id' => $participant_id,
#    	'record_type' => $record_type,
#        'record_value' => $record_value,
#        'previous_value' => $previous_value,
#        'comment' => $comment,
#        'date_coverage_type' => $date_coverage_type,
#        'date_coverage_id' => $date_coverage_value_id
#    }
#    );

}

sub store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
    my @team_nodes = $node->findnodes("//sports-event/team");
    my @player_nodes = $node->findnodes("//sports-event/player");
    
    $self->{record_record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'records');
    
    # Assumption is one set of records per event
    $self->{cleared_old} = 0;   
    foreach my $node (@team_nodes) {
        $self->store_from_doc('teams', $node, $publisher_id);    	
    }    
    foreach my $node (@player_nodes) {    	
        $self->store_from_doc('players', $node, $publisher_id);       
    }    
}

1;