# $Id: Season.pm 3078 2016-04-07 19:58:31Z ian $

package FeedfetcherDeluxe::Entities::Season;

use vars qw(@ISA);
use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Event;
use FeedfetcherDeluxe::Entities::Period;
use FeedfetcherDeluxe::Entities::Events_sub_season;
use FeedfetcherDeluxe::Entities::Sub_season;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::SportsmlDocument;


#use Core::DbUtil;
#use Core::XmlUtil;

# NOTE:
# The code in this module needs to be split up so that a main DateCoverage module is requested to handle the 
# locating, storage, and return of date coverage information (events, career, season) and the Event and Season
# modules handle only those things specific to events and seasons, respectively.

=head1 NAME

Core::Entities::Affiliation

=head1 SYNOPSIS
Figures out the season or event context for a node. 

=head2 Some variables

=item contex

The name gathered from the document, such as 'event' or 'post-season'

=item date_coverage_type

The table to store to: seasons, sub_seasons, events, affiliations (e.g. for career league affiliation)

=item date_coverage_value

The value stored in the date_coverage_type table (e.g. 2007, season-regular, l.mlb.com)

=item date_coverage_value_id

The seasons, sub_seasons, events, or affiliations record id field value

=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN { @ISA = qw ( FeedfetcherDeluxe::Entities::Entity ); }

 #      $self->{log}->log("[DBG] ");

sub setup_sql {
	my ($self) = @_;

	# Note: Not viable for all tables
#	$self->{'retrieve_id_by_key'} = $self->{dbh}->prepare(
#		qq{
#        SELECT id
#        FROM seasons 
#        WHERE season_key=?
#        AND league_id=?
#        AND publisher_id=?
#    }
#	);
}

sub store_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
    my $season_node =
      solonode_optional( $node, "//sports-content-code[\@code-type='season']" );
    if ($season_node) {
    	
    }
}

sub get_season {
    my ( $self, $node, $publisher_id ) = @_;
	
    my $context;
    my $date_coverage_type;
    my $date_coverage_value;
    my $date_coverage_value_id;

    ( $context, $date_coverage_type, $date_coverage_value,
            $date_coverage_value_id ) = $self->get_coverage($node, $publisher_id);
    
    return ( $context, $date_coverage_type, $date_coverage_value,
            $date_coverage_value_id ) if ($date_coverage_type) =~ /season/;
    
    # If contextual coverage type is not some sort of season, dig deeper to find that
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{log});
    my $season = $sportsml_document->get_content_code_key('season');
    my $season_type = $sportsml_document->get_content_code_key('season-type');
    # Make sure that date_coverage_type, on storage, is seen as season

# Debug code
#    print "season type: $season_type\n";
    $season_type = "season-$season_type" if ! ($season_type eq 'season');
    $date_coverage_type = $season_type;
    $context = $season_type;
    
    return ( $context, $date_coverage_type, $date_coverage_value,
            $date_coverage_value_id ) if ! $season;
    
    
    ( $context, $date_coverage_type, $date_coverage_value,
        $date_coverage_value_id ) = $self->date_coverage_store( $node, $context, $season_type, $season, $publisher_id );

    return ( $context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id );
}

sub get_coverage_node_in_context_of_season {
    my ( $self, $coverage_node, $find_season ) = @_;
    my $ret_val = $coverage_node;

    if ( $coverage_node && $find_season ) {
        my $date_coverage_type =
          required_attribute( $coverage_node, 'date-coverage-type' );
        if ( $date_coverage_type =~ /event/ ||  $date_coverage_type !~ /season/ ) {
            $ret_val = undef;
        }
    }
    return $ret_val;
}

sub get_season_data_from_sport_content_codes {
    my ( $self, $node ) = @_;

    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{log});
    my $season = $sportsml_document->get_content_code_key('season');
    my $context;

    # Don't create 'season-' if there's nothing in s-c-c
    return (undef, undef, undef) if ! defined $season;

    my $season_type = $sportsml_document->get_content_code_key('season-type');

# Debug code
#    print "season_type: $season_type\n";
    if (defined $season_type) {
        $season_type = 'season-regular' if ($season_type =~ /^\s*$/);
        $season_type = "season-".$season_type if $season_type !~ /season/;
    }
    else {
    	$season_type = 'season-regular';
    }
    $context = $season_type;
    
    my $season_value = $season;
    $season_value =~ s/[^\d]//g;

    return ( $context, $season_type, $season_value );	
}

sub get_coverage_data {
    my ( $self, $node, $publisher_id, $find_season ) = @_;
    my $coverage_node;
    my $context;
    my $date_coverage_type;
    my $date_coverage_value;
    my $date_coverage_value_id;
	
    my $node_name = $node->nodeName;
#    print "node name: $node_name\n";

    # Search explicitly for an event-metadata node if called in the context of a sports-event node
    if ( $node_name eq 'sports-event' ) {
        $coverage_node = firstnode( $node, 'event-metadata[@date-coverage-type and (@date-coverage-value or @event-key)]' );
#                print "found coverage node in sports-event\n";
        
    }
    else {
    	# Changed - xpath was child::*/*[@date-coverage-type] but that was failing
        $coverage_node = firstnode( $node, 'child::*[@date-coverage-type and (@date-coverage-value or @event-key)]' );
# Debug code        
#        $self->{log}->log("[DBG] ***** looking in terms of child::*[date-coverage-type]");
#        $self->{log}->log("[DBG] ***** Node: $node_name");
    }
    if ( !$coverage_node ) {
		# In the case where a searc is being done in the context of e.g. a player-stats node
#   <statistic-metadata team-coverage="multi-team" date-coverage-type="season-regular" date-coverage-value="2009" />
#   <player>
#      <player-metadata xts:team-key="l.mlb.com-t.4" team-key="l.mlb.com-t.4">
#        <name full="Carlos Pena" />
#        <player-metadata-baseball />
#      </player-metadata>
#      <player-stats>
        $coverage_node = firstnode( $node, 'descendant-or-self::*[@date-coverage-type and (@date-coverage-value or @event-key)]' );
#		my $dct = (defined $coverage_node)? $coverage_node->getAttribute('date-coverage-type') : "not found";
#        $self->{log}->log("[DBG] ***** dct: $dct looking in terms of descendant-or-self::*[date-coverage-type] node_name: $node_name coverage_node: $coverage_node find_season: $find_season");
    }
#    my $found = (defined $coverage_node) ? 'yes': 'no';
#    my $coverage_node_name = ($found eq 'yes') ? $coverage_node->nodeName : 'unknown';
#    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
#    my $found_2 = (defined $coverage_node) ? 'yes': 'no';
#    $self->{log}->log("[DBG] ***** Found in descendant: $found Found after extra check: $found_2 find_season: $find_season node_name: $node_name coverage_node_name: $coverage_node_name");

    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
    if ( !$coverage_node ) {

# Try for a sibling node containing season information (e.g. teams in player career stats files)
#<statistic-metadata date-coverage-type="season-regular" date-coverage-value="2006" team-coverage="single-team" />
# <team id="l.nfl.com-t.31">
        $coverage_node = firstnode( $node, 'preceding-sibling::*[@date-coverage-type and (@date-coverage-value or @event-key)][1]' );
#        $self->{log}->log("[DBG] ***** looking in terms of preceding-sibling::*[date-coverage-type][1]");

    }
    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );

    if ( !$coverage_node ) {
        $coverage_node = firstnode( $node, 'ancestor::sports-event/event-metadata' );
#        print "found coverage node: ".$coverage_node->nodeName()." node_name: $node_name\n" if (defined $coverage_node);
    }
    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
    if ( !$coverage_node ) {
#        print "looking in: ancestor::*/*[\@date-coverage-type]\n";
        $coverage_node = firstnode( $node, 'ancestor::*/*[@date-coverage-type and (@date-coverage-value or @event-key)]' );
    }
    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
#    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
#    $self->{log}->log("[DBG] ***** node_name: $node_name");

    # Do extra looking if we're supposed to be finding a season
    # This if block basically looks for a sub-node that has a sports-content-code node with code-type attribute of 'season-type'
    # Extra digging is done to find the year as well. This allows sub_seasons and events_sub_seasons tables to be populated.
    if ( !$coverage_node && $find_season ) {
        $coverage_node = firstnode( $node, 'child::*/*/sports-content-code[@code-type="season-type"]' );
        if ( $coverage_node ) {
            $date_coverage_type = required_attribute( $coverage_node, 'code-key' );
            $date_coverage_type = ($date_coverage_type =~ /season/) ? $date_coverage_type : "season-$date_coverage_type";
            $context = $date_coverage_type;
        	$date_coverage_value = attribute( $coverage_node, 'season' );
        	if (! $date_coverage_value ) {
        		my $doc_season_node = firstnode( $node, '//sports-content-codes/sports-content-code[@code-type="season"]' );
        		if ( ! $doc_season_node ) {
        			return ( undef, undef, undef, undef );
        		}
        		$date_coverage_value = attribute( $doc_season_node, 'code-key' );
                $context = $date_coverage_type;
        	}

            return ($context, $date_coverage_type, $date_coverage_value);  
        }
        else {
            my ( $scc_context, $scc_date_coverage_type, $scc_date_coverage_value ) = $self->get_season_data_from_sport_content_codes( $node );

# Debug code            
#    $self->{log}->log("[DBG] Looking in SCC");
             if (defined $scc_context) {
                return ($scc_context, $scc_date_coverage_type, $scc_date_coverage_value);  
             }
        }
    }


    $coverage_node = $self->get_coverage_node_in_context_of_season( $coverage_node, $find_season );
# Debug code
#    $self->{log}->log("[DBG] coverage_node: $coverage_node");
    if ( !$coverage_node ) {
        $coverage_node =
          firstnode( $node, '//event-metadata[@date-coverage-type and (@date-coverage-value or @event-key)][1]' );
    }

    # Return undefined values if no node found
    if ( !$coverage_node ) {
        return ( undef, undef, undef, undef );
    }

    $date_coverage_type = required_attribute( $coverage_node, 'date-coverage-type' );
#    print "date coverage type: $date_coverage_type\n";
    $context = $date_coverage_type;
    

#    if ($node_name eq 'player-stats') {
#        print "date_coverage_type: $date_coverage_type\n";
#    }
    
    $date_coverage_value = attribute( $coverage_node, 'date-coverage-value' );

    # Career context has no date-coverage-value so return what can be returned here
    # since following this, if date-coverage-value is not available undef is returned.
    if ($date_coverage_type =~ 'career' || $date_coverage_value =~ /career/) {
#    	if ($context eq 'career') {
            return ('career', $date_coverage_type, $date_coverage_value);
#    	}
#    	else {
#            # return context (date_coverage_type), date_coverage_type, date_coverage_value
#            return ($context, $date_coverage_type, $date_coverage_value);
#    	}
    }
    
#    if ($date_coverage_type =~ /period/) {
#		my $node_name = $node->nodeName;
#		my $coverage_node_name = $coverage_node->nodeName;
#		$self->{log}->log("[DBG] node name: $node_name coverage node name: $coverage_node_name");
#
#    	my $period_entity = FeedfetcherDeluxe::Entities::Period->new( $self->{dbh}, $self->{log} );
#    	$date_coverage_value = $period_entity->find_period_id($coverage_node, $publisher_id);
#    }
    
    # If the date coverage type is event and there was not a date coverage value, look for an event key
    if ( $date_coverage_type =~ /event/ ) {
    	if (! $date_coverage_value) {
        	$date_coverage_value = attribute( $coverage_node, 'event-key' );
    	}
        # Return emptyhanded if no valid date coverage value found
        if ( ! $date_coverage_value ) {
            return ( undef, undef, undef, undef );
        }
        $context = $date_coverage_type;
    
    }
    elsif (! defined $date_coverage_value) {
        return ( undef, undef, undef, undef );
    }
    
    $date_coverage_value =~ s/\s*//g;
           my $coverage_node_name = $coverage_node->nodeName();
#    print "coverage_node_name: $coverage_node_name date_coverage_type: $date_coverage_type date_coverage_value: $date_coverage_value context: $context\n";
    

    return ($context, $date_coverage_type, $date_coverage_value);	
}

sub get_context {
	my ( $self, $node ) = @_;
	
	my $context_node;
	$context_node = firstnode( $node, 'ancestor::sports-event/event-metadata' );
	if (defined $context_node) {
		
	}
	else {
		$context_node = firstnode( $node, 'ancestor::statistic/statistic-metadata' );
	}
	
	return undef unless (defined $context_node);
	my $context = $context_node->getAttribute('date-coverage-type');
#	print "context: $context\n";
	return $context;
}

# NOTE
# This is not a perfect fit for season since date coverage type could very well be an event
# This sub is called from outside of the module
sub get_coverage {
	my ( $self, $node, $publisher_id ) = @_;

#    $self->{log}->log("[DBG] ***** GET COVERAGE");
	my $coverage_node;

	my $season_id;
	my $sub_season_id;
	my $league_id;
	my $context;
	my $date_coverage_type;
	my $date_coverage_value;
	my $date_coverage_value_id;


    ($context, $date_coverage_type, $date_coverage_value) = $self->get_coverage_data($node, $publisher_id);

    if ( $context eq 'season' || $context eq 'season-' ) {
       my ( $scc_context, $scc_date_coverage_type, $scc_date_coverage_value ) = $self->get_season_data_from_sport_content_codes( $node );
       $context='season-regular' if $context eq 'season-';

       if (defined $scc_context) {
           # If the season year is the same
           if ( $date_coverage_value eq $scc_date_coverage_value && defined $scc_date_coverage_type ) {
       	        # Update values for date_coverage_type and context (e.g. 'season' to 'post-season')
                $date_coverage_type = $scc_date_coverage_type;
                $context = $scc_context;
            }
       }
    }

    if ($context =~ /period/) {
		my $node_name = $node->nodeName;
		$context = $self->get_context($node);
		return ( undef, undef, undef, undef ) unless (defined $context);
		
        $date_coverage_type = 'periods';
    	my $period_entity = FeedfetcherDeluxe::Entities::Period->new( $self->{dbh}, $self->{log} );
    	$date_coverage_value_id = $period_entity->find_period_id($node, $publisher_id);
#    $self->{log}->log("[DBG] ***** node_name: $node_name context: $context date_coverage_type: $date_coverage_type date_coverage_value: $date_coverage_value date_coverage_value_id: $date_coverage_value_id");
    	return ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id);
    }

    if ( !$context ) {
        # Essentially, return undefined values if no node found
		return ( $context, $date_coverage_type, $date_coverage_value,
			$date_coverage_value_id );
	}

    my $to_return_context;
    my $to_return_date_coverage_type;
    my $to_return_date_coverage_value;
    my $to_return_date_coverage_value_id;
    

    ( $to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id ) = 
        $self->date_coverage_store( $node, $context, $date_coverage_type, $date_coverage_value, $publisher_id );

#    if ($to_return_context eq 'period') {
#        return ( $to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id );	
#    }
    

    # Leave now if we have career stats as context        
    if ($to_return_context eq 'career') {
        # $to_return_date_coverage_type will be 'affiliations' here
        return ( $to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id );	
    }

    # The immediate context is event, but try to find and store any season information too
    if ( $to_return_context =~ /event/ ) {
#    	print "$to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id\n";
        # $to_return_date_coverage_type will be 'events' here
    	# Try again, this time passing a flag to look for season data
        ($context, $date_coverage_type, $date_coverage_value) = $self->get_coverage_data( $node, $publisher_id, 1 );

        # Sometimes there's no season data. Don't try to store what's not there.
        unless ( $context eq 'event' ) {
            if ( $context eq 'season' || $context eq 'season-' ) {
               my ( $scc_context, $scc_date_coverage_type, $scc_date_coverage_value ) = $self->get_season_data_from_sport_content_codes( $node );
                # If the season year is the same

                if (defined $scc_context) {
                    if ( $date_coverage_value eq $scc_date_coverage_value && defined $scc_date_coverage_type ) {
                        # Update values for date_coverage_type and context (e.g. 'season' to 'post-season')
#                        $date_coverage_type = $scc_date_coverage_type;
#                        print "date_coverage_value: $date_coverage_value scc_date_coverage_value: $scc_date_coverage_value scc_date_coverage_type: $scc_date_coverage_type\n";
#                        $date_coverage_type = 'sub_seasons';
                        $date_coverage_type = $scc_date_coverage_type;
                        $context = $scc_context;
                    }
                }
            }

            ( $context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id ) = 
                $self->date_coverage_store( $node, $context, $date_coverage_type, $date_coverage_value, $publisher_id );
 
             # Only proceed if something has been found
            if ( $context ) {
                 # Store season-regular if the season is just 'season'
                if ( $context eq 'season' ) {
#                	$self->{log}->log("[DBG] store_sub_season_1");
                    $sub_season_id = $self->store_sub_season( "season-regular", $date_coverage_value, "season-regular", $date_coverage_value_id);
                }
                else {
        	       $sub_season_id = $date_coverage_value_id;
                }

                # Store events_sub_seasons information
                $self->store_event_sub_season( $to_return_date_coverage_value_id, $sub_season_id );
            }
        }
    }
    else {
    	# If some sort of season has been found, the type is sub_seasons
        if ($to_return_context =~ /season/) {
            $to_return_date_coverage_type = 'sub_seasons';
        }
        # We've already covered periods, career, and events. Try to find season info in s-c-c section, otherwise return undef (failure)
        else {
            my ( $scc_context, $scc_date_coverage_type, $scc_date_coverage_value ) = $self->get_season_data_from_sport_content_codes( $node );
        	
            # Make an attempt to store any season data if it's in top-level s-c-c data
            if ($scc_context =~ /season/) {
                # Update values for date_coverage_type and context (e.g. 'season' to 'post-season')
#                $context = $scc_context;
#               $date_coverage_type = $scc_date_coverage_type;
#                $date_coverage_type = 'sub_seasons';

#                $date_coverage_value = $scc_date_coverage_value;

                # Store season information just found in s-c-c section
                ( $to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id ) = 
                    $self->date_coverage_store( $node, $scc_context, $scc_date_coverage_type, $scc_date_coverage_value, $publisher_id ) if $scc_context =~ /season/;
                $to_return_date_coverage_type = 'sub_seasons';
            }
            else {
                return (undef, undef, undef, undef);
            }
        }
    }
    
#    $self->{log}->log("[DBG] ***** to_return_context: $to_return_context, date_coverage_type: $date_coverage_type, date_coverage_value: $date_coverage_value");
    if ($to_return_context =~ /event/) {
    	$to_return_context = $self->get_context($node);
    }
    return ( $to_return_context, $to_return_date_coverage_type, $to_return_date_coverage_value, $to_return_date_coverage_value_id );
}

sub get_league_id {
    my ( $self, $node, $publisher_id ) = @_;
	
    my $affiliation_entity =
      FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh},
        $self->{log} );

	my $league_id;
	my $league_key;
	
    # TODO: This node can be the child of a sibling node. Find out if firstnode is best way to get this node or if
    # sometimes other leagues might be in same document.
    
    my $node_name = $node->nodeName();
        
    my $league_node =
      firstnode( $node, "descendant::sports-content-code[\@code-type='league']" );
#        $self->{log}->log("[DBG] ***** found league node at descendant") if defined $league_node;
	if (defined $league_node) {
		$league_key = $league_node->getAttribute('code-key' );
		$league_key = undef if $league_key =~ /^\s*$/;
	}
    unless (defined $league_node && defined $league_key) {
        $league_node = firstnode( $node, "preceding::sports-content-codes[1]/sports-content-code[\@code-type='league']" );
		if (defined $league_node) {
			$league_key = $league_node->getAttribute('code-key' );
			$league_key = undef if $league_key =~ /^\s*$/;
		}
    }

    unless (defined $league_node && defined $league_key) {
        $league_node = firstnode( $node, "//sports-content-codes/sports-content-code[\@code-type='league']" );
		if (defined $league_node) {
			$league_key = $league_node->getAttribute('code-key' );
			$league_key = undef if $league_key =~ /^\s*$/;
		}
    }

    
    if (defined $league_node && defined $league_key) {
        $league_id = $affiliation_entity->retrieve_id_by_key(
            {
                'affiliation_key'  => $league_key,
                'affiliation_type' => 'league',
                'publisher_id'     => $publisher_id
            }
        );
    }
    if ( ! defined $league_id && defined $league_key) {

        # Create an affiliation record
        $league_id = $affiliation_entity->create(
            {
                'affiliation_key'  => $league_key,
                'affiliation_type' => 'league',
                'publisher_id'     => $publisher_id
            }
        );
    }
    return $league_id;
}

sub date_coverage_store {
    my ( $self, $node, $context, $date_coverage_type, $date_coverage_value, $publisher_id ) = @_;
	
	my $league_id;
	my $season_id;
	my $sub_season_id;
	
	my $node_name = $node->nodeName();
	
    my $event_entity =
      FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );
    my $affiliation_entity =
      FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh},
        $self->{log} );

	my $date_coverage_value_id;

    my $league_id = $self->get_league_id($node, $publisher_id);
##    $self->{log}->log("[DBG] ***** node_name: $node_name league_id: $league_id");
#
#    if ($date_coverage_type =~ /period/) {
#		my $node_name = $node->nodeName;
##		my $coverage_node_name = $coverage_node->nodeName;
##		$self->{log}->log("[DBG] node name: $node_name coverage node name: $coverage_node_name");
#
#    	my $period_entity = FeedfetcherDeluxe::Entities::Period->new( $self->{dbh}, $self->{log} );
#    	$date_coverage_value_id = $period_entity->find_period_id($node, $publisher_id);
##    $self->{log}->log("[DBG] ***** node_name: $node_name date_coverage_value_id: $date_coverage_value_id");
#    	
#    }

    
    if ( $date_coverage_type =~ /event/ ) {
        $date_coverage_type = 'events';
        $date_coverage_value_id =
          $event_entity->retrieve_id_by_key( {'event_key'=>$date_coverage_value,'publisher_id'=>$publisher_id} );
        if (! $date_coverage_value_id ) {
        	$self->{log}->log("[ADV] ***** Event not found for date coverage value $date_coverage_value");
        }
    }
    
    elsif ( $date_coverage_type =~ /career/ || $date_coverage_value =~ /career/) {
        $date_coverage_type     = 'affiliations';
        # Don't set date_coverage_value_id
#        $date_coverage_value_id = $league_id;
    }
    
    
    # Look for any sort of season date coverage type
    elsif ( $date_coverage_type =~ /season/ ) {
#    $self->{log}->log("[DBG] date_coverage_type: $date_coverage_type date_coverage_value: $date_coverage_value");
    	
    	$_ = $date_coverage_value;
    	s/[^\d]//g;
    	$date_coverage_value = $_;


    	if (defined $league_id) {
            $season_id = $self->retrieve_id_by_key( 
              {
                'season_key' => $date_coverage_value,
          	    'league_id' => $league_id,
                'publisher_id' => $publisher_id 
            }
            );
    	}
    	else {
            $season_id = $self->retrieve_id_by_key( 
              {
                'season_key' => $date_coverage_value,
                'publisher_id' => $publisher_id 
            }
          );
    	}
        if ( !$season_id ) {
        	if (! defined $league_id) {
                $self->{log}->log("[ADV] season_id not found for $date_coverage_value. Creating season entry now.");
                $season_id = $self->create(
                    {
                        'season_key'   => $date_coverage_value,
                        'publisher_id' => $publisher_id
                    }
                );
        	}
        	else {
                $self->{log}->log("[ADV] season_id not found for $date_coverage_value, league_id: $league_id. Creating season entry now.");
                $season_id = $self->create(
                    {
                        'season_key'   => $date_coverage_value,
                        'league_id'    => $league_id,
                        'publisher_id' => $publisher_id
                    }
                );        		
        	}
        }
        # Look for things like 'season-regular' and then deal with sub-seasons
        if ( $date_coverage_type ne 'season' ) {
           if ($date_coverage_type eq 'season-') {
#           print "season: $date_coverage_type\n";
           	    $context = 'season-regular';
#            	$date_coverage_type = 'sub_seasons';
#                $sub_season_id = $self->store_sub_season( $date_coverage_type, $date_coverage_value, $context, $season_id);
                $sub_season_id = $self->store_sub_season( 'season-regular', $date_coverage_value, $context, $season_id);
                $date_coverage_value_id = $sub_season_id;
            }
            else {
#                $date_coverage_type = 'sub_seasons';
    #           $sub_season_id = $self->store_sub_season( $date_coverage_type, $date_coverage_value, $context, $season_id);
                $sub_season_id = $self->store_sub_season( $date_coverage_type, $date_coverage_value, $context, $season_id);
                # verify sub-season is done

                $date_coverage_value_id = $sub_season_id;
            }
        }
        else {
            # Store sub_season as 'season_regular'
            
        	$sub_season_id = $self->store_sub_season( 'season-regular', $date_coverage_value, "season-regular", $season_id);
#            $date_coverage_type     = 'sub_seasons';
#            $date_coverage_type     = 'seasons';
            $date_coverage_value_id = $season_id;
        }
    }

    return ( $context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id );
}

sub store_sub_season {
	my ( $self, $date_coverage_type, $date_coverage_value, $sub_season_key, $season_id ) = @_;
    
#    print "date_coverage_type: $date_coverage_type date_coverage_value: $date_coverage_value sub_season_key: $sub_season_key season_id: $season_id\n";
    
    my $sub_season_id;
    my $sub_season_entity =
        FeedfetcherDeluxe::Entities::Sub_season->new( $self->{dbh}, $self->{log} );
        
    $sub_season_key =~ s/-/_/g;
    $sub_season_key = $date_coverage_value . "_" . $sub_season_key;
#    print "new sub_season_key: $sub_season_key\n";

    my $sub_season_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'sub_seasons');
    $sub_season_record_mapper->retrieve({'sub_season_key' => $sub_season_key, 'season_id' => $season_id});
    $sub_season_record_mapper->set({'sub_season_type' => $date_coverage_type});
    $sub_season_id = $sub_season_record_mapper->save();

#    $sub_season_id = $sub_season_entity->retrieve_id_by_key( {'sub_season_key' => $sub_season_key, 'season_id' => $season_id} );
#
#    if ( !$sub_season_id ) {
#
#        $sub_season_id = $sub_season_entity->create(
#            {
#                'sub_season_type' => $date_coverage_type,
#                'sub_season_key'  => $sub_season_key,
#                'season_id'       => $season_id
#            }
#        );
#    }
    return $sub_season_id;
}

sub store_event_sub_season {
    my ( $self, $event_id, $sub_season_id ) = @_;
	
	return unless defined $event_id;

#    my $events_sub_season_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'events_sub_seasons');
#    $events_sub_season_record_mapper->retrieve({'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id});
#    $events_sub_season_record_mapper->save();

     my $events_sub_season_entity = FeedfetcherDeluxe::Entities::Events_sub_season->new( $self->{dbh}, $self->{log} );
     my $events_sub_seasons_count = $events_sub_season_entity->retrieve_row_count( {'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id} );
     $events_sub_season_entity->create( {'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id} ) if ($events_sub_seasons_count eq '0');


#   my $events_sub_season_entity = FeedfetcherDeluxe::Entities::Events_sub_season->new( $self->{dbh}, $self->{log} );
#    # Changed to create only if not there already
#	my $events_sub_season_hash = $events_sub_season_entity->retrieve_by_key( {'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id} );
##    $events_sub_season_entity->delete( {'event_id'=>$event_id} );
#    $events_sub_season_entity->create( {'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id} ) unless defined $events_sub_season_hash;
#    $events_sub_season_entity->create( {'event_id'=>$event_id, 'sub_season_id'=>$sub_season_id} );
}

1;
