# $Id: Site.pm 2953 2012-05-30 22:15:42Z ian $

package FeedfetcherDeluxe::Entities::Site;

use vars qw(@ISA);

use strict;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Location;
use FeedfetcherDeluxe::Entities::Display_name;
use FeedfetcherDeluxe::Entities::Sports_property;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;

=head1 NAME

FeedFetcherDeluxe::Entities::Site

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub  store_from_doc {
    my ( $self, $node, $publisher_id ) = @_;

    my $site_id;
    my $site_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'sites');
    
    # Not currently used, but planned
    my $location_id;
    # Assuming event-metadata node
    return undef if (! $node->nodeName eq 'event-metadata');
    
    my $site_key;
    my $site_name;
    my $site_metadata_node = firstnode($node, 'site/site-metadata');
    my $site_name_node = firstnode($node, 'site/site-metadata/name');
    my $location_node = firstnode($node, 'site/site-metadata/home-location');
    
    if (defined $location_node) {
        my $location_entity = FeedfetcherDeluxe::Entities::Location->new($self->{dbh}, $self->{log});
        $location_id = $location_entity->store_from_doc($location_node);
    }
    
    $site_key = $node->getAttribute('site-key');
    if (defined $site_name_node) {
        $site_name = $site_name_node->getAttribute('full');
    }
    
    # Try more to get a site key
    if (! defined $site_key) {
    	if (defined $site_metadata_node) {
            $site_key = $site_metadata_node->getAttribute('site-key');
    	}
        return undef if ! defined $site_key;
    }
    
    $site_mapper->retrieve(
    {
        'site_key'=>$site_key, 
        'publisher_id'=>$publisher_id
    }
    );

    $site_mapper->set(
        {
        	'location_id'=>$location_id
        }
    );
    $site_id = $site_mapper->save();
    
    my $sports_property_entity = FeedfetcherDeluxe::Entities::Sports_property->new($self->{dbh}, $self->{log});
    $sports_property_entity->process_all($site_metadata_node, 'sites', $site_id, $publisher_id);

    if (defined $site_name) {
        my $display_name_entity = FeedfetcherDeluxe::Entities::Display_name->new( $self->{dbh}, $self->{log} );
        my $display_name_entity_id=$display_name_entity->retrieve_id_by_key(
       {
            'entity_type'=>'sites', 
            'entity_id'=>$site_id, 
            'language'=>'en-US'
        }
        );
        if (! $display_name_entity_id) {
            $display_name_entity->create(
            {
                'entity_type'=>'sites', 
                'entity_id'=>$site_id, 
                'language'=>'en-US',
                'full_name'=>$site_name
            }
            );          
        }
        else {
            $display_name_entity->update(
            {
                'entity_type'=>'sites', 
                'entity_id'=>$site_id, 
                'language'=>'en-US',
                'full_name'=>$site_name
            },
                {'id'=>$display_name_entity_id}
            );
        }
    }
    
    return $site_id;
}

1;