# $Id: Soccer_action_participant.pm 2993 2012-08-14 22:00:29Z ian $

package FeedfetcherDeluxe::Entities::Soccer_action_participant;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Soccer_action_play;
use FeedfetcherDeluxe::XtossUtil;
use Core::XmlUtil;

#use Core::DbUtil;

=head1 NAME

FeedFetcherDeluxe::Entities::Soccer_action_participant

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;
}

sub store_from_doc {
    my ($self, $node, $publisher_id, $play_id) = @_;
    
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});
    my $player_idref = $node->getAttribute('player-idref') || $node->getAttribute('recipient-idref');
#    $player_idref = $player_idref || $node->getAttribute('recipient-idref');
    
    my $person_id = $person_entity->find_id_from_idref($node, $player_idref, $publisher_id);
    if (! $person_id) {
        $self->{log}->log("[ADV] soccer_action_participant add failure: no player information found in document for player-idref '$player_idref'");               
        return;
    }

    my $team_idref = attribute($node, 'team-idref');
    my $team_id;
    # Make an extra effort to find the team for a player if that attribute has not been attached to the node
    if (! defined $team_idref) {
        $team_idref = $person_entity->find_team_idref_from_player_idref($node, $player_idref);
#        print "team_idref: $team_idref\n";
    }
    if (defined $team_idref) {
        my $team_entity = FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
        $team_id = $team_entity->find_id_from_idref($node, $team_idref, $publisher_id);
    }
    else {
        $self->{log}->log("[ADV] soccer_action_participant add failure: no team information found in document for player-idref '$player_idref'");             
        return;
    }

	my $role = attribute($node, 'role');
    if (! defined $role) {
        $role = attribute($node, 'participant-role');
    }	
    $role = undef if $role =~ /^\s*$/;
	if (! defined $role) {
        # Possible soccer-assisting-player child nodes
        my $node_name = $node->nodeName;
        if ($node_name =~ /attempt/) {
    	   $role = 'shooter';
        }
        elsif ($node_name =~ /assisting/) {
            $role = 'assist';
        }
        elsif ($node_name =~ /score$/) {
            $role = 'scorer';
        }
#        else {
##    $self->{log}->log("[DBG] node_name: $node_name");
#	       $role = 'scorer';
#        }
	}
    
#    my $soccer_action_play_id =  $self->retrieve_id_by_key(
#        {
#            'soccer_action_play_id'  => $play_id,
#            'person_id' => $person_id,
#            'participant_role'=>$role
#        }
#    );
#    if (! $soccer_action_play_id) {
#        $self->create(
#            {
#                'soccer_action_play_id'  => $play_id,
#                'person_id' => $person_id,
#                'participant_role'=>$role
#            }
#        );
#    }
#    else {
#        $self->update(
#        {
#                'participant_role'=>$role
#        }
#        , {
#            'soccer_action_play_id'  => $play_id,
#            'person_id' => $person_id
#        }
#        );
#   }
   
   
    my $soccer_action_participant_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new(
        $self->{dbh}, $self->{log}, 'soccer_action_participants'
    );

    # Get a count of the rows that are there, in case cleanup is needed
#    my $action_participant_row_count =  $self->retrieve_row_count(
#        {
#            'soccer_action_play_id'  => $play_id,
#            'person_id' => $person_id,
#            'participant_role'=>$role
#        }
#    );

    # This should not happen, but if old duplicate rows are in the db, this will clear them out
#    if ($action_participant_row_count > 1) {
#        $self->delete(
#        {
#            'soccer_action_play_id'  => $play_id,
#            'person_id' => $person_id,
#            'participant_role'=>$role
#        }
#        );
#        $action_participant_row_count = 0;
#    }

    # The spec says that there will be only one participant per play, therefore we are willing to
    # do an update overwriting a previous person_id to fix an error from a previous file
    $soccer_action_participant_record_mapper->retrieve(
        {
            'soccer_action_play_id'  => $play_id,
            'person_id' => $person_id
        }
    );
    $soccer_action_participant_record_mapper->set(
        {
            'team_id' => $team_id,
            'participant_role'=>$role           
        }
    );
    $soccer_action_participant_record_mapper->save();
   
}

sub store_all_from_doc {
    my ($self, $node, $publisher_id, $event_state_id) = @_;
    
    my $soccer_action_play_entity = FeedfetcherDeluxe::Entities::Soccer_action_play->new($self->{dbh}, $self->{log});

    my $play_id;
    if ($event_state_id) {
        $play_id = $soccer_action_play_entity->retrieve_id_by_key({'soccer_event_state_id'=>$event_state_id});
    }
    else {
        $play_id = $soccer_action_play_entity->retrieve_max_id();        
    } 
    $play_id = 0 if ! defined $play_id;

    # We have to delete since errors can occur in player for a play and the role for a participant may not be present
    # i.e. we may not be able to uniquely identify a participant for updating so bad records could creep in
    my $delete_sql = <<EOS;
            DELETE FROM soccer_action_participants
            WHERE soccer_action_play_id = $play_id
EOS
    my $sth = $self->{dbh}->prepare($delete_sql);
    $sth->execute();

#    print "node name: ".$node->nodeName."\n";
    $self->store_from_doc($node, $publisher_id, $play_id);

    my @participant_nodes = $node->findnodes("child::action-soccer-play-participant");
    foreach my $participant_node (@participant_nodes) {
        $self->store_from_doc($participant_node, $publisher_id, $play_id);
    }
    my @participant_nodes = $node->findnodes("child::action-soccer-assisting-player");
    foreach my $participant_node (@participant_nodes) {
        $self->store_from_doc($participant_node, $publisher_id, $play_id);
    }
#    $self->store_from_doc($node, $publisher_id, $play_id);
}

1;