# $Id: SportsmlDocument.pm 2772 2010-05-06 18:12:59Z ian $

package FeedfetcherDeluxe::Entities::SportsmlDocument;

use DBI();
use Core::Log;
use Core::DbUtil;
use Core::XmlUtil;
#use FeedfetcherDeluxe::Entities::Publisher;

=head1 NAME
Core::Entities::Entity

=head1 SYNOPSIS
	my $sportsml_document = Core::Entities::SportsmlDocument->new($doc, $log);
	$self->log("[DBG] fixture name: ".$sportsml_document->{'fixture_name'});

=head1 DESCRIPTION
	SportsmlDocument encapsulates methods that allow retrieval of core data from a sportsml
	document, such as content codes, publisher, event key, etc.

=cut

sub new {
    my ($class, $doc, $log) = @_;

	my $self = {};
	bless $self, $class;

	$self->initialize($doc, $log);

	return $self;
}

sub initialize {
	my ($self, $doc, $log) = @_;

	$self->{doc} = $doc;
	$self->{"log"} = $log;
	$self->{data} = $self->get_basic_data($doc);	
}

# Probably should move this
# used in get_basic_data
sub get_league_key {
	my ($metanode) = @_;

	my $general_league = "l.sportsml.com.general";
	my $league_query =
		'sports-content-codes/sports-content-code[@code-type="league"]';

	my $league_content_node = firstnode($metanode, $league_query, 1);

	return $general_league if !$league_content_node;

	my $league = required_attribute($league_content_node, "code-key");

	return $general_league if !$league;

	return $league;
}

sub get_sports_conent_node {
    my ($self, $node) = @_;
	
    # To ensure we are in the scope of the current document, first find the <sports-content node, then look at sports-content-codes
    # - if top, find sports-content-codes
    # - otherwise, find top, then find sports-content-codes
    # - look for the one in question.
    my $base_node;
        $base_node = solonode_optional($node, "ancestor::sports-content");
        if ( ! defined $base_node ) {
            $base_node = solonode($node, "child::sports-content");
        }
        if ( ! defined $base_node ) {
            $base_node = $node;          
        }
        
#    }
    my $base_node_name = $base_node->nodeName;
#    $self->{log}->log("[DBG]  base node: $base_node_name");
    $node_name = $base_node->nodeName;
#   $self->{log}->log("[DBG]  node name: $node_name");
    
    return $base_node;
}

sub get_content_code_key {
    my ($self, $code_type) = @_;

    # TODO
    # use get_sports_conent_codes_node
    my $content_codes_nodes_node = $self->get_sports_conent_node($self->{doc});
    my $content_node = firstnode($content_codes_nodes_node, "descendant::sports-content-codes/sports-content-code[\@code-type='$code_type']");

    my $code_key;
    if ($content_node) {
        $code_key = attribute($content_node, 'code-key');
    }
    else {
#    	my $node_name = $self->{doc}->nodeName;
#        $self->{log}->log("[DBG] could not find content node via $node_name for content type $code_type");    	
    }
    return $code_key;
}

# TODO: Needs more testing
sub get_content_codes {
	my ($self, @content_codes_nodes) = @_;

	my @sport_content_nodes = ();
	foreach my $content_codes_node (@content_codes_nodes) {
		my @current_sport_content_nodes =
			$content_codes_node->getChildrenByTagName("sports-content-code");
		push @sport_content_nodes, @current_sport_content_nodes;
	}

	my %codes = ();
	my %code_names = ();
	my $doc_id = $self->{doc_id};

	foreach my $node (@sport_content_nodes) {
		my $code_type = $node->getAttribute('code-type');
		# xtoss
		my $code_name = $node->getAttribute('code-name');
		my $code_key = $node->getAttribute('code-key');
		$self->{'log'}->log("[ADV] Required attribute 'code-type' " .
			"not defined for 'sports-content-code' ($doc_id)") if !$code_type;
		next if (!$code_type || !$code_key);
		my $list = $codes{$code_type};
		$list = [] if !defined $list;
		push @$list, $code_key;
		$codes{$code_type} = $list;

		# imarsman 2/09/07
		my $code_name_key = "$code_type.$code_key";

		$code_names{$code_name_key} = $code_name;
		# $self->log("[DBG] code-key: $code_key");
		# $self->log("[DBG] code-name: $code_name");
		# $self->log("[DBG] code-name key: $code_name_key");

		# xtoss - need code here to record as part of this hash the
		# content code NAME in addition to its KEY and TYPE
		# (needed for publisher-name, in addition to other things)
	}

	# OLD retval - just the codes
	# return /%codes;
	# NEW retval - content codes plus a hash of content code names
	my %ret_val = ();
	$ret_val{'codes'}=\%codes;
	$ret_val{'code_names'}=\%code_names;

	return \%ret_val;
}

sub get_basic_data {
	my ($self, $doc) = @_;

	my $data = {};

    # TODO
    # use get_sports_conent_codes_node

	my $metanode = $data->{'metanode'} = solonode_optional($doc, "//sports-content/sports-metadata");
		
	return $data if ! $metanode;

	my $content_codes_node = $data->{'content_codes_node'} = solonode($metanode, "sports-content-codes");

	$data->{'doc_id'} = required_attribute($metanode, "doc-id");
	$self->{doc_id} = $data->{'doc_id'};

	# OLD content code fetch
	#my $content_codes = $self->get_content_codes($content_codes_node);
	# NEW content code fetch
	my $content_code_data = $self->get_content_codes($content_codes_node);
	my $content_codes = $content_code_data->{'codes'};
	my $code_names = $content_code_data->{'code_names'};
	# $self->log("[DBG] content: $content_codes");


	$data->{'content_codes'} = $content_codes;
	$data->{'code_names'} = $code_names;

	$data->{'fixture_key'} = required_attribute($metanode, "fixture-key");

	# xtoss
	$data->{'fixture_name'} = required_attribute($metanode, "fixture-name");
	$data->{'document_class'} = required_attribute($metanode, "document-class");

	my $publisher_keys = $content_codes->{'publisher'};
	die "No publisher-key in document" if !$publisher_keys || @$publisher_keys == 0;
	$data->{'publisher_key'} = @{$publisher_keys}[0];

	# xtoss
	my $source_keys = $content_codes->{'source'};
	$data->{'source_key'} = @{$source_keys}[0];

	$self->{'log'}->log(
		"[ADV] More than one publisher in sports-content-codes. " .
		"Using first one. (" . $self->{doc_id} . ")") if
		@$publisher_keys > 1;

	$data->{'league_key'} = get_league_key($metanode);
	$data->{date_time} = required_attribute($metanode, 'date-time');

	# Add the items from the data hash as class attributes
	while (my ($key, $value) = each(%$data)){
   		$self->{$key} = $value;
   	}

	# Behaviour as per DBStore.pm get_basic_data
	return $data;
}

sub data {
    my ($self) = @_;
    return \$self->{data};	
}

sub log {
	my ($self) = @_;
	
	return $self->{log};
}

sub document {
	my ($self) = @_;
	
	return $self->{document};
}

1;