# $Id: Standing.pm 2123 2008-07-23 19:22:08Z ian $

package FeedfetcherDeluxe::Entities::Standing;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Standing_subgroup;
#use Core::DbUtil;
use Core::XmlUtil;
use Core::Util;
use FeedfetcherDeluxe::Entities::Season;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Affiliation;


=head1 NAME

FeedFetcherDeluxe::Entities::Standing

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub  store_from_doc {
    my ( $self, $participant_type, $node, $publisher_id ) = @_;
    
    # stadings_type from s-c-c
    
}

sub  store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
    my $season_entity = FeedfetcherDeluxe::Entities::Season->new($self->{dbh}, $self->{log});
     
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{log});
    my $standing_subgroup_entity = FeedfetcherDeluxe::Entities::Standing_subgroup->new( $self->{dbh}, $self->{log} );
    
    # Should we bail with no standing_type?

    my $standing_type = $sportsml_document->get_content_code_key('standing-type');
    my $standing_affiliation = $sportsml_document->get_content_code_key('standing-affiliation');
    my $standing_affiliation_key = $sportsml_document->get_content_code_key("$standing_affiliation");
    my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, $self->{log} );
    my $standing_affiliation_id = $affiliation_entity->find_affiliation_id($standing_affiliation_key, $standing_affiliation, $publisher_id);
    return unless (defined $standing_type && defined $standing_affiliation && defined $standing_affiliation_key && defined $standing_affiliation_id);
    
    my @standings_nodes = $node->findnodes("//standing");
    foreach my $standing_node (@standings_nodes) {

#        my $standing_affiliation_node = solonode_optional($standing_node, "standing-metadata/sports-content-codes/sports-content-code[\@code-type='$standing_type']");
#        if ( defined $standing_affiliation_node ) {
#            $standing_affiliation_key = $standing_affiliation_node->getAttribute('code-key');
#            $standing_affiliation_id = $affiliation_entity->find_affiliation_id($standing_affiliation_key, $standing_type, $publisher_id);          
#        }

        my ($context, $date_coverage_type, $date_coverage_value, $date_coverage_value_id) = $season_entity->get_coverage($standing_node, $publisher_id);

        # Get the current date/time
        my $timestamp = Core::Util::current_ISO8601_time();
        
        # What to do with this?
        
        my $standing_id = $self->retrieve_id_by_key( 
        {
            'standing_type' => $standing_type,
            'affiliation_id' => $standing_affiliation_id,
            'sub_season_id' => $date_coverage_value_id
        }
        );
        
        if ( ! defined $standing_id ) {
        	$standing_id = $self->create(
        	{
                'standing_type' => $standing_type,
                'affiliation_id' => $standing_affiliation_id,
                'sub_season_id' => $date_coverage_value_id,
                'last_updated' => $timestamp
        	}
        	);
        }
        else {
            $self->update(
            {
                'last_updated' => $timestamp
            }, 
            {
            	'id' => $standing_id
            }
            );
        }
        $standing_subgroup_entity->store_all_from_doc( $standing_type, $standing_id, $standing_affiliation_id, $standing_node, $publisher_id );
    }    
    
}

1;