# $Id: Standing_subgroup.pm 3077 2016-04-07 19:56:16Z ian $

package FeedfetcherDeluxe::Entities::Standing_subgroup;

use FeedfetcherDeluxe::Entities::Entity;
#use Core::DbUtil;
use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Outcome_total;

=head1 NAME

FeedFetcherDeluxe::Entities::Standing_subgroup

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub  get_subgroup_id {
    my ( $self, $standing_id, $affiliation_id, $competition_scope, $alignment_scope, $duration_scope ) = @_;
    
    my $sql = qq{
        SELECT id
        FROM $self->{table_name}
        WHERE standing_id=?
        AND affiliation_id=?
        AND competition_scope=?
    };
    
    my @args = ();
    push @args, $standing_id;
    push @args, $affiliation_id;
    push @args, $competition_scope;
    
    if ( defined $alignment_scope ) {
        $sql .= "\nAND alignment_scope=?";
        push @args, $alignment_scope;
    }
    else {
        $sql .= "\nAND alignment_scope IS NULL";    	
    }
    
    if ( defined $duration_scope ) {
        $sql .= "\nAND duration_scope=?";
        push @args, $duration_scope;
    }
    else {
        $sql .= "\nAND duration_scope IS NULL";            	
    }
#    $self->{log}->log("[DBG] $sql");
    
    my $sth = $self->{dbh}->prepare($sql);
    $sth->execute(@args);    
    my $hashref = $sth->fetchrow_hashref();
    $sth->finish;
    return undef if ( ! defined $hashref );

    $ret_val = $hashref->{id};
    return $ret_val;
}

sub store_from_doc {
}


sub store_all_from_doc {
    my ( $self, $standing_type, $standing_id, $affiliation_id, $node, $publisher_id ) = @_;
    
    my $outcome_total_entity = FeedfetcherDeluxe::Entities::Outcome_total->new( $self->{dbh}, $self->{log} );
    my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, $self->{log} );
    my $team_entity = FeedfetcherDeluxe::Entities::Team->new($self->{dbh}, $self->{log});
    my $person_entity = FeedfetcherDeluxe::Entities::Person->new($self->{dbh}, $self->{log});

    # Get local affiliation if it's available, otherwise leave at default incoming value
    my $standing_affiliation_node = solonode_optional($node, "standing-metadata/sports-content-codes/sports-content-code[\@code-type='$standing_type']");
    if ( defined $standing_affiliation_node ) {
        my $standing_affiliation_key = $standing_affiliation_node->getAttribute('code-key');
        $affiliation_id = $affiliation_entity->find_affiliation_id($standing_affiliation_key, $standing_type, $publisher_id);          
    }
    if (! defined $affiliation_id) {
       $self->{log}->log("[ADV] No affilation found. Cannot store standing_subgroup data.");
       return;
    }

    my $node_name = $node->nodeName();

    my @team_nodes = $node->findnodes("team");
    my @player_holder_nodes = $node->findnodes("player");

    # If findnodes found elements
    my @outcome_holder_nodes = ( @team_nodes) ?  @team_nodes: @player_nodes;
    my $node_name = $node->nodeName();
#            print "store_all_from_doc node_name: $node_name\n";
    # In case no team or player nodes
    return unless @outcome_holder_nodes;
    
    # Spin thru outcome holder nodes
    foreach my $outcome_holder_node (@outcome_holder_nodes) {
        my $node_name = $outcome_holder_node->nodeName;
        
        my $outcome_holder_type = ($node_name eq 'team') ? 'team': 'player';
        my $metadata_node = solonode($outcome_holder_node, $outcome_holder_type."-metadata");
        my $outcome_holder_key = $metadata_node->getAttribute($outcome_holder_type."-key");
        next if (! $outcome_holder_key || $outcome_holder_key eq '');
        
        my $outcome_holder_id;
        
        if ($outcome_holder_type eq 'team') {
            $outcome_holder_id = $team_entity->retrieve_id_by_key({'team_key'=>$outcome_holder_key, 'publisher_id'=>$publisher_id});
        }

        if ($outcome_holder_type eq 'player') {
            $outcome_holder_id = $person_entity->retrieve_id_by_key({'person_key'=>$outcome_holder_key, 'publisher_id'=>$publisher_id});
        }
#        print "found outcome_holder_id: $outcome_holder_id\n";
        
        my @outcome_nodes = $outcome_holder_node->findnodes($outcome_holder_type."-stats/outcome-totals");
        my $stats_node = solonode($outcome_holder_node, $outcome_holder_type."-stats");
        my $rank_node = solonode_optional($stats_node, "rank");
        my $rank_node_rank;
        if ($rank_node) {
        	$rank_node_rank = $rank_node->getAttribute('value');
        }
        
        push @outcome_nodes, $stats_node;

        foreach my $outcome_node (@outcome_nodes) {
            my $standing_points = $outcome_node->getAttribute('standing_points');
            my $rank = $outcome_node->getAttribute('rank');
            $rank = $rank_node_rank if ! defined $rank;
            my $events_played = $outcome_node->getAttribute('events-played');
            my $wins = $outcome_node->getAttribute('wins');
            my $wins_regulation_overtime = $outcome_node->getAttribute('xts:wins-regulation-overtime'); 
            my $losses = $outcome_node->getAttribute('losses');
            my $winning_percentage = $outcome_node->getAttribute('winning-percentage');

            if ( defined $wins && defined $losses && (! defined $winning_percentage) ) {
                my $total = $wins + $losses;
                my $avg;
                if ( $total eq '0' ) {
                    $avg = 0;
                }
                else {
                	$avg = $wins/$total;
                }
                $winning_percentage = $avg;
            }
            if ( defined $winning_percentage ) {
                $winning_percentage =~ s/^\./0\./;
                $winning_percentage = sprintf("%.3f", $winning_percentage);
            }
            
            my $ties = $outcome_node->getAttribute('ties');
            my $losses_overtime = $outcome_node->getAttribute('losses-overtime');
            my $undecideds = $outcome_node->getAttribute('undecideds');
            my $points_scored_for = $outcome_node->getAttribute('points-scored-for');
            my $points_scored_against = $outcome_node->getAttribute('points-scored-against');
            my $points_difference = $outcome_node->getAttribute('points-difference');
            my $standing_points = $outcome_node->getAttribute('standing-points');
            my $games_back = $outcome_node->getAttribute('games-back');
            my $streak_type = $outcome_node->getAttribute('streak-type');
            my $streak_duration = $outcome_node->getAttribute('streak-duration');
            my $streak_total = $outcome_node->getAttribute('streak-total');
            my $streak_start = $outcome_node->getAttribute('streak-start');
            my $streak_end = $outcome_node->getAttribute('streak-end');
            my $result_effect = $outcome_node->getAttribute('result-effect');
            my $sets_for = $outcome_node->getAttribute('sets-for');
            my $sets_against = $outcome_node->getAttribute('sets-against');
            
            my $alignment_scope = $outcome_node->getAttribute('alignment-scope');
            my $duration_scope = $outcome_node->getAttribute('duration-scope');
            my $site_scope = $outcome_node->getAttribute('site-scope');
            # Is this an attribute?
            my $scoping_label = $outcome_node->getAttribute('scoping-label');
            my $competition_scope = $outcome_node->getAttribute('competition-scope');
            $competition_scope = (defined $competition_scope) ? $competition_scope: 'all';
            my $wins_overtime = $outcome_node->getAttribute('xts:wins-overtime');
            
            my $standing_subgroup_id = $self->get_subgroup_id( $standing_id, $affiliation_id, $competition_scope, $alignment_scope, $duration_scope );

            if ( ! defined $standing_subgroup_id ) {
                $standing_subgroup_id = $self->create(
                    {
                        'standing_id'   => $standing_id,
                        'affiliation_id' => $affiliation_id,
                        'competition_scope' => $competition_scope,
#                        'competition_scope_id'   => $competition_scope_id,
                        'alignment_scope' => $alignment_scope,
                        'duration_scope' => $duration_scope,
                        'site_scope' => $site_scope,
                        'scoping_label' => $scoping_label
                    }
                );
            }
            else {
            	$self->update(
            	{
                        'alignment_scope' => $alignment_scope, 
                        'duration_scope' => $duration_scope,
                        'site_scope' => $site_scope,
                        'scoping_label' => $scoping_label,
                        'competition_scope'   => $competition_scope
#                        'competition_scope_id'   => $competition_scope_id
                },
                {
                        'id' => $standing_subgroup_id
                }
                );
            }
            
# Debug code            
#            $self->{log}->log("[DBG] wins_regulation_overtime: $wins_regulation_overtime");
            
            my $outcome_total_id = $outcome_total_entity->retrieve_id_by_key(
            {
            	'standing_subgroup_id'=>$standing_subgroup_id, 
            	'outcome_holder_id'=>$outcome_holder_id
            }
            );
            
            if ( ! defined $outcome_total_id ) {
                $outcome_total_id = $outcome_total_entity->create(
                    {
                        'standing_subgroup_id'   => $standing_subgroup_id,
                        'outcome_holder_type' => ($outcome_holder_type eq 'team') ? 'teams': 'players',
                        'outcome_holder_id' => $outcome_holder_id,
                        'rank' => $rank,
                        'events_played' => $events_played,
                        'wins' => $wins,
                        'wins_regulation_overtime' => $wins_regulation_overtime,
                        'losses' => $losses,
                        'ties' => $ties,
                        'losses_overtime' => $losses_overtime,
                        'undecideds' => $undecideds,
                        'winning_percentage' => $winning_percentage,
                        'points_scored_for' => $points_scored_for,
                        'points_scored_against' => $points_scored_against,
                        'points_difference' => $points_difference,
                        'standing_points' => $standing_points,
                        'games_back' => $games_back,
                        'streak_type' => $streak_type,
                        'streak_duration' => $streak_duration,
                        'streak_total' => $streak_total,
                        'streak_start' => $streak_start,
                        'streak_end' => $streak_end,
                        'result_effect' => $result_effect,
                        'sets_for' => $sets_for,
                        'sets_against' => $sets_against,
                        'wins_overtime' => $wins_overtime
                    }
                );
            }
            else {
                $outcome_total_entity->update(
                {
                    'outcome_holder_type' => ($outcome_holder_type eq 'team') ? 'teams': 'players',
                    'rank' => $rank,
                    'events_played' => $events_played,
                    'wins' => $wins,
                    'wins_regulation_overtime' => $wins_regulation_overtime,
                    'losses' => $losses,
                    'ties' => $ties,
                    'losses_overtime' => $losses_overtime,
                    'undecideds' => $undecideds,
                    'winning_percentage' => $winning_percentage,
                    'points_scored_for' => $points_scored_for,
                    'points_scored_against' => $points_scored_against,
                    'points_difference' => $points_difference,
                    'standing_points' => $standing_points,
                    'games_back' => $games_back,
                    'streak_type' => $streak_type,
                    'streak_duration' => $streak_duration,
                    'streak_total' => $streak_total,
                    'streak_start' => $streak_start,
                    'streak_end' => $streak_end,
                    'result_effect' => $result_effect,
                    'sets_for' => $sets_for,
                    'sets_against' => $sets_against,
                    'wins_overtime' => $wins_overtime
                },
                {
                    'id' => $outcome_total_id
                }
                );
            }
        }
    }
}

1;