# $Id: Team.pm 2953 2012-05-30 22:15:42Z ian $

package FeedfetcherDeluxe::Entities::Team;

use Core::XmlUtil;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Event;
use FeedfetcherDeluxe::Entities::Position;
use FeedfetcherDeluxe::Entities::Display_name;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;
use FeedfetcherDeluxe::Entities::Sports_property;

#use Core::DbUtil;
#use Core::XmlUtil;

=head1 NAME

Core::Entities::Team

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;

}

sub initialize_local {
    my ($self) = @_;
    
    $self->{display_name_record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'display_names');

}

sub find_id_from_idref {
    my ( $self, $node, $idref, $publisher_id ) = @_;
    
    my $team_id;
    my $team_key;
    my $id;
        
    $id = $self->retrieve_id_by_key({'team_key'=>$idref, 'publisher_id'=>$publisher_id});
    
    return $id if defined $id;
        
    my $team_metadata = firstnode($node, "//team[\@id='$idref']/team-metadata");
        
    if (! $team_metadata) {
       $team_metadata = firstnode($node, "//team-metadata[\@team-key='$idref']");
    }

    if (! $team_metadata) {
       $team_metadata = firstnode($node, "//team-metadata[\@id='$idref']");
    }

    if ($team_metadata) {
        $team_key = required_attribute($team_metadata, 'team-key');
    }

    # Create a stub team entry if no metadata is found.
    # The assumption is that in a later document proper team metadata will be
    # available which can then be used to populate team related tables.
    if (! $team_key) {

        my $team_id_count = $self->retrieve_row_count({'team_key'=>$idref, 'publisher_id'=>$publisher_id});
        
        if ($team_id_count == 0) {
            $self->{log}->log("[ADV] No team key and/or team metadata found for team-idref $idref. Creating stub team entry.");
            $id = $self->create(
                {
                    'team_key'   => $idref,
                    'publisher_id' => $publisher_id
                }
            );        
        }
        return $id;
    }

	# Retrieve team id by team-key
    $id = $self->retrieve_id_by_key({'team_key'=>$team_key, 'publisher_id'=>$publisher_id});

	# Create a team entry if none exists.
    if (! $id) {
        my $team_node = solonode_optional($team_metadata, "ancestor::team");
        $id = $self->store_from_doc($team_node, $publisher_id);
    }
    
    return $id;
}

sub get_id_from_node {
    my ( $self, $team_node, $publisher_id ) = @_;

    my  $team_metadata = solonode( $team_node, 'team-metadata' );
    return undef if ! $team_metadata;
    
    my $team_key = attribute( $team_metadata, 'team-key' ) if $team_metadata;

     my $team_id = $self->retrieve_id_by_key(
         {
             'team_key'     => $team_key,
             'publisher_id' => $publisher_id
        }
     );
    return $team_id;
}

sub store_all_from_doc {
	# TODO!!!
	# Store in teams_documents
	# See what's done at
	# DbStore->xtoss_store_doc_content_codes
	
    my ( $self, $node, $publisher_id ) = @_;
    my $team_key;
    my $team_id;
    my $team_metadata;
    my $display_name_entity = FeedfetcherDeluxe::Entities::Display_name->new( $self->{dbh}, $self->{log} );
    
    my @team_nodes = $node->findnodes("//team");
    foreach my $team_node (@team_nodes) {
        $team_metadata = solonode_optional($team_node, 'team-metadata') if $team_node;
        next if (! defined $team_metadata);
        $team_key = attribute( $team_metadata, 'team-key' );
        $team_key =~ s/\s//g;
        
        # Bail out if there is no valid team key or if it's an empty string.
        # Otherwise, team records with empty team keys will be created.        
        next if (! $team_key || $team_key eq '');

        my $team_name_node = solonode_optional($team_node, 'team-metadata/name');
        my $first_name;
        my $last_name;
        my $full_name;
        my $nickname;
        my $abbreviation;
        
        if ($team_name_node) {
            $first_name = $team_name_node->getAttribute('first');
            $last_name = $team_name_node->getAttribute('last');
            $full_name = $team_name_node->getAttribute('full');
            $nickname = $team_name_node->getAttribute('nickname');
            $abbreviation = $team_name_node->getAttribute('abbreviation');
            # Fall through to defining team name full from first and last if no full name attribute
            unless (defined $full_name) {
            	$full_name = "$first_name $last_name";
            }
        }
        
        $team_id = $self->retrieve_id_by_key(
        {
        	'team_key'=>$team_key, 
        	'publisher_id'=>$publisher_id
        }
        );

        if (! $team_id) {
            $team_id = $self->create({'team_key'=>$team_key, 'publisher_id'=>$publisher_id});
        }
        $self->{display_name_record_mapper}->reset();
        $self->{display_name_record_mapper}->retrieve(
        {
            'entity_type'=>'teams', 
            'entity_id'=>$team_id, 
            'language'=>'en-US'
        }
        );
        $self->{display_name_record_mapper}->set(
        {
            'first_name'=>$first_name, 
            'last_name'=>$last_name, 
            'full_name'=>$full_name,
            'abbreviation'=>$abbreviation,
            'alias'=>$nickname
        }
        );
        $self->{display_name_record_mapper}->save();

        my $sports_property_entity = FeedfetcherDeluxe::Entities::Sports_property->new($self->{dbh}, $self->{log});
        $sports_property_entity->process_all($team_metadata, 'teams', $team_id, $publisher_id);

        
#        if (! $team_id) {
#            $team_id = $self->create({'team_key'=>$team_key, 'publisher_id'=>$publisher_id});
#            $display_name_entity->create(
#            {
#            	'entity_type'=>'teams', 
#            	'entity_id'=>$team_id, 
#            	'language'=>'en-US', 
#            	'first_name'=>$first_name, 
#            	'last_name'=>$last_name, 
#            	'full_name'=>$full_name
#            }
#            );
#        }
#        else {
#        	$display_name_entity_id=$display_name_entity->retrieve_id_by_key(
#        	{
#        		'entity_type'=>'teams', 
#        		'entity_id'=>$team_id, 
#        		'language'=>'en-US'
#        	}
#        	);
#        	if (! $display_name_entity_id) {
#                $display_name_entity->create(
#                {
#                	'entity_type'=>'teams', 
#                	'entity_id'=>$team_id, 
#                	'language'=>'en-US', 
#                	'first_name'=>$first_name, 
#                	'last_name'=>$last_name, 
#                	'full_name'=>$full_name,
#                	'abbreviation'=>$abbreviation,
#                	'alias'=>$nickname
#                }
#                );    		
#        	}
#        	else {
#                $display_name_entity->update(
#                    {
#                    	'entity_type'=>'teams', 
#                    	'entity_id'=>$team_id, 
#                    	'language'=>'en-US', 
#                	    'first_name'=>$first_name, 
#                	    'last_name'=>$last_name, 
#                	    'full_name'=>$full_name,
#                        'abbreviation'=>$abbreviation,
#                        'alias'=>$nickname
#                    },
#                    {'id'=>$display_name_entity_id}
#                );
#        	}
#        }
    }
}

# Find document's league id
sub find_league_id {
	my ( $self, $node, $publisher_id ) = @_;
	
	my $affiliation_entity = FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh}, $self->{log} );
	my $sportsml_document_entity = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{log});
	
	my $league_key = $sportsml_document_entity->get_content_code_key('league');
	return undef unless defined $league_key;
	
	my $league_id = $affiliation_entity->retrieve_id_by_key(
	{
		'affiliation_type' => 'league',
		'affiliation_key' => $league_key,
		'publisher_id' => $publisher_id
	}
	);

	# may or may not be undef, depending on whether the league affiliation was found
	return $league_id;
}

1;