# $Id: Team_phase.pm 2884 2011-08-23 22:22:11Z ian $

package FeedfetcherDeluxe::Entities::Team_phase;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Season;
use FeedfetcherDeluxe::Entities::Display_name;
use FeedfetcherDeluxe::Entities::Affiliation;
use FeedfetcherDeluxe::Entities::Team;

#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

Core::Entities::Team_phase

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN { @ISA = qw ( FeedfetcherDeluxe::Entities::Entity ); }

sub setup_sql {
	my ($self) = @_;

	$self->{'delete_by_team_id'} = $self->{dbh}->prepare(
		qq{
        SELECT id
        FROM $self->{table_name} 
        WHERE $self->{name}_key=?
        AND affiliation_type=?
        AND publisher_id=?
    }
	);

}

sub process_league_directory_team {
	my ( $self, $team_node, $publisher_id ) = @_;


	my $affiliation_id;
	my $affiliation_entity =
	  FeedfetcherDeluxe::Entities::Affiliation->new( $self->{dbh},
		$self->{log} );
	my $season_entity =
	  FeedfetcherDeluxe::Entities::Season->new( $self->{dbh}, $self->{log} );
	my $display_name_entity =
	  FeedfetcherDeluxe::Entities::Display_name->new( $self->{dbh},
		$self->{log} );
	my $team_entity =
	  FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );

#    my $team_phase_entity = FeedfetcherDeluxe::Entities::Team_phase->new($self->{dbh}, $self->{log});

	my $team_key = attribute_xpath( $team_node, 'team-metadata/@team-key' )
	  or die "No team-key supplied";

	my $league_node = solonode_optional( $team_node,
		'team-metadata/affiliation[@membership-type="league"]' );
	my $league_key = attribute( $league_node, 'membership-key' )
	  ;    # or die "No team-key supplied";

	my $league_id = $affiliation_entity->retrieve_id_by_key(
		{
			'affiliation_key'  => $league_key,
			'affiliation_type' => 'league',
			'publisher_id'     => $publisher_id
		}
	);
	if ( !$league_id ) {
		$league_id = $affiliation_entity->create(
			(
				{
					'affiliation_key'  => $league_key,
					'affiliation_type' => 'league',
					'publisher_id'     => $publisher_id
				}
			)
		);
	}

	my $display_name_full =
	  attribute_xpath( $team_node, 'team-metadata/name/@full' );
	my $display_name_first =
	  attribute_xpath( $team_node, 'team-metadata/name/@first' );
	my $display_name_last =
	  attribute_xpath( $team_node, 'team-metadata/name/@last' );
	my $display_name_nickname =
	  attribute_xpath( $team_node, 'team-metadata/name/@nickname' );
	my $display_name_abbreviation =
	  attribute_xpath( $team_node, 'team-metadata/name/@abbreviation' );

# HERE: if teams table already has a row with these matches:
#	teams.team_key == $team_key
#	teams.publisher_id == $publisher_id
# then update:
#	display_names.entity_type, entity_id, full_name, first_name, last_name, abbreviation
# else:
#	create new row in teams
#	create new row with display_names.entity_type, entity_id, full_name, first_name, last_name, abbreviation
#
# ALSO: wipe out any old team_phases for this team (NOTE: OLD xtoss table name was team_affiliation_phases...
#	xtoss14 should shorten that name to be just team_phases

	my $team_id = $team_entity->retrieve_id_by_key( {'team_key'=>$team_key, 'publisher_id'=>$publisher_id} );
	if ( !$team_id ) {

		# Create team record
		$team_id = $team_entity->create(
			{ 'team_key' => $team_key, 'publisher_id' => $publisher_id } );

		# Create display name record to store more detailed naming information
	}
	my $display_name_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new( $self->{dbh}, $self->{log}, 'display_names' );
	$display_name_record_mapper->retrieve(
		{
			'language'    => 'en-US',
			'entity_type' => 'teams',
			'entity_id'   => $team_id
		}
	);
	$display_name_record_mapper->set(
        {
            'full_name'    => $display_name_full,
            'first_name'   => $display_name_first,
            'last_name'    => $display_name_last,
            'alias'        => $display_name_nickname,
            'abbreviation' => $display_name_abbreviation
        }	);
	$display_name_record_mapper->save();

#	my $display_name_id = $display_name_entity->retrieve_id_by_key(
#		{ 'entity_type' => 'teams', 'entity_id' => $team_id } );
#	if ( !$display_name_id ) {
#		$display_name_entity->create(
#			{
#				'entity_type'  => 'teams',
#				'entity_id'    => $team_id,
#				'language'     => 'en-US',
#				'full_name'    => $display_name_full,
#				'first_name'   => $display_name_first,
#				'last_name'    => $display_name_last,
#				'alias'        => $display_name_nickname,
#				'abbreviation' => $display_name_abbreviation
#			}
#		);
#	} else {
#		$display_name_entity->update(
#			{
#				'entity_type'  => 'teams',
#				'entity_id'    => $team_id,
#				'language'     => 'en-US',
#				'full_name'    => $display_name_full,
#				'first_name'   => $display_name_first,
#				'last_name'    => $display_name_last,
#				'alias'        => $display_name_nickname,
#				'abbreviation' => $display_name_abbreviation
#			},
#			{ 'id' => $display_name_id }
#		);
#
#	}

	my $organizational_phase_start_season = attribute_xpath( $team_node,
		'team-metadata/organizational-phase/@start-season' );
	my $organizational_phase_end_season = attribute_xpath( $team_node,
		'team-metadata/organizational-phase/@end-season' );
	my $organizational_phase_start_date_time = attribute_xpath( $team_node,
		'team-metadata/organizational-phase/@start-date-time' );
	my $organizational_phase_end_date_time = attribute_xpath( $team_node,
		'team-metadata/organizational-phase/@end-date-time' );

	$organizational_phase_start_date_time =~ s/\s//g;
	$organizational_phase_end_date_time   =~ s/\s//g;

	# Fragile, need to check for well-formedness
	$organizational_phase_start_date_time = undef
	  if $organizational_phase_start_date_time eq '';
	$organizational_phase_end_date_time = undef
	  if $organizational_phase_start_date_time eq '';

	$organizational_phase_start_season =~ s/\s//g;
	$organizational_phase_start_season = ''
	  if $organizational_phase_start_season !~ /\d+/;
	$organizational_phase_end_season =~ s/\s//g;
	$organizational_phase_end_season = ''
	  if $organizational_phase_end_season !~ /\d+/;

	my $start_season_id;
	my $end_season_id;

	# Need to look this up
	if ( $organizational_phase_start_season ne '' ) {
		$start_season_id = $season_entity->retrieve_id_by_key(
			{
				'season_key'   => $organizational_phase_start_season,
				'league_id'    => $league_id,
				'publisher_id' => $publisher_id
			}
		);
		if ( !$start_season_id ) {
			$start_season_id = $season_entity->create(
				{
					'season_key'   => $organizational_phase_start_season,
					'league_id'    => $league_id,
					'publisher_id' => $publisher_id
				}
			);
		}
	}

	# Need to look this up
	if ( $organizational_phase_end_season ne '' ) {
		$end_season_id = $season_entity->retrieve_id_by_key(
			{
				'season_key'   => $organizational_phase_end_season,
				'league_id'    => $league_id,
				'publisher_id' => $publisher_id
			}
		);
		if ( !$end_season_id ) {
			$end_season_id = $season_entity->create(
				{
					'season_key'   => $organizational_phase_end_season,
					'league_id'    => $league_id,
					'publisher_id' => $publisher_id
				}
			);
		}
	}

	my $end_season_id;

	my @sub_affliation_nodes = nodes( $team_node, 'team-metadata/affiliation' );
	$self->delete( { 'team_id' => $team_id } );

	foreach my $sub_affliation_node (@sub_affliation_nodes) {

# HERE: for each child affiliation:
# 	insert: team_phases.affiliation_id, start_season, end_season, start_date_time, end_date_time

		my $affiliation_key =
		  attribute_xpath( $sub_affliation_node, '@membership-key' );
		my $affiliation_type =
		  attribute_xpath( $sub_affliation_node, '@membership-type' );

		my $affiliation_id = $affiliation_entity->retrieve_id_by_key(
			{
				'affiliation_key'  => $affiliation_key,
				'affiliation_type' => $affiliation_type,
				'publisher_id'     => $publisher_id
			}
		);
		if ( !$affiliation_id ) {

			# Create an affiliation record
			$affiliation_id = $affiliation_entity->create(
				{
					'affiliation_key'  => $affiliation_key,
					'affiliation_type' => $affiliation_type,
					'publisher_id'     => $publisher_id
				}
			);
		}

		$self->create(
			{
				'team_id'         => $team_id,
				'start_season_id' => $start_season_id,
				'end_season_id'   => $end_season_id,
				'affiliation_id'  => $affiliation_id,
				'start_date_time' => $organizational_phase_start_date_time,
				'end_date_time'   => $organizational_phase_end_date_time
			}
		);
	}
}
1;
