# $Id: Wagering_total_score_line.pm 2123 2008-07-23 19:22:08Z ian $

package FeedfetcherDeluxe::Entities::Wagering_total_score_line;

use FeedfetcherDeluxe::Entities::Entity;
use FeedfetcherDeluxe::Entities::Bookmaker;
use FeedfetcherDeluxe::Entities::Team;
use FeedfetcherDeluxe::Entities::Person;
use FeedfetcherDeluxe::Entities::Event;

#use Core::DbUtil;
use Core::XmlUtil;

=head1 NAME

Core::Entities::Wagering_total_score_line

=head1 SYNOPSIS


=head1 Description

See Entity.pm to get an understanding of what basic capabilities this module inherits. Any methods or functions
found here are specific to this entity.

=cut

BEGIN{@ISA = qw ( FeedfetcherDeluxe::Entities::Entity );} 

sub setup_sql {
    my ($self) = @_;
}

sub store_from_doc {
    my ( $self, $node, $event_id, $team_id, $person_id, $publisher_id ) = @_;

=pd
    <wagering-stats> can be found as a child of event, team, or player
    However, storage requirements stipulate that event id and team id must
    be found, whereas player id does not need to be.
    
    Event id cannot be assumed to refer to a document-wide event.
=cut

    my $id;
    my $bookmaker_id;
    
    my $bookmaker_entity =
      FeedfetcherDeluxe::Entities::Bookmaker->new( $self->{dbh}, $self->{log} );

    my $wagering_stats_node = solonode_optional($node, "child::wagering-stats");
    return unless defined $wagering_stats_node;

    my @wagering_total_score_line_nodes = $wagering_stats_node->findnodes("wagering-total-score");
    foreach my $wagering_total_score_line_node (@wagering_total_score_line_nodes) {

        $bookmaker_id = $bookmaker_entity->store_from_doc($wagering_total_score_line_node, $publisher_id);
        return $id unless $bookmaker_id;
        
        my $date_time = attribute($wagering_total_score_line_node, 'date-time');
        $date_time = $self->sportsml_date2sql_date_local($date_time);

        my $comment = attribute($wagering_total_score_line_node, 'comment');
        my $vigorish = attribute($wagering_total_score_line_node, 'vigorish');
        my $rotation_key = attribute($wagering_total_score_line_node, 'rotation-key');
        my $total = attribute($wagering_total_score_line_node, 'total');
        my $total_opening = attribute($wagering_total_score_line_node, 'total-opening');
        my $prediction = attribute($wagering_total_score_line_node, 'prediction');

        my $select_id;
        my $select_field;
        
        if ($team_id) {
        	$select_id = $team_id;
        	$select_field = 'team_id';
        }
        elsif ($person_id) {
        	$select_id = $person_id;
            $select_field = 'person_id';
        }
        else {
        	return $id;
        }
        
        my $id = $self->retrieve_id_by_key(
        {
        	'bookmaker_id' => $bookmaker_id,
            'event_id' => $event_id,
            'date_time' => $date_time,
            $select_field => $select_id
        }
        );

#        $self->{log}->log("[DBG] id: $id bookmaker id: $bookmaker_id");
        if (! $id) {
        	$self->create(
        	{
        		'event_id' => $event_id,
        		'team_id' => $team_id,
        		'person_id' => $person_id,
        		'bookmaker_id' => $bookmaker_id,
        		'date_time' => $date_time,
        		'comment' => $comment,
                'vigorish' => $vigorish,
        		'rotation_key' => $rotation_key,
        		'total' => $total,
                'total_opening' => $total_opening,
        		'prediction' => $prediction
        	}
        	);
        }
        else {
        # $self->{log}->log("[DBG] updating...");
            $self->update
            (
            {
                'event_id' => $event_id,
                'team_id' => $team_id,
                'person_id' => $person_id,
                'bookmaker_id' => $bookmaker_id,
                'date_time' => $date_time,
                'comment' => $comment,
                'vigorish' => $vigorish,
                'rotation_key' => $rotation_key,
                'total' => $total,
                'total_opening' => $total_opening,
                'prediction' => $prediction
            },
            {
                'id' => $id
            }
            );
        	
        }
    }
    
}

sub store_all_from_doc {
    my ( $self, $node, $publisher_id ) = @_;
    
    my $team_entity =
      FeedfetcherDeluxe::Entities::Team->new( $self->{dbh}, $self->{log} );
    my $person_entity =
      FeedfetcherDeluxe::Entities::Person->new( $self->{dbh}, $self->{log} );
    my $event_entity =
      FeedfetcherDeluxe::Entities::Event->new( $self->{dbh}, $self->{log} );

    my @sports_event_nodes = $node->findnodes("//sports-event");
    foreach my $sports_event_node (@sports_event_nodes) {

        my $event_metadata_node = solonode_optional($sports_event_node, "child::event-metadata");
        next unless defined $event_metadata_node;
        
        my $event_key = attribute($event_metadata_node, 'event-key');

        my $event_id = $event_entity->retrieve_id_by_key(
            {
                'event_key' => $event_key,
                'publisher_id' => $publisher_id
            }
        );
        
        my @team_nodes = $sports_event_node->findnodes("child::team");
        foreach my $team_node (@team_nodes) {
   
            my $team_key;
            my $team_id;
            my $person_key;
            my $person_id;
    
            my $team_metadata_node = solonode_optional($team_node, "child::team-metadata");
            my $player_metadata_node = solonode_optional($team_node, "player/player-metadata");
    
            if ($team_metadata_node) {
                $team_key = attribute($team_metadata_node, 'team-key');
                if ($team_key) {
                    $team_id = $team_entity->retrieve_id_by_key(
                    {
                        'team_key' => $team_key,
                        'publisher_id' => $publisher_id
                    }
                    );
                }
            }
            next unless defined $team_metadata_node;
            
            if ($player_metadata_node) {
                $person_key = attribute($player_metadata_node, 'player-key');
                if ($person_key) {
                    $person_id = $person_entity->retrieve_id_by_key(
                    {
                        'person_key' => $person_key,
                        'publisher_id' => $publisher_id
                    }
                    );
                }
            }
            $self->store_from_doc($team_node, $event_id, $team_id, $person_id, $publisher_id);
        }

    }
    
}

1;