# $Id: PbpStore.pm 3075 2016-04-07 19:52:23Z ian $

package FeedfetcherDeluxe::PbpStore;

use XML::LibXML;
use Carp;
use FeedfetcherDeluxe::XtossUtil;

use Core::Util;
use Core::XmlUtil;
use Core::LineParser;
use Core::ActiveFixtures;
use Core::DbUtil;
use FeedfetcherDeluxe::Entities::SportsmlDocument;
use FeedfetcherDeluxe::Entities::American_football_action_participant;
use FeedfetcherDeluxe::Entities::American_football_event_state;
use FeedfetcherDeluxe::Entities::Baseball_event_state;
use FeedfetcherDeluxe::Entities::Baseball_action_pitches;
use FeedfetcherDeluxe::Entities::Basketball_action_participant;
use FeedfetcherDeluxe::Entities::Ice_hockey_action_participant;
use FeedfetcherDeluxe::Entities::Ice_hockey_action_play;
use FeedfetcherDeluxe::Entities::Soccer_action_participant;
use FeedfetcherDeluxe::Entities::Event;
use FeedfetcherDeluxe::Entities::Publisher;
use FeedfetcherDeluxe::Entities::Event_state;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;

use strict;

#my $loadflexsport = 0;
#my $loadxtoss = 1;
my $xtoss_table_prefix = '';

sub new {
    my ($class, $cfg, $log) = @_;

	my $self = {};
	bless $self, $class;
	$self->_initialize($cfg, $log);

	return $self;
}

sub _initialize {
	my ($self, $cfg, $log) = @_;

	$self->{dbh};
	$self->{xtutil};
	$self->{log} = $log;
	$self->{cfg} = $cfg;
	$self->_parse($cfg->required_param('pbp_file'));		
}

sub _parse {
	my ($self, $file) = @_;

	my @lines = Core::LineParser::load($file);
	my ($table_code, $table_name);
#    $self->{log}->log("[DBG] ***** PARSING *****");
    
	# Load play-by-play definitions
	# ** BY DEFAULT: from pbp.def
	LINE: foreach my $line (@lines) {
		if ($line =~ /^\s*\#.*$/) {
#			$self->{log}->log("[DBG] ***** SKIPPING *****");
			next LINE;
		}
		# Proceed if the line begins with a bang
		if ($line =~ /^!(.*)$/) {
			# Split on colon (bang not part of $1)
			my ($table_type, $table_code1, $table_code2, $sport_key) = trims(split(/:/, $1));
			$table_name = $table_code1;
			$table_code = join(":", $table_type , $table_code1 , $table_code2);
			# sport_key is a number
			$self->{sport_map}->{$sport_key} = $table_code 
				if $table_type eq 'emd';
			push @{$self->{tables}}, $table_code;
		} else {
			die "Error parsing $file: No table name defined" if 
				!defined $table_code;
			my ($field, $def) = trims(split(/:/, $line));
			# Deal with xts namespace field delimiters
			# (turning from piping char to colon)
			$field =~ s/^xts\|/xts\:/;
			# Add the line to the list of fields as long as it's not commented out
			push @{$self->{fields}->{$table_code}}, $field;
			# Added by imarsman - 18 May, 2007 to allow easier lookup of field sets based on sportsml table name
			push @{$self->{fields_by_table_code}->{$table_name}}, $field;
		}
	}
}

sub store {
	my ($self, $dbh, $event_node, $event_key, $document_id) = @_;
	
	$self->{document_id} = $document_id;
	$self->{document_event_context} = undef;

	# Since database handle isn't available until here, we can't set the class variable for
	# it until here, nor can we instantiate the XtossUtil class instance
	$self->{dbh} = $dbh;
	$self->{xtutil} = new FeedfetcherDeluxe::XtossUtil($self->{dbh}, $self->{log});

    # Defined here since dbh isn't actually available until store called
    $self->{record_mapper} = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, undef);

    # This is a pretty clean way of getting publisher information
    # The entity should also be used to get values currently obtained in the loop below
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($event_node, $self->{'log'});

    my $publisher_key = $sportsml_document->{'publisher_key'};

    my $publisher_entity = FeedfetcherDeluxe::Entities::Publisher->new($self->{dbh}, $self->{log});
    my $publisher_id = $publisher_entity->retrieve_id_by_key({'publisher_key'=>$publisher_key});
 
	my $event_entity = FeedfetcherDeluxe::Entities::Event->new($self->{dbh}, $self->{log});
    my $event_id = $event_entity->retrieve_id_by_key({'event_key'=>$event_key, 'publisher_id'=>$publisher_id});

    unless (defined $event_id) {
    	$self->{log}->log("[DBG] Could not find events table entry for event key $event_key. Not processing plays for this event.");
    	return;
    }

    # Process new general event_states tables
    my $event_state_entity = FeedfetcherDeluxe::Entities::Event_state->new( $self->{dbh}, $self->{log} );
    $event_state_entity->store_all_from_doc($event_node, $publisher_id);

    my $action_listing = $sportsml_document->get_content_code_key('action-listing');
    my $play_sequence_number = $sportsml_document->get_content_code_key('play-sequence-number');
#	my $action_listing = attribute_xpath($event_node, q{ancestor::*/*/sports-content-codes/} .q{sports-content-code[@code-type='action-listing']/@code-key});
#	my $play_sequence_number = attribute_xpath($event_node, q{ancestor::*/*/sports-content-codes/} .q{sports-content-code[@code-type='play-sequence-number']/@code-key});
	my $date_coverage_type = attribute_xpath($event_node, 'event-metadata/@date-coverage-type');
	my $date_coverage_value = attribute_xpath($event_node, 'event-metadata/@date-coverage-value');

	my $node_name = $event_node->nodeName();
#	$self->{log}->log("[DBG] Processing for event-key: $event_key node_name: $node_name");
	
	# Run through the table definitions loaded in the _parse sub
	foreach my $table_code (@{$self->{tables}}) {
		my ($table_type, $table_name, $table_path) = split /:/, $table_code;
		my $complete_action = $action_listing eq 'complete' && $table_type eq 'play';

# Debug output
#        $self->{log}->log("[DBG] table: $table_name");
		
		# This is handled separately by play entities, e.g. ice_hockey_action_play
		next if $table_type eq 'participant';	

		# Translate the table name from sportsml to xtoss
		# This is important
		my $table = $self->{xtutil}->get_xtoss_name($table_name);
		$table = dash2underscore($table);

   		# If the action is complete, remove all old entries so that the completed set can be re-added
   		# Done now in add_event_states_record
  		if ($complete_action){
    		# $dbh->do(qq{DELETE FROM $table WHERE event_id=? AND } . q{date_coverage_type=?}, undef, $event_key, $date_coverage_type);
  		}

		$table_path =~ s/^\s*[\/]+//g;
		my @specific_nodes = $event_node->findnodes("descendant::$table_path");

#		my @specific_nodes = nodes($event_node, $table_path);
##	my $count = @specific_nodes;
#	$self->{log}->log("[DBG] table path: $table_path count: $count");

		foreach my $specific_node (@specific_nodes) {
			my $id = $self->process_node(
				$specific_node, $dbh, $event_id, $publisher_id, $table_name, 
				$table_code, $table_type, $table, undef,  
				$action_listing, $date_coverage_type, 
				$date_coverage_value, $action_listing, $play_sequence_number
			);
		}
	}
    my $sportsml_document_entity = FeedfetcherDeluxe::Entities::SportsmlDocument->new($event_node, $self->{log}); 
    my $action_listing = $sportsml_document_entity->get_content_code_key('action-listing');

    # For complete listings documents, go through and remove any old actions that may have remained from previous documents
    if ($action_listing eq 'complete') {
        if ($self->{sportsdb_sport_name} eq 'ice_hockey') {
            my $ice_hockey_action_play_entity = FeedfetcherDeluxe::Entities::Ice_hockey_action_play->new($self->{dbh}, $self->{log});
            $ice_hockey_action_play_entity->clear_bad_plays($event_id, $self->{document_id}, $self->{document_event_context});
       }
    }
}

sub process_node {
	my ($self, $specific_node, $dbh, $event_id, $publisher_id, $table_name, $table_code, 
		$table_type, $table, $play_id, $action_listing, 
		$date_coverage_type, $date_coverage_value, $action_listing, $play_sequence_number) = @_;

	if ($table_type eq 'emd') {
		return $self->process_emd($specific_node, $dbh, $event_id, $publisher_id, $table_name, $table_code, $table_type, $table, $play_id, $action_listing, $play_sequence_number);
	} else {
		# process plays yes or no based on flag in cfg
		return $self->process_play($specific_node, $dbh, $event_id, 
			$publisher_id, $table_name, $table_code, $table_type, $table, $play_id,
			$action_listing, $date_coverage_type, $date_coverage_value) if $self->{cfg}->param("action_tables") eq 'active';
	}
}

sub process_emd {
	my ($self, $specific_node, $dbh, $event_id, $publisher_id, $table_name, $table_code, $table_type, $table, $play_id, $action_listing, $play_sequence_number) = @_;

	my @all_names = @{$self->{fields}->{$table_code}};
	
	my $tbl_name_ret;
	my $table_id;

	my $xtoss_table = $self->{xtutil}->get_xtoss_name($table_name);

	if ($xtoss_table =~ /_event_states/) {
		 ($tbl_name_ret, $table_id) = $self->add_event_states_record($table, $event_id, $publisher_id, $specific_node, \@all_names);
	}
	return;
}

# Add an event states record to a table based on attributes found in $node
# and a field array passed in $fields
# NOTE
# for action nodes, the field list comes from the sport name concatenated to event-metadata
# and looked up in pbp.def. Thus, if a field is in an event states table it needs to be
# defined in the corresponding event-metadata-[SPORTNAME] section of the pbp.def file
sub add_event_states_record {
	my ($self,  $xtoss_table, $event_id, $publisher_id, $node, $fields) = @_;
	
    my $node_name = $node->nodeName;

#    my 
#    if ($sport =~ /ice_hockey/) {
#    }

	my @fields_array;
	# Deal with xts namespaced items to avoid having them make their way
	# into table field code
	foreach my $field (@$fields) {
		$field =~ s/xts\://;
		push @fields_array, $field;
	}
	
	my $ret_val = 0;
	my $action = "";
	my $sport = $xtoss_table;
	$sport =~ s/^(.*?)_event_.*$/$1/;
	$sport =~ s/[^\w]+//g;

    # Some files contain no useful information and/or are not in the list of fixtures to have event data processed (e.g. injury-phase)
    # They do, however, have stub tags like <event-metadata-basketball period-value="0" />, which can cause this sub to be
    # invoked with no valid event_id (see ticket 1057)
    return ('', 0, $action) if (! defined $event_id);
#    print "event id: $event_id node: $node_name\n"; 
	
	
 	# Test for current state based on whether it's a metadata node versus action play node
    my $current_state = ($node_name =~ /metadata/) ? 1: 0;
#    print "current_state\n" if ($current_state eq '1');
    
	my $fields = " ".join(' ', @fields_array)." ";
		
	my @attribute_list = $node->attributes();
		
	my $name;
	my $value;
	my %h = ();
	
	foreach my $attr (@attribute_list) {
		# getName is not documented!
		$name = $attr->getName();
		
		
        # Allow only attribute nodes
#        next if ! ($attr->nodeType() =~ /XML::LibXML::XML_ATTRIBUTE_NODE/);
#       Old way that relied on return value found to not always be the same across perl versions 
        my $node_type = scalar $attr;
#       print "node type: ".$node_type." attr node type: ".$attr->nodeType()." constant: ".XML::LibXML::XML_ATTRIBUTE_NODE."\n";
#		next if ! ($node_type =~ /Attr/ || $attr->nodeType() =~ /XML::LibXML::XML_ATTRIBUTE_NODE/);
        next if ! ($attr->nodeType() == XML::LibXML::XML_ATTRIBUTE_NODE);
		$value = $attr->getValue();
#		print "name: $name value: $value fields: @fields_array\n" if $name =~ /^xts/;
		
		# Changed 2009.03.30 by ian - Skip this attribute if its value is empty
		next if $value =~ /^\s*$/;
		$name =~ s/^xts\://;
		
		if ($fields =~ /\s$name\s/) {
			($name, $value) = $self->{xtutil}->get_xtoss_column_info($name, $value, $publisher_id, $xtoss_table, $node);
			if ($name =~ /runner.*?id/) {
				my $nn = $node->nodeName();
			}
			
			$name =~ s/-/_/g;
			$h{$name} = $value;
		}
	}

	# Make sure shootouts have period value of 5 (in case this is missing)
    if ($node_name =~ /action-ice-hockey/ && attribute($node, 'score-attempt-type') eq 'shootout') {
    	$h{'period_value'} = 5;
    }
    my @keys = keys %h;
#    print "keys: @keys\n";
    if (keys %h == 0) {
        return ("${sport}_event_state_id", 0, $action);
    }
	
	my $context;
	my $event_metadata_node = firstnode($node, 'ancestor::event-metadata');
    $event_metadata_node = firstnode($node, 'preceding::event-metadata') if ! defined $event_metadata_node;
	$context = attribute($event_metadata_node, 'date-coverage-type') if defined $event_metadata_node;
	if (defined $context && (! defined $self->{document_event_context})) {
		$self->{document_event_context} = $context;
	}
	
    my $sportsml_document = FeedfetcherDeluxe::Entities::SportsmlDocument->new($node, $self->{'log'});
    my $action_listing = $sportsml_document->get_content_code_key('action-listing');
 		
    my @t_keys = keys %h;
    my @t_values = values %h;
 
    my $delete_sql;
    my $sth;
 
#    # Delete any current state records
#    if ($current_state eq '1') {
#        # Delete any old items
#        $delete_sql = qq{
#            DELETE FROM $xtoss_table
#            WHERE event_id=?
#            AND current_state=1
#        };
##            AND context=?
#
#        $sth = $self->{dbh}->prepare($delete_sql);
#        $sth->execute($event_id);   
##        $sth->execute($event_id, $context);   
#        $sth->finish;
#    }
    
    # Sequence values can be missing or non-numerical, leading event-states to action-plays meaningfulness
	# issues.
	if ($sport =~ /american_football/ && $current_state eq '0') {
		if ((! $h{'sequence_number'}) || ($h{'sequence_number'} !~ /^\d+$/)) {
			# Return a failed result if no sequence number and not current state (i.e. this is not a metadata node but an action node)
			return ('american_football_event_state_id', 0, $action);
		}
	}

    # If we're in a non-current-state item, don't proceed if we don't have the data to do a proper lookup
    if ($sport =~ /ice_hockey/ && $current_state eq '0') {
        return ('ice_hockey_event_state_id', 0, '') if (
	        ! defined $h{'action_key'} && 
    	    ! defined $h{'sequence_number'} &&
        	(! (defined $h{'period_time_elapsed'}) || $h{'period_time_elapsed'} eq '' || $h{'period_time_elapsed'} !~ /^\d+:\d+$/)
        );
    }
	
	my $existing_id;
	my $select_sql;
	my $insert_sql;
	my $update_sql;
	$action = 'insert';
    my $sequence_number = $h{'sequence_number'};
    $h{'context'} = $context;

	my %tests = ();
	# event_id, current_state, and context help define a test for uniqueness, but do not 
	# individually or together do so
	$tests{'event_id'} = $event_id;
	# Make sure not to link to a current_state=1 record. This would be an error
	# and would cause problems later when trying to clear current state records
	# mistakenly linked to by, say, action play records.
#    $tests{'current_state'} = '0';
    $tests{'current_state'} = $current_state;
    
    $tests{'context'} = $h{'context'};

	# sequence_number is a test for uniqueness per record (also others like period_time_...)
	if ($current_state eq '0') {
    	$tests{'sequence_number'} = $h{'sequence_number'};
	}

    # FIND OUT - do we need to check by participant role for football pbp too?
    # Conclusion, no, this is a play participant thing, handled separately
#    if ($sport =~ /football/) {
#        $tests{'participant_role'} = $h{'participant_role'};       
#    }

    if ($sport =~ /ice_hockey/) {
    	if (defined $h{'action_key'}) {
    		$tests{'action_key'} = $h{'action_key'};
    	}
    	# Redundant since sequence-number already added to tests outside of this logic, above
    	elsif (defined $h{'sequence_number'}) {
    		$tests{'sequence_number'} = $h{'sequence_number'};
    	}
    	else {
    		$tests{'period_time_elapsed'} = $h{'period_time_elapsed'};
    	}
    }
	
#    print "sport: $sport sequence number: $h{'sequence_number'}\n";
	# Use minutes elapsed for soccer as a test for insert/update
    if ($sport =~ /soccer/) {
        if (defined $h{'sequence_number'}) {
            $tests{'sequence_number'} = $h{'sequence_number'};
        }
        elsif (defined $h{'minutes_elapsed'}) {
            $tests{'minutes_elapsed'} = $h{'minutes_elapsed'};
        }
    }

	# For non-current-state cases, make sure there is something unique to identify a record with (to the
	# best of our ability) and leave if not
	unless ($current_state eq '1') {
		my %unique_tests = %tests;
		# event_id, current_state, and context help define a test for uniqueness, but do not 
		# individually or together do so
		$unique_tests{event_id} = undef;
		unless ($sport =~ /ice_hockey/) {
		  $unique_tests{context} = undef;
		}
		$unique_tests{current_state} = undef;
		my @unique_tests_keys = keys %unique_tests;
		my @test_keys = grep { defined $unique_tests{$_} } @unique_tests_keys;
		
		# Leave if nothing unique to test with
		if (scalar @test_keys eq '0') {
			next;
		}
	}

#	my $set_sub_clause = join(" ", map { "AND $_=?" } grep { defined $tests{$_} } keys %tests);
#	my @select_clause_values = grep { defined $_ } values %tests;
##    print "sub_clause: $set_sub_clause @select_clause_values\n";
#	my $select_sql = qq{SELECT max(id) FROM $xtoss_table WHERE 1=1 $set_sub_clause};	
#    print "context: $context\n";
    my $mapper_tests = {};
    if ($current_state eq '0') {
        $mapper_tests = \%tests;
    }
    else {
        $mapper_tests->{event_id} = $tests{event_id};
        $mapper_tests->{context} = $tests{context};
        $mapper_tests->{current_state} = $tests{current_state};
    }
        
    $self->{record_mapper}->reset($xtoss_table);
#    my $event_states_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, $xtoss_table);
    my $event_states_record_mapper = $self->{record_mapper};
    $event_states_record_mapper->retrieve($mapper_tests);

#    my $old_context = $event_states_record_mapper->get_field_value("context");
#    my $old_document_id = $event_states_record_mapper->get_field_value("document_id") || $self->{document_id};
#    my $old_sequence_number = $event_states_record_mapper->get_field_value("sequence_number");
#    if ($sport =~ /ice_hockey/) {
#        $h{'context'} = $old_context;
#        $h{'document_id'} = $old_document_id;
##        print "old context: $context old document id: $old_document_id old sequence number: $old_sequence_number\n";
#    }

    my $mapper_action = $event_states_record_mapper->get_action();
    $event_states_record_mapper->set_existing_as_null();
#    print "table: $xtoss_table mapper action: $mapper_action\n";
    
	# There's no point looking for an existing record if we've just deleted it
#	unless ($current_state eq '1') {
# 		$sth = $self->{dbh}->prepare($select_sql);
#		$sth->execute(@select_clause_values);
#
#		my @arr = $sth->fetchrow_array();
#		$sth->finish;
#	
#		if (@arr->[0]) {
#			$existing_id = @arr->[0];
#	        $action = 'update';
#		}
#		# Redundant, but just in case
#		else {
#			$action = 'insert';
#		}
#	}
		
	# sport-specific for now until a way is found to do this less explicitly
	if ($sport =~ /baseball/) {
		$h{'runner_on_first'} = undef;
		$h{'runner_on_second'} = undef
		$h{'runner_on_third'} = undef;

		$h{'runner_on_first'} = (defined $h{'runner_on_first_id'}) ? 1: undef;
		$h{'runner_on_second'} = (defined $h{'runner_on_second_id'}) ? 1: undef;
		$h{'runner_on_third'} = (defined $h{'runner_on_third_id'}) ? 1: undef;
	}

#    if (defined $existing_id) {
#        $action = 'update';
#    }
#    else {
#    	$action = 'insert';
#    } 
    if ($mapper_action eq 'update') {
        if ($current_state eq '1') {
            $h{'current_state'} = $current_state;
            $h{'event_id'} = $event_id;
        }
    }
    else {
        $h{'current_state'} = $current_state;
        $h{'event_id'} = $event_id;
    }
    # TODO: May be unecessary
#    if ($sport =~ /ice_hockey/) {
#        $h{'document_id'} = $old_document_id;
#    }
#    else {
        $h{'document_id'} = $self->{document_id};
#    }
    
    my @k = keys %h;
#    print "keys: @k event id: $event_id\n";
    $event_states_record_mapper->set(\%h);
    $ret_val = $event_states_record_mapper->save();
    
#    print "event state id: $ret_val\n";
#	if ($action eq 'update') {
#		if ($current_state eq '1') {
#			$h{'current_state'} = $current_state;
#		}
#		
#		my $set_sub_clause = join(", ", map { "$_=?" } grep { defined $h{$_} } keys %h);
#		my @clause_values = grep { defined $_ } values %h;
#		$update_sql = qq{ UPDATE $xtoss_table SET $set_sub_clause WHERE id=? };
#		
#		my $sth = $self->{dbh}->prepare($update_sql);
#		$sth->execute(@clause_values, $existing_id);
#		
#		$ret_val = $existing_id;
#	}	
#	else {
#		$h{'current_state'} = $current_state;
#		$h{'event_id'} = $event_id;
#	
#		# Basically found at http://search.cpan.org/dist/DBI/DBI.pm
#		# while looking up "dbi perl last_insert_id"
#		my @insert_fields = sort keys %h;
#		# Changed to make ref to %h a pointer
#		#        my @values = @{%h}{@insert_fields};
#    	my @values = @{\%h}{@insert_fields};
#    	$insert_sql = sprintf "insert into %s (%s) values (%s)",
#        	$xtoss_table, join(",", @insert_fields), join(",", ("?")x@insert_fields);
# 
#    	my $sth = $self->{dbh}->prepare($insert_sql);
#    	$sth->execute(@values);
#
#		$sth = $self->{dbh}->prepare($select_sql);
#	
#		$sth->execute(@select_clause_values);
#		my @arr = $sth->fetchrow_array();
#		$sth->finish;
#
#		if (@arr->[0]) {
#			$ret_val = @arr->[0];
#		}
#	}

    return ("${sport}_event_state_id", $ret_val, $mapper_action);
#	return ("${sport}_event_state_id", $ret_val, $action);
}

sub get_event_states_id {
	my ($self, $event_id, $node, $publisher_id, $xtoss_table, $table_code) = @_;
	
	my $ret_val;
	
	# my $event_id = $h->{'event_id'};
	# my $node = $h->{'node'};
	
	# Figure out the sport name from the incoming table name
	my $sport = $xtoss_table;
	
	$sport =~ s/-/_/g;
	$sport =~ s/^(.*?)_action.*$/$1/;
	
	# NOTE
	# Here we getting fields for action tables via a lookup of event-metadata-[SPORTNAME] in pbp.def
	# Thus, any fields we want to go into an event states table need to be defined there!
	
	# We are assuming a consistent naming convention here, baseball_event_states, american_football_event_states, ...
	my $states_table_name = $sport."_event_states";
	# To be used later when clearing out bad plays
	$self->{sportsdb_sport_name} = $sport;
	# Ensure no dashes get through
	$states_table_name =~ s/-/_/g;
	my $sportsml_table_name = my $table = $self->{xtutil}->get_sportsml_name($states_table_name);
#	my @all_names = @{$self->{fields}->{$table_code}};
	my @all_names = @{$self->{fields_by_table_code}->{$sportsml_table_name}};	
	# Figure out the name of the method
	
	# Invoke the method that will get a new id

	my ($test_key, $ret_val, $action) = $self->add_event_states_record($states_table_name, $event_id, $publisher_id, $node, \@all_names);
#   $self->{log}->log("[DBG] ***** states_table_name, event_id, publisher_id, node, all_names: $states_table_name, $event_id, $publisher_id, $node, @all_names");
    
#	$self->{log}->log("[DBG] test_key, ret_val, action: $test_key, $ret_val, $action");
	return ($test_key, $ret_val, $action);
}



sub process_play {
 	my ($self, $specific_node, $dbh, $event_id, $publisher_id, $table_name, $table_code, 
    $table_type, $table, $play_id, $pub_key, $action_listing, 
    $date_coverage_type, $date_coverage_value) = @_;

    my $node_name = $specific_node->nodeName();
    
	my $original_table_name = $table_name;
	# Translate table to its xtoss equivalent
 	$table = $table_name = $self->{xtutil}->get_xtoss_name($table_name);

	my $event_states_table_id_name;
	my $event_states_table_id;
	my $action = '';
	
#	print "table name: $table $original_table_name\n";

	#Don't proceed if action substitutions node doesn't pass required attribute test
	if ($table_name =~ /baseball_action_substitutions/) {
		my $node_test = attribute($specific_node, 'person-original-idref');
		return if ! $node_test;
	}

	my $play_type = attribute($specific_node, 'play-type');
	if ($table_name =~ /_action_/) {
		($event_states_table_id_name, $event_states_table_id, $action) = $self->get_event_states_id($event_id, $specific_node, $publisher_id, $table_name, $table_code);
	}
#   print "table name: $table $original_table_name\n";
	

	# Don't proceed if we got back a zero from the states generation call.
	if ($event_states_table_id eq '0') {
		return 0;
	}
#        print "table name: $table_name $event_states_table_id_name, $event_states_table_id\n";

	my $complete_action = $action_listing eq 'complete' && $table_type eq 'play';
  
  	my @names = @{$self->{fields}->{$table_code}};
  	my %values;
#  	print "names: @names\n";

	my %final_names = ();
 
	# Use short circuit logic to test two possible sources for sequence number value
	my $sequence_number = attribute($specific_node, 'play-sequence-number') || attribute($specific_node, 'sequence-number');

#	my $sequence_number_test = attribute($specific_node, 'play-sequence-number') || ;
#	if ($sequence_number_test) {
#		$sequence_number = $sequence_number_test;
#	}
#	elsif (! defined $sequence_number) {
#		$sequence_number_test = attribute($specific_node, 'sequence-number')
#	}
#	
	if (! defined $sequence_number) {
		my $sequence_number_node = solonode_optional($specific_node, "//sports-content-code[\@code-type='play-sequence-number']");
		if ($sequence_number_node) {
			$sequence_number = required_attribute($sequence_number_node, 'code-key');
		}
	}

	if ($table_name =~ /_event_states/) {
		if ($sequence_number) {
			$final_names{'sequence_number'} = $sequence_number;
			$values{'sequence_number'} = $sequence_number;			
		}
	}
	# This will work for some but not all action play tables
	else {
		my $sport = $table_name;
	
		$sport =~ s/-/_/g;
		$sport =~ s/^(.*?)_.*$/$1/;
		if ($action eq 'update') {
			# $self->{log}->log("[DBG] DELETING for event: $event_id SEQUENCE NUMBER: $sequence_number");
			
			# my $delete_sql = qq{DELETE FROM $table_name WHERE ${sport}_event_state_id = ?};
			
			# $self->{log}->log("[DBG] $delete_sql");
	   		# my $sth = $self->{dbh}->prepare($delete_sql);
    		# $sth->execute($event_states_table_id);		
		}
	}
#	print "PRE node_name: $node_name play-type: $play_type event_states_table_id: $event_states_table_id sequence number: $sequence_number action: $action\n";

  	foreach my $attr_name (@names) {
  		my $attr = attribute($specific_node, $attr_name);
     	
     	$attr_name =~ s/^xts\://;
 		($attr_name, $attr) = $self->{xtutil}->get_xtoss_column_info($attr_name, $attr, $publisher_id, $table, $specific_node);
#     	print "attr_name: $attr_name attr: $attr event_states_table_id: $event_states_table_id\n" if $attr_name =~ /^penalty/;

        # Specific jiggery-pokery
        # NOTE: attr_name may be a translated version (to match db field name 
        # (e.g. player-idref->person-id)not necessarily the same as the initial value
        # Snag soccer position id translated value
        if ($node_name eq 'action-soccer-substitution' && $attr_name eq 'person_original_position_id') {
#	       	print "node_name: $node_name attr_name: $attr_name attr: $attr\n";
            $final_names{'person_replacing_position_id'} = $attr;
            $values{'person_replacing_position_id'} = $attr;
        }
        
    	$final_names{$attr_name} = $attr if (! exists($final_names{$attr_name}) && ! $attr =~ /^\s*$/);
    	$values{$attr_name} = $attr if ! exists($values{$attr_name});
  	}

    # play_type is not always specified in node. Use value from node name if no attribute.
    if ($original_table_name =~ /action-american-football-(.*)$/) {
        my $play_type = $specific_node->getAttribute('play-type');
        $final_names{'play_type'} = $play_type || $1;
        $values{'play_type'} = $play_type || $1;
    }
	# Here we add any field values that are not explicitly picked up from pbp.def
	# If we don't do this here code following will cause a return from the method
	# since no valid fields to store will have been found.
	if ($node_name =~ /action-ice-hockey-(.*)$/) {
		# Default to play-type attribute if it's available
		my $play_type = $specific_node->getAttribute('play-type');
		$final_names{'play_type'} = $play_type || $1;
		$values{'play_type'} = $play_type || $1;
	}
    if ($node_name eq 'action-ice-hockey-score') {
        $final_names{'play_result'} = 'goal';
        $values{'play_result'} = 'goal';       
    }
    if ($node_name eq 'action-ice-hockey-score-attempt') {
        $final_names{'play_result'} = 'no-goal';
        $values{'play_result'} = 'no-goal';
    }
	if ($node_name eq 'action-soccer-score') {
        $final_names{'play_result'} = 'goal';
        $values{'play_result'} = 'goal';
	}
    if ($node_name =~ /action-basketball-(.*)$/) {
        # Default to play-type attribute if it's available
        my $play_type = $specific_node->getAttribute('play-type');
        $play_type = $play_type || $1;
        # A basketball penalty is a foul
        if ($play_type =~ /penalty/) {$play_type='foul'};
        $final_names{'play_type'} = $play_type;
        $values{'play_type'} = $play_type;
    }

    @names = keys %final_names;
    
    return 0 if ! @names;


#	print "post node_name: $node_name play-type: $play_type event_states_table_id: $event_states_table_id sequence number: $sequence_number action: $action\n";

    # re-do names 
	@names = keys %final_names;

	my %xtoss_final_names = %final_names;
	my %xtoss_values = %values;

    $self->{record_mapper}->reset($table);
#    my $action_play_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, $table);
    my $action_play_record_mapper = $self->{record_mapper};
	
    $action_play_record_mapper->retrieve({"$event_states_table_id_name"=>$event_states_table_id});
    $action = $action_play_record_mapper->get_action();
    $action_play_record_mapper->set(\%values);
    my $id = $action_play_record_mapper->save();
    
#    $event_states_record_mapper->retrieve(\%tests);

	# Ensure that even though the event states record has been updated,
	# a record to update for the current table _does_. If not, the action
	# needs to be 'insert'
#	if ($action eq 'update') {
#		# $event_states_table_id_name, $event_states_table_id
#        my $select_sql = "SELECT max(id) FROM $table WHERE $event_states_table_id_name = $event_states_table_id";
#        my $sth = $self->{dbh}->prepare($select_sql);
#        $sth->execute();
#
#        my @arr = $sth->fetchrow_array();
#        $sth->finish;
#    
#        if (! defined @arr->[0]) {
#            $action = 'insert';
#        }
#	}    
    
#	if ($action eq 'update') {
#
#        my @xtoss_names = keys %xtoss_final_names;
##        print "names: @xtoss_names\n";
#        my @values_list = map { $xtoss_values{$_} } 
#                grep { defined $xtoss_values{$_} } @xtoss_names; 
#
#        my $final_size = scalar @values_list;
#        if ($final_size == 0) {
#            $self->{log}->log("[DBG] No attributes to save for node $node_name, table $table.");
#            return 0;
#        }
#        
#	   	my $update_list = join(", ", map { dash2underscore($_) . '=?' } 
#	       	grep { defined $xtoss_values{$_} } @xtoss_names);
#
#        
#    	my $sql = qq{
#    		UPDATE $table SET $update_list 
#       		WHERE $event_states_table_id_name=? 
#    	};
#    	$dbh->do(
#    		$sql, undef, @values_list, $event_states_table_id
#       	); 
#	}
#	else {
#		$xtoss_final_names{$event_states_table_id_name} = $event_states_table_id;
#		$xtoss_values{$event_states_table_id_name} = $event_states_table_id;
#
#		my @xtoss_names = keys %xtoss_final_names;
#        my $final_size = scalar @xtoss_names;
#        if ($final_size == 0) {
#            $self->{log}->log("[DBG] No attributes to save for node $node_name, table $table.");
#            return 0;
#        }
#
#		my $insert_list = join(", ", map { dash2underscore($_) } @xtoss_names);
#    	my $values_list = join(", ", map { '?' } @xtoss_names);
#    	my $sql = qq{
#    		INSERT INTO $table ($insert_list) 
#    		VALUES ($values_list) 
#    	};
#    	$dbh->do($sql, undef, map { $xtoss_values{$_} } @xtoss_names);	
#	}

    if ($table eq 'american_football_action_plays') {
        my  $american_football_action_participant_entity = FeedfetcherDeluxe::Entities::American_football_action_participant->new($self->{dbh}, $self->{log});
     	$american_football_action_participant_entity->store_all_from_doc($specific_node, $publisher_id, $event_states_table_id);
    }
    if ($table eq 'baseball_action_plays') {
#       $self->{log}->log("[DBG] baseball_action_plays");
        my  $baseball_action_pitches_entity = FeedfetcherDeluxe::Entities::Baseball_action_pitches->new($self->{dbh}, $self->{log});
        $baseball_action_pitches_entity->store_all_from_doc($specific_node, $publisher_id, $event_states_table_id);
    }
    if ($table eq 'basketball_action_plays') {
#       $self->{log}->log("[DBG] basketball_action_plays");
        my  $basketball_action_participant_entity = FeedfetcherDeluxe::Entities::Basketball_action_participant->new($self->{dbh}, $self->{log});
        $basketball_action_participant_entity->store_all_from_doc($specific_node, $publisher_id, $event_states_table_id);
    }
    # For hockey action plays, need to add the participants
    if ($table eq 'ice_hockey_action_plays') {
        my  $ice_hockey_action_participant_entity = FeedfetcherDeluxe::Entities::Ice_hockey_action_participant->new($self->{dbh}, $self->{log});
        $ice_hockey_action_participant_entity->store_all_from_doc($specific_node, $publisher_id, $event_states_table_id);
    }

    if ($table eq 'soccer_action_plays') {
        my  $soccer_action_participant_entity = FeedfetcherDeluxe::Entities::Soccer_action_participant->new($self->{dbh}, $self->{log});
        $soccer_action_participant_entity->store_all_from_doc($specific_node, $publisher_id, $event_states_table_id);
    }
    return $id;
#  	return $values{id};
}

# Returns a table code, really a concatenation of table type,
# table name, and table path
sub get_table_code {
	my ($self, $sport_key) = @_;

	return $self->{sport_map}->{$sport_key};
}

sub event_meta_table {
	my ($self, $sport_key) = @_;

	# Table code is the concatenation of table_type, table_name, and table_path
	my $table_code = $self->get_table_code($sport_key) or return undef;
	my ($table_type, $table_name, $table_path) = split /:/, $table_code;

	# Table is the second part, having dashes converted to underscore
	my $table = dash2underscore($table_name);
	
	return $table;
}

sub get_db_fields {
	my ($self, $sport_key) = @_;

	my $table_code = $self->get_table_code($sport_key) or return undef;
	my @fields = @{$self->{fields}->{$table_code}};
	return map { dash2underscore($_) } @fields;
}


1;
