# $Id: SpecificStatsStore.pm 2977 2012-07-19 22:33:06Z ian $

package FeedfetcherDeluxe::SpecificStatsStore;

use strict;

use DBI();

use Core::DbUtil;
use Core::Util;
use FeedfetcherDeluxe::XtossUtil;
use FeedfetcherDeluxe::Entities::MetaRecordMapper;

#my $loadflexsport = 0;
#my $loadxtoss = 1;

my $xtoss_table_prefix = '';


sub new {
    my ($class, $dbh, $table_defs, $cfg, $log) = @_;

	my $self = {};
	bless $self, $class;
	$self->_initialize($dbh, $table_defs, $cfg, $log);
	return $self;
}

sub _initialize {
	my ($self, $dbh, $table_defs, $cfg, $log) = @_;

	$self->{'cfg'} = $cfg;
	$self->{'log'} = $log;
	$self->{'dbh'} = $dbh;
	$self->{'table_defs'} = $table_defs;
	$self->{xtutil} = new FeedfetcherDeluxe::XtossUtil($dbh, $self->{log});
#	$self->_prepare_stats($table_defs);
}

sub trap {
    my ($self, $sub, @params) = @_;

    $self->{log}->trap($sub, $self->{doc_id}, @params);
}

sub _prepare_stats {
	my ($self, $table_defs) = @_;

	my $dbh = $self->{'dbh'};

	$self->{"xtoss_stats_delete_from_metadata_select"} =
		$dbh->prepare("DELETE FROM stats WHERE stats.stat_repository_type=? AND stats.stat_repository_id=?");


	foreach my $table (keys %$table_defs) {
		my $xtoss_table = $self->{xtutil}->get_xtoss_name($table);		
		
	# FIXME Need to ensure this query is really doing what it's supposed to.
	# Also need to ensure it works with SQL Server.
	# Probably works with MySQL
	# Probably more compatible to do a sub-select.
	my $test_sql = qq{
		DELETE
		FROM $xtoss_table
		USING stats
		WHERE stats.stat_repository_type = '$table'
		AND stats.stat_repository_id = $xtoss_table.id
		AND stats.stat_repository_id=?
		AND stats.stat_holder_id=?
		AND stats.stat_coverage_type=?
		AND stats.stat_coverage_id=?
		
	};
		
	# Used to compile a list of stats to
	# * Find the source table id to clear that record
	# * Delete all records matcing that id
	# NOTE The line
	# AND (stats.stat_coverage_id=? OR stats.stat_coverage_id IS NULL)
	# is needed to ensure that stats lines with stats_coverage_type of 'career'
	# are found properly. Career is not an entry in any table, unlike seasons,
	# so, without the OR clause we'd be missing things.
	my $xtoss_stats_select_by_metadata = qq{
			SELECT stats.stat_repository_id
			FROM stats
			WHERE stats.stat_repository_type = '$xtoss_table'
			AND stats.stat_holder_type=?
			AND stats.stat_holder_id=?
			AND stats.stat_coverage_type=?
			AND (stats.stat_coverage_id=? OR stats.stat_coverage_id IS NULL)
		AND (
			(stat_membership_type IS NULL AND stat_membership_id IS NULL) OR
			(stat_membership_type = ? and stat_membership_id = ?) 
		)
		AND context=?
	};
	
	my $stats_delete_sql = qq{
		DELETE
		FROM $xtoss_table
		WHERE $xtoss_table\.id = ?
	};

	$self->{"xtoss_${xtoss_table}_select_by_metadata"} =
		$dbh->prepare($xtoss_stats_select_by_metadata);

	$self->{"xtoss_${xtoss_table}_delete_by_metadata"} =
		$dbh->prepare($stats_delete_sql);

	# xtossql: *_stats / select (maxid)
		$self->{"latest_xtoss_${xtoss_table}"} = 
			$dbh->prepare("SELECT max(id) from $xtoss_table");

	# xtossql: *_stats / delete
		$self->{"xtoss_${xtoss_table}_delete"} =
			$dbh->prepare("DELETE FROM $table WHERE id=?");


	# here is where we are "unshifting in" the ID field... unnecessarily. 
	# but I can't figure out how to stop it.
	# I figure that map { '?' } is expecting the value of an ID field... so that's
	# a candidate for getting-rid-of. But it seems like we need to use perl-map
	# to turn that @xtoss_fields structure into a string. But I could be wrong.

	my @xtoss_fields = @{$table_defs->{$table}};
	
    # Allow filtering of field names through XtossUtil.pm lookup
	my @new_fields;
	foreach my $field (@xtoss_fields) {
		my ($new_field_name, $value) = $self->{xtutil}->get_xtoss_column_info($field);
		$new_field_name =~ s/-/_/g;
		$new_field_name =~ s/^xts\://;
#		print "new_field_name: $new_field_name\n";
		push(@new_fields, $new_field_name);
	}
#	$self->{'log'}->log("[DBG] xtoss_table: $xtoss_table new fields: @new_fields");
    @xtoss_fields = @new_fields;
	# End filtering of field names
	
	# unshift @xtoss_fields, 'id';
	my $xtoss_field_clause = join(", ", @xtoss_fields);
	my $xtoss_placeholders = join(", ", map { '?' } @xtoss_fields);
	

	# xtossql: *_stats / insert
		$self->{"xtoss_${xtoss_table}_insert"} =
			$dbh->prepare("INSERT INTO $xtoss_table ($xtoss_field_clause) " .
			"VALUES ($xtoss_placeholders);");

#		print "statement: "."INSERT INTO $xtoss_table ($xtoss_field_clause) VALUES ($xtoss_placeholders);"."\n";
	}

	$self->_prepare_team_stats;
	$self->_prepare_player_stats;
}

# TODO delete
sub _prepare_team_stats {
	my ($self) = @_;

	my $dbh = $self->{'dbh'};

	# xtossql: stats / delete -- for teams
	$self->{'xtoss_stats_team_delete'} = $dbh->prepare(q{
		DELETE FROM stats
		WHERE  stat_holder_type='teams' AND stat_holder_id=?
		AND stat_coverage_type=? AND stat_coverage_id=?
		AND stat_repository_type=?
		AND (
			(stat_membership_type IS NULL AND stat_membership_id IS NULL) OR
			(stat_membership_type = ? and stat_membership_id = ?) 
		)
		AND context=?
		});

	# xtossql: stats / insert -- for teamss
	# fixit: add in support for column named 'final'
	$self->{'xtoss_stats_team_insert'} =
		$dbh->prepare(q{INSERT INTO stats (
		stat_holder_type, 
		stat_holder_id, 
		stat_coverage_type, 
		stat_coverage_id,
		stat_repository_type, 
		stat_repository_id, 
		stat_membership_type, 
		stat_membership_id, 
		context
		)
		VALUES ('teams', ?, ?, ?, ?, ?, ?, ?, ?)});

	# xtossql: stats / select -- for teams
	$self->{'xtoss_stats_team_select'} =
		$dbh->prepare("SELECT stat_repository_type, stat_repository_id " .
		"FROM stats " .
		"WHERE  stat_holder_type='teams' AND stat_holder_id=? " .
		"AND stat_coverage_type=? AND stat_coverage_id=?");
}

# TODO delete
sub _prepare_player_stats {
	my ($self) = @_;

	my $dbh = $self->{'dbh'};

	# xtossql: stats / delete -- for players
	$self->{'xtoss_stats_player_delete'} = $dbh->prepare(q{
		DELETE FROM stats
		WHERE  stat_holder_type='persons' AND stat_holder_id=?
		AND stat_coverage_type=? AND stat_coverage_id=?
		AND stat_repository_type=?
		AND (
			(stat_membership_type IS NULL AND stat_membership_id IS NULL) OR
			(stat_membership_type = ? AND stat_membership_id = ?) 
		)
		AND context=?
		});

	# xtossql: stats / insert -- for players
	# fixit: add in support for column named 'final'
	$self->{'xtoss_stats_player_insert'} =
		$dbh->prepare(q{INSERT INTO stats (
		stat_holder_type, 
		stat_holder_id, 
		stat_coverage_type, 
		stat_coverage_id,
		stat_repository_type, 
		stat_repository_id, 
		stat_membership_type, 
		stat_membership_id,
		context
		)
		VALUES ('persons', ?, ?, ?, ?, ?, ?, ?, ?)});

	# TODO: Delete
	# xtossql: stats / select -- for players
	$self->{'xtoss_stats_player_select'} =
		$dbh->prepare("SELECT stat_repository_type, stat_repository_id " .
		"FROM stats " .
		"WHERE  stat_holder_type='persons' AND stat_holder_id=? " .
		"AND stat_coverage_type=? AND stat_coverage_id=?");

}

# TODO delete
sub map_team_stat {
	my ($self, $team_id, $stat_coverage_type, $stat_coverage_id, 
		$stat_repository_type, $stat_repository_id, $stat_membership_type, $stat_membership_id, $context
	) = @_;
	
#	my $tests = {
#		'stat_holder_type'=>'teams',
#		'stats_holder_id'=>$team_id,
#		'stat_coverage_type'=>$stat_coverage_type,
#		'stat_coverage_id'=>$stat_coverage_id,
#		'stat_repository_type'=>$stat_repository_type,
#		'stat_repository_id'=>$stat_repository_id,
#		'context'=>$context
#	};
#    my $stats_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, 'stats');
#    $stats_record_mapper->retrieve(\%tests);
#    my $mapper_action = $stats_record_mapper->get_action();
#    $self->{log}->log("[DBG] mapper action: $mapper_action");

	# Delete the old, if there is one. Still makes sense. (actually, probably overkill)
	# The removal of data in the stats table and the table pointed to it is done in
	# delete_stats_by_metadata_group
	$self->{'xtoss_stats_team_delete'}->execute(
		$team_id, 
		$stat_coverage_type, 
		$stat_coverage_id, 
		$stat_repository_type, 
		$stat_membership_type, 
		$stat_membership_id, 
		$context
	);

	# foo here is where we insert the stat_repository_type
	$self->{'xtoss_stats_team_insert'}->execute(
		$team_id, 
		$stat_coverage_type, 
		$stat_coverage_id, 
		$stat_repository_type, 
		$stat_repository_id, 
		$stat_membership_type, 
		$stat_membership_id,
		$context
	);
}

# TODO delete
sub map_player_stat {
	my ($self, $player_id, $stat_coverage_type, $stat_coverage_id,
		$stat_repository_type, $stat_repository_id, $stat_membership_type, $stat_membership_id, $context) = @_;
		
#	my ($attr_name, $attr) = $self->{xtutil}->get_xtoss_column_info('stats-coverage-type', $stat_coverage_type, '', 'stats');
#	my $stats_coverage_type = $attr;
   my $stats_coverage_type = $stat_coverage_type;

	# stat_holder_type: the holder of the stats - e.g. persons
	# stat_holder_id: the id for the holder
	# context: the context of the stat: e.g. career, event, event-play
	# stat_coverage_id: the id for the event or season etc.
	# membership_type: teams, players
	# membership_id: id for membership_type

	# Delete the old, if there is one. Still makes sense. (actually, probably overkill)
	# The removal of data in the stats table and the table pointed to it is done in
	# delete_stats_by_metadata_group
	$self->{'xtoss_stats_player_delete'}->execute(
		$player_id, 
		$stat_coverage_type,
		$stat_coverage_id, 
		$stat_repository_type, 
		$stat_membership_type, 
		$stat_membership_id, 
		$context
	);

	# Here is where we insert the stat_repository_type into the stats table
	$self->{'xtoss_stats_player_insert'}->execute(
		$player_id,
		$stats_coverage_type, 
		$stat_coverage_id, 
		$stat_repository_type, 
		$stat_repository_id, 
		$stat_membership_type, 
		$stat_membership_id,
		$context
	);
}

# TODO delete - probably
# Delete stats from the table pointed to in the stats table (e.g. american_football_defensive_stats)
sub delete_stats_by_metadata_group {
	my ($self, $stat_holder_type, $stat_holder_id, $stat_coverage_type, $stat_coverage_id, $stat_repository_type, $membership_type, $membership_id, $context) = @_;

	my $table = $stat_repository_type;

	# NOTE: the variable "context" was "stat_coverage_type", but this did not match the query, which deletes based on
	# the context value (e.g. event vs. events)
	
	# Find by:
	# stat_holder_type: the holder of the stats - e.g. persons
	# stat_holder_id: the id for the holder
	# context: the context of the stat: e.g. career, event, event-play
	# stat_coverage_id: the id for the event or season etc.
	# membership_type: teams, players
	# membership_id: id for membership_type
	$self->{"xtoss_${table}_select_by_metadata"}->execute(
		$stat_holder_type,
		$stat_holder_id,
		$stat_coverage_type,
		$stat_coverage_id,
		$membership_type,
		$membership_id,
		$context
	);

	my $id;
	while (my @row = $self->{"xtoss_${table}_select_by_metadata"}->fetchrow_array()) {
#		$self->{log}->log("[DBG] ** DELETING for ${table}");
		$id = $row[0];
#		$self->{log}->log("[DBG] ** ID: $id");
		# Delete the target table row
		$self->{"xtoss_${table}_delete_by_metadata"}->execute($id);
		# Delete the stats table row now that it's pointing to a non-existent row
		$self->{"xtoss_stats_delete_from_metadata_select"}->execute($table, $id);
	}
	$self->{"xtoss_${table}_select_by_metadata"}->finish;

#	$self->{"xtoss_${table}_delete_by_metadata"}->execute($stat_holder_type, $stat_holder_id, $context, $stat_coverage_id);
}

sub store_stats_group {
	my ($self, $group, $rec) = @_;
	
	my $table = $group;

#    my $stats_record_mapper = FeedfetcherDeluxe::Entities::MetaRecordMapper->new($self->{dbh}, $self->{log}, $table);
#    $stats_record_mapper->retrieve(\%tests);
#    my $mapper_action = $stats_record_mapper->get_action();
#    $self->{log}->log("[DBG] mapper action: $mapper_action");
	
	my @rec = map { defined $_ && length($_) == 0 ? undef : $_ } @$rec;
	
	my $xtoss_table = $self->{xtutil}->get_xtoss_name($table);
	
	eval {
		my $rv =	$self->{"xtoss_${xtoss_table}_insert"}->execute(@rec);
	};
	if ($@) {
		$self->{'log'}->log("[DBG] Insert to $xtoss_table failed. Storage will continue for other data. Here is the error: $@");
    	return (undef, undef);
  	}
	
	# fixit now: below must be filled with the ID value of the specific stats row we just inserted.
	# my $stat_repository_id = $dbh->last_insert_id();
	
#	$self->{"latest_xtoss_${xtoss_table}"}->execute();
#	my @new_specific_stat_id = $self->{"latest_xtoss_${xtoss_table}"}->fetchrow_array();
#	$self->{"latest_xtoss_${xtoss_table}"}->finish;
#
#	my $mynewid = $new_specific_stat_id[0];
    my $mynewid = Core::DbUtil::retrieve_max_id($self->{dbh}, ${xtoss_table});
    
	return ($table, $mynewid);
}

1;