# $Id$

use strict;
package Util::FileUtils;
use Cwd;


sub new {
    my ($class) = @_;

    my $self = {};
    bless $self, $class;

    # Windows Perl should return somthing like MSWin32, so 'mswin' should do it
    # Linux should be 'linux' and OSX should be 'darwin' - both are unix separators
    my $sep = ($^O =~ /mswin/i) ? '\\' : '/';
    $self->{cwd} = getcwd();
    $self->{sep} = $sep;
        
    return $self;   
}

# Get base path for script
sub get_base_path {
    my ($self) = @_;
    
    return $self->{cwd}
}

# Clean a path by ensuring its separators characters are OS-valid
sub clean_path {
    my ($self, $path) = @_;

    my $sep = $self->{sep};
    my $un_sep = ($self->{sep} eq '/') ? "\\\\" : "\\/";
    $path =~ s/$un_sep/$sep/g;
    $path =~ s/$sep{2,}/$sep/g;

    return $path;
}

# Read a file into a string
sub read_file {
    my ($self, $path) = @_;
    
    my $text;
    {
        local( $/, *FH );
        open( FH, $path ) or print "Cannot open $path\n";
        $text = <FH>;
        close FH;
    }
    return $text;
}

# Look through a set of allow and deny patterns, sent in as comma-delimited strings
# and decide whethe ther filename in question passes or fails
sub filter_fname {
    my ($self, $fname, $allow_str, $deny_str) = @_;
    
    # Don't let NOTHING through if no allow filters defined
    $allow_str = '.*' unless defined $allow_str;
    $allow_str =~ s/^\s*(.*?)\s$/$1/;
    my @filters_allow = split(/\s*\,\s*/, $allow_str);
    
    $deny_str =~ s/^\s*(.*?)\s$/$1/;
    my @filters_deny = split(/\s*\,\s*/, $deny_str);
    
    my $keep = 0;
    
    for my $filter (@filters_allow) {
        if ($fname =~ /$filter/) {
            $keep = 1;
            last;
        }
    }
    for my $filter (@filters_deny) {
        if ($fname =~ /$filter/) {
            $keep = 0;
            return $keep;
        }
    }
    return $keep;
}

1;