<?xml version="1.0"?>

<xsl:stylesheet
	version="1.0"
	extension-element-prefixes="redirect"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:redirect="org.apache.xalan.xslt.extensions.Redirect"
	>


<xsl:strip-space elements="*"/>

<xsl:output method="html"/>

<xsl:template match="/">
	<html>
	<head>
	<xsl:text disable-output-escaping="yes"><![CDATA[<link href="/resources/sportsml.css" type="text/css" rel="stylesheet"/>]]></xsl:text>
	</head>
	<body>

	<xsl:choose>
		<xsl:when test="/*/@error-count='1'">
			<h2 class="error-count">1 error was recorded.</h2>
		</xsl:when>
		<xsl:when test="/*/@error-count &gt; '1'">
			<h2 class="error-count"><xsl:value-of select="/*/@error-count"/> errors were recorded.</h2>
		</xsl:when>
	</xsl:choose>

	<xsl:choose>
		<xsl:when test="/*/@result-count='0'">
			<h2 class="result-count">No matches found in database.</h2>
		</xsl:when>
		<xsl:when test="/*/@result-count='1' and /*/@result-count=/*/@total-count">
			<h2 class="result-count">The 1 matching document was returned:</h2>
		</xsl:when>
		<xsl:when test="/*/@result-count &gt; '1' and /*/@result-count=/*/@total-count">
			<h2 class="result-count">All <xsl:value-of select="/*/@result-count"/> matching documents were returned:</h2>
		</xsl:when>
		<xsl:otherwise>
			<h2 class="result-count"><xsl:value-of select="/*/@total-count"/> total matches were found in the database.<br/>
				The most recent <xsl:value-of select="/*/@result-count"/> documents were returned:</h2>
		</xsl:otherwise>
	</xsl:choose>

	<!--
		If error-count == 1
			1 error was recorded.
		elseif error-count > 1
			X errors were recorded.
		
		If result-count == 0
			No matches found in database.
		Else If result-count == total-count == 1
			The 1 matching document was returned:
		Else If result-count == total-count > 1
			All RESULT-COUNT matching documents were returned:
		Else
			TOTAL-COUNT total matches were found in the database.
			The most recent RESULT-COUNT documents were returned:
	-->
	
	<table border="1" cellpadding="4" width="100%" class="listing">
		<xsl:apply-templates />
	</table>
	</body>
	</html>

</xsl:template>

<xsl:template match="sports-content">
	<tr valign="top">
		<td><b><xsl:value-of select="sports-metadata/sports-title"/></b><br/>
		<font size="-1"><xsl:value-of select="article/nitf/body/body.head/abstract/p"/></font>
		</td>
		<td nowrap="nowrap" class="date">
			<xsl:call-template name="formatted-date-time">
				<xsl:with-param name="date-value" select="sports-metadata/@date-time"/>
			</xsl:call-template>
		</td>
		<td class="link">
			<xsl:element name="a">
				<xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="sports-metadata/@doc-id"/>&amp;stylesheet=sportsml2html</xsl:attribute>
				HTML
			</xsl:element>
		</td>
		<td class="link">
			<xsl:element name="a">
				<xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="sports-metadata/@doc-id"/></xsl:attribute>
				SportsML
			</xsl:element>
		</td>
	</tr>
</xsl:template>

<xsl:template name="formatted-date-time">
	<xsl:param name="date-value"/>
	<xsl:variable name="hour">
		<xsl:value-of select="substring($date-value,10,2)"/>
	</xsl:variable>
	<xsl:variable name="month">
		<xsl:value-of select="substring($date-value,5,2)"/>
	</xsl:variable>
	<xsl:variable name="temp-date">
		<xsl:choose>
			<xsl:when test="$hour = 0">12</xsl:when>
			<xsl:when test="$hour &lt; 10"><xsl:value-of select="$hour - 0"/></xsl:when>
			<xsl:when test="$hour &lt; 13"><xsl:value-of select="$hour - 0"/></xsl:when>
			<xsl:when test="$hour &lt; 22"><xsl:value-of select="$hour - 12"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$hour - 12"/></xsl:otherwise>
		</xsl:choose>:<xsl:value-of select="substring($date-value,12,2)"/>
		<xsl:choose><xsl:when test="$hour &lt; 12">am</xsl:when>
			<xsl:otherwise>pm</xsl:otherwise>
		</xsl:choose>,<xsl:text> </xsl:text><xsl:choose>
			<xsl:when test="$month = 1">Jan</xsl:when>
			<xsl:when test="$month = 2">Feb</xsl:when>
			<xsl:when test="$month = 3">Mar</xsl:when>
			<xsl:when test="$month = 4">Apr</xsl:when>
			<xsl:when test="$month = 5">May</xsl:when>
			<xsl:when test="$month = 6">Jun</xsl:when>
			<xsl:when test="$month = 7">Jul</xsl:when>
			<xsl:when test="$month = 8">Aug</xsl:when>
			<xsl:when test="$month = 9">Sep</xsl:when>
			<xsl:when test="$month = 10">Oct</xsl:when>
			<xsl:when test="$month = 11">Nov</xsl:when>
			<xsl:when test="$month = 12">Dec</xsl:when>
			<xsl:otherwise>error: <xsl:value-of select="$month"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<xsl:value-of select="substring($date-value,7,2)-0"/>
	</xsl:variable>

	<xsl:value-of select="$temp-date"/>
	<!--
	<xsl:value-of select="concat(substring($date-value,10,2),':',substring($date-value,12,2),', ',substring($date-value,5,2),'/',substring($date-value,7,2)-0,'/',substring($date-value,1,4))"/>
	-->
	
</xsl:template>

</xsl:stylesheet>