<?xml version="1.0"?>

<xsl:stylesheet
	version="1.0"
	extension-element-prefixes="redirect"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:redirect="org.apache.xalan.xslt.extensions.Redirect"
	>


<xsl:strip-space elements="*"/>

<xsl:output method="html"/>

<xsl:template match="/">
	<html>
	<head>
	<xsl:text disable-output-escaping="yes"><![CDATA[<link href="/resources/sportsml.css" type="text/css" rel="stylesheet"/>]]></xsl:text>
	</head>
	<body>

	<xsl:choose>
		<xsl:when test="/*/@error-count='1'">
			<h4 class="error-count">1 error was recorded.</h4>
		</xsl:when>
		<xsl:when test="/*/@error-count &gt; '1'">
			<h4 class="error-count"><xsl:value-of select="/*/@error-count"/> errors were recorded.</h4>
		</xsl:when>
	</xsl:choose>

	<xsl:choose>
		<xsl:when test="/*/@result-count='0'">
			<h4 class="result-count">No matches found in database.</h4>
		</xsl:when>
		<xsl:when test="/*/@result-count='1' and /*/@result-count=/*/@total-count">
			<h4 class="result-count">The 1 matching linescore was returned:</h4>
		</xsl:when>
		<xsl:when test="/*/@result-count &gt; '1' and /*/@result-count=/*/@total-count">
			<h4 class="result-count">All <xsl:value-of select="/*/@result-count"/> matching linescores were returned:</h4>
			<!--
			<h5>Note: Currently fixing a bug in scoreboards. They may be a bit off in the next few days.</h5>
			-->
		</xsl:when>
		<xsl:otherwise>
			<h4 class="result-count"><xsl:value-of select="/*/@total-count"/> total linescores were found in the database.<br/>
				The most recent <xsl:value-of select="/*/@result-count"/> linescores were returned:</h4>
			<!--
			<h5>Note: Currently fixing a bug in scoreboards. They may be a bit off in the next few days.</h5>
			-->
		</xsl:otherwise>
	</xsl:choose>

	<table border="0" cellpadding="3">
		<xsl:apply-templates />
	</table>
	</body>
	</html>

</xsl:template>

<xsl:template match="sports-content">
		<tr class="scoreHeading">
			<td class="dateLabel">
				<xsl:call-template name="formatted-date-time">
					<xsl:with-param name="date-value" select="sports-event/event-metadata/@start-date-time"/>
				</xsl:call-template>
			</td>
			<xsl:for-each select="sports-event/team[1]/team-stats/sub-score">
				<td class="periodLabel"><xsl:value-of select="@period-value"/></td>
			</xsl:for-each>
			<xsl:if test="sports-event/event-metadata/@event-status!='pre-event'">
				<xsl:for-each select="sports-event/team[1]/team-stats/@score">
					<td class="totalLabel">Total</td>
				</xsl:for-each>
			</xsl:if>
		</tr>
		<xsl:for-each select="sports-event/team">
			<tr class="scoreData">
			<td class="teamName"><b><xsl:value-of select="team-metadata/name/@first"/><xsl:text> </xsl:text><xsl:value-of select="team-metadata/name/@last"/></b></td>
			<xsl:for-each select="team-stats/sub-score">
				<td class="subScore"><xsl:value-of select="@score"/></td>
			</xsl:for-each>
			<xsl:if test="../event-metadata/@event-status!='pre-event'">
				<xsl:for-each select="team-stats">
					<td class="totalScore"><xsl:value-of select="@score"/></td>
				</xsl:for-each>
			</xsl:if>
			</tr>
		</xsl:for-each>
		<tr class="scoreLinks"><td colspan="4">
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
					<xsl:when test="@formal-name='pre-event-coverage'">
						<a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>preview</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
					<xsl:when test="@formal-name='matchup'">
						<xsl:text> | </xsl:text>
						<a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>matchup</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			<!--
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
                                        <xsl:when test="@formal-name='lineup'">
                                                <xsl:text> | </xsl:text>
                                                <a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>lineup</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			-->
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
					<xsl:when test="@formal-name='post-event-coverage'">
						<xsl:text> | </xsl:text>
						<a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>writeup</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
			                <xsl:when test="@formal-name='event-stats'">
                                                <xsl:text> | </xsl:text>
                                                <a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>box score</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			<xsl:for-each select="sports-event/event-metadata/sports-property">
				<xsl:choose>
                                        <xsl:when test="@formal-name='event-stats-composite'">
                                                <xsl:text> | </xsl:text>
                                                <a><xsl:attribute name="href">getDocuments.php?doc-ids=<xsl:value-of select="@value"/>&amp;stylesheet=sportsml2html</xsl:attribute>composite box score</a>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</td></tr>
		<tr class="scoreSpacer"><td colspan="4">
			<xsl:text> </xsl:text>
		</td></tr>
</xsl:template>

<xsl:template name="formatted-date-time">
	<xsl:param name="date-value"/>
	<xsl:variable name="hour">
		<xsl:value-of select="substring($date-value,10,2)"/>
	</xsl:variable>
	<xsl:variable name="month">
		<xsl:value-of select="substring($date-value,5,2)"/>
	</xsl:variable>
	<xsl:variable name="temp-date">
		<xsl:choose>
			<xsl:when test="$hour = 0">12</xsl:when>
			<xsl:when test="$hour &lt; 10"><xsl:value-of select="$hour - 0"/></xsl:when>
			<xsl:when test="$hour &lt; 13"><xsl:value-of select="$hour - 0"/></xsl:when>
			<xsl:when test="$hour &lt; 22"><xsl:value-of select="$hour - 12"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$hour - 12"/></xsl:otherwise>
		</xsl:choose>:<xsl:value-of select="substring($date-value,12,2)"/>
		<xsl:choose><xsl:when test="$hour &lt; 12">am</xsl:when>
			<xsl:otherwise>pm</xsl:otherwise>
		</xsl:choose>,<xsl:text> </xsl:text><xsl:choose>
			<xsl:when test="$month = 1">January</xsl:when>
			<xsl:when test="$month = 2">February</xsl:when>
			<xsl:when test="$month = 3">March</xsl:when>
			<xsl:when test="$month = 4">April</xsl:when>
			<xsl:when test="$month = 5">May</xsl:when>
			<xsl:when test="$month = 6">June</xsl:when>
			<xsl:when test="$month = 7">July</xsl:when>
			<xsl:when test="$month = 8">August</xsl:when>
			<xsl:when test="$month = 9">September</xsl:when>
			<xsl:when test="$month = 10">October</xsl:when>
			<xsl:when test="$month = 11">November</xsl:when>
			<xsl:when test="$month = 12">December</xsl:when>
			<xsl:otherwise>error: <xsl:value-of select="$month"/></xsl:otherwise>
		</xsl:choose>
		<xsl:text> </xsl:text>
		<xsl:value-of select="substring($date-value,7,2)-0"/>
	</xsl:variable>

	<xsl:value-of select="$temp-date"/>
	<!--
	<xsl:value-of select="concat(substring($date-value,10,2),':',substring($date-value,12,2),', ',substring($date-value,5,2),'/',substring($date-value,7,2)-0,'/',substring($date-value,1,4))"/>
	-->
	
</xsl:template>

</xsl:stylesheet>
